@@ -28,7 +28,6 @@
 #ifndef CXXTOOLS_APPLICATION_H
 #define CXXTOOLS_APPLICATION_H
 
-#include <cxxtools/api.h>
 #include <cxxtools/eventloop.h>
 #include <cxxtools/application.h>
 #include <cxxtools/connectable.h>
@@ -62,11 +61,11 @@ namespace cxxtools {
      *
      * There are convenience methods available for easier access to functionality of
      * the underlying event loop. commitEvent() delegates to EventLoop::commitEvent(),
-     * queueEvent() delegates to EventLoop::delegateEvent() and processEvents() delegates
+     * queueEvent() delegates to EventLoop::queueEvent() and processEvents() delegates
      * to EventLoop::processEvents() without making it necessary to first obtain the
      * event loop manually.
      */
-    class CXXTOOLS_API Application : public Connectable
+    class Application : public Connectable
     {
             void construct();
 
@@ -101,12 +100,36 @@ namespace cxxtools {
             int argc() const
             { return _argc; }
 
-            char** argv() const
+            int& argc()
+            { return _argc; }
+
+            const char* const* argv() const
+            { return _argv; }
+
+            char** argv()
             { return _argv; }
 
             ApplicationImpl& impl()
             { return *_impl; }
 
+            /// Puts a event in the queue but does not wake the loop.
+            void queueEvent(const Event& event)
+            { loop().queueEvent(event); }
+
+            /// Puts a event in the queue and wakes the loop so that the event is processed.
+            void commitEvent(const Event& event)
+            { loop().commitEvent(event); }
+
+            /// Puts a priority event in the queue but does not wake the loop.
+            /// Priority events are processed before any non priority event.
+            void queuePriorityEvent(const Event& event)
+            { loop().queuePriorityEvent(event); }
+
+            /// Puts a priority event in the queue and wakes the loop so that the event is processed.
+            /// Priority events are processed before any non priority event.
+            void commitPriorityEvent(const Event& event)
+            { loop().commitPriorityEvent(event); }
+
         protected:
             void init(EventLoopBase& loop);
 
