@@ -28,7 +28,7 @@
 #ifndef CXXTOOLS_CHAR_H
 #define CXXTOOLS_CHAR_H
 
-#include <cxxtools/api.h>
+#include <cxxtools/config.h>
 #include <string>
 #include <cstring>
 #include <ios>
@@ -72,23 +72,8 @@ namespace cxxtools
             : _value(0)
             {}
 
-            //! Constructs a character using the given char as base for the character value.
-            Char(char ch)
-            : _value(value_type(static_cast<unsigned char>(ch)))
-            {}
-
-            //! Constructs a character using the given char as base for the character value.
-            Char(unsigned char ch)
-            : _value( value_type(ch) )
-            {}
-
-            //! Constructs a character using the given char as base for the character value.
-            Char(wchar_t ch)
-            : _value( value_type(static_cast<uint16_t>(ch)) )
-            {}
-
-            //! Constructs a character using the given unsigned 32-bit as base for the character value.
-            explicit Char(value_type ch)
+            //! Constructs a character using the given value as base for the character value.
+            Char(value_type ch)
             : _value(ch)
             {}
 
@@ -114,9 +99,6 @@ namespace cxxtools
                 return Char(0);
             }
 
-            Char& operator=(const Char& ch)
-            { _value = ch._value; return *this; }
-
             /**
              * @brief Returns the internal value (unsigned 32 bits) of this character.
              * @return The 32-bit-value of this character.
@@ -140,104 +122,106 @@ namespace cxxtools
             friend bool operator==(const Char& a, const Char& b)
             { return a.value() == b.value(); }
             friend bool operator==(const Char& a, wchar_t b)
-            { return a.value() == b; }
+            { return a.value() == static_cast<value_type>(b); }
             friend bool operator==(wchar_t a, const Char& b)
-            { return a == b.value(); }
+            { return static_cast<value_type>(a) == b.value(); }
             friend bool operator==(const Char& a, char b)
-            { return a.value() == b; }
+            { return a.value() == static_cast<value_type>(b); }
             friend bool operator==(char a, const Char& b)
-            { return a == b.value(); }
+            { return static_cast<value_type>(a) == b.value(); }
 
             //! @brief Returns $true$ if the a and b are not the same character; $false$ otherwise.
             //! @return $true$ if the a and b are not the same character; $false$ otherwise.
             friend bool operator!=(const Char& a, const Char& b)
             { return a.value() != b.value(); }
             friend bool operator!=(const Char& a, wchar_t b)
-            { return a.value() != b; }
+            { return a.value() != static_cast<value_type>(b); }
             friend bool operator!=(wchar_t a, const Char& b)
-            { return a != b.value(); }
+            { return static_cast<value_type>(a) != b.value(); }
             friend bool operator!=(const Char& a, char b)
-            { return a.value() != b; }
+            { return a.value() != static_cast<value_type>(b); }
             friend bool operator!=(char a, const Char& b)
-            { return a != b.value(); }
+            { return static_cast<value_type>(a) != b.value(); }
 
             //! @brief Returns $true$ if the numeric value of a is less than the numeric value of b; $false$ otherwise.
             //! @return $true$ if the numeric value of a is less than the numeric value of b; $false$ otherwise.
             friend bool operator<(const Char& a, const Char& b)
             { return a.value() < b.value(); }
             friend bool operator<(const Char& a, wchar_t b)
-            { return a.value() < b; }
+            { return a.value() < static_cast<value_type>(b); }
             friend bool operator<(wchar_t a, const Char& b)
-            { return a < b.value(); }
+            { return static_cast<value_type>(a) < b.value(); }
             friend bool operator<(const Char& a, char b)
-            { return a.value() < b; }
+            { return a.value() < static_cast<value_type>(b); }
             friend bool operator<(char a, const Char& b)
-            { return a < b.value(); }
+            { return static_cast<value_type>(a) < b.value(); }
 
             //! @brief Returns $true$ if the numeric value of a is greater than the numeric value of b; $false$ otherwise.
             //! @return $true$ if the numeric value of a is greater than the numeric value of b; $false$ otherwise.
             friend bool operator>(const Char& a, const Char& b)
             { return a.value() > b.value(); }
             friend bool operator>(const Char& a, wchar_t b)
-            { return a.value() > b; }
+            { return a.value() > static_cast<value_type>(b); }
             friend bool operator>(wchar_t a, const Char& b)
-            { return a > b.value(); }
+            { return static_cast<value_type>(a) > b.value(); }
             friend bool operator>(const Char& a, char b)
-            { return a.value() > b; }
+            { return a.value() > static_cast<value_type>(b); }
             friend bool operator>(char a, const Char& b)
-            { return a > b.value(); }
+            { return static_cast<value_type>(a) > b.value(); }
 
             //! @brief Returns $true$ if the numeric value of a is equal or less than the numeric value of b; $false$ otherwise.
             //! @return $true$ if the numeric value of a is equal or less than the numeric value of b; $false$ otherwise.
             friend  bool operator<=(const Char& a, const Char& b)
             { return a.value() <= b.value(); }
             friend  bool operator<=(const Char& a, wchar_t b)
-            { return a.value() <= b; }
+            { return a.value() <= static_cast<value_type>(b); }
             friend  bool operator<=(wchar_t a, const Char& b)
-            { return a <= b.value(); }
+            { return static_cast<value_type>(a) <= b.value(); }
             friend  bool operator<=(const Char& a, char b)
-            { return a.value() <= b; }
+            { return a.value() <= static_cast<value_type>(b); }
             friend  bool operator<=(char a, const Char& b)
-            { return a <= b.value(); }
+            { return static_cast<value_type>(a) <= b.value(); }
 
             //! @brief Returns $true$ if the numeric value of a is equals or greater than the numeric value of b; $false$ otherwise.
             //! @return $true$ if the numeric value of a is equals or greater than the numeric value of b; $false$ otherwise.
             friend  bool operator>=(const Char& a, const Char& b)
             { return a.value() >= b.value(); }
             friend  bool operator>=(const Char& a, wchar_t b)
-            { return a.value() >= b; }
+            { return a.value() >= static_cast<value_type>(b); }
             friend  bool operator>=(wchar_t a, const Char& b)
-            { return a >= b.value(); }
+            { return static_cast<value_type>(a) >= b.value(); }
             friend  bool operator>=(const Char& a, char b)
-            { return a.value() >= b; }
+            { return a.value() >= static_cast<value_type>(b); }
             friend  bool operator>=(char a, const Char& b)
-            { return a >= b.value(); }
+            { return static_cast<value_type>(a) >= b.value(); }
 
         private:
             value_type _value;
     };
 
+    /// @cond internal
     struct MBState
     {
         MBState()
         : n(0)
         {}
 
-        int n;
+        uint8_t n;
         union {
             Char::value_type wchars[4];
             char mbytes[16];
         } value;
     };
+    /// @endcond internal
 
-    CXXTOOLS_API std::ostream& operator<< (std::ostream& out, Char ch);
+    std::ostream& operator<< (std::ostream& out, Char ch);
 
 } // namespace cxxtools
 
 
 namespace std {
 
-    /// @cond INTERNAL
+    /// @cond internal
     template<>
     struct char_traits<cxxtools::Char>
     {
@@ -275,6 +259,7 @@ namespace std {
 
         inline static int_type not_eof(const int_type& c);
     };
+    /// @endcond internal
 
     inline void char_traits<cxxtools::Char>::assign(char_type& c1, const char_type& c2)
     {
@@ -336,14 +321,14 @@ namespace std {
     inline char_traits<cxxtools::Char>::char_type*
     char_traits<cxxtools::Char>::move(char_type* s1, const char_type* s2, int_type n)
     {
-        return (cxxtools::Char*)std::memmove(s1, s2, n * sizeof(cxxtools::Char));
+        return static_cast<cxxtools::Char*>(std::memmove(static_cast<void*>(s1), static_cast<const void*>(s2), n * sizeof(cxxtools::Char)));
     }
 
 
     inline char_traits<cxxtools::Char>::char_type*
     char_traits<cxxtools::Char>::copy(char_type* s1, const char_type* s2, size_t n)
     {
-        return (cxxtools::Char*)std::memcpy(s1, s2, n * sizeof(cxxtools::Char));
+        return static_cast<cxxtools::Char*>(std::memcpy(static_cast<void*>(s1), static_cast<const void*>(s2), n * sizeof(cxxtools::Char)));
     }
 
 
@@ -421,14 +406,14 @@ namespace std {
         @ingroup Unicode
     */
     template <>
-    class CXXTOOLS_API ctype< cxxtools::Char > : public ctype_base {
+    class ctype< cxxtools::Char > : public ctype_base {
 
 #else
     /** @brief Ctype localization facet
         @ingroup Unicode
     */
     template <>
-    class CXXTOOLS_API ctype<cxxtools::Char> : public ctype_base, public locale::facet {
+    class ctype<cxxtools::Char> : public ctype_base, public locale::facet {
 
 #endif
 
@@ -546,7 +531,7 @@ class ctype_base
 
 namespace cxxtools {
 
-CXXTOOLS_API std::ctype_base::mask ctypeMask(const Char& ch);
+std::ctype_base::mask ctypeMask(const Char& ch);
 
 inline int isalpha(const Char& ch)
 {
@@ -603,9 +588,9 @@ inline int isspace(const Char& ch)
     return ctypeMask(ch) & std::ctype_base::space;
 }
 
-CXXTOOLS_API Char tolower(const Char& ch);
+Char tolower(const Char& ch);
 
-CXXTOOLS_API Char toupper(const Char& ch);
+Char toupper(const Char& ch);
 
 } // namespace cxxtools
 
