@@ -26,7 +26,6 @@
 #ifndef CXXTOOLS_CLOCK_H
 #define CXXTOOLS_CLOCK_H
 
-#include <cxxtools/api.h>
 #include <cxxtools/datetime.h>
 #include <cxxtools/timespan.h>
 
@@ -38,34 +37,40 @@ namespace cxxtools
     The clock class can be used like a stop-watch by calling Clock::start()
     and Clock::stop(). The latter method returns the elapsed time.
 */
-class CXXTOOLS_API Clock
+class Clock
 {
     public:
         /** @brief Constructs a Clock
         */
         Clock();
 
+        Clock(const Clock& cl);
+
+        Clock& operator=(const Clock& cl);
+
         /** @brief Destructor
         */
-        ~Clock();
+        virtual ~Clock();
 
-        /** @brief Start the clock
+        /** @brief Start the clock and return the current system ticks.
         */
-        void start();
+        Timespan start();
 
-        /** @brief Stop the clock
+        /** @brief Stop the clock. Can be called several times.
 
-            Returns the elapsed time since start was called.
+            Returns the elapsed time since start was called. Since stop does not
+            change the start time it can be called several times in order to get
+            the elapsed time.
         */
-        Timespan stop();
+        Timespan stop() const;
 
         /** @brief Returns the system time
         */
-        static DateTime getSystemTime();
+        static UtcDateTime getSystemTime();
 
         /** @brief Returns the current local time
         */
-        static DateTime getLocalTime();
+        static LocalDateTime getLocalTime();
 
         /** @brief Returns the timespan since a fixed point in the past
 
