@@ -30,7 +30,6 @@
 #ifndef CXXTOOLS_CONVERT_H
 #define CXXTOOLS_CONVERT_H
 
-#include <cxxtools/api.h>
 #include <cxxtools/config.h>
 #include <cxxtools/string.h>
 #include <cxxtools/stringstream.h>
@@ -58,26 +57,32 @@ inline void convert(String& s, const String& str)
     s = str;
 }
 
-CXXTOOLS_API void convert(String& s, const std::string& value);
+void convert(String& s, const std::string& value);
 
-CXXTOOLS_API void convert(String& s, bool value);
+void convert(String& s, bool value);
 
-CXXTOOLS_API void convert(String& s, char value);
-CXXTOOLS_API void convert(String& s, wchar_t value);
-CXXTOOLS_API void convert(String& s, Char value);
-CXXTOOLS_API void convert(String& s, unsigned char value);
-CXXTOOLS_API void convert(String& s, signed char value);
+void convert(String& s, char value);
+void convert(String& s, wchar_t value);
+void convert(String& s, Char value);
+void convert(String& s, unsigned char value);
+void convert(String& s, signed char value);
 
-CXXTOOLS_API void convert(String& s, short value);
-CXXTOOLS_API void convert(String& s, unsigned short value);
-CXXTOOLS_API void convert(String& s, int value);
-CXXTOOLS_API void convert(String& s, unsigned int value);
-CXXTOOLS_API void convert(String& s, long value);
-CXXTOOLS_API void convert(String& s, unsigned long value);
+void convert(String& s, short value);
+void convert(String& s, unsigned short value);
+void convert(String& s, int value);
+void convert(String& s, unsigned int value);
+void convert(String& s, long value);
+void convert(String& s, unsigned long value);
+#ifdef HAVE_LONG_LONG
+void convert(String& s, long long value);
+#endif
+#ifdef HAVE_UNSIGNED_LONG_LONG
+void convert(String& s, unsigned long long value);
+#endif
 
-CXXTOOLS_API void convert(String& s, float value);
-CXXTOOLS_API void convert(String& s, double value);
-CXXTOOLS_API void convert(String& s, long double value);
+void convert(String& s, float value);
+void convert(String& s, double value);
+void convert(String& s, long double value);
 
 template <typename T>
 inline void convert(String& s, const T& value)
@@ -91,30 +96,30 @@ inline void convert(String& s, const T& value)
 // Conversions from cxxtools::String
 //
 
-CXXTOOLS_API void convert(bool& n, const String& str);
+void convert(bool& n, const String& str);
 
-CXXTOOLS_API void convert(char& n, const String& str);
-CXXTOOLS_API void convert(wchar_t& n, const String& str);
-CXXTOOLS_API void convert(Char& n, const String& str);
-CXXTOOLS_API void convert(unsigned char& n, const String& str);
-CXXTOOLS_API void convert(signed char& n, const String& str);
+void convert(char& n, const String& str);
+void convert(wchar_t& n, const String& str);
+void convert(Char& n, const String& str);
+void convert(unsigned char& n, const String& str);
+void convert(signed char& n, const String& str);
 
-CXXTOOLS_API void convert(short& n, const String& str);
-CXXTOOLS_API void convert(unsigned short& n, const String& str);
-CXXTOOLS_API void convert(int& n, const String& str);
-CXXTOOLS_API void convert(unsigned int& n, const String& str);
-CXXTOOLS_API void convert(long& n, const String& str);
-CXXTOOLS_API void convert(unsigned long& n, const String& str);
+void convert(short& n, const String& str);
+void convert(unsigned short& n, const String& str);
+void convert(int& n, const String& str);
+void convert(unsigned int& n, const String& str);
+void convert(long& n, const String& str);
+void convert(unsigned long& n, const String& str);
 #ifdef HAVE_LONG_LONG
-CXXTOOLS_API void convert(long long& n, const String& str);
+void convert(long long& n, const String& str);
 #endif
 #ifdef HAVE_UNSIGNED_LONG_LONG
-CXXTOOLS_API void convert(unsigned long long& n, const String& str);
+void convert(unsigned long long& n, const String& str);
 #endif
 
-CXXTOOLS_API void convert(float& n, const String& str);
-CXXTOOLS_API void convert(double& n, const String& str);
-CXXTOOLS_API void convert(long double& n, const String& str);
+void convert(float& n, const String& str);
+void convert(double& n, const String& str);
+void convert(long double& n, const String& str);
 
 template <typename T>
 inline void convert(T& t, const String& str)
@@ -135,24 +140,30 @@ inline void convert(std::string& s, const std::string& str)
     s = str;
 }
 
-CXXTOOLS_API void convert(std::string& s, const String& str);
+void convert(std::string& s, const String& str);
 
-CXXTOOLS_API void convert(std::string& s, bool value);
+void convert(std::string& s, bool value);
 
-CXXTOOLS_API void convert(std::string& s, char value);
-CXXTOOLS_API void convert(std::string& s, signed char value);
-CXXTOOLS_API void convert(std::string& s, unsigned char value);
+void convert(std::string& s, char value);
+void convert(std::string& s, signed char value);
+void convert(std::string& s, unsigned char value);
 
-CXXTOOLS_API void convert(std::string& s, short value);
-CXXTOOLS_API void convert(std::string& s, unsigned short value);
-CXXTOOLS_API void convert(std::string& s, int value);
-CXXTOOLS_API void convert(std::string& s, unsigned int value);
-CXXTOOLS_API void convert(std::string& s, long value);
-CXXTOOLS_API void convert(std::string& s, unsigned long value);
+void convert(std::string& s, short value);
+void convert(std::string& s, unsigned short value);
+void convert(std::string& s, int value);
+void convert(std::string& s, unsigned int value);
+void convert(std::string& s, long value);
+void convert(std::string& s, unsigned long value);
+#ifdef HAVE_LONG_LONG
+void convert(std::string& s, long long value);
+#endif
+#ifdef HAVE_UNSIGNED_LONG_LONG
+void convert(std::string& s, unsigned long long value);
+#endif
 
-CXXTOOLS_API void convert(std::string& s, float value);
-CXXTOOLS_API void convert(std::string& s, double value);
-CXXTOOLS_API void convert(std::string& s, long double value);
+void convert(std::string& s, float value);
+void convert(std::string& s, double value);
+void convert(std::string& s, long double value);
 
 template <typename T>
 inline void convert(std::string& s, const T& value)
@@ -166,28 +177,28 @@ inline void convert(std::string& s, const T& value)
 // Conversions from std::string
 //
 
-CXXTOOLS_API void convert(bool& n, const std::string& str);
+void convert(bool& n, const std::string& str);
 
-CXXTOOLS_API void convert(char& n, const std::string& str);
-CXXTOOLS_API void convert(signed char& n, const std::string& str);
-CXXTOOLS_API void convert(unsigned char& n, const std::string& str);
+void convert(char& n, const std::string& str);
+void convert(signed char& n, const std::string& str);
+void convert(unsigned char& n, const std::string& str);
 
-CXXTOOLS_API void convert(short& n, const std::string& str);
-CXXTOOLS_API void convert(unsigned short& n, const std::string& str);
-CXXTOOLS_API void convert(int& n, const std::string& str);
-CXXTOOLS_API void convert(unsigned int& n, const std::string& str);
-CXXTOOLS_API void convert(long& n, const std::string& str);
-CXXTOOLS_API void convert(unsigned long& n, const std::string& str);
+void convert(short& n, const std::string& str);
+void convert(unsigned short& n, const std::string& str);
+void convert(int& n, const std::string& str);
+void convert(unsigned int& n, const std::string& str);
+void convert(long& n, const std::string& str);
+void convert(unsigned long& n, const std::string& str);
 #ifdef HAVE_LONG_LONG
-CXXTOOLS_API void convert(long long& n, const std::string& str);
+void convert(long long& n, const std::string& str);
 #endif
 #ifdef HAVE_UNSIGNED_LONG_LONG
-CXXTOOLS_API void convert(unsigned long long& n, const std::string& str);
+void convert(unsigned long long& n, const std::string& str);
 #endif
 
-CXXTOOLS_API void convert(float& n, const std::string& str);
-CXXTOOLS_API void convert(double& n, const std::string& str);
-CXXTOOLS_API void convert(long double& n, const std::string& str);
+void convert(float& n, const std::string& str);
+void convert(double& n, const std::string& str);
+void convert(long double& n, const std::string& str);
 
 template <typename T>
 inline void convert(T& t, const std::string& str)
@@ -203,28 +214,28 @@ inline void convert(T& t, const std::string& str)
 // Conversions from const char* (null-terminated)
 //
 
-CXXTOOLS_API void convert(bool& n, const char* str);
+void convert(bool& n, const char* str);
 
-CXXTOOLS_API void convert(char& n, const char* str);
-CXXTOOLS_API void convert(signed char& n, const char* str);
-CXXTOOLS_API void convert(unsigned char& n, const char* str);
+void convert(char& n, const char* str);
+void convert(signed char& n, const char* str);
+void convert(unsigned char& n, const char* str);
 
-CXXTOOLS_API void convert(short& n, const char* str);
-CXXTOOLS_API void convert(unsigned short& n, const char* str);
-CXXTOOLS_API void convert(int& n, const char* str);
-CXXTOOLS_API void convert(unsigned int& n, const char* str);
-CXXTOOLS_API void convert(long& n, const char* str);
-CXXTOOLS_API void convert(unsigned long& n, const char* str);
+void convert(short& n, const char* str);
+void convert(unsigned short& n, const char* str);
+void convert(int& n, const char* str);
+void convert(unsigned int& n, const char* str);
+void convert(long& n, const char* str);
+void convert(unsigned long& n, const char* str);
 #ifdef HAVE_LONG_LONG
-CXXTOOLS_API void convert(long long& n, const char* str);
+void convert(long long& n, const char* str);
 #endif
 #ifdef HAVE_UNSIGNED_LONG_LONG
-CXXTOOLS_API void convert(unsigned long long& n, const char* str);
+void convert(unsigned long long& n, const char* str);
 #endif
 
-CXXTOOLS_API void convert(float& n, const char* str);
-CXXTOOLS_API void convert(double& n, const char* str);
-CXXTOOLS_API void convert(long double& n, const char* str);
+void convert(float& n, const char* str);
+void convert(double& n, const char* str);
+void convert(long double& n, const char* str);
 
 
 //
@@ -543,7 +554,7 @@ inline unsigned long long formatAbs(long long i, bool& isNeg)
 }
 #endif
 
-#ifdef HAVE_UNSIGNEDLONG_LONG
+#ifdef HAVE_UNSIGNED_LONG_LONG
 //! @internal @brief Returns the absolute value of \a i
 inline unsigned long long formatAbs(unsigned long long i, bool& isNeg)
 {
@@ -637,12 +648,12 @@ inline OutIterT putInt(OutIterT it, T i)
 
 
 template <typename OutIterT, typename T, typename FormatT>
-inline OutIterT putFloat(OutIterT it, T d, const FormatT& fmt, int precision)
+inline OutIterT putFloat(OutIterT it, T value, const FormatT& fmt, int precision)
 {
     typedef typename FormatT::CharT CharT;
 
-    // 1. Test for not-a-number with d != d
-    if( d != d ) 
+    // 1. Test for not-a-number with value != value
+    if( value != value ) 
     {
         for(const CharT* nanstr = fmt.nan(); *nanstr != 0; ++nanstr)
         {
@@ -654,13 +665,13 @@ inline OutIterT putFloat(OutIterT it, T d, const FormatT& fmt, int precision)
     }
 
     // 2. check sign
-    if(d < 0.0)
+    if(value < 0.0)
     {
         *it = fmt.minus();
         ++it;
     }
 
-    T num = std::fabs(d);
+    T num = std::fabs(value);
 
     // 3. Test for infinity
     if( num == std::numeric_limits<T>::infinity() ) 
@@ -680,21 +691,21 @@ inline OutIterT putFloat(OutIterT it, T d, const FormatT& fmt, int precision)
         precision = bufsize;
 
     CharT fract[bufsize + 1];
-    fract[bufsize] = CharT('\0');
+    fract[precision] = CharT('\0');
 
     int exp = static_cast<int>(std::floor(std::log10(num))) + 1;
 
-    num *= std::pow(T(10.0), static_cast<int>(precision) - exp);
+    num *= std::pow(T(10.0), precision - exp);
     num += .5;
 
     bool notZero = false;
-    for (unsigned short d = precision; d > 0; --d)
+    for (unsigned short dd = precision; dd > 0; --dd)
     {
-        T n = num / 10.0;
-        T fl = std::floor(n) * 10.0;
+        T n = num / T(10.0);
+        T fl = std::floor(n) * T(10.0);
         unsigned char v = static_cast<unsigned char>(num - fl);
-        notZero |= (v != 0);
-        fract[d - 1] = notZero ? fmt.toChar(v) : CharT('\0');
+        notZero |= (v != 0 && v < 10);
+        fract[dd - 1] = notZero ? fmt.toChar(v) : CharT('\0');
         num = n;
     }
 
@@ -715,21 +726,21 @@ inline OutIterT putFloat(OutIterT it, T d, const FormatT& fmt, int precision)
             ++exp;
         }
 
-        for (int d = 0; fract[d]; ++d)
+        for (int dd = 0; fract[dd]; ++dd)
         {
-            *it = fract[d]; ++it;
+            *it = fract[dd]; ++it;
         }
     }
     else
     {
-        for (int d = 0; fract[d]; ++d)
+        for (int dd = 0; fract[dd]; ++dd)
         {
             if (exp-- == 0)
             {
                 *it = '.';
                 ++it;
             }
-            *it = fract[d]; ++it;
+            *it = fract[dd]; ++it;
         }
 
         while (exp-- > 0)
@@ -743,11 +754,11 @@ inline OutIterT putFloat(OutIterT it, T d, const FormatT& fmt, int precision)
 
 
 template <typename OutIterT, typename T>
-inline OutIterT putFloat(OutIterT it, T d)
+inline OutIterT putFloat(OutIterT it, T value)
 {
-    const int precision = std::numeric_limits<T>::digits10 + 1;
+    const int precision = std::numeric_limits<T>::digits10 - 1;
     FloatFormat<char> fmt;
-    return putFloat(it, d, fmt, precision);
+    return putFloat(it, value, fmt, precision);
 }
 
 
@@ -760,14 +771,17 @@ InIterT getSign(InIterT it, InIterT end, bool& pos, const FormatT& fmt)
     while (it != end && isspace(*it))
         ++it;
 
-    if(*it == fmt.minus())
+    if (it != end)
     {
-        pos = false;
-        ++it;
-    }
-    else if( *it == fmt.plus() )
-    {
-        ++it;
+        if (*it == fmt.minus())
+        {
+            pos = false;
+            ++it;
+        }
+        else if ( *it == fmt.plus() )
+        {
+            ++it;
+        }
     }
 
     return it;
@@ -805,10 +819,9 @@ InIterT getInt(InIterT it, InIterT end, bool& ok, T& n, const FormatT& fmt)
     // parse number
     UnsignedInt u = 0;
     const UnsignedInt base = fmt.base;
-    unsigned char d = 0;
     while(it != end)
     {    
-        d = fmt.toDigit(*it);
+        unsigned char d = fmt.toDigit(*it);
         
         if(d >= base)
             break;
@@ -983,6 +996,7 @@ InIterT getFloat(InIterT it, InIterT end, bool& ok, T& n, const FormatT& fmt)
         if(digit >= fmt.base)
             return it;
         
+        ok = true;
         n *= 10;
         n += digit; 
     }
