@@ -28,27 +28,29 @@
 #ifndef CXXTOOLS_DATE_H
 #define CXXTOOLS_DATE_H
 
-#include <cxxtools/api.h>
 #include <string>
 #include <stdexcept>
+#include <cxxtools/timespan.h>
 
 namespace cxxtools
 {
 
 class SerializationInfo;
 
-class CXXTOOLS_API InvalidDate : public std::invalid_argument
+class InvalidDate : public std::invalid_argument
 {
     public:
         InvalidDate();
 
+        explicit InvalidDate(const std::string& what);
+
         ~InvalidDate() throw()
         {}
 };
 
-CXXTOOLS_API void greg2jul(unsigned& jd, int y, int m, int d);
+void greg2jul(unsigned& jd, int y, int m, int d);
 
-CXXTOOLS_API void jul2greg(unsigned jd, int& y, int& m, int& d);
+void jul2greg(unsigned jd, int& y, int& m, int& d);
 
 /*
   Notes:
@@ -156,11 +158,22 @@ class Date
         : _julian(0)
         {}
 
-        /** \brief Copy constructor.
-        */
-        Date(const Date& date)
-        : _julian(date._julian)
-        {}
+        /** \brief create Date from string using format
+
+            Valid format codes are:
+
+              %Y   4 digit year
+              %y   2 digit year
+              %m   month (1-12)
+              %2m  month (01-12)
+              %O   english monthname (Jan-Dec)
+              %d   day (1-31)
+              %2d  day (01-31)
+              ?    arbitrary character
+              *    skip non digit characters
+              #    skip word
+         */
+        explicit Date(const std::string& d, const std::string& fmt = "%Y-%m-%d");
 
         /** \brief Constructs a Date from given values
 
@@ -214,7 +227,7 @@ class Date
         */
         int year() const;
 
-        /** @brief Return day of the week, starting with sunday
+        /** @brief Return day of the week, starting with sunday (=0)
         */
         unsigned dayOfWeek() const;
 
@@ -230,12 +243,6 @@ class Date
         */
         bool leapYear() const;
 
-        // TODO: move to cxxtools:.System
-        //static Date localDate();
-
-        // TODO: move to cxxtools:.System
-        //static Date universalDate();
-
         /** @brief Assignment operator
         */
         Date& operator=(const Date& date)
@@ -246,7 +253,7 @@ class Date
         Date& operator+=(int days)
         { _julian += days; return *this; }
 
-        /** @brief Substract days from the date
+        /** @brief Subtract days from the date
         */
         Date& operator-=(int days)
         { _julian -= days; return *this; }
@@ -293,9 +300,27 @@ class Date
 
         friend inline Date operator+(const Date& d, int days);
 
+        friend inline Date operator-(const Date& d, int days);
+
         friend inline Date operator+(int days, const Date& d);
 
-        friend inline int operator-(const Date& a, const Date& b);
+        friend inline Days operator-(const Date& a, const Date& b);
+
+        /** \brief format Date into a string using a format string
+
+            Valid format codes are:
+
+              %d   day (01-31)
+              %1d  day (1-31)
+              %m   month (01-12)
+              %1m  month (1-12)
+              %O   english monthname (Jan-Dec)
+              %Y   4 digit year
+              %y   2 digit year
+              %w   day of week (0-6 sunday=6)
+              %W   day of week (1-7 sunday=7)
+         */
+        std::string toString(const std::string& fmt = "%Y-%m-%d") const;
 
         /** \brief Returns the date in ISO-format
 
@@ -303,7 +328,8 @@ class Date
 
             \return Date as iso formated string.
         */
-        std::string toIsoString() const;
+        std::string toIsoString() const
+        { return toString("%Y-%m-%d"); }
 
         /** \brief Interprets a string as a date-string in ISO-format
 
@@ -315,7 +341,8 @@ class Date
             \return Date result
             \throw IllegalArgument
         */
-        static Date fromIsoString(const std::string& s);
+        static Date fromIsoString(const std::string& s)
+        { return Date(s, "%Y-%m-%d"); }
 
     public:
         /** \brief Returns true if values describe a valid date
@@ -331,14 +358,9 @@ class Date
         unsigned _julian;
 };
 
-CXXTOOLS_API void operator >>=(const SerializationInfo& si, Date& date);
-
-CXXTOOLS_API void operator <<=(SerializationInfo& si, const Date& date);
-
-CXXTOOLS_API void convert(std::string& str, const Date& date);
-
-CXXTOOLS_API void convert(Date& date, const std::string& s);
+void operator >>=(const SerializationInfo& si, Date& date);
 
+void operator <<=(SerializationInfo& si, const Date& date);
 
 inline void Date::get(int& y, unsigned& m, unsigned& d) const
 {
@@ -421,43 +443,19 @@ inline bool Date::leapYear() const
 }
 
 
-inline bool Date::isValid(int y, int m, int d)
-{
-    if(m<1 || m>12 || d<1 || d>31)
-    {
-        return false;
-    }
-
-    return true;
-}
-
-
-inline std::string Date::toIsoString() const
-{
-    std::string str;
-    convert(str, *this);
-    return str;
-}
-
-
-inline Date Date::fromIsoString(const std::string& s)
-{
-    Date date;
-    convert(date, s);
-    return date;
-}
-
-
 inline Date operator+(const Date& d, int days)
 { return Date(d._julian + days); }
 
+inline Date operator-(const Date& d, int days)
+{ return Date(d._julian - days); }
+
 
 inline Date operator+(int days, const Date& d)
 { return Date(days + d._julian); }
 
 
-inline int operator-(const Date& a, const Date& b)
-{ return a._julian - b._julian; }
+inline Days operator-(const Date& a, const Date& b)
+{ return Days(a._julian - b._julian); }
 
 }
 
