@@ -28,7 +28,6 @@
 #ifndef cxxtools_Formatter_h
 #define cxxtools_Formatter_h
 
-#include <cxxtools/api.h>
 #include <cxxtools/string.h>
 #include <string>
 #include <cxxtools/config.h>
@@ -59,6 +58,9 @@ class Formatter
         virtual void addValueStdString(const std::string& name, const std::string& type,
                               const std::string& value);
 
+        virtual void addValueChar(const std::string& name, const std::string& type,
+                              char value);
+
         virtual void addValueBool(const std::string& name, const std::string& type,
                               bool value);
 
@@ -69,6 +71,12 @@ class Formatter
                               unsigned_type value);
 
         virtual void addValueFloat(const std::string& name, const std::string& type,
+                              float value);
+
+        virtual void addValueDouble(const std::string& name, const std::string& type,
+                              double value);
+
+        virtual void addValueLongDouble(const std::string& name, const std::string& type,
                               long double value);
 
         virtual void addNull(const std::string& name, const std::string& type);
