@@ -28,7 +28,6 @@
 #ifndef cxxtools_System_IOStream_h
 #define cxxtools_System_IOStream_h
 
-#include <cxxtools/api.h>
 #include <cxxtools/streambuffer.h>
 #include <iostream>
 #include <algorithm>
@@ -197,27 +196,53 @@ class BasicIOStream : public std::basic_iostream<CharT>
 };
 
 
-class CXXTOOLS_API IStream : public BasicIStream<char>
+class IStream : public BasicIStream<char>, public Connectable
 {
     public:
-        explicit IStream(size_t bufferSize = 8192, bool extend = false);
+        explicit IStream(size_t bufferSize = 8192);
 
         ~IStream();
 
-        explicit IStream(IODevice& device, size_t bufferSize = 8192, bool extend = false);
+        explicit IStream(IODevice& device, size_t bufferSize = 8192);
 
-        StreamBuffer& buffer();
+        // needed to avoid confusion.
+        // clear is defined in Connectable also
+        void clear()
+        { BasicIStream<char>::clear(); }
+
+        StreamBuffer& buffer()
+        { return _buffer; }
+
+        std::streamsize in_avail()
+        { return buffer().in_avail(); }
 
         IODevice* attachDevice(IODevice& device);
 
-        IODevice* attachedDevice();
+        IODevice* attachedDevice()
+        { return _buffer.device(); }
+
+        void setSelector(SelectorBase* parent)
+        { attachedDevice()->setSelector(parent); }
+
+        void setSelector(SelectorBase& parent)
+        { attachedDevice()->setSelector(parent); }
+
+        void beginRead()
+        { buffer().beginRead(); }
+
+        void endRead();
+
+        Signal<IStream&> inputReady;
 
     private:
+        void onInput(StreamBuffer&);
+        void onOutput(StreamBuffer&);
+
         StreamBuffer _buffer;
 };
 
 
-class CXXTOOLS_API OStream : public BasicOStream<char>
+class OStream : public BasicOStream<char>, public Connectable
 {
     public:
         explicit OStream(size_t bufferSize = 8192, bool extend = false);
@@ -226,18 +251,44 @@ class CXXTOOLS_API OStream : public BasicOStream<char>
 
         ~OStream();
 
-        StreamBuffer& buffer();
+        // needed to avoid confusion.
+        // clear is defined in Connectable also
+        void clear()
+        { BasicOStream<char>::clear(); }
+
+        StreamBuffer& buffer()
+        { return _buffer; }
+
+        std::streamsize out_avail()
+        { return buffer().out_avail(); }
 
         IODevice* attachDevice(IODevice& device);
 
-        IODevice* attachedDevice();
+        IODevice* attachedDevice()
+        { return _buffer.device(); }
+
+        void setSelector(SelectorBase* parent)
+        { attachedDevice()->setSelector(parent); }
+
+        void setSelector(SelectorBase& parent)
+        { attachedDevice()->setSelector(parent); }
+
+        void beginWrite()
+        { buffer().beginWrite(); }
+
+        void endWrite()
+        { buffer().endWrite(); }
+
+        Signal<OStream&> outputReady;
 
     private:
+        void onOutput(StreamBuffer&);
+
         StreamBuffer _buffer;
 };
 
 
-class CXXTOOLS_API IOStream : public BasicIOStream<char>
+class IOStream : public BasicIOStream<char>, public Connectable
 {
     public:
         explicit IOStream(size_t bufferSize = 8192, bool extend = false);
@@ -246,13 +297,50 @@ class CXXTOOLS_API IOStream : public BasicIOStream<char>
 
         ~IOStream();
 
-        StreamBuffer& buffer();
+        // needed to avoid confusion.
+        // clear is defined in Connectable also
+        void clear()
+        { BasicIOStream<char>::clear(); }
+
+        StreamBuffer& buffer()
+        { return _buffer; }
+
+        std::streamsize in_avail()
+        { return buffer().in_avail(); }
+
+        std::streamsize out_avail()
+        { return buffer().out_avail(); }
 
         IODevice* attachDevice(IODevice& device);
 
-        IODevice* attachedDevice();
+        IODevice* attachedDevice()
+        { return _buffer.device(); }
+
+        void setSelector(SelectorBase* parent)
+        { attachedDevice()->setSelector(parent); }
+
+        void setSelector(SelectorBase& parent)
+        { attachedDevice()->setSelector(parent); }
+
+        void beginRead()
+        { buffer().beginRead(); }
+
+        void endRead();
+
+        void beginWrite()
+        { buffer().beginWrite(); }
+
+        void endWrite()
+        { buffer().endWrite(); }
+
+        Signal<IOStream&> inputReady;
+
+        Signal<IOStream&> outputReady;
 
     private:
+        void onInput(StreamBuffer&);
+        void onOutput(StreamBuffer&);
+
         StreamBuffer _buffer;
 };
 
