@@ -29,31 +29,58 @@
 #ifndef CXXTOOLS_JSONPARSER_H
 #define CXXTOOLS_JSONPARSER_H
 
-#include <cxxtools/api.h>
 #include <cxxtools/string.h>
+#include <cxxtools/serializationerror.h>
 
 namespace cxxtools
 {
-    class DeserializerBase;
+    class JsonDeserializer;
 
-    class CXXTOOLS_API JsonParser
+    class JsonParserError : public SerializationError
+    {
+            friend class JsonParser;
+            unsigned _lineNo;
+            mutable std::string _msg;
+
+        public:
+            JsonParserError(const std::string& msg, unsigned lineNo)
+                : SerializationError(msg),
+                  _lineNo(lineNo)
+            { }
+
+            ~JsonParserError() throw()
+            { }
+
+            const char* what() const throw();
+    };
+
+    class JsonParser
     {
             class JsonStringParser
             {
+                    JsonParser* _jsonParser;
                     String _str;
                     unsigned _count;
                     unsigned short _value;
+                    unsigned short _surrogateValue;
 
                     enum
                     {
                         state_0,
                         state_esc,
-                        state_hex
+                        state_hex,
+                        state_highsurrogate0,
+                        state_lowsurrogate0,
+                        state_highsurrogate_esc,
+                        state_lowsurrogate_esc,
+                        state_highsurrogate_hex,
+                        state_lowsurrogate_hex
                     } _state;
 
                 public:
-                    JsonStringParser()
-                        : _state(state_0)
+                    explicit JsonStringParser(JsonParser* jsonParser)
+                        : _jsonParser(jsonParser),
+                          _state(state_0)
                         { }
 
                     bool advance(Char ch);
@@ -63,12 +90,20 @@ namespace cxxtools
 
                     const String& str() const
                     { return _str; }
+
+                    void str(const String& s)
+                    { _str = s; }
             };
 
+            // make non copyable:
+            JsonParser(const JsonParser&);
+            JsonParser& operator=(const JsonParser&);
+
         public:
             JsonParser();
+            ~JsonParser();
 
-            void begin(DeserializerBase& handler)
+            void begin(JsonDeserializer& handler)
             {
                 _state = state_0;
                 _token.clear();
@@ -83,12 +118,15 @@ namespace cxxtools
             {
                 state_0,
                 state_object,
+                state_object_plainname,
                 state_object_name,
                 state_object_after_name,
                 state_object_value,
                 state_object_e,
+                state_object_next_member0,
                 state_object_next_member,
                 state_array,
+                state_array_value0,
                 state_array_value,
                 state_array_e,
                 state_string,
@@ -104,9 +142,13 @@ namespace cxxtools
 
             String _token;
 
-            DeserializerBase* _deserializer;
+            JsonDeserializer* _deserializer;
             JsonStringParser _stringParser;
             JsonParser* _next;
+            unsigned _lineNo;
+
+            void doThrow(const std::string& msg);
+            void throwInvalidCharacter(Char ch);
     };
 }
 
