@@ -26,16 +26,16 @@
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
+#ifndef CXXTOOLS_POOL_H
+#define CXXTOOLS_POOL_H
+
 #include <cxxtools/smartptr.h>
-#include <cxxtools/noncopyable.h>
 #include <cxxtools/mutex.h>
 #include <vector>
 
 namespace cxxtools
 {
-  /**
-   This class is a factory for objects wich are default constructable.
-   */
+  /// This class is a factory for objects wich are default constructable.
   template <class T>
   class DefaultCreator
   {
@@ -44,21 +44,27 @@ namespace cxxtools
       { return new T(); }
   };
 
-  /**
-   A Pool is a container for pooled objects.
+  /** A Pool is a container for pooled objects
 
-   It maintains a list of object instances, which are not in use. If a
-   program needs a instance, it can request one with the get-method. The
-   pool returns a smart pointer to an instance. When the pointer is released,
-   the instance is put back into the pool or dropped, if the pool has already
-   a maximum number of instances.
+      It maintains a list of object instances which are not in use. If a
+      program needs an instance, it can request one with the get-method. The
+      pool returns a smart pointer to an instance. When the pointer is released,
+      the instance is put back into the pool or dropped if the pool already has 
+      the maximum number of instances.
    */
   template <typename ObjectType,
             typename CreatorType = DefaultCreator<ObjectType>,
             template <class> class OwnershipPolicy = RefLinked,
             template <class> class DestroyPolicy = DeletePolicy>
-  class Pool : private NonCopyable
+  class Pool
   {
+#if __cplusplus >= 201103L
+      Pool(const Pool&) = delete;
+      Pool& operator=(const Pool&) = delete;
+#else
+      Pool(const Pool&) { }
+      Pool& operator=(const Pool&) { return *this; }
+#endif
     public:
       class Ptr : public OwnershipPolicy<ObjectType>,
                   public DestroyPolicy<ObjectType>
@@ -79,10 +85,12 @@ namespace cxxtools
 
         public:
           Ptr()
-            : object(0)
+            : object(0),
+              pool(0)
             {}
           Ptr(ObjectType* ptr)
-            : object(ptr)
+            : object(ptr),
+              pool(0)
             { OwnershipPolicyType::link(*this, ptr); }
           Ptr(const Ptr& ptr)
             : object(ptr.object),
@@ -223,3 +231,5 @@ namespace cxxtools
   };
 
 }
+
+#endif
