@@ -40,6 +40,8 @@
 namespace cxxtools
 {
 
+class SerializationInfo;
+
 /**
  QueryParams represents parameters from a HTML-Form.
 
@@ -53,6 +55,8 @@ namespace cxxtools
  */
 class QueryParams
 {
+    friend void operator<<= (cxxtools::SerializationInfo& si, const QueryParams& p);
+
   public:
     struct value_type
     {
@@ -215,6 +219,19 @@ class QueryParams
       return *this;
     }
 
+    /// remove unnamed parameter
+    QueryParams& remove()
+    { remove(std::string()); return *this; }
+
+    /// set unnamed parameter
+    QueryParams& set(const std::string& value)
+    {
+      remove();
+      add(value);
+      return *this;
+    }
+
+
     //
     // named parameter
     //
@@ -247,12 +264,31 @@ class QueryParams
       return *this;
     }
 
+    /// remove named parameter
+    QueryParams& remove(const std::string& name);
+
+    /// sets named parameter
+    QueryParams& set(const std::string& name, const std::string& value)
+    {
+      remove(name);
+      add(name, value);
+      return *this;
+    }
+
     QueryParams& add(const QueryParams& other)
     {
       _values.insert(_values.end(), other._values.begin(), other._values.end());
       return *this;
     }
 
+    /// returns all names of parameters
+    template <typename output_iterator>
+    void getNames(output_iterator o) const
+    {
+      for (values_type::const_iterator i = _values.begin(); i != _values.end(); ++i)
+        *o++ = (*i).name;
+    }
+
     /// removes all data
     void clear()
     { _values.clear(); }
@@ -280,10 +316,17 @@ class QueryParams
 
 };
 
+
 /// output QueryParams in url-syntax
 inline std::ostream& operator<< (std::ostream& out, const QueryParams& p)
 { return out << p.getUrl(); }
 
+inline std::istream& operator>> (std::istream& in, QueryParams& p)
+{ p.parse_url(in); return in; }
+
+/// make QueryParams serializable
+void operator<<= (cxxtools::SerializationInfo& si, const QueryParams& p);
+
 }
 
 #endif // CXXTOOLS_QUERY_PARAMS_H
