@@ -32,6 +32,8 @@
 #include <queue>
 #include <cxxtools/mutex.h>
 #include <cxxtools/condition.h>
+#include <cxxtools/timespan.h>
+#include <cxxtools/scopedincrement.h>
 
 namespace cxxtools
 {
@@ -53,8 +55,8 @@ namespace cxxtools
 
         private:
             mutable Mutex _mutex;
-            Condition _notEmpty;
-            Condition _notFull;
+            mutable Condition _notEmpty;
+            mutable Condition _notFull;
             std::deque<value_type> _queue;
             size_type _maxSize;
             size_type _numWaiting;
@@ -73,6 +75,23 @@ namespace cxxtools
              */
             value_type get();
 
+            /** @brief Returns the next element if the queue is not empty.
+
+                This method returns the next element. If the queue is empty,
+                the thread will wait up to timeout milliseconds until a element
+                is available.
+
+                If the queue was empty after the timeout, a pair of a default
+                constructed value_type and the value false are returned.
+                Otherwise the next element is removed and returned together
+                with a true value.
+
+                Spurious wakeups are ignored so that when that happens, the
+                method might return before the timeout is over even when the
+                queue is empty.
+             */
+            std::pair<value_type, bool> get(const Milliseconds& timeout);
+
             /** @brief Returns the next element if the queue is not empty.
 
                 If the queue is empty, a default constructed value_type is returned.
@@ -88,6 +107,13 @@ namespace cxxtools
              */
             void put(const_reference element, bool force = false);
 
+            /** @brief Removes one specific element.
+
+                This method removes one specific element from the queue.
+                It returns true, when the element was found and removed.
+             */
+            bool remove(const_reference element);
+
             /// @brief Returns true, if the queue is empty.
             bool empty() const
             {
@@ -95,6 +121,29 @@ namespace cxxtools
                 return _queue.empty();
             }
 
+            /// @brief Waits until timeout or queue empty.
+            /// Waits until timeout or queue empty. Returns true
+            /// when queue is not empty.
+            bool waitEmpty(Milliseconds timeout = -1) const
+            {
+                MutexLock lock(_mutex);
+                if (timeout >= Milliseconds(0))
+                {
+                    Timespan until = Timespan::gettimeofday() + timeout;
+                    Timespan remaining;
+                    while (!_queue.empty()
+                      && (remaining = until - Timespan::gettimeofday()) > Timespan(0))
+                        _notEmpty.wait(_mutex, remaining);
+                }
+                else
+                {
+                    while (!_queue.empty())
+                        _notEmpty.wait(lock);
+                }
+
+                return !_queue.empty();
+            }
+
             /// @brief Returns the number of elements currently in queue.
             size_type size() const
             {
@@ -130,10 +179,9 @@ namespace cxxtools
     {
         MutexLock lock(_mutex);
 
-        ++_numWaiting;
+        ScopedIncrement<size_type> inc(_numWaiting);
         while (_queue.empty())
             _notEmpty.wait(lock);
-        --_numWaiting;
 
         value_type element = _queue.front();
         _queue.pop_front();
@@ -146,6 +194,34 @@ namespace cxxtools
         return element;
     }
 
+    template <typename T>
+    std::pair<typename Queue<T>::value_type, bool> Queue<T>::get(const Milliseconds& timeout)
+    {
+        typedef typename Queue<T>::value_type value_type;
+        typedef typename std::pair<value_type, bool> return_type;
+
+        MutexLock lock(_mutex);
+
+        ScopedIncrement<size_type> inc(_numWaiting);
+        if (_queue.empty())
+        {
+            if (_notEmpty.wait(lock, timeout) == false)
+            {
+                return return_type(value_type(), false);
+            }
+        }
+
+        value_type element = _queue.front();
+        _queue.pop_front();
+
+        if (!_queue.empty())
+            _notEmpty.signal();
+
+        _notFull.signal();
+
+        return return_type(element, true);
+    }
+
     template <typename T>
     std::pair<typename Queue<T>::value_type, bool> Queue<T>::tryGet()
     {
@@ -184,6 +260,16 @@ namespace cxxtools
             _notFull.signal();
     }
 
+    template <typename T>
+    bool Queue<T>::remove(const_reference element)
+    {
+        MutexLock lock(_mutex);
+        for (typename std::deque<T>::iterator it = _queue.begin(); it != _queue.end(); ++it)
+            if (*it == element)
+                return true;
+        return false;
+    }
+
     template <typename T>
     void Queue<T>::maxSize(size_type m)
     {
