@@ -61,9 +61,11 @@ namespace cxxtools
       void checkerr(int ret) const;
 
     public:
+      /// create a uninitialized regex object.
       Regex()
       { }
 
+      /// create a regex object with a const char*.
       explicit Regex(const char* ex, int cflags = REG_EXTENDED)
         : expr(0)
       {
@@ -74,6 +76,7 @@ namespace cxxtools
         }
       }
 
+      /// create a regex object with std::string.
       explicit Regex(const std::string& ex, int cflags = REG_EXTENDED)
         : expr(0)
       {
@@ -84,10 +87,28 @@ namespace cxxtools
         }
       }
 
-      bool match(const std::string& str_, RegexSMatch& smatch, int eflags = 0) const;
-      bool match(const std::string& str_, int eflags = 0) const;
+      /// Returns true if str starting from p matches the regular expression. The matches are decribed in the smatch object.
+      bool matchp(const std::string& str_, std::string::size_type p, RegexSMatch& smatch, int eflags = 0) const;
 
+      /// Returns true if str starting from p matches the regular expression.
+      bool matchp(const std::string& str_, std::string::size_type p, int eflags = 0) const;
+
+      /// Returns true if str matches the regular expression. The matches are decribed in the smatch object.
+      bool match(const std::string& str_, RegexSMatch& smatch, int eflags = 0) const
+      { return matchp(str_, 0, smatch, eflags); }
+
+      /// Returns true if str matches the regular expression.
+      bool match(const std::string& str_, int eflags = 0) const
+      { return matchp(str_, 0, eflags); }
+
+      /// Replaces occurence of the regular expression in str with expr. The expr is expanded using RegexSMatch::format.
+      /// If the 3rd parameter is true, all occurences are replaced.
+      std::string subst(const std::string& str, const std::string& expr, bool all = true);
+
+      /// Destroys the regular expression. This is normally done by the destructor.
       void free()  { expr = 0; }
+
+      /// Returns true, if the object does not have a valid regular expression.
       bool empty() const    { return expr.getPointer() == 0; }
   };
 
@@ -111,6 +132,8 @@ namespace cxxtools
       regoff_t offsetBegin(unsigned n) const   { return matchbuf[n].rm_so; }
       /// returns the end position of the n-th expression
       regoff_t offsetEnd(unsigned n) const     { return matchbuf[n].rm_eo; }
+      /// returns the size of the n-th expression
+      regoff_t size(unsigned n) const     { return matchbuf[n].rm_eo - matchbuf[n].rm_so; }
 
       /// returns true if the n-th element is set.
       bool has(unsigned n) const
@@ -119,7 +142,7 @@ namespace cxxtools
       std::string get(unsigned n) const;
       /// replace each occurence of "$n" with the n-th element (n: 0..9).
       std::string format(const std::string& s) const;
-      /// returns the n-th element. No range checking is done.
+      /// returns the n-th element. No range check is done.
       std::string operator[] (unsigned n) const
         { return get(n); }
   };
