@@ -30,13 +30,16 @@
 #define CXXTOOLS_REMOTECLIENT_H
 
 #include <cstddef>
+#include <cxxtools/timespan.h>
 
 namespace cxxtools
 {
     class IComposer;
     class IDecomposer;
     class IRemoteProcedure;
+    class SelectorBase;
 
+    /// Base class for rpc client.
     class RemoteClient
     {
         public:
@@ -55,8 +58,17 @@ namespace cxxtools
 
             virtual void cancel() = 0;
 
-            virtual void wait(std::size_t msecs = WaitInfinite) = 0;
+            virtual void wait(Milliseconds msecs = WaitInfinite) = 0;
 
+            virtual Milliseconds timeout() const = 0;
+            virtual void timeout(Milliseconds t) = 0;
+
+            virtual Milliseconds connectTimeout() const = 0;
+            virtual void connectTimeout(Milliseconds t) = 0;
+
+            virtual void setSelector(SelectorBase* selector) = 0;
+
+            void setSelector(SelectorBase& selector) { setSelector(&selector); }
     };
 }
 
