@@ -78,6 +78,10 @@ class IRemoteProcedure
 };
 
 
+/** Base class for Remote procedure class.
+ *
+ */
+
 template <typename R>
 class RemoteProcedureBase : public IRemoteProcedure
 {
@@ -92,7 +96,22 @@ class RemoteProcedureBase : public IRemoteProcedure
             _result.setFault(rc, msg);
         }
 
-        const R& result()
+        /// access result value.
+        const R& value() const
+        {
+            return _result.value();
+        }
+
+        /** Finalize call and return result.
+         *  Exceptions are thrown when the call resulted in a exception.
+         *  The method should be called just once since the exception state
+         *  is partly reset.
+         */
+#if __cplusplus >= 201103L
+        R&& result()
+#else
+        R& result()
+#endif
         {
             return _result.get();
         }
@@ -102,9 +121,13 @@ class RemoteProcedureBase : public IRemoteProcedure
             return _result.failed();
         }
 
-        Signal< const RemoteResult<R> & > finished;
+        Signal< RemoteResult<R> & > finished;
 
-        const R& end(std::size_t msecs = RemoteClient::WaitInfinite)
+#if __cplusplus >= 201103L
+        R&& end(Milliseconds msecs = RemoteClient::WaitInfinite)
+#else
+        R& end(Milliseconds msecs = RemoteClient::WaitInfinite)
+#endif
         {
             _result.client().wait(msecs);
             return _result.get();
@@ -119,6 +142,12 @@ class RemoteProcedureBase : public IRemoteProcedure
 };
 
 
+//! @endcond internal
+
+/// Template for calling a remote procedure.
+
+// generated with `remoteprocedure.pl -n 10`
+//! @cond internal
 template <typename R,
           typename A1 = cxxtools::Void,
           typename A2 = cxxtools::Void,
@@ -162,7 +191,11 @@ class RemoteProcedure : public RemoteProcedureBase<R>
             this->client().beginCall(this->_r, *this, argv, 10);
         }
 
-        const R& call(const A1& a1, const A2& a2, const A3& a3, const A4& a4, const A5& a5, const A6& a6, const A7& a7, const A8& a8, const A9& a9, const A10& a10)
+#if __cplusplus >= 201103L
+        R&& call(const A1& a1, const A2& a2, const A3& a3, const A4& a4, const A5& a5, const A6& a6, const A7& a7, const A8& a8, const A9& a9, const A10& a10)
+#else
+        R& call(const A1& a1, const A2& a2, const A3& a3, const A4& a4, const A5& a5, const A6& a6, const A7& a7, const A8& a8, const A9& a9, const A10& a10)
+#endif
         {
             this->_result.clearFault();
 
@@ -183,7 +216,11 @@ class RemoteProcedure : public RemoteProcedureBase<R>
             return this->_result.get();
         }
 
-        const R& operator()(const A1& a1, const A2& a2, const A3& a3, const A4& a4, const A5& a5, const A6& a6, const A7& a7, const A8& a8, const A9& a9, const A10& a10)
+#if __cplusplus >= 201103L
+        R&& operator()(const A1& a1, const A2& a2, const A3& a3, const A4& a4, const A5& a5, const A6& a6, const A7& a7, const A8& a8, const A9& a9, const A10& a10)
+#else
+        R& operator()(const A1& a1, const A2& a2, const A3& a3, const A4& a4, const A5& a5, const A6& a6, const A7& a7, const A8& a8, const A9& a9, const A10& a10)
+#endif
         {
             return this->call(a1, a2, a3, a4, a5, a6, a7, a8, a9, a10);
         }
@@ -244,7 +281,11 @@ class RemoteProcedure<R, A1, A2, A3, A4, A5, A6, A7, A8, A9,
             this->client().beginCall(this->_r, *this, argv, 9);
         }
 
-        const R& call(const A1& a1, const A2& a2, const A3& a3, const A4& a4, const A5& a5, const A6& a6, const A7& a7, const A8& a8, const A9& a9)
+#if __cplusplus >= 201103L
+        R&& call(const A1& a1, const A2& a2, const A3& a3, const A4& a4, const A5& a5, const A6& a6, const A7& a7, const A8& a8, const A9& a9)
+#else
+        R& call(const A1& a1, const A2& a2, const A3& a3, const A4& a4, const A5& a5, const A6& a6, const A7& a7, const A8& a8, const A9& a9)
+#endif
         {
             this->_result.clearFault();
 
@@ -264,7 +305,11 @@ class RemoteProcedure<R, A1, A2, A3, A4, A5, A6, A7, A8, A9,
             return this->_result.get();
         }
 
-        const R& operator()(const A1& a1, const A2& a2, const A3& a3, const A4& a4, const A5& a5, const A6& a6, const A7& a7, const A8& a8, const A9& a9)
+#if __cplusplus >= 201103L
+        R&& operator()(const A1& a1, const A2& a2, const A3& a3, const A4& a4, const A5& a5, const A6& a6, const A7& a7, const A8& a8, const A9& a9)
+#else
+        R& operator()(const A1& a1, const A2& a2, const A3& a3, const A4& a4, const A5& a5, const A6& a6, const A7& a7, const A8& a8, const A9& a9)
+#endif
         {
             return this->call(a1, a2, a3, a4, a5, a6, a7, a8, a9);
         }
@@ -323,7 +368,11 @@ class RemoteProcedure<R, A1, A2, A3, A4, A5, A6, A7, A8,
             this->client().beginCall(this->_r, *this, argv, 8);
         }
 
-        const R& call(const A1& a1, const A2& a2, const A3& a3, const A4& a4, const A5& a5, const A6& a6, const A7& a7, const A8& a8)
+#if __cplusplus >= 201103L
+        R&& call(const A1& a1, const A2& a2, const A3& a3, const A4& a4, const A5& a5, const A6& a6, const A7& a7, const A8& a8)
+#else
+        R& call(const A1& a1, const A2& a2, const A3& a3, const A4& a4, const A5& a5, const A6& a6, const A7& a7, const A8& a8)
+#endif
         {
             this->_result.clearFault();
 
@@ -342,7 +391,11 @@ class RemoteProcedure<R, A1, A2, A3, A4, A5, A6, A7, A8,
             return this->_result.get();
         }
 
-        const R& operator()(const A1& a1, const A2& a2, const A3& a3, const A4& a4, const A5& a5, const A6& a6, const A7& a7, const A8& a8)
+#if __cplusplus >= 201103L
+        R&& operator()(const A1& a1, const A2& a2, const A3& a3, const A4& a4, const A5& a5, const A6& a6, const A7& a7, const A8& a8)
+#else
+        R& operator()(const A1& a1, const A2& a2, const A3& a3, const A4& a4, const A5& a5, const A6& a6, const A7& a7, const A8& a8)
+#endif
         {
             return this->call(a1, a2, a3, a4, a5, a6, a7, a8);
         }
@@ -399,7 +452,11 @@ class RemoteProcedure<R, A1, A2, A3, A4, A5, A6, A7,
             this->client().beginCall(this->_r, *this, argv, 7);
         }
 
-        const R& call(const A1& a1, const A2& a2, const A3& a3, const A4& a4, const A5& a5, const A6& a6, const A7& a7)
+#if __cplusplus >= 201103L
+        R&& call(const A1& a1, const A2& a2, const A3& a3, const A4& a4, const A5& a5, const A6& a6, const A7& a7)
+#else
+        R& call(const A1& a1, const A2& a2, const A3& a3, const A4& a4, const A5& a5, const A6& a6, const A7& a7)
+#endif
         {
             this->_result.clearFault();
 
@@ -417,7 +474,11 @@ class RemoteProcedure<R, A1, A2, A3, A4, A5, A6, A7,
             return this->_result.get();
         }
 
-        const R& operator()(const A1& a1, const A2& a2, const A3& a3, const A4& a4, const A5& a5, const A6& a6, const A7& a7)
+#if __cplusplus >= 201103L
+        R&& operator()(const A1& a1, const A2& a2, const A3& a3, const A4& a4, const A5& a5, const A6& a6, const A7& a7)
+#else
+        R& operator()(const A1& a1, const A2& a2, const A3& a3, const A4& a4, const A5& a5, const A6& a6, const A7& a7)
+#endif
         {
             return this->call(a1, a2, a3, a4, a5, a6, a7);
         }
@@ -472,7 +533,11 @@ class RemoteProcedure<R, A1, A2, A3, A4, A5, A6,
             this->client().beginCall(this->_r, *this, argv, 6);
         }
 
-        const R& call(const A1& a1, const A2& a2, const A3& a3, const A4& a4, const A5& a5, const A6& a6)
+#if __cplusplus >= 201103L
+        R&& call(const A1& a1, const A2& a2, const A3& a3, const A4& a4, const A5& a5, const A6& a6)
+#else
+        R& call(const A1& a1, const A2& a2, const A3& a3, const A4& a4, const A5& a5, const A6& a6)
+#endif
         {
             this->_result.clearFault();
 
@@ -489,7 +554,11 @@ class RemoteProcedure<R, A1, A2, A3, A4, A5, A6,
             return this->_result.get();
         }
 
-        const R& operator()(const A1& a1, const A2& a2, const A3& a3, const A4& a4, const A5& a5, const A6& a6)
+#if __cplusplus >= 201103L
+        R&& operator()(const A1& a1, const A2& a2, const A3& a3, const A4& a4, const A5& a5, const A6& a6)
+#else
+        R& operator()(const A1& a1, const A2& a2, const A3& a3, const A4& a4, const A5& a5, const A6& a6)
+#endif
         {
             return this->call(a1, a2, a3, a4, a5, a6);
         }
@@ -542,7 +611,11 @@ class RemoteProcedure<R, A1, A2, A3, A4, A5,
             this->client().beginCall(this->_r, *this, argv, 5);
         }
 
-        const R& call(const A1& a1, const A2& a2, const A3& a3, const A4& a4, const A5& a5)
+#if __cplusplus >= 201103L
+        R&& call(const A1& a1, const A2& a2, const A3& a3, const A4& a4, const A5& a5)
+#else
+        R& call(const A1& a1, const A2& a2, const A3& a3, const A4& a4, const A5& a5)
+#endif
         {
             this->_result.clearFault();
 
@@ -558,7 +631,11 @@ class RemoteProcedure<R, A1, A2, A3, A4, A5,
             return this->_result.get();
         }
 
-        const R& operator()(const A1& a1, const A2& a2, const A3& a3, const A4& a4, const A5& a5)
+#if __cplusplus >= 201103L
+        R&& operator()(const A1& a1, const A2& a2, const A3& a3, const A4& a4, const A5& a5)
+#else
+        R& operator()(const A1& a1, const A2& a2, const A3& a3, const A4& a4, const A5& a5)
+#endif
         {
             return this->call(a1, a2, a3, a4, a5);
         }
@@ -609,7 +686,11 @@ class RemoteProcedure<R, A1, A2, A3, A4,
             this->client().beginCall(this->_r, *this, argv, 4);
         }
 
-        const R& call(const A1& a1, const A2& a2, const A3& a3, const A4& a4)
+#if __cplusplus >= 201103L
+        R&& call(const A1& a1, const A2& a2, const A3& a3, const A4& a4)
+#else
+        R& call(const A1& a1, const A2& a2, const A3& a3, const A4& a4)
+#endif
         {
             this->_result.clearFault();
 
@@ -624,7 +705,11 @@ class RemoteProcedure<R, A1, A2, A3, A4,
             return this->_result.get();
         }
 
-        const R& operator()(const A1& a1, const A2& a2, const A3& a3, const A4& a4)
+#if __cplusplus >= 201103L
+        R&& operator()(const A1& a1, const A2& a2, const A3& a3, const A4& a4)
+#else
+        R& operator()(const A1& a1, const A2& a2, const A3& a3, const A4& a4)
+#endif
         {
             return this->call(a1, a2, a3, a4);
         }
@@ -673,7 +758,11 @@ class RemoteProcedure<R, A1, A2, A3,
             this->client().beginCall(this->_r, *this, argv, 3);
         }
 
-        const R& call(const A1& a1, const A2& a2, const A3& a3)
+#if __cplusplus >= 201103L
+        R&& call(const A1& a1, const A2& a2, const A3& a3)
+#else
+        R& call(const A1& a1, const A2& a2, const A3& a3)
+#endif
         {
             this->_result.clearFault();
 
@@ -687,7 +776,11 @@ class RemoteProcedure<R, A1, A2, A3,
             return this->_result.get();
         }
 
-        const R& operator()(const A1& a1, const A2& a2, const A3& a3)
+#if __cplusplus >= 201103L
+        R&& operator()(const A1& a1, const A2& a2, const A3& a3)
+#else
+        R& operator()(const A1& a1, const A2& a2, const A3& a3)
+#endif
         {
             return this->call(a1, a2, a3);
         }
@@ -734,7 +827,11 @@ class RemoteProcedure<R, A1, A2,
             this->client().beginCall(this->_r, *this, argv, 2);
         }
 
-        const R& call(const A1& a1, const A2& a2)
+#if __cplusplus >= 201103L
+        R&& call(const A1& a1, const A2& a2)
+#else
+        R& call(const A1& a1, const A2& a2)
+#endif
         {
             this->_result.clearFault();
 
@@ -747,7 +844,11 @@ class RemoteProcedure<R, A1, A2,
             return this->_result.get();
         }
 
-        const R& operator()(const A1& a1, const A2& a2)
+#if __cplusplus >= 201103L
+        R&& operator()(const A1& a1, const A2& a2)
+#else
+        R& operator()(const A1& a1, const A2& a2)
+#endif
         {
             return this->call(a1, a2);
         }
@@ -792,7 +893,11 @@ class RemoteProcedure<R, A1,
             this->client().beginCall(this->_r, *this, argv, 1);
         }
 
-        const R& call(const A1& a1)
+#if __cplusplus >= 201103L
+        R&& call(const A1& a1)
+#else
+        R& call(const A1& a1)
+#endif
         {
             this->_result.clearFault();
 
@@ -804,7 +909,11 @@ class RemoteProcedure<R, A1,
             return this->_result.get();
         }
 
-        const R& operator()(const A1& a1)
+#if __cplusplus >= 201103L
+        R&& operator()(const A1& a1)
+#else
+        R& operator()(const A1& a1)
+#endif
         {
             return this->call(a1);
         }
@@ -846,7 +955,11 @@ class RemoteProcedure<R,
             this->client().beginCall(this->_r, *this, argv, 0);
         }
 
-        const R& call()
+#if __cplusplus >= 201103L
+        R&& call()
+#else
+        R& call()
+#endif
         {
             this->_result.clearFault();
 
@@ -857,13 +970,19 @@ class RemoteProcedure<R,
             return this->_result.get();
         }
 
-        const R& operator()()
+#if __cplusplus >= 201103L
+        R&& operator()()
+#else
+        R& operator()()
+#endif
         {
             return this->call();
         }
 };
 
+//! @endcond internal
+// end generated with `remoteprocedure.pl -n 10`
 
 }
 
-#endif // CXXTOOLS_REMOTEPROCEDURE_H
+#endif
