@@ -39,7 +39,8 @@ class IRemoteResult
 {
     public:
         IRemoteResult()
-            : _failed(false)
+            : _failed(false),
+              _rc(0)
             { }
 
         void setFault(int rc, const std::string& msg)
@@ -76,27 +77,45 @@ template <typename R>
 class RemoteResult : public IRemoteResult
 {
     public:
+        RemoteResult()
+            : _client(0)
+            { }
+
         explicit RemoteResult(RemoteClient& client)
-            : _client(client)
+            : _client(&client)
         {
         }
 
+        const R& value() const
+        {
+            return _result;
+        }
+
         R& value()
         {
             return _result;
         }
 
-        const R& get() const
+#if __cplusplus >= 201103L
+        R&& get()
         {
-            _client.endCall();
+            _client->endCall();
+            checkFault();
+            return std::move(_result);
+        }
+#else
+        R& get()
+        {
+            _client->endCall();
             checkFault();
             return _result;
         }
+#endif
 
-        RemoteClient& client()   { return _client; }
+        RemoteClient& client()   { return *_client; }
 
     private:
-        RemoteClient& _client;
+        RemoteClient* _client;
         R _result;
 };
 
