@@ -27,17 +27,24 @@
 #define CXXTOOLS_SELECTABLE_H
 
 #include <cxxtools/signal.h>
-#include <cxxtools/noncopyable.h>
-#include <cxxtools/api.h>
 #include <cxxtools/selector.h>
+#include <cxxtools/timespan.h>
 
 namespace cxxtools
 {
 
 class SelectableImpl;
 
-class CXXTOOLS_API Selectable : protected NonCopyable
+class Selectable
 {
+#if __cplusplus >= 201103L
+        Selectable(const Selectable&) = delete;
+        Selectable& operator=(const Selectable&)= delete;
+#else
+        Selectable(const Selectable&) { }
+        Selectable& operator=(const Selectable&) { return *this; }
+#endif
+
     public:
         static const std::size_t WaitInfinite = Selector::WaitInfinite;
 
@@ -53,11 +60,14 @@ class CXXTOOLS_API Selectable : protected NonCopyable
         //! @brief Destructor
         virtual ~Selectable();
 
+        /// Sets the selector for this device.
         void setSelector(SelectorBase* parent);
 
-        SelectorBase* selector();
+        /// For convenience the application may pass a reference to a selector.
+        void setSelector(SelectorBase& parent)
+        { setSelector(&parent); }
 
-        const SelectorBase* selector() const;
+        SelectorBase* selector() const;
 
         //! @brief Closes the I/O device
         /*!
@@ -65,7 +75,7 @@ class CXXTOOLS_API Selectable : protected NonCopyable
         */
         void close();
 
-        bool wait(std::size_t msecs = WaitInfinite);
+        bool wait(Milliseconds msecs = WaitInfinite);
 
         //! @brief Test if the I/O device object is enabled
         /*!
@@ -97,13 +107,15 @@ class CXXTOOLS_API Selectable : protected NonCopyable
         void setState(State state);
 
         //! @brief Closes the Selector
-        virtual void onClose() = 0;
+        virtual void onClose();
 
-        virtual bool onWait(std::size_t msecs) = 0;
+        virtual bool onWait(Timespan timeout);
 
-        virtual void onAttach(SelectorBase&) = 0;
+        virtual void onAttach(SelectorBase&)
+        { }
 
-        virtual void onDetach(SelectorBase&) = 0;
+        virtual void onDetach(SelectorBase&)
+        { }
 
     private:
         SelectorBase* _parent;
