@@ -28,19 +28,19 @@
 #ifndef cxxtools_SerializationError_h
 #define cxxtools_SerializationError_h
 
-#include <cxxtools/api.h>
 #include <string>
 #include <stdexcept>
 
 namespace cxxtools
 {
+class SerializationInfo;
 
 /** @brief Error during serialization of a type
 
     This Exception indicates a error during serialization caused by
     missing or invalid object attributes.
 */
-class CXXTOOLS_API SerializationError : public std::runtime_error
+class SerializationError : public std::runtime_error
 {
     public:
         /** @brief Construct with a message
@@ -53,12 +53,14 @@ class CXXTOOLS_API SerializationError : public std::runtime_error
         static void doThrow(const std::string& msg);
 };
 
-class CXXTOOLS_API SerializationMemberNotFound : public SerializationError
+class SerializationMemberNotFound : public SerializationError
 {
         std::string _member;
 
     public:
-        SerializationMemberNotFound(const std::string& member);
+        explicit SerializationMemberNotFound(const std::string& member);
+        SerializationMemberNotFound(const SerializationInfo& si, const std::string& member);
+        SerializationMemberNotFound(const SerializationInfo& si, unsigned idx);
         ~SerializationMemberNotFound() throw()
         { }
 
@@ -67,6 +69,16 @@ class CXXTOOLS_API SerializationMemberNotFound : public SerializationError
 
 };
 
+class SerializationConversionError : public SerializationError
+{
+    public:
+        explicit SerializationConversionError(const std::string& msg)
+            : SerializationError(msg)
+            { }
+
+        static void doThrow(const SerializationInfo& si, const std::string& typefrom, const std::string& typeto, const std::string& value = std::string());
+};
+
 } // namespace cxxtools
 
 
