@@ -29,35 +29,103 @@
 #ifndef cxxtools_SerializationInfo_h
 #define cxxtools_SerializationInfo_h
 
-#include <cxxtools/api.h>
 #include <cxxtools/string.h>
-#include <cxxtools/convert.h>
-#include <cxxtools/serializationerror.h>
 #include <vector>
 #include <set>
 #include <map>
 #include <list>
 #include <deque>
 #include <limits>
-#include <typeinfo>
+#include <iosfwd>
 #include <cxxtools/config.h>
 
+#if __cplusplus >= 201103L
+
+#include <forward_list>
+#include <unordered_set>
+#include <unordered_map>
+#include <tuple>
+#include <array>
+#include <type_traits>
+
+#endif
+
 namespace cxxtools
 {
 
 /** @brief Represents arbitrary types during serialization.
-*/
-class CXXTOOLS_API SerializationInfo
+
+    A SerializationInfo represent arbitrary types during serialization. It is
+    a flexible structure, which can hold all informations to reconstruct a
+    object.
+
+    The types of informations are:
+
+     - a category (Void, Value, Object or Array)
+     - a scalar value
+     - a name of the type
+     - a ordered list of SerializationInfo objects
+
+    Those informations are filled when objects are serialized using a
+    serialization operator and a serializer can use those to create a
+    serialized representation of a object.
+
+    A deserializer is used to convert a serialized representation into a
+    SerializationInfo and a deserialization operator is used to convert the
+    SerializationInfo back to the desired object.
+
+    To serialize and deserialize a object, those operators are needed.
+    For all standard types there are predefined operators. For user types
+    like own classes a operator can easily be written. The signature is
+    always the same.
+
+    Serialization operator is defined as:
+
+        void operator <<= (cxxtools::SerializationInfo& si, const YourType& object);
+
+    It must collect all informations from `YourType` and put them to the
+    cxxtools::SerializationInfo.
+
+    The deserialization operator is defined as:
+
+        void operator >>= (const cxxtools::SerializationInfo& si, YourType& object);
+
+    It must fetch the informations from SerializationInfo and fill the passed object
+    to reconstruct it.
+
+    With those 2 operators a serializer is able to to convert the object to
+    some format and the deserializers to convert it back to the object.
+
+    Since for standard container types operators are defined using templates,
+    The operators above are also able to serializer and deserialize e.g. a
+    `std::vector<YourType>`.
+
+    The scalar value may have several types. When accessing the value it is
+    typically converted implicitly when possible. So when e.g. the
+    SerializationInfo is set to a numeric value and a string value is read, you
+    get a string representation of the numeric value.
+
+ */
+
+class SerializationInfo
 {
-    typedef std::vector<SerializationInfo> Nodes;
+        typedef std::deque<SerializationInfo> Nodes;
+        friend void operator <<=(SerializationInfo& si, const SerializationInfo& ssi);
 
     public:
         enum Category {
             Void = 0, Value = 1, Object = 2, Array = 6
         };
 
-        class Iterator;
-        class ConstIterator;
+        /// iterator over subnodes.
+        typedef Nodes::iterator Iterator;
+        /// const iterator over subnodes.
+        typedef Nodes::const_iterator ConstIterator;
+
+        /// iterator over subnodes, std style name.
+        typedef Nodes::iterator iterator;
+        /// const iterator over subnodes, std style name.
+        typedef Nodes::const_iterator const_iterator;
 
 #ifdef HAVE_LONG_LONG
         typedef long long int_type;
@@ -75,10 +143,17 @@ class CXXTOOLS_API SerializationInfo
 
         SerializationInfo(const SerializationInfo& si);
 
-        ~SerializationInfo()
-        { _releaseValue(); }
+        ~SerializationInfo();
 
-        void reserve(size_t n);
+        SerializationInfo& operator =(const SerializationInfo& si);
+
+#if __cplusplus >= 201103L
+
+        SerializationInfo(SerializationInfo&& si) noexcept;
+
+        SerializationInfo& operator=(SerializationInfo&& si);
+
+#endif
 
         Category category() const
         {
@@ -90,16 +165,6 @@ class CXXTOOLS_API SerializationInfo
             _category = cat;
         }
 
-        SerializationInfo* parent()
-        {
-            return _parent;
-        }
-
-        const SerializationInfo* parent() const
-        {
-            return _parent;
-        }
-
         const std::string& typeName() const
         {
             return _type;
@@ -108,6 +173,8 @@ class CXXTOOLS_API SerializationInfo
         void setTypeName(const std::string& type)
         {
             _type = type;
+            if (_category == Void)
+                _category = Object;
         }
 
         const std::string& name() const
@@ -144,8 +211,8 @@ class CXXTOOLS_API SerializationInfo
 #endif
 
         void setValue(float value)               { _setFloat(value); }
-        void setValue(double value)              { _setFloat(value); }
-        void setValue(long double value)         { _setFloat(value); }
+        void setValue(double value)              { _setDouble(value); }
+        void setValue(long double value)         { _setLongDouble(value); }
         void setNull();
 
         /** @brief Deserialization of flat data-types
@@ -181,11 +248,11 @@ class CXXTOOLS_API SerializationInfo
             { value = static_cast<unsigned long long>(_getUInt("unsigned long long", std::numeric_limits<unsigned long long>::max())); }
 #endif
         void getValue(float& value) const
-            { value = static_cast<float>(_getFloat("float", std::numeric_limits<float>::max())); }
+            { value = _getFloat(); }
         void getValue(double& value) const
-            { value = static_cast<double>(_getFloat("double", std::numeric_limits<double>::max())); }
+            { value = _getDouble(); }
         void getValue(long double& value) const
-            { value = static_cast<long double>(_getFloat("long double", std::numeric_limits<long double>::max())); }
+            { value = _getLongDouble(); }
 
         /** @brief Serialization of flat member data-types
         */
@@ -199,19 +266,31 @@ class CXXTOOLS_API SerializationInfo
 
         /** @brief Serialization of member data
         */
-        SerializationInfo& addMember(const std::string& name);
+        SerializationInfo& addMember(const std::string& name = std::string());
+
+        /** @brief Returns a member; adds when not yet found
+         */
+        SerializationInfo& getAddMember(const std::string& name)
+        {
+            SerializationInfo* si = findMember(name);
+            if (si == 0)
+                return addMember(name);
+            return *si;
+        }
 
         /** @brief Deserialization of member data
 
             @throws SerializationError when member is not found.
         */
         const SerializationInfo& getMember(const std::string& name) const;
+        SerializationInfo& getMember(const std::string& name);
 
         /** @brief Deserialization of member data
 
             @throws std::range_error when index is too large
         */
         const SerializationInfo& getMember(unsigned idx) const;
+        SerializationInfo& getMember(unsigned idx);
 
         /** @brief Deserialization of member data.
 
@@ -244,40 +323,47 @@ class CXXTOOLS_API SerializationInfo
 
         size_t memberCount() const
         {
-            return _nodes.size();
+            return nodes().size();
         }
 
-        Iterator begin();
+        Iterator begin()
+        {
+            return nodes().begin();
+        }
 
-        Iterator end();
+        Iterator end()
+        {
+            return nodes().end();
+        }
 
-        ConstIterator begin() const;
+        ConstIterator begin() const
+        {
+            return nodes().begin();
+        }
 
-        ConstIterator end() const;
-
-        SerializationInfo& operator =(const SerializationInfo& si);
+        ConstIterator end() const
+        {
+            return nodes().end();
+        }
 
         void clear();
 
         void swap(SerializationInfo& si);
 
-        bool isNull() const     { return _t == t_none && _category == Void; }
-        bool isString() const   { return _t == t_string; }
-        bool isString8() const  { return _t == t_string8; }
-        bool isChar() const     { return _t == t_char; }
-        bool isBool() const     { return _t == t_bool; }
-        bool isInt() const      { return _t == t_int; }
-        bool isUInt() const     { return _t == t_uint; }
-        bool isFloat() const    { return _t == t_float; }
+        bool isNull() const       { return _t == t_none && (_category == Void || _category == Value); }
+        bool isString() const     { return _t == t_string; }
+        bool isString8() const    { return _t == t_string8; }
+        bool isChar() const       { return _t == t_char; }
+        bool isBool() const       { return _t == t_bool; }
+        bool isInt() const        { return _t == t_int; }
+        bool isUInt() const       { return _t == t_uint; }
+        bool isFloat() const      { return _t == t_float; }
+        bool isDouble() const     { return _t == t_double; }
+        bool isLongDouble() const { return _t == t_ldouble; }
 
-        void dump(std::ostream& out, const std::string& praefix = std::string()) const;
-
-    protected:
-        void setParent(SerializationInfo& si)
-        { _parent = &si; }
+        void dump(std::ostream& out, const std::string& prefix = std::string()) const;
 
     private:
-        SerializationInfo* _parent;
         Category _category;
         std::string _name;
         std::string _type;
@@ -290,14 +376,22 @@ class CXXTOOLS_API SerializationInfo
         void _setBool(bool value);
         void _setInt(int_type value);
         void _setUInt(unsigned_type value);
-        void _setFloat(long double value);
+        void _setFloat(float value);
+        void _setDouble(double value);
+        void _setLongDouble(long double value);
 
         bool _getBool() const;
         wchar_t _getWChar() const;
         char _getChar() const;
         int_type _getInt(const char* type, int_type min, int_type max) const;
         unsigned_type _getUInt(const char* type, unsigned_type max) const;
-        long double _getFloat(const char* type, long double max) const;
+        float _getFloat() const;
+        double _getDouble() const;
+        long double _getLongDouble() const;
+        Nodes& nodes();
+        const Nodes& nodes() const;
+        // assignment without name
+        void assignData(const SerializationInfo& si);
 
         union U
         {
@@ -306,16 +400,18 @@ class CXXTOOLS_API SerializationInfo
             bool _b;
             int_type _i;
             unsigned_type _u;
-            long double _f;
+            float _f;
+            double _d;
+            long double _ld;
         } _u;
 
-        String* _StringPtr()                    { return reinterpret_cast<String*>(_u._s); }
+        String* _StringPtr()                    { return reinterpret_cast<String*>(&_u); }
         String& _String()                       { return *_StringPtr(); }
-        const String* _StringPtr() const        { return reinterpret_cast<const String*>(_u._s); }
+        const String* _StringPtr() const        { return reinterpret_cast<const String*>(&_u); }
         const String& _String() const           { return *_StringPtr(); }
-        std::string* _String8Ptr()              { return reinterpret_cast<std::string*>(_u._s); }
+        std::string* _String8Ptr()              { return reinterpret_cast<std::string*>(&_u); }
         std::string& _String8()                 { return *_String8Ptr(); }
-        const std::string* _String8Ptr() const  { return reinterpret_cast<const std::string*>(_u._s); }
+        const std::string* _String8Ptr() const  { return reinterpret_cast<const std::string*>(&_u); }
         const std::string& _String8() const     { return *_String8Ptr(); }
 
         enum T
@@ -327,169 +423,20 @@ class CXXTOOLS_API SerializationInfo
           t_bool,
           t_int,
           t_uint,
-          t_float
+          t_float,
+          t_double,
+          t_ldouble
         } _t;
 
-        Nodes _nodes;             // objects/arrays
+        Nodes* _nodes;             // objects/arrays
 };
 
 
-class SerializationInfo::Iterator
-{
-    public:
-        Iterator();
-
-        Iterator(const Iterator& other);
-
-        Iterator(SerializationInfo* info);
-
-        Iterator& operator=(const Iterator& other);
-
-        Iterator& operator++();
-
-        SerializationInfo& operator*();
-
-        SerializationInfo* operator->();
-
-        bool operator!=(const Iterator& other) const;
-
-        bool operator==(const Iterator& other) const;
-
-    private:
-        SerializationInfo* _info;
-};
-
-
-class SerializationInfo::ConstIterator
-{
-    public:
-        ConstIterator();
-
-        ConstIterator(const ConstIterator& other);
-
-        ConstIterator(const SerializationInfo* info);
-
-        ConstIterator& operator=(const ConstIterator& other);
-
-        ConstIterator& operator++();
-
-        const SerializationInfo& operator*() const;
-
-        const SerializationInfo* operator->() const;
-
-        bool operator!=(const ConstIterator& other) const;
-
-        bool operator==(const ConstIterator& other) const;
-
-    private:
-        const SerializationInfo* _info;
-};
-
-
-inline SerializationInfo::Iterator::Iterator()
-: _info(0)
-{}
-
-
-inline SerializationInfo::Iterator::Iterator(const Iterator& other)
-: _info(other._info)
-{}
-
-
-inline SerializationInfo::Iterator::Iterator(SerializationInfo* info)
-: _info(info)
-{}
-
-
-inline SerializationInfo::Iterator& SerializationInfo::Iterator::operator=(const Iterator& other)
-{
-    _info = other._info;
-    return *this;
-}
-
-
-inline SerializationInfo::Iterator& SerializationInfo::Iterator::operator++()
-{
-    ++_info;
-    return *this;
-}
-
-
-inline SerializationInfo& SerializationInfo::Iterator::operator*()
-{
-    return *_info;
-}
-
-
-inline SerializationInfo* SerializationInfo::Iterator::operator->()
-{
-    return _info;
-}
-
-
-inline bool SerializationInfo::Iterator::operator!=(const Iterator& other) const
-{
-    return _info != other._info;
-}
-
-
-inline bool SerializationInfo::Iterator::operator==(const Iterator& other) const
-{
-    return _info == other._info;
-}
-
-
-inline SerializationInfo::ConstIterator::ConstIterator()
-: _info(0)
-{}
-
-
-inline SerializationInfo::ConstIterator::ConstIterator(const ConstIterator& other)
-: _info(other._info)
-{}
-
-
-inline SerializationInfo::ConstIterator::ConstIterator(const SerializationInfo* info)
-: _info(info)
-{}
-
-
-inline SerializationInfo::ConstIterator& SerializationInfo::ConstIterator::operator=(const ConstIterator& other)
-{
-    _info = other._info;
-    return *this;
-}
-
-
-inline SerializationInfo::ConstIterator& SerializationInfo::ConstIterator::operator++()
-{
-    ++_info;
-    return *this;
-}
-
-
-inline const SerializationInfo& SerializationInfo::ConstIterator::operator*() const
-{
-    return *_info;
-}
-
-
-inline const SerializationInfo* SerializationInfo::ConstIterator::operator->() const
-{
-    return _info;
-}
-
-
-inline bool SerializationInfo::ConstIterator::operator!=(const ConstIterator& other) const
-{
-    return _info != other._info;
-}
-
-
-inline bool SerializationInfo::ConstIterator::operator==(const ConstIterator& other) const
-{
-    return _info == other._info;
-}
+inline SerializationInfo::SerializationInfo()
+: _category(Void),
+  _t(t_none),
+  _nodes(0)
+{ }
 
 
 inline void operator >>=(const SerializationInfo& si, SerializationInfo& ssi)
@@ -498,10 +445,7 @@ inline void operator >>=(const SerializationInfo& si, SerializationInfo& ssi)
 }
 
 
-inline void operator <<=(SerializationInfo& si, const SerializationInfo& ssi)
-{
-    si = ssi;
-}
+void operator <<=(SerializationInfo& si, const SerializationInfo& ssi);
 
 
 inline void operator >>=(const SerializationInfo& si, bool& n)
@@ -677,7 +621,7 @@ inline void operator >>=(const SerializationInfo& si, float& n)
 inline void operator <<=(SerializationInfo& si, float n)
 {
     si.setValue(n);
-    si.setTypeName("double");
+    si.setTypeName("float");
 }
 
 
@@ -694,6 +638,19 @@ inline void operator <<=(SerializationInfo& si, double n)
 }
 
 
+inline void operator >>=(const SerializationInfo& si, long double& n)
+{
+    si.getValue(n);
+}
+
+
+inline void operator <<=(SerializationInfo& si, long double n)
+{
+    si.setValue(n);
+    si.setTypeName("double");
+}
+
+
 inline void operator >>=(const SerializationInfo& si, std::string& n)
 {
     si.getValue(n);
@@ -714,19 +671,32 @@ inline void operator <<=(SerializationInfo& si, const char* n)
 }
 
 
-inline void operator >>=(const SerializationInfo& si, cxxtools::String& n)
+inline void operator >>=(const SerializationInfo& si, String& n)
 {
     si.getValue(n);
 }
 
 
-inline void operator <<=(SerializationInfo& si, const cxxtools::String& n)
+inline void operator <<=(SerializationInfo& si, const String& n)
 {
     si.setValue(n);
     si.setTypeName("string");
 }
 
 
+inline void operator >>=(const SerializationInfo& si, Char& n)
+{
+    si.getValue(n);
+}
+
+
+inline void operator <<=(SerializationInfo& si, const Char& n)
+{
+    si.setValue(n);
+    si.setTypeName("char");
+}
+
+
 template <typename T, typename A>
 inline void operator >>=(const SerializationInfo& si, std::vector<T, A>& vec)
 {
@@ -745,10 +715,9 @@ inline void operator <<=(SerializationInfo& si, const std::vector<T, A>& vec)
 {
     typename std::vector<T, A>::const_iterator it;
 
-    si.reserve(vec.size());
     for(it = vec.begin(); it != vec.end(); ++it)
     {
-        SerializationInfo& newSi = si.addMember(std::string());
+        SerializationInfo& newSi = si.addMember();
         newSi <<= *it;
     }
 
@@ -757,28 +726,17 @@ inline void operator <<=(SerializationInfo& si, const std::vector<T, A>& vec)
 }
 
 
-inline void operator >>=(const SerializationInfo& si, std::vector<int>& vec)
+inline void operator>>=(const SerializationInfo& si, std::vector<bool>::reference bit)
 {
-    vec.clear();
-    for(SerializationInfo::ConstIterator it = si.begin(); it != si.end(); ++it)
-    {
-        vec.resize( vec.size() + 1 );
-        *it >>= vec.back();
-    }
+    bool v;
+    si >>= v;
+    bit = v;
 }
 
-inline void operator <<=(SerializationInfo& si, const std::vector<int>& vec)
+
+inline void operator<<=(SerializationInfo& si, std::vector<bool>::reference bit)
 {
-    std::vector<int>::const_iterator it;
-
-    for(it = vec.begin(); it != vec.end(); ++it)
-    {
-        SerializationInfo& newSi = si.addMember(std::string());
-        newSi <<= *it;
-    }
-
-    si.setTypeName("array");
-    si.setCategory(SerializationInfo::Array);
+    si <<= static_cast<bool>(bit);
 }
 
 
@@ -801,7 +759,7 @@ inline void operator <<=(SerializationInfo& si, const std::list<T, A>& list)
 
     for(it = list.begin(); it != list.end(); ++it)
     {
-        SerializationInfo& newSi = si.addMember(std::string());
+        SerializationInfo& newSi = si.addMember();
         newSi <<= *it;
     }
 
@@ -829,7 +787,7 @@ inline void operator <<=(SerializationInfo& si, const std::deque<T, A>& deque)
 
     for(it = deque.begin(); it != deque.end(); ++it)
     {
-        SerializationInfo& newSi = si.addMember(std::string());
+        SerializationInfo& newSi = si.addMember();
         newSi <<= *it;
     }
 
@@ -858,7 +816,7 @@ inline void operator <<=(SerializationInfo& si, const std::set<T, C, A>& set)
 
     for(it = set.begin(); it != set.end(); ++it)
     {
-        SerializationInfo& newSi = si.addMember(std::string());
+        SerializationInfo& newSi = si.addMember();
         newSi <<= *it;
     }
 
@@ -887,7 +845,7 @@ inline void operator <<=(SerializationInfo& si, const std::multiset<T, C, A>& mu
 
     for(it = multiset.begin(); it != multiset.end(); ++it)
     {
-        SerializationInfo& newSi = si.addMember(std::string());
+        SerializationInfo& newSi = si.addMember();
         newSi <<= *it;
     }
 
@@ -921,7 +879,8 @@ inline void operator >>=(const SerializationInfo& si, std::map<K, V, P, A>& map)
     {
         typename std::pair<K, V> v;
         *it >>= v;
-        map.insert(v);
+        typename std::map<K, V, P, A>::value_type vv(v.first, v.second);
+        map.insert(vv);
     }
 }
 
@@ -933,7 +892,7 @@ inline void operator <<=(SerializationInfo& si, const std::map<K, V, P, A>& map)
 
     for(it = map.begin(); it != map.end(); ++it)
     {
-        SerializationInfo& newSi = si.addMember(std::string());
+        SerializationInfo& newSi = si.addMember();
         newSi <<= *it;
     }
 
@@ -950,7 +909,8 @@ inline void operator >>=(const SerializationInfo& si, std::multimap<K, V, P, A>&
     {
         typename std::pair<K, V> v;
         *it >>= v;
-        multimap.insert(v);
+        typename std::multimap<K, V, P, A>::value_type vv(v.first, v.second);
+        multimap.insert(vv);
     }
 }
 
@@ -962,7 +922,7 @@ inline void operator <<=(SerializationInfo& si, const std::multimap<T, C, P, A>&
 
     for(it = multimap.begin(); it != multimap.end(); ++it)
     {
-        SerializationInfo& newSi = si.addMember(std::string());
+        SerializationInfo& newSi = si.addMember();
         newSi <<= *it;
     }
 
@@ -970,6 +930,331 @@ inline void operator <<=(SerializationInfo& si, const std::multimap<T, C, P, A>&
     si.setCategory(SerializationInfo::Array);
 }
 
+template <typename T, size_t N>
+void operator >>=(const SerializationInfo& si, T (&v)[N])
+{
+    for (size_t n = 0; n < N; ++n)
+        si.getMember(n) >>= v[n];
+}
+
+template <typename T, size_t N>
+void operator <<=(SerializationInfo& si, const T (&v)[N])
+{
+    for (size_t n = 0; n < N; ++n)
+        si.addMember() <<= v[n];
+
+    si.setTypeName("array");
+    si.setCategory(SerializationInfo::Array);
+}
+
+#if __cplusplus >= 201103L
+
+template <typename T, typename A>
+inline void operator >>=(const SerializationInfo& si, std::forward_list<T, A>& list)
+{
+    list.clear();
+    for (size_t n = si.memberCount(); n > 0; --n)
+    {
+        list.push_front(T());
+        si.getMember(n-1) >>= list.front();
+    }
+}
+
+
+template <typename T, typename A>
+inline void operator <<=(SerializationInfo& si, const std::forward_list<T, A>& list)
+{
+    typename std::forward_list<T, A>::const_iterator it;
+
+    for(it = list.begin(); it != list.end(); ++it)
+    {
+        SerializationInfo& newSi = si.addMember();
+        newSi <<= *it;
+    }
+
+    si.setTypeName("list");
+    si.setCategory(SerializationInfo::Array);
+}
+
+
+template <typename T, typename H, typename P, typename A>
+inline void operator >>=(const SerializationInfo& si, std::unordered_set<T, H, P, A>& unordered_set)
+{
+    unordered_set.clear();
+    for(SerializationInfo::ConstIterator it = si.begin(); it != si.end(); ++it)
+    {
+        T t;
+        *it >>= t;
+        unordered_set.insert(t);
+    }
+}
+
+
+template <typename T, typename H, typename P, typename A>
+inline void operator <<=(SerializationInfo& si, const std::unordered_set<T, H, P, A>& unordered_set)
+{
+    typename std::unordered_set<T, H, P, A>::const_iterator it;
+
+    for(it = unordered_set.begin(); it != unordered_set.end(); ++it)
+    {
+        SerializationInfo& newSi = si.addMember();
+        newSi <<= *it;
+    }
+
+    si.setTypeName("set");
+    si.setCategory(SerializationInfo::Array);
+}
+
+
+template <typename T, typename H, typename P, typename A>
+inline void operator >>=(const SerializationInfo& si, std::unordered_multiset<T, H, P, A>& unordered_multiset)
+{
+    unordered_multiset.clear();
+    for(SerializationInfo::ConstIterator it = si.begin(); it != si.end(); ++it)
+    {
+        T t;
+        *it >>= t;
+        unordered_multiset.insert(t);
+    }
+}
+
+
+template <typename T, typename H, typename P, typename A>
+inline void operator <<=(SerializationInfo& si, const std::unordered_multiset<T, H, P, A>& unordered_multiset)
+{
+    typename std::unordered_multiset<T, H, P, A>::const_iterator it;
+
+    for(it = unordered_multiset.begin(); it != unordered_multiset.end(); ++it)
+    {
+        SerializationInfo& newSi = si.addMember();
+        newSi <<= *it;
+    }
+
+    si.setTypeName("set");
+    si.setCategory(SerializationInfo::Array);
+}
+
+
+template <typename K, typename V, typename H, typename P, typename A>
+inline void operator >>=(const SerializationInfo& si, std::unordered_map<K, V, H, P, A>& unordered_map)
+{
+    unordered_map.clear();
+    for(SerializationInfo::ConstIterator it = si.begin(); it != si.end(); ++it)
+    {
+        typename std::pair<K, V> v;
+        *it >>= v;
+        typename std::unordered_map<K, V, H, P, A>::value_type vv(v.first, v.second);
+        unordered_map.insert(vv);
+    }
+}
+
+
+template <typename K, typename V, typename H, typename P, typename A>
+inline void operator <<=(SerializationInfo& si, const std::unordered_map<K, V, H, P, A>& unordered_map)
+{
+    typename std::unordered_map<K, V, H, P, A>::const_iterator it;
+
+    for(it = unordered_map.begin(); it != unordered_map.end(); ++it)
+    {
+        SerializationInfo& newSi = si.addMember();
+        newSi <<= *it;
+    }
+
+    si.setTypeName("map");
+    si.setCategory(SerializationInfo::Array);
+}
+
+
+template <typename K, typename V, typename H, typename P, typename A>
+inline void operator >>=(const SerializationInfo& si, std::unordered_multimap<K, V, H, P, A>& unordered_multimap)
+{
+    unordered_multimap.clear();
+    for(SerializationInfo::ConstIterator it = si.begin(); it != si.end(); ++it)
+    {
+        typename std::pair<K, V> v;
+        *it >>= v;
+        typename std::unordered_multimap<K, V, H, P, A>::value_type vv(v.first, v.second);
+        unordered_multimap.insert(vv);
+    }
+}
+
+
+template <typename K, typename V, typename H, typename P, typename A>
+inline void operator <<=(SerializationInfo& si, const std::unordered_multimap<K, V, H, P, A>& unordered_multimap)
+{
+    typename std::unordered_multimap<K, V, H, P, A>::const_iterator it;
+
+    for(it = unordered_multimap.begin(); it != unordered_multimap.end(); ++it)
+    {
+        SerializationInfo& newSi = si.addMember();
+        newSi <<= *it;
+    }
+
+    si.setTypeName("map");
+    si.setCategory(SerializationInfo::Array);
+}
+
+namespace helper
+{
+    template <int index, typename ... Types>
+    struct TupleSerializer
+    {
+        void serialize(SerializationInfo& si, const std::tuple<Types...>& tuple) const
+        {
+            TupleSerializer<index - 1, Types...>{}.serialize(si, tuple);
+            si.addMember() <<= std::get<index>(tuple);
+        }
+
+        void deserialize(const SerializationInfo& si, std::tuple<Types...>& tuple) const
+        {
+            TupleSerializer<index - 1, Types...>{}.deserialize(si, tuple);
+            si.getMember(index) >>= std::get<index>(tuple);
+        }
+    };
+
+    template <typename ... Types>
+    struct TupleSerializer<0, Types ...>
+    {
+        void serialize(SerializationInfo& si, const std::tuple<Types...>& tuple) const
+        {
+            si.addMember() <<= std::get<0>(tuple);
+        }
+
+        void deserialize(const SerializationInfo& si, std::tuple<Types...>& tuple) const
+        {
+            si.getMember(0) >>= std::get<0>(tuple);
+        }
+    };
+}
+
+template <class... Types>
+void operator >>=(const SerializationInfo& si, std::tuple<Types...>& tuple)
+{
+    const auto size = std::tuple_size<std::tuple<Types...>>::value;
+    helper::TupleSerializer<size - 1, Types...>{}.deserialize(si, tuple);
+}
+
+template <typename ... Types>
+void operator <<=(SerializationInfo& si, const std::tuple<Types...>& tuple)
+{
+    const auto size = std::tuple_size<std::tuple<Types...>>::value;
+    helper::TupleSerializer<size - 1, Types...>{}.serialize(si, tuple);
+
+    si.setTypeName("tuple");
+    si.setCategory(SerializationInfo::Array);
+}
+
+template <typename T, size_t N>
+void operator >>=(const SerializationInfo& si, std::array<T, N>& array)
+{
+    for (size_t n = 0; n < N; ++n)
+        si.getMember(n) >>= array[n];
+}
+
+template <typename T, size_t N>
+void operator <<=(SerializationInfo& si, const std::array<T, N>& array)
+{
+    for (size_t n = 0; n < N; ++n)
+        si.addMember() <<= array[n];
+
+    si.setTypeName("array");
+    si.setCategory(SerializationInfo::Array);
+}
+
+/// @internal
+template <typename ENUM>
+class EnumClassSerializer
+{
+    const ENUM& _value;
+
+public:
+    explicit EnumClassSerializer(const ENUM& e)
+        : _value(e)
+        { }
+
+    const ENUM& value() const   { return _value; }
+};
+
+/// @internal
+template <typename ENUM>
+void operator <<=(SerializationInfo& si, EnumClassSerializer<ENUM> e)
+{
+    si <<= static_cast<typename std::underlying_type<ENUM>::type>(e.value());
+}
+
+/// @internal
+template <typename ENUM>
+class EnumClassDeserializer
+{
+    ENUM& _value;
+
+public:
+    explicit EnumClassDeserializer(ENUM& e)
+        : _value(e)
+        { }
+
+    ENUM& value() const   { return _value; }
+};
+
+/// @internal
+template <typename ENUM>
+void operator <<=(SerializationInfo& si, EnumClassDeserializer<ENUM> e)
+{
+    si <<= static_cast<typename std::underlying_type<ENUM>::type>(e.value());
+}
+
+/// @internal
+template <typename ENUM>
+void operator >>=(const SerializationInfo& si, EnumClassDeserializer<ENUM> e)
+{
+    si >>= reinterpret_cast<typename std::underlying_type<ENUM>::type&>(e.value());
+}
+
+/** Helper for serializing strongly typed enums.
+ 
+    To serialize a enum this function returns a helper class, which can
+    serialize the underlying type using the serialization operator.
+
+    @code
+        enum class Foo : char
+        {
+            FOO, BAR, BAZ
+        };
+
+        Foo foo(Foo::FOO);
+
+        cxxtools::SerializationInfo si;
+        si <<= cxxtools::EnumClass(foo);
+    @endcode
+
+ */
+template <typename ENUM>
+EnumClassSerializer<ENUM> EnumClass(const ENUM& e)
+{ return EnumClassSerializer<ENUM>(e); }
+
+/** Helper for deserializing strongly typed enums.
+ 
+    To deserialize a enum this function returns a helper class, which can
+    serialize and deserialize the underlying type using the proper operators.
+
+    @code
+        enum class Foo : char
+        {
+            FOO, BAR, BAZ
+        };
+
+        Foo foo(Foo::FOO);
+
+        cxxtools::SerializationInfo si;
+        si >>= cxxtools::EnumClass(foo);
+    @endcode
+
+ */
+template <typename ENUM>
+EnumClassDeserializer<ENUM> EnumClass(ENUM& e)
+{ return EnumClassDeserializer<ENUM>(e); }
+
+#endif // __cplusplus >= 201103L
 
 inline std::ostream& operator<< (std::ostream& out, const SerializationInfo& si)
 {
@@ -977,6 +1262,29 @@ inline std::ostream& operator<< (std::ostream& out, const SerializationInfo& si)
     return out;
 }
 
+/** Converts one object to another using serialization operators.
+
+    If a object is serializable (i.e. it has a operator<<=(SerializationInfo&, T)),
+    and another is deserializable (i.e. it has a operator>>=(const SerializationInfo&, ...))
+    this function can be used to convert one to the other.
+
+    @code
+        std::set<int> foo = getSetFromSomewhere();
+        std::vector<long> bar = cxxtools::serialization_cast<std::vector<long> >(foo);
+        // or with C++11:
+        auto bar = cxxtools::serialization_cast<std::vector<long> >(foo);
+    @endcode
+ */
+template <typename R, typename T>
+R serialization_cast(const T& t)
+{
+    SerializationInfo si;
+    si <<= t;
+    R r;
+    si >>= r;
+    return r;
+}
+
 } // namespace cxxtools
 
 
