@@ -42,8 +42,7 @@ namespace cxxtools
 
       The most common output iterator is a std::back_inserter.
 
-      Example (prints username and uid on a unix system:
-
+      Example (print username and uid on a unix system):
       \code
         std::ifstream passwd("/etc/passwd");
         std::string line;
@@ -60,17 +59,21 @@ namespace cxxtools
   template <typename outputIterator, typename characterType>
   void split(characterType ch, const std::basic_string<characterType>& line, outputIterator it)
   {
-    std::basic_string<characterType> s(line);
-    typename std::basic_string<characterType>::size_type pos;
+    if (line.empty())
+      return;
 
-    while ((pos = s.find(ch)) != std::basic_string<characterType>::npos)
+    typename std::basic_string<characterType>::size_type b;
+    typename std::basic_string<characterType>::size_type e;
+
+    b = 0;
+    while ((e = line.find(ch, b)) != std::basic_string<characterType>::npos)
     {
-      *it = s.substr(0, pos);
+      (*it) = std::basic_string<characterType>(line, b, e - b);
       ++it;
-      s.erase(0, pos + 1);
+      b = e + 1;
     }
 
-    *it = s;
+    *it = std::basic_string<characterType>(line, b);
     ++it;
   }
 
@@ -86,7 +89,7 @@ namespace cxxtools
       \code
         std::string line = ...;
         std::vector<std::string> tokens;
-        cxxtools::split("[ \t]+", line, std::back_inserter(tokens));
+        cxxtools::split(" \t", line, std::back_inserter(tokens));
       \endcode
    
 
@@ -94,17 +97,21 @@ namespace cxxtools
   template <typename outputIterator, typename characterType>
   void split(const characterType* chars, const std::basic_string<characterType>& line, outputIterator it)
   {
-    std::basic_string<characterType> s(line);
-    typename std::basic_string<characterType>::size_type pos;
+    if (line.empty())
+      return;
 
-    while ((pos = s.find_first_of(chars)) != std::basic_string<characterType>::npos)
+    typename std::basic_string<characterType>::size_type b;
+    typename std::basic_string<characterType>::size_type e;
+
+    b = 0;
+    while ((e = line.find_first_of(chars, b)) != std::basic_string<characterType>::npos)
     {
-      *it = s.substr(0, pos);
+      (*it) = std::basic_string<characterType>(line, b, e - b);
       ++it;
-      s.erase(0, pos + 1);
+      b = e + 1;
     }
 
-    *it = s;
+    *it = std::basic_string<characterType>(line, b);
     ++it;
   }
 
@@ -128,17 +135,21 @@ namespace cxxtools
   template <typename outputIterator, typename characterType>
   void split(const std::basic_string<characterType>& chars, const std::basic_string<characterType>& line, outputIterator it)
   {
-    std::basic_string<characterType> s(line);
-    typename std::basic_string<characterType>::size_type pos;
+    if (line.empty())
+      return;
 
-    while ((pos = s.find_first_of(chars)) != std::basic_string<characterType>::npos)
+    typename std::basic_string<characterType>::size_type e;
+    typename std::basic_string<characterType>::size_type b;
+
+    b = 0;
+    while ((e = line.find_first_of(chars, b)) != std::basic_string<characterType>::npos)
     {
-      *it = s.substr(0, pos);
+      (*it) = std::basic_string<characterType>(line, b, e - b);
       ++it;
-      s.erase(0, pos + 1);
+      b = e + 1;
     }
 
-    *it = s;
+    *it = std::basic_string<characterType>(line, b);
     ++it;
   }
 
@@ -159,17 +170,21 @@ namespace cxxtools
   template <typename outputIterator>
   void split(const Regex& re, const std::string& line, outputIterator it)
   {
-    std::string s(line);
-    RegexSMatch sm;
+    if (line.empty())
+      return;
 
-    while (re.match(s, sm))
+    RegexSMatch sm;
+    typename std::string::size_type b;
+
+    b = 0;
+    while (re.matchp(line, b, sm))
     {
-      *it = s.substr(0, sm.offsetBegin(0));
+      (*it) = std::string(line, b, sm.offsetBegin(0) - b);
       ++it;
-      s.erase(0, sm.offsetEnd(0));
+      b = sm.offsetEnd(0);
     }
 
-    *it = s;
+    *it = std::string(line, b);
     ++it;
   }
 
