@@ -30,12 +30,10 @@
 #ifndef CXXTOOLS_STRING_H
 #define CXXTOOLS_STRING_H
 
-#include <cxxtools/api.h>
 #include <cxxtools/char.h>
 
 #include <string>
 #include <iterator>
-#include <stdexcept>
 #include <iosfwd>
 
 #include <cxxtools/config.h>
@@ -46,7 +44,7 @@ namespace std {
     @ingroup Unicode
 */
 template <>
-class CXXTOOLS_API basic_string< cxxtools::Char > {
+class basic_string< cxxtools::Char > {
     public:
         typedef cxxtools::Char value_type;
         typedef size_t size_type;
@@ -104,6 +102,14 @@ class CXXTOOLS_API basic_string< cxxtools::Char > {
         template <typename InputIterator>
         basic_string(InputIterator begin, InputIterator end, const allocator_type& a = allocator_type());
 
+#if __cplusplus >= 201103L
+
+        basic_string(basic_string&& str) noexcept;
+
+        basic_string& operator=(basic_string&& str);
+
+#endif
+
         ~basic_string();
 
     public:
@@ -309,6 +315,8 @@ class CXXTOOLS_API basic_string< cxxtools::Char > {
 //
         size_type find(cxxtools::Char ch, size_type pos = 0) const;
 
+        size_type find(wchar_t ch, size_type pos = 0) const;
+
         size_type rfind(const basic_string& str, size_type pos = npos) const;
 
         size_type rfind(const cxxtools::Char* str, size_type pos, size_type n) const;
@@ -317,6 +325,8 @@ class CXXTOOLS_API basic_string< cxxtools::Char > {
 
         size_type rfind(cxxtools::Char ch, size_type pos = npos) const;
 
+        size_type rfind(wchar_t ch, size_type pos = npos) const;
+
         size_type find_first_of(const basic_string& str, size_type pos = 0) const
         { return this->find_first_of( str.data(), pos, str.size() ); }
 
@@ -448,19 +458,23 @@ class CXXTOOLS_API basic_string< cxxtools::Char > {
         uint32_t& shortStringMagic()                  { return _data.u.shortdata[_shortStringSize - 1]; }
         size_type shortStringLength() const           { return _shortStringSize - 1 - shortStringMagic(); }
         size_type shortStringCapacity() const         { return _shortStringSize - 1; }
-        void setShortStringLength(size_type n)        { shortStringData()[n] = cxxtools::Char::null(); shortStringMagic() = _shortStringSize - n - 1; }
+        void setShortStringLength(size_type n)
+        {
+            shortStringData()[n] = cxxtools::Char::null();
+            shortStringMagic() = static_cast<uint32_t>(_shortStringSize - n - 1);
+        }
         void shortStringAssign(const cxxtools::Char* str, size_type n)
         {
             traits_type::copy(shortStringData(), str, n);
             shortStringData()[n] = cxxtools::Char::null();
-            shortStringMagic() = _shortStringSize - n - 1;
+            shortStringMagic() = static_cast<uint32_t>(_shortStringSize - n - 1);
         }
         void shortStringAssign(const wchar_t* str, size_type n)
         {
             for (size_type nn = 0; nn < n; ++nn)
                 shortStringData()[nn] = str[nn];
             shortStringData()[n] = cxxtools::Char::null();
-            shortStringMagic() = _shortStringSize - n - 1;
+            shortStringMagic() = static_cast<uint32_t>(_shortStringSize - n - 1);
         }
 
         const cxxtools::Char* longStringData() const    { return _data.u.ptr._begin; }
@@ -561,7 +575,7 @@ class CXXTOOLS_API basic_string< cxxtools::Char > {
     inline bool operator>(const basic_string<cxxtools::Char>& a, const wchar_t* b)
     { return a.compare(b) > 0; }
 
-    CXXTOOLS_API ostream& operator<< (ostream& out, const basic_string<cxxtools::Char>& str);
+    ostream& operator<< (ostream& out, const basic_string<cxxtools::Char>& str);
 
 } // namespace std
 
