@@ -28,7 +28,6 @@
 #ifndef cxxtools_TextBuffer_h
 #define cxxtools_TextBuffer_h
 
-#include <cxxtools/api.h>
 #include <cxxtools/char.h>
 #include <cxxtools/textcodec.h>
 #include <cxxtools/conversionerror.h>
@@ -322,8 +321,12 @@ class BasicTextBuffer : public std::basic_streambuf<CharT>
 
             bool atEof = false;
             const std::streamsize bufavail = _ebufmax - _ebufsize;
-            size = bufavail < size ? bufavail : size;
-            if(size)
+            const std::streamsize in_avail = _target->rdbuf()->in_avail();
+            if (bufavail < size)
+                size = bufavail;
+            if (in_avail > 0 && in_avail < size)
+                size = in_avail;
+            if (size > 0)
             {
                 n = _target->rdbuf()->sgetn( _ebuf + _ebufsize, size );
                 _ebufsize += n;
@@ -339,26 +342,25 @@ class BasicTextBuffer : public std::basic_streambuf<CharT>
             char_type* toNext            = toBegin;
 
             typename CodecType::result r = CodecType::noconv;
-            if(_codec)
-                r = _codec->in(_state, fromBegin, fromEnd, fromNext, toBegin, toEnd, toNext);
-
-            if(r == CodecType::noconv)
+            if (_codec)
             {
-                // copy characters and advance fromNext and toNext
-                int n =_ebufsize > _ibufmax ? _ibufmax : _ebufsize ;
+                r = _codec->in(_state, fromBegin, fromEnd, fromNext, toBegin, toEnd, toNext);
+                std::streamsize consumed = fromNext - fromBegin;
+                if(consumed)
+                {
+                    std::char_traits<extern_type>::move( _ebuf, _ebuf + consumed, _ebufsize );
+                    _ebufsize -= consumed;
+                }
+            }
+            else
+            {
+                // copy characters and advance toNext
+                int n = _ebufsize > _ibufmax ? _ibufmax : _ebufsize;
                 this->copyBytes(toBegin, fromBegin, n);
                 _ebufsize -= n;
-                fromNext += n;
                 toNext += n;
             }
 
-            std::streamsize consumed = fromNext - fromBegin;
-            if(consumed)
-            {
-                std::char_traits<extern_type>::move( _ebuf, _ebuf + consumed, _ebufsize );
-                _ebufsize -= consumed;
-            }
-
             std::streamsize generated = toNext - toBegin;
             if(generated)
             {
@@ -407,7 +409,7 @@ class BasicTextBuffer : public std::basic_streambuf<CharT>
     The internal type is cxxtools::Char. The external type is $char$.
     See BasicTextBuffer for a more detailed description.
 */
-class CXXTOOLS_API TextBuffer : public BasicTextBuffer<cxxtools::Char, char>
+class TextBuffer : public BasicTextBuffer<cxxtools::Char, char>
 {
     public:
         typedef TextCodec<cxxtools::Char, char> Codec;
