@@ -26,15 +26,17 @@
  * License along with this library; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  */
-#if !defined(CXXTOOLS_SYSTEM_THREAD_H)
-#define CXXTOOLS_SYSTEM_THREAD_H
 
-#include <cxxtools/noncopyable.h>
+#ifndef CXXTOOLS_THREAD_H
+#define CXXTOOLS_THREAD_H
+
 #include <cxxtools/callable.h>
 #include <cxxtools/function.h>
 #include <cxxtools/method.h>
+#include <cxxtools/timespan.h>
 
-namespace cxxtools {
+namespace cxxtools
+{
 
     /** @brief Platform independent threads
 
@@ -57,8 +59,15 @@ namespace cxxtools {
         can give up CPU time either by calling Thread::yield() or sleep() to
         stop for a specified periode of time.
     */
-    class Thread : protected NonCopyable
+    class Thread
     {
+#if __cplusplus >= 201103L
+            Thread(const Thread&) = delete;
+            Thread& operator=(const Thread&) = delete;
+#else
+            Thread(const Thread&) { }
+            Thread& operator=(const Thread&) { return *this; }
+#endif
         public:
             //! @brief Status of a thread
             enum State
@@ -132,10 +141,21 @@ namespace cxxtools {
 
             /** @brief Sleep for some time
 
-                The calling thread sleeps for \a ms milliseconds. Throws a
-                SystemError on failure.
+                The calling thread sleeps for a specified timespan.
+                Since the Milliseconds class has conversion from double to Milliseconds
+                it is possible to pass just a number. Another possibility is to pass
+                another Timespan object. The cxxtools::Seconds is also derived from
+                Timespan an hence perfectly ok.
+
+                Example:
+
+                @code
+                  cxxtools::Thread::sleep(500);  // waits 500 milliseconds
+                  cxxtools::Thread::sleep(cxxtools::Seconds(0.5));  // waits 500 milliseconds
+                @endcode
+
             */
-            static void sleep(unsigned int ms);
+            static void sleep(const Milliseconds& ts);
 
         protected:
             //! @brief Detaches the thread
