@@ -30,7 +30,6 @@
 #ifndef CXXTOOLS_TIME_H
 #define CXXTOOLS_TIME_H
 
-#include <cxxtools/api.h>
 #include <cxxtools/timespan.h>
 #include <string>
 #include <stdexcept>
@@ -41,11 +40,15 @@ namespace cxxtools
 
 class SerializationInfo;
 
-class CXXTOOLS_API InvalidTime : public std::invalid_argument
+class InvalidTime : public std::invalid_argument
 {
     public:
         InvalidTime();
 
+        explicit InvalidTime(const std::string& what)
+            : std::invalid_argument(what)
+            { }
+
         ~InvalidTime() throw()
         {}
 };
@@ -69,69 +72,109 @@ class Time
         static const unsigned MSecsPerHour     = 3600000;
         static const unsigned MSecsPerMinute   = 60000;
         static const unsigned MSecsPerSecond   = 1000;
+        static const uint64_t USecsPerDay = static_cast<uint64_t>(MSecsPerDay) * 1000;
 
         /** \brief Creates a Time set to zero.
         */
         Time()
-        : _msecs(0)
+        : _usecs(0)
         {}
 
+        /** \brief create Time from string using format
+
+            Valid format codes are:
+
+              %H   hours (0-23)
+              %I   hours (0-11)
+              %2I  hours (00-11)
+              %M   minutes (0-59)
+              %2M  minutes (00-59)
+              %S   seconds (0-59)
+              %2S  seconds (00-59)
+              %j   fractional seconds (1-6 digits, optionally leading '.')
+              %J   fractional seconds (1-6 digits, with leading '.')
+              %K   milliseconds (3 digits, with leading '.')
+              %k   milliseconds (3 digits)
+              %U   microseconds (6 digits, with leading '.')
+              %u   microseconds (6 digits)
+              %p   AM/PM
+              ?    arbitrary character
+              *    skip non digit characters
+              #    skip word
+         */
+        explicit Time(const std::string& d, const std::string& fmt = "%H:%M:%S%j");
+
         /** \brief Creates a Time from given values.
 
             InvalidTime is thrown if one or more of the values are out of range
         */
-        inline Time(unsigned h, unsigned m, unsigned s = 0, unsigned ms = 0)
+        Time(unsigned h, unsigned m, unsigned s = 0, unsigned ms = 0, unsigned usec = 0)
         {
-            set(h, m, s, ms);
+            set(h, m, s, ms, usec);
         }
 
         /** \brief Returns the hour-part of the Time.
         */
         unsigned hour() const
         {
-            return _msecs / MSecsPerHour;
+            return unsigned(_usecs / 1000 / 1000 / 60 / 60);
         }
 
         /** \brief Returns the minute-part of the Time.
         */
         unsigned minute() const
         {
-            return (_msecs % MSecsPerHour) / MSecsPerMinute;
+            return unsigned(_usecs / 1000 / 1000 / 60) % 60;
         }
 
-        /** \brief Returns the second-part of the Time.
+        /** \brief Returns the second part of the Time.
         */
         unsigned second() const
         {
-            return (_msecs / 1000) % SecondsPerMinute;
+            return unsigned(_usecs / 1000 / 1000) % 60;
         }
 
-        /** \brief Returns the millisecond-part of the Time.
+        /** \brief Returns the millisecond part of the Time.
         */
         unsigned msec() const
         {
-            return _msecs % 1000;
+            return unsigned(_usecs / 1000) % 1000;
+        }
+
+        /** \brief Returns the microseconds of the Time.
+         *
+         *  Note that the microseconds contain the milliseconds also.
+        */
+        unsigned usec() const
+        {
+            return unsigned(_usecs % 1000000);
         }
 
         unsigned totalMSecs() const
-        { return _msecs; }
+        { return _usecs / 1000; }
+
+        uint64_t totalUSecs() const
+        { return _usecs; }
 
         void setTotalMSecs(unsigned msecs)
-        { _msecs = msecs; }
+        { _usecs = static_cast<uint64_t>(msecs) * 1000; }
+
+        void setTotalUSecs(uint64_t m)
+        { _usecs = m; }
 
         /** \brief Sets the time.
 
-            Sets the time to a new hour, minute, second, milli-second.
+            Sets the time to a new hour, minute, second, millisecond.
             InvalidTime is thrown if one or more of the values are out of range
         */
-        void set(unsigned hour, unsigned min, unsigned sec, unsigned msec = 0)
+        void set(unsigned hour, unsigned min, unsigned sec, unsigned msec = 0, unsigned usec = 0)
         {
-            if ( ! isValid(hour, min, sec , msec) )
+            if ( ! isValid(hour, min, sec, msec, usec) )
             {
                 throw InvalidTime();
             }
 
-            _msecs = (hour*SecondsPerHour + min*SecondsPerMinute + sec) * 1000 + msec;
+            _usecs = (((((static_cast<uint64_t>(hour) * 60 + min) * 60) + sec) * 1000) + msec) * 1000 + usec;
         }
 
         /** @brief Get the time values
@@ -146,133 +189,107 @@ class Time
             ms = msec();
         }
 
-        /** @brief Adds seconds to the time
+        /** @brief Get the time values
 
-            This method does not change the time, but returns the time
-            with the seconds added.
+            Gets the hour, minute, second, millisecond and microsecond parts of the time.
+            Note that the microseconds include the millisecons.
         */
-        Time addSecs(int secs) const
+        void get(unsigned& h, unsigned& m, unsigned& s, unsigned& ms, unsigned& us) const
         {
-            return addMSecs(secs * 1000);
+            h = hour();
+            m = minute();
+            s = second();
+            ms = msec();
+            us = usec();
         }
 
-        /** @brief Determines seconds until another time
-        */
-        int secsTo(const Time &t) const
-        {
-            return static_cast<int>( msecsTo(t) / 1000 );
-        }
+        /** \brief format Time into a string using a format string
 
-        /** @brief Adds milliseconds to the time
+            Valid format codes are:
 
-            This method does not change the time, but returns the time
-            with the milliseconds added.
-        */
-        inline Time addMSecs(int64_t ms) const
-        {
-            Time t;
-            if (ms < 0)
-            {
-                int64_t negdays = (MSecsPerDay - ms) / MSecsPerDay;
-                t._msecs = static_cast<unsigned>((_msecs + ms + negdays * MSecsPerDay) % MSecsPerDay);
-            }
-            else
-            {
-                t._msecs = static_cast<unsigned>((_msecs + ms) % MSecsPerDay);
-            }
-
-            return t;
-        }
-
-        /** @brief Determines milliseconds until another time
-        */
-        int64_t msecsTo(const Time &t) const
-        {
-            if(t._msecs > _msecs)
-                return t._msecs - _msecs;
-
-            return MSecsPerDay - (_msecs - t._msecs);
-        }
+              %1H  hours (0-23)
+              %I   hours (00-11)
+              %1I  hours (0-11)
+              %M   minutes (00-59)
+              %1M  minutes (0-59)
+              %S   seconds (00-59)
+              %1S  seconds (0-59)
+              %j   fractional seconds (1-6 digits, optionally leading '.')
+              %J   fractional seconds (1-6 digits, with leading '.')
+              %K   milliseconds (3 digits, with leading '.')
+              %k   milliseconds (3 digits)
+              %U   microseconds (6 digits, with leading '.')
+              %u   microseconds (6 digits)
+              %p   am/pm
+              %P   AM/PM
+         */
+        std::string toString(const std::string& fmt = "%H:%M:%S%j") const;
 
         /** @brief Assignment operator
         */
         Time& operator=(const Time& other)
-        { _msecs=other._msecs; return *this; }
+        { _usecs=other._usecs; return *this; }
 
         /** @brief Equal comparison operator
         */
         bool operator==(const Time& other) const
-        { return _msecs == other._msecs; }
+        { return _usecs == other._usecs; }
 
         /** @brief Inequal comparison operator
         */
         bool operator!=(const Time& other) const
-        { return _msecs != other._msecs; }
+        { return _usecs != other._usecs; }
 
         /** @brief Less-than comparison operator
         */
         bool operator<(const Time& other) const
-        { return _msecs < other._msecs; }
+        { return _usecs < other._usecs; }
 
         /** @brief Less-than-or-equal comparison operator
         */
         bool operator<=(const Time& other) const
-        { return _msecs <= other._msecs; }
+        { return _usecs <= other._usecs; }
 
         /** @brief Greater-than comparison operator
         */
         bool operator>(const Time& other) const
-        { return _msecs > other._msecs; }
+        { return _usecs > other._usecs; }
 
         /** @brief Greater-than-or-equal comparison operator
         */
         bool operator>=(const Time& other) const
-        { return _msecs >= other._msecs; }
+        { return _usecs >= other._usecs; }
 
         /** @brief Assignment by sum operator
         */
-        Time& operator+=(const Timespan& ts)
-        {
-            int64_t msecs = ( _msecs + ts.totalMSecs() ) % MSecsPerDay;
-            msecs = msecs < 0 ? MSecsPerDay + msecs : msecs;
-            _msecs = static_cast<unsigned>(msecs);
-            return *this;
-        }
+        Time& operator+=(const Timespan& ts);
 
         /** @brief Assignment by difference operator
         */
-        Time& operator-=(const Timespan& ts)
-        {
-            int64_t msecs = ( _msecs - ts.totalMSecs() ) % MSecsPerDay;
-            msecs = msecs < 0 ? MSecsPerDay + msecs : msecs;
-            _msecs = static_cast<unsigned>(msecs);
-            return *this;
-        }
+        Time& operator-=(const Timespan& ts);
 
         /** @brief Addition operator
         */
-        friend Time operator+(const Time& time, const Timespan& ts)
-        {
-            return time.addMSecs( ts.totalMSecs() );
-        }
+        friend Time operator+(const Time& time, const Timespan& ts);
 
-        /** @brief Substraction operator
+        /** @brief Subtraction operator
         */
-        friend Time operator-(const Time& time, const Timespan& ts)
-        {
-            return time.addMSecs( -ts.totalMSecs() );
-        }
+        friend Time operator-(const Time& time, const Timespan& ts);
 
-        /** @brief Substraction operator
+        /** @brief Subtraction operator
         */
         friend Timespan operator-(const Time& a, const Time& b)
         {
-            return b.msecsTo(a) * 1000;
+            return Microseconds(
+                a._usecs >= b._usecs
+                    ? double(a._usecs - b._usecs)
+                    : -double(b._usecs - a._usecs));
         }
 
         /** \brief Returns the time in ISO-format (hh:mm:ss.hhh)
         */
-        std::string toIsoString() const;
+        std::string toIsoString() const
+        { return toString(); }
 
         /** \brief Returns true if values are a valid time
         */
@@ -281,40 +298,49 @@ class Time
             return h < 24 && m < 60 && s < 60 && ms < 1000;
         }
 
+        /** \brief Returns true if values are a valid time
+        */
+        static bool isValid(unsigned h, unsigned m, unsigned s, unsigned ms, unsigned usec)
+        {
+            return h < 24 && m < 60 && s < 60 && ms < 1000 && usec < 1000000;
+        }
+
         /** \brief Convert from an ISO time string
 
             Interprets the passed string as a time-string in ISO-format
             (hh:mm:ss.hhh) and returns a Time-object. If the string is not
             in ISO-format, InvalidTime is thrown.
         */
-        static Time fromIsoString(const std::string& s);
+        static Time fromIsoString(const std::string& s)
+        { return Time(s); }
 
     private:
         //! @internal
-        unsigned _msecs;
+        uint64_t _usecs;
     };
 
-    CXXTOOLS_API void operator >>=(const SerializationInfo& si, Time& time);
+    void operator >>=(const SerializationInfo& si, Time& time);
 
-    CXXTOOLS_API void operator <<=(SerializationInfo& si, const Time& time);
+    void operator <<=(SerializationInfo& si, const Time& time);
 
-    CXXTOOLS_API void convert(std::string& str, const cxxtools::Time& time);
-
-    CXXTOOLS_API void convert(cxxtools::Time& time, const std::string& s);
-
-    inline std::string Time::toIsoString() const
+    /** @brief Addition operator
+    */
+    inline Time operator+(const Time& time, const Timespan& ts)
     {
-        std::string str;
-        convert(str, *this);
-        return str;
+        Time t(time);
+        t += ts;
+        return t;
     }
 
-    inline Time Time::fromIsoString(const std::string& s)
+    /** @brief Subtraction operator
+    */
+    inline Time operator-(const Time& time, const Timespan& ts)
     {
-        Time time;
-        convert(time, s);
-        return time;
+        Time t(time);
+        t -= ts;
+        return t;
     }
+
 }
 
 #endif // CXXTOOLS_TIME_H
