@@ -28,12 +28,11 @@
 
 #include <cxxtools/signal.h>
 #include <cxxtools/timespan.h>
-#include <cxxtools/api.h>
-#include <vector>
-#include <cstddef>
 
 namespace cxxtools {
 
+    class DateTime;
+    class UtcDateTime;
     class SelectorBase;
 
     /** @brief Notifies clients in constant intervals
@@ -53,28 +52,32 @@ namespace cxxtools {
 
         int main()
         {
-            cxxtools::Timer timer;
-            connect(timer.timeout, onTimer);
-
             cxxtools::EventLoop loop;
-            loop.add(timer);
+
+            cxxtools::Timer timer(&loop);
+            connect(timer.timeout, onTimer);
             timer.start(1000);
 
             loop.run();
             return 0;
         }
         @endcode
+
+        Note that a timer can process just one interval at once. When one of
+        the methods is called while the timer is active, the previous interval
+        will be stopped.
     */
-    class CXXTOOLS_API Timer
+    class Timer
     {
         class Sentry;
 
         public:
             /** @brief Default constructor
 
-                Constructs an inactive timer.
+                Constructs an inactive timer and optionally sets the selector,
+                which must be done anyway to run the timer.
             */
-            Timer();
+            explicit Timer(SelectorBase* selector = 0);
 
             /** @brief Destructor
 
@@ -87,6 +90,9 @@ namespace cxxtools {
 
             void setSelector(SelectorBase* s);
 
+            void setSelector(SelectorBase& s)
+            { setSelector(&s); }
+
             /** @brief Returs true if timer is active
             */
             bool active() const;
@@ -95,7 +101,7 @@ namespace cxxtools {
 
                 Returns the current interval of the timer in milliseconds.
             */
-            std::size_t interval() const;
+            const Timespan& interval() const;
 
             /** @brief Starts the timer
 
@@ -103,9 +109,44 @@ namespace cxxtools {
                 Timer needs to be registered with a Selector or event loop,
                 otherwise the timeout signal will not be sent.
 
-                @param interval Timeout interval in milliseconds
+                @param interval Timeout interval as cxxtools::Timespan
             */
-            void start(std::size_t interval);
+            void start(const Milliseconds& interval);
+
+            /** @brief Starts the timer
+
+                Start a timer. The start time is passed as a cxxtools::DateTime.
+                If the start time is in the past, the timer starts on the next
+                cycle. Past timer ticks are not catched up.
+                
+                The Timer needs to be registered with a Selector or event loop,
+                otherwise the timeout signal will not be sent.
+
+                @param interval Timeout interval as a cxxtools::Timespan
+            */
+            void start(const DateTime& startTime, const Milliseconds& interval, bool localtime = true);
+
+            /** @brief Starts the timer
+
+                Start a timer. The timer will tick once after the specified
+                time span.
+
+                The Timer needs to be registered with a Selector or event loop,
+                otherwise the timeout signal will not be sent.
+
+                @param interval Timeout interval as a cxxtools::Timespan
+            */
+            void after(const Milliseconds& interval);
+
+            /** @brief Starts the timer
+
+                Start a timer. The timer will tick once at the specified point
+                in time.
+
+                @param tickTime The time, when the timer should tick.
+            */
+            void at(const DateTime& tickTime, bool localtime = true);
+            void at(const UtcDateTime& tickTime);
 
             /** @brief Stops the timer
 
@@ -123,7 +164,7 @@ namespace cxxtools {
             */
             bool update();
 
-            bool update(const Timespan& now);
+            bool update(const Milliseconds& now);
 
             /** @brief Notifies about interval timeouts
 
@@ -131,16 +172,32 @@ namespace cxxtools {
             */
             Signal<> timeout;
 
-            const Timespan& finished() const
+            /** @brief Notifies about interval timeouts
+
+                This signal is sent if the interval time has expired.
+                As a parameter the due time is passed as local time.
+            */
+            Signal<DateTime> timeoutts;
+
+            /** @brief Notifies about interval timeouts
+
+                This signal is sent if the interval time has expired.
+                As a parameter the due time is passed in utc.
+            */
+            Signal<DateTime> timeoutUtc;
+
+            /** @brief Returns the timespan since epoch when the timer expires
+             */
+            Timespan finished() const
             { return _finished; }
 
         private:
             Sentry* _sentry;
             SelectorBase* _selector;
             bool          _active;
-            std::size_t   _interval;
-            Timespan      _remaining;
+            Timespan      _interval;
             Timespan      _finished;
+            bool          _once;
     };
 
 }
