@@ -26,7 +26,6 @@
 #ifndef cxxtools_xml_EndElement_h
 #define cxxtools_xml_EndElement_h
 
-#include <cxxtools/xml/api.h>
 #include <cxxtools/xml/node.h>
 #include <cxxtools/string.h>
 
@@ -52,7 +51,7 @@ namespace cxxtools {
          * @see StartElement
          * @see Node
          */
-        class CXXTOOLS_XML_API EndElement : public Node
+        class EndElement : public Node
         {
             public:
                 /**
