@@ -28,7 +28,6 @@
 #ifndef cxxtools_xmlrpc_Client_h
 #define cxxtools_xmlrpc_Client_h
 
-#include <cxxtools/xmlrpc/api.h>
 #include <cxxtools/remoteclient.h>
 #include <string>
 
@@ -38,14 +37,14 @@ namespace xmlrpc {
 
 class ClientImpl;
 
-class CXXTOOLS_XMLRPC_API Client : public RemoteClient
+class Client : public RemoteClient
 {
         ClientImpl* _impl;
 
-        Client(Client&) { }
-        void operator= (const Client&) { }
-
     protected:
+        Client(Client& c) : _impl(c._impl) { }
+        Client& operator= (const Client& c) { _impl = c._impl; return *this; }
+
         void impl(ClientImpl* i) { _impl = i; }
 
     public:
@@ -55,15 +54,24 @@ class CXXTOOLS_XMLRPC_API Client : public RemoteClient
 
         virtual ~Client();
 
+        /// This method is used internally to initiate a rpc request.
         void beginCall(IComposer& r, IRemoteProcedure& method, IDecomposer** argv, unsigned argc);
 
+        /// This method is used internally to finalize a rpc request.
         void endCall();
 
+        /// This method is used internally to initiate a rpc request.
         void call(IComposer& r, IRemoteProcedure& method, IDecomposer** argv, unsigned argc);
 
-        std::size_t timeout() const;
+        /// Returns the timeout for syncronous requests.
+        /// When the timeout exprires before the request returns, a cxxtools::IOTimeout is thrown.
+        /// On negative timeout the client will wait infinitely. This is the default.
+        Milliseconds timeout() const;
+        /// Sets the timeout for syncronous requests.
+        void timeout(Milliseconds t);
 
-        void timeout(std::size_t t);
+        Milliseconds connectTimeout() const;
+        void connectTimeout(Milliseconds t);
 
         std::string url() const;
 
