@@ -28,13 +28,12 @@
 #ifndef cxxtools_xmlrpc_Scanner_h
 #define cxxtools_xmlrpc_Scanner_h
 
-#include <cxxtools/xmlrpc/api.h>
 #include <cxxtools/string.h>
 
 namespace cxxtools
 {
 
-class DeserializerBase;
+class Deserializer;
 class IComposer;
 
 namespace xml
@@ -47,7 +46,7 @@ class Node;
 namespace xmlrpc
 {
 
-class CXXTOOLS_XMLRPC_API Scanner
+class Scanner
 {
     enum State
     {
@@ -84,13 +83,13 @@ class CXXTOOLS_XMLRPC_API Scanner
         ~Scanner()
         {}
 
-        void begin(DeserializerBase& handler, IComposer& composer);
+        void begin(Deserializer& handler, IComposer& composer);
 
         bool advance(const xml::Node& node);
 
     private:
         State _state;
-        DeserializerBase* _deserializer;
+        Deserializer* _deserializer;
         IComposer* _composer;
         String _value;
         String _type;
