@@ -27,6 +27,7 @@
  */
 
 #include <cxxtools/base64codec.h>
+#include <cctype>
 
 namespace cxxtools
 {
@@ -66,8 +67,43 @@ inline uint8_t fromBase64(char b64)
     return b64dec[(int)b64];
 }
 
+// returns number of available non space bytes up to N
+unsigned short numBytesN(unsigned short N, const MBState& s, const char* fromBegin, const char* fromEnd)
+{
+    unsigned short count = s.n;
+    while (count < N && fromBegin < fromEnd)
+        if (!std::isspace(*fromBegin++))
+            ++count;
+    return count;
 }
 
+// returns the next non space byte
+char readByte(MBState& s, const char*& fromNext)
+{
+    if (s.n > 0)
+    {
+        char ret = s.value.mbytes[0];
+        std::memmove(s.value.mbytes, s.value.mbytes + 1, s.n - 1);
+        --s.n;
+        return ret;
+    }
+    else
+    {
+        while (std::isspace(*fromNext))
+            ++fromNext;
+        return *fromNext++;
+    }
+}
+
+}
+
+
+Base64Codec::Base64Codec(size_t ref)
+: TextCodec<char, char>(ref),
+  _maxcol(76),
+  _lineend("\r\n"),
+  _padding(true)
+{}
 
 Base64Codec::result Base64Codec::do_in(MBState& s,
                                        const char* fromBegin,
@@ -80,26 +116,33 @@ Base64Codec::result Base64Codec::do_in(MBState& s,
     fromNext = fromBegin;
     toNext = toBegin;
 
-    while( (fromEnd - fromNext) >= 4 && (toEnd - toNext) >= 3 )
+    while ( numBytesN(4, s, fromNext, fromEnd) >= 4 && (toEnd - toNext) >= 3 )
     {
-        uint8_t first  = fromBase64( *(fromNext++) );
-        uint8_t second = fromBase64( *(fromNext++) );
-        uint8_t third  = fromBase64( *(fromNext++) );
-        uint8_t fourth = fromBase64( *(fromNext++) );
+        uint8_t first  = fromBase64(readByte(s, fromNext));
+        uint8_t second = fromBase64(readByte(s, fromNext));
+        uint8_t third  = fromBase64(readByte(s, fromNext));
+        uint8_t fourth = fromBase64(readByte(s, fromNext));
 
         *(toNext++) = (first << 2) + (second >> 4);
 
-        if(third != 64)
+        if (third != 64)
             *(toNext++) = (second << 4) + (third >> 2);
 
-        if(fourth != 64)
+        if (fourth != 64)
             *(toNext++) = (third << 6) + (fourth);
     }
 
-    if( fromEnd == fromNext )
-        return std::codecvt_base::ok;
+    while (fromNext < fromEnd && s.n < 4)
+    {
+        while (fromNext < fromEnd && std::isspace(*fromNext))
+            ++fromNext;
+        if (fromNext < fromEnd)
+            s.value.mbytes[s.n++] = *fromNext++;
+    }
 
-    return std::codecvt_base::partial;
+    return numBytesN(1, s, fromNext, fromEnd) > 0
+                ? std::codecvt_base::partial
+                : std::codecvt_base::ok;
 }
 
 
@@ -114,81 +157,42 @@ Base64Codec::result Base64Codec::do_out(cxxtools::MBState& state,
     fromNext = fromBegin;
     toNext = toBegin;
 
-    const char* first = 0;
-    const char* second = 0;
-    const char* third = 0;
-
-    if(fromEnd - fromNext < 1)
-        return std::codecvt_base::partial;
-
-    if(toEnd - toNext < 4)
-        return std::codecvt_base::partial;
-
-    switch( state.n )
+    unsigned char& col = reinterpret_cast<unsigned char&>(state.value.mbytes[15]);
+    if (state.n == 0)
     {
-        case 2:
-            first  = &state.value.mbytes[0];
-            second = &state.value.mbytes[1];
-            third  = fromNext++;
-            break;
+        state.n = 1;
+        col = 0;
+    }
 
-        case 1:
-            if(fromEnd - fromNext < 2)
-            {
-                state.value.mbytes[1] = *fromNext++;
-                state.n = 2;
+    while (fromEnd - fromNext > 0)
+    {
+        if (state.n == 4)
+        {
+            if (toEnd - toNext < 4)
                 return std::codecvt_base::partial;
+
+            if (_maxcol > 0 && static_cast<unsigned>(col) + 4 > _maxcol)
+            {
+                if (toEnd - toNext < static_cast<int>(4 + _lineend.size()))
+                    return std::codecvt_base::partial;
+                for (unsigned n = 0; n < _lineend.size(); ++n)
+                    *toNext++ = _lineend[n];
+                col = 0;
             }
 
-            first  = &state.value.mbytes[0];
-            second = fromNext++;
-            third  = fromNext++;
-            break;
+            col += 4;
 
-        default:
-            first  = fromNext++;
-            second = fromNext++;
-            third  = fromNext++;
-            break;
-    }
-
-    while (true)
-    {
-        *toNext++   = toBase64( (static_cast<unsigned char>(*first) >> 2) & 0x3f );
-        *(toNext++) = toBase64( ((static_cast<unsigned char>(*first) << 4) + (static_cast<unsigned char>(*second) >> 4)) & 0x3f );
-        *(toNext++) = toBase64( (static_cast<unsigned char>(*second << 2) + (static_cast<unsigned char>(*third) >> 6)) & 0x3f );
-        *(toNext++) = toBase64( static_cast<unsigned char>(*third) & 0x3f );
-
-        if(toEnd - toNext < 4)
-        {
-            state = MBState();
-            return std::codecvt_base::partial;
+            *toNext++   = toBase64( (static_cast<unsigned char>(state.value.mbytes[0]) >> 2) & 0x3f );
+            *(toNext++) = toBase64( ((static_cast<unsigned char>(state.value.mbytes[0]) << 4)
+                                        + (static_cast<unsigned char>(state.value.mbytes[1]) >> 4)) & 0x3f );
+            *(toNext++) = toBase64( (static_cast<unsigned char>(state.value.mbytes[1] << 2)
+                                        + (static_cast<unsigned char>(state.value.mbytes[2]) >> 6)) & 0x3f );
+            *(toNext++) = toBase64( static_cast<unsigned char>(state.value.mbytes[2]) & 0x3f );
+            state.n = 1;
         }
 
-        if( fromEnd - fromNext < 3 )
-            break;
-
-        first =  fromNext++;
-        second = fromNext++;
-        third =  fromNext++;
-    }
-
-    switch( fromEnd - fromNext )
-    {
-        case 2:
-            state.value.mbytes[0] = *fromNext++;
-            state.value.mbytes[1] = *fromNext++;
-            state.n = 2;
-            break;
-
-        case 1:
-            state.value.mbytes[0] = *fromNext++;
-            state.n = 1;
-            break;
-
-        default:
-            state = MBState();
-            break;
+        state.value.mbytes[state.n-1] = *fromNext++;
+        ++state.n;
     }
 
     return std::codecvt_base::ok;
@@ -202,28 +206,58 @@ Base64Codec::result Base64Codec::do_unshift(MBState& state,
 {
     toNext = toBegin;
 
-    if(toEnd - toBegin < 4)
+    unsigned char& col = reinterpret_cast<unsigned char&>(state.value.mbytes[15]);
+    if (state.n == 0)
     {
-        return std::codecvt_base::partial;
+        state.n = 1;
+        col = 0;
     }
 
+    if (_maxcol > 0 && static_cast<unsigned>(col) + 4 > _maxcol)
+    {
+        if (toEnd - toNext < static_cast<int>(4 + _lineend.size()))
+            return std::codecvt_base::partial;
+        for (unsigned n = 0; n < _lineend.size(); ++n)
+            *toNext++ = _lineend[n];
+        col = 0;
+    }
+
+    col += 4;
+
+    if (toEnd - toBegin < 4)
+        return std::codecvt_base::partial;
+
     switch(state.n)
     {
+        case 4:
+            *toNext++   = toBase64( (static_cast<unsigned char>(state.value.mbytes[0]) >> 2) & 0x3f );
+            *(toNext++) = toBase64( ((static_cast<unsigned char>(state.value.mbytes[0]) << 4)
+                                        + (static_cast<unsigned char>(state.value.mbytes[1]) >> 4)) & 0x3f );
+            *(toNext++) = toBase64( (static_cast<unsigned char>(state.value.mbytes[1] << 2)
+                                        + (static_cast<unsigned char>(state.value.mbytes[2]) >> 6)) & 0x3f );
+            *(toNext++) = toBase64( static_cast<unsigned char>(state.value.mbytes[2]) & 0x3f );
+            break;
+
+        case 3:
+            *toNext++   = toBase64( (static_cast<unsigned char>(state.value.mbytes[0]) >> 2) & 0x3f );
+            *(toNext++) = toBase64( ((static_cast<unsigned char>(state.value.mbytes[0]) << 4)
+                                        + (static_cast<unsigned char>(state.value.mbytes[1]) >> 4)) & 0x3f );
+            *(toNext++) = toBase64( (static_cast<unsigned char>(state.value.mbytes[1]) << 2) &  0x3f );
+            if (_padding)
+                *(toNext++) = '=';
+            break;
+
         case 2:
-            *toNext++   = toBase64( (static_cast<unsigned char>(state.value.mbytes[0]) >> 2) & 0x3f );
-            *(toNext++) = toBase64( ((static_cast<unsigned char>(state.value.mbytes[0]) << 4) + (static_cast<unsigned char>(state.value.mbytes[1]) >> 4)) & 0x3f );
-            *(toNext++) = toBase64( (static_cast<unsigned char>(state.value.mbytes[1]) << 2) &  0x3f );
-            *(toNext++) = '=';
-            break;
-
-        case 1:
             *toNext++   = toBase64( (static_cast<unsigned char>(state.value.mbytes[0]) >> 2) & 0x3f );
             *(toNext++) = toBase64( (static_cast<unsigned char>(state.value.mbytes[0]) << 4) & 0x3f );
-            *(toNext++) = '=';
-            *(toNext++) = '=';
+            if (_padding)
+            {
+                *(toNext++) = '=';
+                *(toNext++) = '=';
+            }
             break;
 
-        case 0:
+        default:
             return std::codecvt_base::noconv;
     }
 
