@@ -26,8 +26,8 @@
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-#ifndef CXXTOOLS_BIN_CLIENTIMPL_H
-#define CXXTOOLS_BIN_CLIENTIMPL_H
+#ifndef CXXTOOLS_BIN_RPCCLIENTIMPL_H
+#define CXXTOOLS_BIN_RPCCLIENTIMPL_H
 
 #include <cxxtools/remoteclient.h>
 #include <cxxtools/bin/formatter.h>
@@ -35,7 +35,9 @@
 #include <cxxtools/iostream.h>
 #include <cxxtools/string.h>
 #include <cxxtools/connectable.h>
-#include <cxxtools/deserializerbase.h>
+#include <cxxtools/bin/deserializer.h>
+#include <cxxtools/refcounted.h>
+#include <cxxtools/timespan.h>
 #include <string>
 #include "scanner.h"
 
@@ -47,40 +49,63 @@ class SelectorBase;
 namespace bin
 {
 
-class RpcClient;
-
-class RpcClientImpl : public Connectable
+class RpcClientImpl : public RefCounted, public Connectable
 {
-        RpcClientImpl(RpcClientImpl&) { }
-        void operator= (const RpcClientImpl&) { }
+        RpcClientImpl(RpcClientImpl&);
+        void operator= (const RpcClientImpl&);
 
     public:
-        RpcClientImpl(const std::string& addr, unsigned short port, const std::string& domain);
+        RpcClientImpl();
 
-        RpcClientImpl(SelectorBase& selector, const std::string& addr, unsigned short port, const std::string& domain);
+        net::TcpSocket& socket()    { return _socket; }
 
-        ~RpcClientImpl();
+        void setSelector(SelectorBase* selector)
+        {
+            _socket.setSelector(selector);
+        }
 
-        void setSelector(SelectorBase& selector)
-        { selector.add(_socket); }
+        void prepareConnect(const net::AddrInfo& addrinfo, const std::string& sslCertificate)
+        {
+            _addrInfo = addrinfo;
+            _socket.close();
+            _sslCertificate = sslCertificate;
+            _ssl = !sslCertificate.empty();
+        }
 
-        void connect(const std::string& addr, unsigned short port, const std::string& domain);
+        void ssl(bool sw)
+        {
+            _ssl = sw;
+        }
+
+        void connect();
 
         void close();
 
+        void setSslVerify(int level, const std::string& ca)
+        {
+            _sslVerifyLevel = level;
+            _sslCa = ca;
+        }
+
         void beginCall(IComposer& r, IRemoteProcedure& method, IDecomposer** argv, unsigned argc);
 
         void endCall();
 
         void call(IComposer& r, IRemoteProcedure& method, IDecomposer** argv, unsigned argc);
 
+        Timespan timeout() const  { return _timeout; }
+        void timeout(Timespan t)  { _timeout = t; if (!_connectTimeoutSet) _connectTimeout = t; }
+
+        Timespan connectTimeout() const  { return _connectTimeout; }
+        void connectTimeout(Timespan t)  { _connectTimeout = t; _connectTimeoutSet = true; }
+
         const IRemoteProcedure* activeProcedure() const
         { return _proc; }
 
-        void wait(std::size_t msecs);
-
         void cancel();
 
+        void wait(Timespan msecs);
+
         const std::string& domain() const
         { return _domain; }
 
@@ -90,25 +115,37 @@ class RpcClientImpl : public Connectable
     private:
         void prepareRequest(const String& name, IDecomposer** argv, unsigned argc);
         void onConnect(net::TcpSocket& socket);
+        void onSslConnect(net::TcpSocket& socket);
         void onOutput(StreamBuffer& sb);
         void onInput(StreamBuffer& sb);
 
-        IRemoteProcedure* _proc;
+        // connection state
         net::TcpSocket _socket;
         IOStream _stream;
+
+        net::AddrInfo _addrInfo;
+        bool _ssl;
+        std::string _sslCertificate;
+        int _sslVerifyLevel;
+        std::string _sslCa;
+        std::string _domain;
+
+        // serialization
         Scanner _scanner;
-        DeserializerBase _deserializer;
+        Deserializer _deserializer;
         Formatter _formatter;
 
         bool _exceptionPending;
+        IRemoteProcedure* _proc;
 
-        std::string _addr;
-        unsigned short _port;
-        std::string _domain;
+        Timespan _timeout;
+        bool _connectTimeoutSet;  // indicates if connectTimeout is explicitely set
+                                  // when not, it follows the setting of _timeout
+        Timespan _connectTimeout;
 };
 
 }
 
 }
 
-#endif // CXXTOOLS_BIN_CLIENTIMPL_H
+#endif // CXXTOOLS_BIN_RPCCLIENTIMPL_H
