@@ -27,19 +27,12 @@
  */
 #include "cxxtools/connectable.h"
 #include "cxxtools/connection.h"
+#include "cxxtools/log.h"
+
+log_define("cxxtools.connectable")
 
 namespace cxxtools {
 
-Connectable::Connectable()
-{}
-
-
-Connectable::Connectable(const Connectable& c)
-{ 
-    this->operator=(c); 
-}
-
-
 Connectable::~Connectable()
 {
     this->clear();
@@ -48,20 +41,20 @@ Connectable::~Connectable()
 
 void Connectable::clear()
 {
-    while( !_connections.empty() ) 
+    while( !_connections.empty() )
     {
-        Connection connection = _connections.front();
-        connection.close();
+        Connection* c = &_connections.front();
+        c->close();
+        if (&_connections.front() == c)
+        {
+            // this should not really happen but just in case we do not want to loop endlessly
+            log_fatal("connection " << static_cast<void*>(c) << " was not removed from " << static_cast<void*>(this));
+            _connections.pop_front();
+        }
     }
 }
 
 
-Connectable& Connectable::operator=(const Connectable& other)
-{
-    return (*this);
-}
-
-
 void Connectable::onConnectionOpen(const Connection& c)
 {
     _connections.push_back(c);
