@@ -25,8 +25,8 @@
  * License along with this library; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  */
+
 #include "cxxtools/connectable.h"
-#include <memory>
 
 namespace cxxtools {
 
@@ -38,38 +38,43 @@ Connection::Connection()
 
 Connection::Connection(Connectable& sender, Slot* slot)
 {
-    std::auto_ptr<ConnectionData> data( new ConnectionData(sender, slot) );
-    _data = data.get();
-    _data->setValid(false);
+    _data = new ConnectionData(sender, slot);
 
-    sender.onConnectionOpen(*this);
-    slot->onConnect(*this);
-   _data->setValid(true);
-    data.release();
+    try
+    {
+        _data->setValid(false);
+
+        sender.onConnectionOpen(*this);
+        slot->onConnect(*this);
+       _data->setValid(true);
+    }
+    catch (...)
+    {
+        delete _data;
+        throw;
+    }
 }
 
 
 Connection::Connection(const Connection& connection)
+    : _data(connection._data)
 {
-    _data = connection._data;
-    _data->ref();
+    if (_data)
+        _data->addRef();
 }
 
 
 Connection::~Connection()
 {
-    if( _data->unref() > 0) {
+    if( !_data || _data->release() > 0) {
         return;
     }
 
     // close the connection if its still valid
-    if( this->valid() ) {
-        this->close();
-    }
+    this->close();
 
     // delete the shared data
     delete _data;
-    _data = 0;
 }
 
 
@@ -78,37 +83,37 @@ void Connection::close()
     if( !this->valid() )
         return;
 
-    _data->slot().onDisconnect( *this );
-    // We set the valid flag here to false since the call above may 
+    Connection c(_data);
+    c._data->slot().onDisconnect( *this );
+    // We set the valid flag here to false since the call above may
     // fail for any reason. If setting the valid flag before, a
-    // connection may pretend to be closed but it is not and it 
-    // may reside e.g. in the list of connections of the 
+    // connection may pretend to be closed but it is not and it
+    // may reside e.g. in the list of connections of the
     // Connectable class and then provoke an infinite loop.
-    _data->setValid(false);
-    _data->sender().onConnectionClose( *this );
+    c._data->setValid(false);
+    c._data->sender().onConnectionClose( c );
 }
 
 
 Connection& Connection::operator=(const Connection& connection)
 {
-    if( 0 == _data->unref()) 
+    if (_data == connection._data)
+        return *this;
+
+    if (_data && _data->release() == 0)
     {
-        this->close();        
-        delete _data;   
+        close();
+        delete _data;
     }
-    
+
     _data = connection._data;
-    _data->ref();
+    if (_data)
+        _data->addRef();
+
     return (*this);
 }
 
 
-bool Connection::operator==(const Connection& connection) const
-{
-    // compare pointers or callable?
-    return _data == connection._data;
-}
-
 } //namespace cxxtools
 
 
