@@ -27,10 +27,12 @@
  */
 
 #include <cxxtools/csvparser.h>
-#include <cxxtools/deserializerbase.h>
+#include <cxxtools/csvdeserializer.h>
 #include <cxxtools/serializationerror.h>
 #include <cxxtools/log.h>
+
 #include <stdexcept>
+#include <sstream>
 
 log_define("cxxtools.csv.parser")
 
@@ -50,7 +52,7 @@ namespace
         else if (column + 1 != noColumns)
         {
             std::ostringstream msg;
-            msg << "number of columns " << (column + 1) << " in line " << lineNo << " does not match expected number of columns " << noColumns;
+            msg << "number of columns " << (column + 1) << " in line " << lineNo << " does not match expected number of columns " << noColumns << " in csv";
             SerializationError::doThrow(msg.str());
         }
     }
@@ -58,7 +60,7 @@ namespace
 }
 const Char CsvParser::autoDelimiter = L'\0';
 
-void CsvParser::begin(DeserializerBase& handler)
+void CsvParser::begin(CsvDeserializer& handler)
 {
     if (_delimiter == autoDelimiter && !_readTitle)
         throw std::logic_error("can't read csv data with auto delimiter but without title");
@@ -217,7 +219,7 @@ void CsvParser::advance(Char ch)
             _column = 0;
             log_debug("new row");
             _deserializer->beginMember(std::string(),
-                std::string(), SerializationInfo::Array);
+                std::string(), SerializationInfo::Object);
             _state = state_datastart;
             // no break
 
@@ -260,6 +262,9 @@ void CsvParser::advance(Char ch)
                 break;
             }
 
+            _state = state_data;
+            // fallthrough
+
         case state_data:
             if (ch == L'\n' || ch == L'\r')
             {
@@ -294,10 +299,7 @@ void CsvParser::advance(Char ch)
         case state_qdata:
             if (ch == _quote)
             {
-                log_debug("value \"" << _value << '"');
-                _deserializer->setValue(_value);
-                _value.clear();
-                _deserializer->leaveMember();  // leave data item
+                //it can be a double _quote, used to allow usage of _quote
                 _state = state_qdata_end;
             }
             else
@@ -307,6 +309,20 @@ void CsvParser::advance(Char ch)
             break;
 
         case state_qdata_end:
+            if (ch == _quote)
+            {
+                //it is indeed a double_quote. Add one.
+                _value += ch;
+                _state = state_qdata;
+                break;
+            }
+
+            //it is not a double _quote. End the _value
+            log_debug("value \"" << _value << '"');
+            _deserializer->setValue(_value);
+            _value.clear();
+            _deserializer->leaveMember();  // leave data item
+
             if (ch == L'\n' || ch == L'\r')
             {
                 checkNoColumns(_column, _noColumns, _lineNo);
@@ -331,7 +347,6 @@ void CsvParser::advance(Char ch)
             break;
     }
 
-    //log_debug("ch=" << ch.narrow() << " _state=" << _state);
 }
 
 void CsvParser::finish()
@@ -359,6 +374,10 @@ void CsvParser::finish()
             break;
 
         case state_qdata_end:
+            log_debug("value \"" << _value << '"');
+            _deserializer->setValue(_value);
+            _value.clear();
+            _deserializer->leaveMember();  // leave data item
             _deserializer->leaveMember();  // leave row
             break;
 
