@@ -31,76 +31,33 @@
 
 namespace cxxtools
 {
-    void DeserializerBase::setCategory(SerializationInfo::Category category)
+    void Deserializer::begin()
     {
-        _current->setCategory(category);
+        clear();
+        _current.push(&_si);
     }
 
-    void DeserializerBase::setName(const std::string& name)
+    void Deserializer::clear()
     {
-        _current->setName(name);
+        while (!_current.empty())
+            _current.pop();
+        _si.clear();
     }
 
-    void DeserializerBase::setTypeName(const std::string& type)
+    void Deserializer::beginMember(const std::string& name, const std::string& type, SerializationInfo::Category category)
     {
-        _current->setTypeName(type);
-    }
-
-    void DeserializerBase::setValue(const String& value)
-    {
-        _current->setValue(value);
-    }
-
-    void DeserializerBase::setValue(const std::string& value)
-    {
-        _current->setValue(value);
-    }
-
-    void DeserializerBase::setValue(const char* value)
-    {
-        _current->setValue(value);
-    }
-
-    void DeserializerBase::setValue(bool value)
-    {
-        _current->setValue(value);
-    }
-
-    void DeserializerBase::setValue(int_type value)
-    {
-        _current->setValue(value);
-    }
-
-    void DeserializerBase::setValue(unsigned_type value)
-    {
-        _current->setValue(value);
-    }
-
-    void DeserializerBase::setValue(long double value)
-    {
-        _current->setValue(value);
-    }
-
-    void DeserializerBase::setNull()
-    {
-        _current->setNull();
-    }
-
-    void DeserializerBase::beginMember(const std::string& name, const std::string& type, SerializationInfo::Category category)
-    {
-        SerializationInfo& child = _current->addMember(name);
+        SerializationInfo& child = current()->addMember(name);
         child.setTypeName(type);
         child.setCategory(category);
-        _current = &child;
+        _current.push(&child);
     }
 
-    void DeserializerBase::leaveMember()
+    void Deserializer::leaveMember()
     {
-        SerializationInfo* p = _current->parent();
-        if( !p )
+        if (_current.size() <= 1)
             SerializationError::doThrow("invalid member");
 
-        _current = p;
+        _current.pop();
     }
 
 } // namespace cxxtools
