@@ -34,6 +34,7 @@
 #include <cxxtools/textstream.h>
 #include <cxxtools/base64codec.h>
 #include <sstream>
+#include <algorithm>
 #include "config.h"
 
 #include <cxxtools/log.h>
@@ -50,123 +51,50 @@ void ClientImpl::ParseEvent::onHttpReturn(unsigned ret, const std::string& text)
 }
 
 ClientImpl::ClientImpl(Client* client)
-: _client(client)
-, _parseEvent(_replyHeader)
-, _parser(_parseEvent, true)
-, _request(0)
-, _stream(8192, true)
-, _chunkedIStream(_stream.rdbuf())
-, _contentLength(0)
-, _readHeader(true)
-, _chunkedEncoding(false)
-, _reconnectOnError(false)
-, _errorPending(false)
+: _client(client),
+  _request(0),
+  _parseEvent(_reply.header()),
+  _parser(_parseEvent, true),
+#ifdef WITH_SSL
+  _ssl(false),
+#endif
+  _stream(8192, true),
+  _chunkedIStream(_stream.rdbuf()),
+  _bodyStream(_stream.rdbuf()),
+  _readHeader(true),
+  _chunkedEncoding(false),
+  _reconnectOnError(false),
+  _errorPending(false)
 {
     _stream.attachDevice(_socket);
     cxxtools::connect(_socket.connected, *this, &ClientImpl::onConnect);
+#ifdef WITH_SSL
+    cxxtools::connect(_socket.sslConnected, *this, &ClientImpl::onSslConnect);
+#endif
     cxxtools::connect(_stream.buffer().outputReady, *this, &ClientImpl::onOutput);
     cxxtools::connect(_stream.buffer().inputReady, *this, &ClientImpl::onInput);
 }
 
 
-ClientImpl::ClientImpl(Client* client, const net::AddrInfo& addrinfo)
-: _client(client)
-, _parseEvent(_replyHeader)
-, _parser(_parseEvent, true)
-, _request(0)
-, _addrInfo(addrinfo)
-, _stream(8192, true)
-, _chunkedIStream(_stream.rdbuf())
-, _contentLength(0)
-, _readHeader(true)
-, _chunkedEncoding(false)
-, _reconnectOnError(false)
-, _errorPending(false)
+void ClientImpl::prepareConnect(const net::AddrInfo& addrinfo, const std::string& sslCertificate)
 {
-    _stream.attachDevice(_socket);
-    cxxtools::connect(_socket.connected, *this, &ClientImpl::onConnect);
-    cxxtools::connect(_stream.buffer().outputReady, *this, &ClientImpl::onOutput);
-    cxxtools::connect(_stream.buffer().inputReady, *this, &ClientImpl::onInput);
+#ifdef WITH_SSL
+    if (addrinfo != _addrInfo || sslCertificate != _sslCertificate || _ssl != _sslCertificate.empty())
+    {
+        _addrInfo = addrinfo;
+        _sslCertificate = sslCertificate;
+        _ssl = !sslCertificate.empty();
+        _socket.close();
+    }
+#else
+    if (addrinfo != _addrInfo)
+    {
+        _addrInfo = addrinfo;
+        _socket.close();
+    }
+#endif
 }
 
-ClientImpl::ClientImpl(Client* client, const net::Uri& uri)
-: _client(client)
-, _parseEvent(_replyHeader)
-, _parser(_parseEvent, true)
-, _request(0)
-, _addrInfo(uri.host(), uri.port())
-, _stream(8192, true)
-, _chunkedIStream(_stream.rdbuf())
-, _username(uri.user())
-, _password(uri.password())
-, _contentLength(0)
-, _readHeader(true)
-, _chunkedEncoding(false)
-, _reconnectOnError(false)
-, _errorPending(false)
-{
-    if (uri.protocol() != "http")
-        throw std::runtime_error("only http is supported by http client");
-
-    _stream.attachDevice(_socket);
-    cxxtools::connect(_socket.connected, *this, &ClientImpl::onConnect);
-    cxxtools::connect(_stream.buffer().outputReady, *this, &ClientImpl::onOutput);
-    cxxtools::connect(_stream.buffer().inputReady, *this, &ClientImpl::onInput);
-}
-
-ClientImpl::ClientImpl(Client* client, SelectorBase& selector, const net::AddrInfo& addrinfo)
-: _client(client)
-, _parseEvent(_replyHeader)
-, _parser(_parseEvent, true)
-, _request(0)
-, _addrInfo(addrinfo)
-, _stream(8192, true)
-, _chunkedIStream(_stream.rdbuf())
-, _contentLength(0)
-, _readHeader(true)
-, _chunkedEncoding(false)
-, _reconnectOnError(false)
-, _errorPending(false)
-{
-    _stream.attachDevice(_socket);
-    cxxtools::connect(_socket.connected, *this, &ClientImpl::onConnect);
-    cxxtools::connect(_stream.buffer().outputReady, *this, &ClientImpl::onOutput);
-    cxxtools::connect(_stream.buffer().inputReady, *this, &ClientImpl::onInput);
-    setSelector(selector);
-}
-
-
-ClientImpl::ClientImpl(Client* client, SelectorBase& selector, const net::Uri& uri)
-: _client(client)
-, _parseEvent(_replyHeader)
-, _parser(_parseEvent, true)
-, _request(0)
-, _addrInfo(uri.host(), uri.port())
-, _stream(8192, true)
-, _chunkedIStream(_stream.rdbuf())
-, _contentLength(0)
-, _readHeader(true)
-, _chunkedEncoding(false)
-, _reconnectOnError(false)
-, _errorPending(false)
-{
-    if (uri.protocol() != "http")
-        throw std::runtime_error("only http is supported by http client");
-
-    _stream.attachDevice(_socket);
-    cxxtools::connect(_socket.connected, *this, &ClientImpl::onConnect);
-    cxxtools::connect(_stream.buffer().outputReady, *this, &ClientImpl::onOutput);
-    cxxtools::connect(_stream.buffer().inputReady, *this, &ClientImpl::onInput);
-    setSelector(selector);
-}
-
-
-void ClientImpl::setSelector(SelectorBase& selector)
-{
-    selector.add(_socket);
-}
-
-
 void ClientImpl::reexecute(const Request& request)
 {
     log_debug("reexecute");
@@ -175,12 +103,21 @@ void ClientImpl::reexecute(const Request& request)
     _stream.buffer().discard();
 
     _socket.connect(_addrInfo);
+#ifdef WITH_SSL
+    if (_ssl)
+    {
+        if (!_sslCertificate.empty())
+            _socket.loadSslCertificateFile(_sslCertificate);
+        _socket.setSslVerify(_sslVerifyLevel, _sslCa);
+        _socket.sslConnect();
+    }
+#endif
 
     sendRequest(request);
     _stream.flush();
 }
 
-void ClientImpl::reexecuteBegin(const Request& request)
+void ClientImpl::reexecuteBegin(const Request& /*request*/)
 {
     log_debug("reexecuteBegin");
 
@@ -199,21 +136,48 @@ void ClientImpl::doparse()
 
 }
 
-const ReplyHeader& ClientImpl::execute(const Request& request, std::size_t timeout)
+const ReplyHeader& ClientImpl::execute(const Request& request, Timespan timeout, Timespan connectTimeout)
 {
     log_trace("execute request " << request.url());
 
-    _replyHeader.clear();
+    if (_chunkedEncoding)
+    {
+        while (_chunkedIStream)
+            _chunkedIStream.get();
+    }
+    else
+    {
+        while (_bodyStream)
+            _bodyStream.get();
+    }
 
-    _socket.setTimeout(timeout);
+    if (connectTimeout < Timespan(0))
+        connectTimeout = timeout;
+
+    _reply.clear();
+
+    _socket.setTimeout(connectTimeout);
 
     bool shouldReconnect = _socket.isConnected();
     if (!shouldReconnect)
     {
         log_debug("connect");
         _socket.connect(_addrInfo);
+
+#ifdef WITH_SSL
+        if (_ssl)
+        {
+            log_debug("ssl connect");
+            if (!_sslCertificate.empty())
+                _socket.loadSslCertificateFile(_sslCertificate);
+            _socket.setSslVerify(_sslVerifyLevel, _sslCa);
+            _socket.sslConnect();
+        }
+#endif
     }
 
+    _socket.setTimeout(timeout);
+
     log_debug("send request");
     sendRequest(request);
     _stream.flush();
@@ -256,49 +220,53 @@ const ReplyHeader& ClientImpl::execute(const Request& request, std::size_t timeo
     if (!_parser.end())
         throw IOError("incomplete HTTP reply header");
 
-    return _replyHeader;
+    _chunkedEncoding = _reply.header().chunkedTransferEncoding();
+
+    if (_chunkedEncoding)
+    {
+        _chunkedIStream.reset();
+    }
+    else
+    {
+        std::size_t n = _reply.header().contentLength();
+        _bodyStream.clear();
+        _bodyStream.icount(n);
+
+        log_debug("content length " << n);
+
+    }
+
+    return _reply.header();
 }
 
 
-void ClientImpl::readBody(std::string& s)
+void ClientImpl::readBody()
 {
-    s.clear();
-
-    _chunkedEncoding = _replyHeader.chunkedTransferEncoding();
-    _chunkedIStream.reset();
-
     if (_chunkedEncoding)
     {
         log_debug("read body with chunked encoding");
 
-        char ch;
-        while (_chunkedIStream.get(ch))
-            s += ch;
-
-        log_debug("eod=" << _chunkedIStream.eod());
+        _reply.bodyStream() << _chunkedIStream.rdbuf();
 
         if (!_chunkedIStream.eod())
+        {
+            _chunkedIStream.setstate(std::ios::failbit);
             throw IOError("error reading HTTP reply body: incomplete chunked data stream");
+        }
     }
-    else
+    else if (_bodyStream.icount() > 0)
     {
-        unsigned n = _replyHeader.contentLength();
+        _reply.bodyStream() << _bodyStream.rdbuf();
 
-        log_debug("read body; content-size: " << n);
-
-        s.reserve(n);
-
-        char ch;
-        while (n-- && _stream.get(ch))
-            s += ch;
-
-        if (_stream.fail())
+        if (_bodyStream.icount() > 0 || !_reply.bodyStream())
+        {
+            _stream.setstate(std::ios::failbit);
             throw IOError("error reading HTTP reply body");
+        }
 
-        //log_debug("body read: \"" << s << '"');
     }
 
-    if (!_replyHeader.keepAlive())
+    if (!_reply.header().keepAlive())
     {
         log_debug("close socket - no keep alive");
         _socket.close();
@@ -310,14 +278,6 @@ void ClientImpl::readBody(std::string& s)
 }
 
 
-std::string ClientImpl::get(const std::string& url, std::size_t timeout)
-{
-    Request request(url);
-    execute(request, timeout);
-    return readBody();
-}
-
-
 void ClientImpl::beginExecute(const Request& request)
 {
     if (_socket.selector() == 0)
@@ -327,7 +287,7 @@ void ClientImpl::beginExecute(const Request& request)
 
     _errorPending = false;
     _request = &request;
-    _replyHeader.clear();
+    _reply.clear();
     if (_socket.isConnected())
     {
         log_debug("we are connected already");
@@ -390,7 +350,12 @@ void ClientImpl::sendRequest(const Request& request)
     static const char* userAgent = "User-Agent";
 
     _stream << request.method() << ' '
-            << request.url() << " HTTP/"
+            << request.url();
+
+    if (!request.qparams().empty() && request.method() == "GET")
+        _stream << '?' << request.qparams();
+
+    _stream << " HTTP/"
             << request.header().httpVersionMajor() << '.'
             << request.header().httpVersionMinor() << "\r\n";
 
@@ -400,7 +365,7 @@ void ClientImpl::sendRequest(const Request& request)
         _stream << it->first << ": " << it->second << "\r\n";
     }
 
-   if (!request.header().hasHeader(contentLength))
+    if (!request.header().hasHeader(contentLength))
     {
         _stream << "Content-Length: " << request.bodySize() << "\r\n";
     }
@@ -457,6 +422,17 @@ void ClientImpl::onConnect(net::TcpSocket& socket)
 
         _errorPending = false;
         socket.endConnect();
+#ifdef WITH_SSL
+        if (_ssl)
+        {
+            if (!_sslCertificate.empty())
+                socket.loadSslCertificateFile(_sslCertificate);
+            socket.setSslVerify(_sslVerifyLevel, _sslCa);
+            socket.beginSslConnect();
+            return;
+        }
+#endif
+
         sendRequest(*_request);
 
         log_debug("request sent - begin write");
@@ -472,6 +448,31 @@ void ClientImpl::onConnect(net::TcpSocket& socket)
     }
 }
 
+#ifdef WITH_SSL
+void ClientImpl::onSslConnect(net::TcpSocket& socket)
+{
+    try
+    {
+        log_trace("onSslConnect");
+
+        _errorPending = false;
+        socket.endSslConnect();
+        sendRequest(*_request);
+
+        log_debug("request sent - begin write");
+        _stream.buffer().beginWrite();
+    }
+    catch (const std::exception& )
+    {
+        _errorPending = true;
+        _client->replyFinished(*_client);
+
+        if (_errorPending)
+            throw;
+    }
+}
+#endif
+
 void ClientImpl::onOutput(StreamBuffer& sb)
 {
     log_trace("ClientImpl::onOutput; out_avail=" << sb.out_avail());
@@ -585,7 +586,7 @@ void ClientImpl::processHeaderAvailable(StreamBuffer& sb)
 
     if( _parser.end() )
     {
-        _chunkedEncoding = _replyHeader.chunkedTransferEncoding();
+        _chunkedEncoding = _reply.header().chunkedTransferEncoding();
 
         _client->headerReceived(*_client);
         _readHeader = false;
@@ -607,10 +608,12 @@ void ClientImpl::processHeaderAvailable(StreamBuffer& sb)
         }
         else
         {
-            _contentLength = _replyHeader.contentLength();
-            log_debug("header received - content-length=" << _contentLength);
+            _bodyStream.clear();
+            _bodyStream.icount(_reply.header().contentLength());
 
-            if (_contentLength > 0)
+            log_debug("header received - content-length=" << _bodyStream.icount());
+
+            if (_bodyStream.icount() > 0)
             {
                 if( sb.in_avail() > 0 )
                 {
@@ -623,7 +626,7 @@ void ClientImpl::processHeaderAvailable(StreamBuffer& sb)
             }
             else
             {
-                if (!_replyHeader.keepAlive())
+                if (!_reply.header().keepAlive())
                 {
                     log_debug("close socket - no keep alive");
                     _socket.close();
@@ -678,7 +681,7 @@ void ClientImpl::processBodyAvailable(StreamBuffer& sb)
                 {
                     log_debug("reply finished");
 
-                    if (!_replyHeader.keepAlive())
+                    if (!_reply.header().keepAlive())
                     {
                         log_debug("close socket - no keep alive");
                         _socket.close();
@@ -693,7 +696,7 @@ void ClientImpl::processBodyAvailable(StreamBuffer& sb)
         }
         else if( _chunkedIStream.eod() )
         {
-            if( _replyHeader.hasHeader("Trailer") )
+            if( _reply.header().hasHeader("Trailer") )
                 _parser.readHeader();
             else
                 _client->replyFinished(*_client);
@@ -714,22 +717,22 @@ void ClientImpl::processBodyAvailable(StreamBuffer& sb)
     }
     else
     {
-        log_debug("content-length(pre)=" << _contentLength);
+        log_debug("content-length(pre)=" << _bodyStream.icount());
 
-        while (_stream.good() && _contentLength > 0 && sb.in_avail() > 0)
+        while (_stream.good() && _bodyStream.good() && _bodyStream.rdbuf()->in_avail() > 0)
         {
-            _contentLength -= _client->bodyAvailable(*_client); // TODO: may throw exception
-            log_debug("content-length(post)=" << _contentLength);
+            _client->bodyAvailable(*_client); // TODO: may throw exception
+            log_debug("content-length(post)=" << _bodyStream.icount());
         }
 
-        if (_stream.fail())
+        if (_stream.fail() || _bodyStream.fail())
             throw IOError("error reading HTTP reply body");
 
-        if( _contentLength <= 0 )
+        if( _bodyStream.icount() <= 0 )
         {
             log_debug("reply finished");
 
-            if (!_replyHeader.keepAlive())
+            if (!_reply.header().keepAlive())
             {
                 log_debug("close socket - no keep alive");
                 _socket.close();
