@@ -138,7 +138,7 @@ void MessageHeader::removeHeader(const char* key)
         {
             unsigned slen = it->second - it->first + std::strlen(it->second) + 1;
 
-            std::memcpy(
+            std::memmove(
                 const_cast<char*>(it->first),
                 it->first + slen,
                 p - it->first + slen);
@@ -200,7 +200,7 @@ char* MessageHeader::htdateCurrent(char* buffer)
     static const char* wday[] = {"Sun", "Mon", "Tue", "Wed", "Thu", "Fri", "Sat"};
     static const char* monthn[] = {"Jan", "Feb", "Mar", "Apr", "May", "Jun",
                                    "Jul", "Aug", "Sep", "Oct", "Nov", "Dec"};
-    sprintf(buffer, "%s, %02d %s %d %02d:%02d:%02d GMT",
+    sprintf(buffer, "%s, %02u %s %d %02u:%02u:%02u GMT",
                     wday[dayOfWeek], day, monthn[month-1], year, hour, min, sec);
 
     return buffer;
