@@ -55,34 +55,46 @@ void Socket::ParseEvent::onUrlParam(const std::string& q)
     _request.qparams(q);
 }
 
-Socket::Socket(ServerImpl& server, net::TcpServer& tcpServer)
+Socket::Socket(ServerImpl& server, net::TcpServer& tcpServer, const std::string& certificateFile, const std::string& privateKeyFile, int sslVerifyLevel, const std::string& sslCa)
     : inputSlot(slot(*this, &Socket::onInput)),
       _tcpServer(tcpServer),
+      _certificateFile(certificateFile),
+      _privateKeyFile(privateKeyFile),
       _server(server),
       _parseEvent(_request),
       _parser(_parseEvent, false),
       _responder(0),
+      _sslVerifyLevel(sslVerifyLevel),
+      _sslCa(sslCa),
       _accepted(false)
 {
     _stream.attachDevice(*this);
     cxxtools::connect(IODevice::inputReady, *this, &Socket::onIODeviceInput);
     cxxtools::connect(_stream.buffer().outputReady, *this, &Socket::onOutput);
     cxxtools::connect(_timer.timeout, *this, &Socket::onTimeout);
+    cxxtools::connect(acceptSslCertificate, *this, &Socket::onAcceptSslCertificate);
 }
 
 Socket::Socket(Socket& socket)
-    : inputSlot(slot(*this, &Socket::onInput)),
+    : net::TcpSocket(),
+      Connectable(*this),
+      inputSlot(slot(*this, &Socket::onInput)),
       _tcpServer(socket._tcpServer),
+      _certificateFile(socket._certificateFile),
+      _privateKeyFile(socket._privateKeyFile),
       _server(socket._server),
       _parseEvent(_request),
       _parser(_parseEvent, false),
       _responder(0),
+      _sslVerifyLevel(socket._sslVerifyLevel),
+      _sslCa(socket._sslCa),
       _accepted(false)
 {
     _stream.attachDevice(*this);
     cxxtools::connect(IODevice::inputReady, *this, &Socket::onIODeviceInput);
     cxxtools::connect(_stream.buffer().outputReady, *this, &Socket::onOutput);
     cxxtools::connect(_timer.timeout, *this, &Socket::onTimeout);
+    cxxtools::connect(acceptSslCertificate, *this, &Socket::onAcceptSslCertificate);
 }
 
 Socket::~Socket()
@@ -95,10 +107,31 @@ void Socket::accept()
 {
     net::TcpSocket::accept(_tcpServer, net::TcpSocket::DEFER_ACCEPT);
 
+    if (!_certificateFile.empty())
+    {
+        loadSslCertificateFile(_certificateFile, _privateKeyFile);
+        setSslVerify(_sslVerifyLevel, _sslCa);
+        beginSslAccept();
+    }
+}
+
+void Socket::postAccept()
+{
+    log_trace("post accept");
+    if (!_certificateFile.empty())
+    {
+        cxxtools::Timespan t = getTimeout();
+        setTimeout(cxxtools::Seconds(10));
+        endSslAccept();
+        setTimeout(t);
+    }
+
     _accepted = true;
 
     _stream.buffer().beginRead();
 
+    log_debug("accepted");
+
     _timer.start(_server.readTimeout());
 }
 
@@ -114,7 +147,7 @@ void Socket::removeSelector()
     _timer.setSelector(0);
 }
 
-void Socket::onIODeviceInput(IODevice& iodevice)
+void Socket::onIODeviceInput(IODevice& /*iodevice*/)
 {
     log_debug("onIODeviceInput");
     inputReady(*this);
@@ -140,7 +173,7 @@ void Socket::onInput(StreamBuffer& sb)
         if (_parser.fail())
         {
             _responder = _server.getDefaultResponder(_request);
-            _responder->replyError(_reply.body(), _request, _reply,
+            _responder->replyError(_reply.bodyStream(), _request, _reply,
                 std::runtime_error("invalid http header"));
             _responder->release();
             _responder = 0;
@@ -158,12 +191,12 @@ void Socket::onInput(StreamBuffer& sb)
             _responder = _server.getResponder(_request);
             try
             {
-                _responder->beginRequest(_stream, _request);
+                _responder->beginRequest(*this, _stream, _request);
             }
             catch (const std::exception& e)
             {
                 _reply.setHeader("Connection", "close");
-                _responder->replyError(_reply.body(), _request, _reply, e);
+                _responder->replyError(_reply.bodyStream(), _request, _reply, e);
                 _responder->release();
                 _responder = 0;
                 sendReply();
@@ -201,7 +234,7 @@ void Socket::onInput(StreamBuffer& sb)
             catch (const std::exception& e)
             {
                 _reply.setHeader("Connection", "close");
-                _responder->replyError(_reply.body(), _request, _reply, e);
+                _responder->replyError(_reply.bodyStream(), _request, _reply, e);
                 _responder->release();
                 _responder = 0;
                 sendReply();
@@ -228,13 +261,13 @@ bool Socket::doReply()
     log_trace("http::Socket::doReply");
     try
     {
-        _responder->reply(_reply.body(), _request, _reply);
+        _responder->reply(_reply.bodyStream(), _request, _reply);
     }
     catch (const std::exception& e)
     {
         log_warn("responder reported error: " << e.what());
         _reply.clear();
-        _responder->replyError(_reply.body(), _request, _reply, e);
+        _responder->replyError(_reply.bodyStream(), _request, _reply, e);
     }
 
     _responder->release();
@@ -354,6 +387,11 @@ void Socket::sendReply()
 
 }
 
+bool Socket::onAcceptSslCertificate(const SslCertificate& cert)
+{
+    return !_server.acceptSslCertificate.isConnected() || _server.acceptSslCertificate(cert);
+}
+
 } // namespace http
 
 } // namespace cxxtools
