@@ -31,8 +31,10 @@
 
 #include "selectableimpl.h"
 #include <cxxtools/iodevice.h>
+#include <cxxtools/timespan.h>
 #include <string>
 #include <iostream>
+#include <exception>
 
 struct pollfd;
 
@@ -80,10 +82,10 @@ namespace cxxtools {
             int fd() const
             { return _fd; }
 
-            void setTimeout(std::size_t msecs)
-            { _timeout = msecs; }
+            void setTimeout(Timespan timeout)
+            { _timeout = timeout; }
 
-            std::size_t timeout() const
+            Timespan timeout() const
             { return _timeout; }
 
             virtual void open(const std::string& path, IODevice::OpenMode mode, bool inherit);
@@ -114,9 +116,9 @@ namespace cxxtools {
 
             void detach(SelectorBase& s);
 
-            virtual bool wait(std::size_t msecs);
+            virtual bool wait(Timespan timeout);
 
-            virtual bool wait(std::size_t msecs, pollfd& pfd);
+            virtual bool wait(Timespan timeout, pollfd& pfd);
 
             virtual void initWait(pollfd& pfd);
 
@@ -129,13 +131,29 @@ namespace cxxtools {
 
             virtual bool checkPollEvent(pollfd& pfd);
 
+            virtual void inputReady();
+
+            virtual void outputReady();
+
         protected:
             IODevice& _device;
             int _fd;
-            std::size_t _timeout;
+            Timespan _timeout;
             pollfd* _pfd;
             DestructionSentry* _sentry;
             bool _errorPending;
+            std::exception_ptr _exception;
+
+            void checkPendingException()
+            {
+                if (_exception)
+                {
+                    std::exception_ptr exception;
+                    _exception.swap(exception);
+                    _errorPending = false;
+                    std::rethrow_exception(exception);
+                }
+            }
     };
 
 } //namespace cxxtools
