@@ -28,19 +28,17 @@
 
 #include "httpclientimpl.h"
 #include "cxxtools/remoteprocedure.h"
-#include "cxxtools/textstream.h"
 #include "cxxtools/jsonformatter.h"
 #include "cxxtools/http/replyheader.h"
 #include "cxxtools/selectable.h"
-#include "cxxtools/utf8codec.h"
 #include "cxxtools/ioerror.h"
 #include "cxxtools/clock.h"
 #include "cxxtools/log.h"
 
 #include <stdexcept>
-#include <string.h>
+#include <strings.h>
 
-log_define("cxxtools.json.client.impl")
+log_define("cxxtools.json.httpclient.impl")
 
 namespace cxxtools
 {
@@ -50,6 +48,8 @@ namespace json
 
 HttpClientImpl::HttpClientImpl()
 : _timeout(Selectable::WaitInfinite),
+  _connectTimeoutSet(false),
+  _connectTimeout(Selectable::WaitInfinite),
   _proc(0),
   _exceptionPending(false),
   _count(0)
@@ -72,7 +72,21 @@ void HttpClientImpl::beginCall(IComposer& r, IRemoteProcedure& method, IDecompos
 
     prepareRequest(method.name(), argv, argc);
 
-    _client.beginExecute(_request);
+    try
+    {
+        _client.beginExecute(_request);
+    }
+    catch (const std::exception& )
+    {
+        IRemoteProcedure* proc = _proc;
+        cancel();
+
+        _exceptionPending = true;
+        proc->onFinished();
+
+        if (_exceptionPending)
+            throw;
+    }
 
     _scanner.begin(_deserializer, r);
 }
@@ -97,7 +111,7 @@ void HttpClientImpl::call(IComposer& r, IRemoteProcedure& method, IDecomposer**
 
     prepareRequest(method.name(), argv, argc);
 
-    _client.execute(_request);
+    _client.execute(_request, timeout(), connectTimeout());
 
     _scanner.begin(_deserializer, r);
 
@@ -105,7 +119,7 @@ void HttpClientImpl::call(IComposer& r, IRemoteProcedure& method, IDecomposer**
     std::istream& is = _client.in();
     while (is.get(ch))
     {
-        if (_scanner.advance(ch))
+        if (_deserializer.advance(ch) != 0)
         {
             log_debug("scanner finished");
             _proc = 0;
@@ -137,10 +151,9 @@ void HttpClientImpl::prepareRequest(const String& name, IDecomposer** argv, unsi
     _request.setHeader("Content-Type", "application/json");
     _request.method("POST");
 
-    TextOStream ts(_request.body(), new Utf8Codec());
     JsonFormatter formatter;
 
-    formatter.begin(ts);
+    formatter.begin(_request.body());
 
     formatter.beginObject(std::string(), std::string());
 
@@ -160,8 +173,6 @@ void HttpClientImpl::prepareRequest(const String& name, IDecomposer** argv, unsi
     formatter.finishObject();
 
     formatter.finish();
-
-    ts.flush();
 }
 
 void HttpClientImpl::onReplyHeader(http::Client& client)
@@ -177,7 +188,7 @@ std::size_t HttpClientImpl::onReplyBody(http::Client& client)
     while (is.rdbuf()->in_avail() && is.get(ch))
     {
         ++count;
-        if (_scanner.advance(ch))
+        if (_deserializer.advance(ch))
         {
             log_debug("scanner finished");
             try
@@ -204,7 +215,7 @@ std::size_t HttpClientImpl::onReplyBody(http::Client& client)
     return count;
 }
 
-void HttpClientImpl::onReplyFinished(http::Client& client)
+void HttpClientImpl::onReplyFinished(http::Client& /*client*/)
 {
     log_debug("onReplyFinished; method=" << static_cast<void*>(_proc));
 
@@ -236,26 +247,27 @@ void HttpClientImpl::onReplyFinished(http::Client& client)
     proc->onFinished();
 }
 
-void HttpClientImpl::wait(std::size_t msecs)
+void HttpClientImpl::wait(Timespan timeout)
 {
     if (!_client.selector())
         throw std::logic_error("cannot run async rpc request without a selector");
 
     Clock clock;
-    if (msecs != RemoteClient::WaitInfinite)
+    if (timeout >= Timespan(0))
         clock.start();
 
-    std::size_t remaining = msecs;
+    Timespan remaining = timeout;
 
     while (activeProcedure() != 0)
     {
         if (_client.selector()->wait(remaining) == false)
             throw IOTimeout();
 
-        if (msecs != RemoteClient::WaitInfinite)
+        if (timeout >= Timespan(0))
         {
-            std::size_t diff = static_cast<std::size_t>(clock.stop().totalMSecs());
-            remaining = diff >= msecs ? 0 : msecs - diff;
+            remaining = timeout - clock.stop();
+            if (remaining < Timespan(0))
+                remaining = Timespan(0);
         }
     }
 }
