@@ -29,7 +29,6 @@
 #include "rpcclientimpl.h"
 #include <cxxtools/log.h>
 #include <cxxtools/remoteprocedure.h>
-#include <cxxtools/utf8codec.h>
 #include <cxxtools/jsonformatter.h>
 #include <cxxtools/ioerror.h>
 #include <cxxtools/clock.h>
@@ -44,21 +43,32 @@ namespace json
 
 RpcClientImpl::RpcClientImpl()
     : _stream(_socket, 8192, true),
+      _ssl(false),
+      _sslVerifyLevel(0),
       _exceptionPending(false),
-      _proc(0)
+      _proc(0),
+      _count(0),
+      _timeout(Selectable::WaitInfinite),
+      _connectTimeoutSet(false),
+      _connectTimeout(Selectable::WaitInfinite)
 {
     cxxtools::connect(_socket.connected, *this, &RpcClientImpl::onConnect);
+    cxxtools::connect(_socket.sslConnected, *this, &RpcClientImpl::onSslConnect);
     cxxtools::connect(_stream.buffer().outputReady, *this, &RpcClientImpl::onOutput);
     cxxtools::connect(_stream.buffer().inputReady, *this, &RpcClientImpl::onInput);
 }
 
-void RpcClientImpl::connect(const std::string& addr, unsigned short port)
+void RpcClientImpl::connect()
 {
-    if (_addr != addr || _port != port)
+    _socket.setTimeout(_connectTimeout);
+    _socket.close();
+    _socket.connect(_addrInfo);
+    if (_ssl)
     {
-        _socket.close();
-        _addr = addr;
-        _port = port;
+        if (!_sslCertificate.empty())
+            _socket.loadSslCertificateFile(_sslCertificate);
+        _socket.setSslVerify(_sslVerifyLevel, _sslCa);
+        _socket.sslConnect();
     }
 }
 
@@ -79,22 +89,36 @@ void RpcClientImpl::beginCall(IComposer& r, IRemoteProcedure& method, IDecompose
 
     prepareRequest(method.name(), argv, argc);
 
-    if (_socket.isConnected())
+    try
     {
-        try
+        if (_socket.isConnected())
         {
-            _stream.buffer().beginWrite();
+            try
+            {
+                _stream.buffer().beginWrite();
+            }
+            catch (const IOError&)
+            {
+                log_debug("write failed, connection is not active any more");
+                _socket.beginConnect(_addrInfo);
+            }
         }
-        catch (const IOError&)
+        else
         {
-            log_debug("write failed, connection is not active any more");
-            _socket.beginConnect(_addr, _port);
+            log_debug("not yet connected - do it now");
+            _socket.beginConnect(_addrInfo);
         }
     }
-    else
+    catch (const std::exception& )
     {
-        log_debug("not yet connected - do it now");
-        _socket.beginConnect(_addr, _port);
+        IRemoteProcedure* proc = _proc;
+        cancel();
+
+        _exceptionPending = true;
+        proc->onFinished();
+
+        if (_exceptionPending)
+            throw;
     }
 
     _scanner.begin(_deserializer, r);
@@ -113,23 +137,67 @@ void RpcClientImpl::endCall()
 
 void RpcClientImpl::call(IComposer& r, IRemoteProcedure& method, IDecomposer** argv, unsigned argc)
 {
-    _proc = &method;
-
-    prepareRequest(_proc->name(), argv, argc);
-
-    if (!_socket.isConnected())
-        _socket.connect(_addr, _port);
-
     try
     {
-        _stream.flush();
+        _proc = &method;
+        StreamBuffer& sb = _stream.buffer();
+
+        if (_socket.isConnected())
+        {
+            log_debug("socket is connected");
+
+            try
+            {
+                prepareRequest(_proc->name(), argv, argc);
+                _socket.setTimeout(timeout());
+                sb.pubsync();
+
+                // try to read from socket to check if still connected
+                // sgetc fills the input buffer but do not consume the character
+                int ch = sb.sgetc();
+                if (ch == StreamBuffer::traits_type::eof())
+                {
+                    log_debug("reading failed");
+                    _socket.close();
+                }
+            }
+            catch (const std::exception& e)
+            {
+                log_debug("request failed: " << e.what());
+                _socket.close();
+            }
+        }
+
+        if (!_socket.isConnected())
+        {
+            log_debug("socket is not connected");
+            _socket.setTimeout(_connectTimeout);
+            _socket.connect(_addrInfo);
+            if (_ssl)
+            {
+                if (!_sslCertificate.empty())
+                    _socket.loadSslCertificateFile(_sslCertificate);
+                _socket.setSslVerify(_sslVerifyLevel, _sslCa);
+                _socket.sslConnect();
+            }
+
+            prepareRequest(_proc->name(), argv, argc);
+            _socket.setTimeout(timeout());
+            sb.pubsync();
+        }
 
         _scanner.begin(_deserializer, r);
 
-        char ch;
-        while (_stream.get(ch))
+        while (true)
         {
-            if (_scanner.advance(ch))
+            int ch = sb.sbumpc();
+            if (ch == StreamBuffer::traits_type::eof())
+            {
+                cancel();
+                throw std::runtime_error("reading result failed");
+            }
+
+            if (_deserializer.advance(ch))
             {
                 _proc = 0;
                 _scanner.finalizeReply();
@@ -139,6 +207,7 @@ void RpcClientImpl::call(IComposer& r, IRemoteProcedure& method, IDecomposer** a
     }
     catch (const RemoteException&)
     {
+        _proc = 0;
         throw;
     }
     catch (const std::exception& e)
@@ -146,14 +215,6 @@ void RpcClientImpl::call(IComposer& r, IRemoteProcedure& method, IDecomposer** a
         cancel();
         throw;
     }
-
-    _proc = 0;
-
-    if (!_stream)
-    {
-        cancel();
-        throw std::runtime_error("reading result failed");
-    }
 }
 
 void RpcClientImpl::cancel()
@@ -164,36 +225,36 @@ void RpcClientImpl::cancel()
     _proc = 0;
 }
 
-void RpcClientImpl::wait(std::size_t msecs)
+void RpcClientImpl::wait(Timespan timeout)
 {
     if (_socket.selector() == 0)
         throw std::logic_error("cannot run async rpc request without a selector");
 
     Clock clock;
-    if (msecs != RemoteClient::WaitInfinite)
+    if (timeout >= Timespan(0))
         clock.start();
 
-    std::size_t remaining = msecs;
+    Timespan remaining = timeout;
 
     while (activeProcedure() != 0)
     {
         if (_socket.selector()->wait(remaining) == false)
             throw IOTimeout();
 
-        if (msecs != RemoteClient::WaitInfinite)
+        if (timeout >= Timespan(0))
         {
-            std::size_t diff = static_cast<std::size_t>(clock.stop().totalMSecs());
-            remaining = diff >= msecs ? 0 : msecs - diff;
+            remaining = timeout - clock.stop();
+            if (remaining < Timespan(0))
+                remaining = Timespan(0);
         }
     }
 }
 
 void RpcClientImpl::prepareRequest(const String& name, IDecomposer** argv, unsigned argc)
 {
-    TextOStream ts(_stream, new Utf8Codec());
     JsonFormatter formatter;
 
-    formatter.begin(ts);
+    formatter.begin(_stream);
 
     formatter.beginObject(std::string(), std::string());
 
@@ -213,8 +274,6 @@ void RpcClientImpl::prepareRequest(const String& name, IDecomposer** argv, unsig
     formatter.finishObject();
 
     formatter.finish();
-
-    ts.flush();
 }
 
 void RpcClientImpl::onConnect(net::TcpSocket& socket)
@@ -223,9 +282,45 @@ void RpcClientImpl::onConnect(net::TcpSocket& socket)
     {
         log_trace("onConnect");
 
-        _exceptionPending = false;
         socket.endConnect();
 
+        _exceptionPending = false;
+        if (_ssl)
+        {
+            if (!_sslCertificate.empty())
+                _socket.loadSslCertificateFile(_sslCertificate);
+            _socket.setSslVerify(_sslVerifyLevel, _sslCa);
+            socket.beginSslConnect();
+            return;
+        }
+
+        _stream.buffer().beginWrite();
+    }
+    catch (const std::exception& )
+    {
+        IRemoteProcedure* proc = _proc;
+        cancel();
+
+        if (!proc)
+            throw;
+
+        _exceptionPending = true;
+        proc->onFinished();
+
+        if (_exceptionPending)
+            throw;
+    }
+}
+
+void RpcClientImpl::onSslConnect(net::TcpSocket& socket)
+{
+    try
+    {
+        log_trace("onSslConnect");
+
+        _exceptionPending = false;
+        socket.endSslConnect();
+
         _stream.buffer().beginWrite();
     }
     catch (const std::exception& )
@@ -281,10 +376,10 @@ void RpcClientImpl::onInput(StreamBuffer& sb)
         if (sb.device()->eof())
             throw IOError("end of input");
 
-        char ch;
-        while (_stream.buffer().in_avail() && _stream.get(ch))
+        while (_stream.buffer().in_avail())
         {
-            if (_scanner.advance(ch))
+            char ch = StreamBuffer::traits_type::to_char_type(_stream.buffer().sbumpc());
+            if (_deserializer.advance(ch))
             {
                 _scanner.finalizeReply();
                 IRemoteProcedure* proc = _proc;
