@@ -34,8 +34,10 @@
 #include <cxxtools/iostream.h>
 #include <cxxtools/string.h>
 #include <cxxtools/deserializer.h>
+#include <cxxtools/jsondeserializer.h>
 #include <cxxtools/connectable.h>
 #include <cxxtools/selector.h>
+#include <cxxtools/refcounted.h>
 #include <string>
 #include "scanner.h"
 
@@ -49,7 +51,7 @@ class IDecomposer;
 namespace json
 {
 
-class RpcClientImpl : public Connectable
+class RpcClientImpl : public RefCounted, public Connectable
 {
         RpcClientImpl(RpcClientImpl&);
         void operator= (const RpcClientImpl&);
@@ -57,25 +59,54 @@ class RpcClientImpl : public Connectable
     public:
         RpcClientImpl();
 
-        void setSelector(SelectorBase& selector)
-        { selector.add(_socket); }
+        net::TcpSocket& socket()    { return _socket; }
 
-        void connect(const std::string& addr, unsigned short port);
+        void setSelector(SelectorBase* selector)
+        {
+            _socket.setSelector(selector);
+        }
+
+        void prepareConnect(const net::AddrInfo& addrinfo, const std::string& sslCertificate)
+        {
+            _addrInfo = addrinfo;
+            _socket.close();
+            _sslCertificate = sslCertificate;
+            _ssl = !sslCertificate.empty();
+        }
+
+        void ssl(bool sw)
+        {
+            _ssl = sw;
+        }
+
+        void connect();
 
         void close();
 
+        void setSslVerify(int level, const std::string& ca)
+        {
+            _sslVerifyLevel = level;
+            _sslCa = ca;
+        }
+
         void beginCall(IComposer& r, IRemoteProcedure& method, IDecomposer** argv, unsigned argc);
 
         void endCall();
 
         void call(IComposer& r, IRemoteProcedure& method, IDecomposer** argv, unsigned argc);
 
+        Timespan timeout() const  { return _timeout; }
+        void timeout(Timespan t)  { _timeout = t; if (!_connectTimeoutSet) _connectTimeout = t; }
+
+        Timespan connectTimeout() const  { return _connectTimeout; }
+        void connectTimeout(Timespan t)  { _connectTimeout = t; _connectTimeoutSet = true; }
+
         const IRemoteProcedure* activeProcedure() const
         { return _proc; }
 
         void cancel();
 
-        void wait(std::size_t msecs);
+        void wait(Timespan msecs);
 
         const std::string& prefix() const
         { return _prefix; }
@@ -86,6 +117,7 @@ class RpcClientImpl : public Connectable
     private:
         void prepareRequest(const String& name, IDecomposer** argv, unsigned argc);
         void onConnect(net::TcpSocket& socket);
+        void onSslConnect(net::TcpSocket& socket);
         void onOutput(StreamBuffer& sb);
         void onInput(StreamBuffer& sb);
 
@@ -93,19 +125,26 @@ class RpcClientImpl : public Connectable
         net::TcpSocket _socket;
         IOStream _stream;
 
-        std::string _addr;
-        unsigned short _port;
+        net::AddrInfo _addrInfo;
+        bool _ssl;
+        std::string _sslCertificate;
+        int _sslVerifyLevel;
+        std::string _sslCa;
         std::string _prefix;
 
         // serialization
         Scanner _scanner;
-        DeserializerBase _deserializer;
+        JsonDeserializer _deserializer;
 
         // current processing
         bool _exceptionPending;
         IRemoteProcedure* _proc;
         Formatter::int_type _count;
 
+        Timespan _timeout;
+        bool _connectTimeoutSet;  // indicates if connectTimeout is explicitely set
+                                  // when not, it follows the setting of _timeout
+        Timespan _connectTimeout;
 };
 
 }
