@@ -29,17 +29,17 @@
 #ifndef CXXTOOLS_JSON_RPCSERVERIMPL_H
 #define CXXTOOLS_JSON_RPCSERVERIMPL_H
 
-#include <map>
-#include <set>
-#include <vector>
-#include <cxxtools/noncopyable.h>
+#include <cxxtools/json/rpcserver.h>
 #include <cxxtools/event.h>
 #include <cxxtools/mutex.h>
 #include <cxxtools/condition.h>
 #include <cxxtools/queue.h>
 #include <cxxtools/signal.h>
+#include <cxxtools/delegate.h>
 #include <cxxtools/connectable.h>
-#include <cxxtools/json/rpcserver.h>
+
+#include <set>
+#include <vector>
 
 namespace cxxtools
 {
@@ -62,14 +62,22 @@ namespace cxxtools
         class ThreadTerminatedEvent;
         class ActiveSocketEvent;
 
-        class RpcServerImpl : private NonCopyable, public Connectable
+        class RpcServerImpl : public Connectable
         {
+#if __cplusplus >= 201103L
+                RpcServerImpl(const RpcServerImpl&) = delete;
+                RpcServerImpl& operator=(const RpcServerImpl&) = delete;
+#else
+                RpcServerImpl(const RpcServerImpl&);
+                RpcServerImpl& operator=(const RpcServerImpl&);
+#endif
+
             public:
                 RpcServerImpl(EventLoopBase& eventLoop, Signal<RpcServer::Runmode>& runmodeChanged, ServiceRegistry& serviceRegistry);
 
                 ~RpcServerImpl();
 
-                void listen(const std::string& ip, unsigned short int port, int backlog);
+                void listen(const std::string& ip, unsigned short int port, const std::string& certificateFile, const std::string& privateKeyFile, int sslVerifyLevel, const std::string& sslCa);
 
                 unsigned minThreads() const
                 { return _minThreads; }
@@ -88,6 +96,8 @@ namespace cxxtools
                 RpcServer::Runmode runmode() const
                 { return _runmode; }
 
+                Delegate<bool, const SslCertificate&> acceptSslCertificate;
+
             private:
                 void runmode(RpcServer::Runmode runmode)
                 {
@@ -112,6 +122,7 @@ namespace cxxtools
                 void start();
 
                 friend class Worker;
+                friend class Socket;
 
                 ////////////////////////////////////////////////////
 
@@ -136,7 +147,6 @@ namespace cxxtools
 
                 bool isTerminating() const
                 { return runmode() == RpcServer::Terminating; }
-
         };
     }
 }
