@@ -28,27 +28,25 @@
  */
 
 #include <cxxtools/serializationinfo.h>
+#include <cxxtools/serializationerror.h>
+#include <cxxtools/convert.h>
+#include <cxxtools/log.h>
+
 #include <stdexcept>
 #include <sstream>
 
+log_define("cxxtools.serializationinfo")
+
 namespace cxxtools
 {
 
-SerializationInfo::SerializationInfo()
-: _parent(0)
-, _category(Void)
-, _t(t_none)
-{ }
-
-
 SerializationInfo::SerializationInfo(const SerializationInfo& si)
-: _parent(si._parent)
-, _category(si._category)
-, _name(si._name)
-, _type(si._type)
-, _u(si._u)
-, _t(si._t)
-, _nodes(si._nodes)
+: _category(si._category),
+  _name(si._name),
+  _type(si._type),
+  _u(si._u),
+  _t(si._t),
+  _nodes(0)
 {
     switch (_t)
     {
@@ -61,122 +59,181 @@ SerializationInfo::SerializationInfo(const SerializationInfo& si)
         default:
             ;
     }
+
+    if (si._nodes)
+        _nodes = new Nodes(*si._nodes);
 }
 
 
 SerializationInfo& SerializationInfo::operator=(const SerializationInfo& si)
 {
-    _parent = si._parent;
-    _category = si._category;
+    if (this == &si)
+        return *this;
+
+    assignData(si);
     _name = si._name;
-    _type = si._type;
+
+    return *this;
+}
+
+
+#if __cplusplus >= 201103L
+
+SerializationInfo::SerializationInfo(SerializationInfo&& si) noexcept
+    : _category(si._category),
+      _name(std::move(si._name)),
+      _type(std::move(si._type)),
+      _u(si._u),
+      _t(si._t),
+      _nodes(si._nodes)
+{
+    if (si._t == t_string)
+    {
+        new (_StringPtr()) String(std::move(*si._StringPtr()));
+    }
+    else if (si._t == t_string8)
+    {
+        new (_String8Ptr()) std::string(std::move(*si._String8Ptr()));
+    }
+
+    si._nodes = 0;
+}
+
+
+SerializationInfo& SerializationInfo::operator=(SerializationInfo&& si)
+{
+    _category = si._category;
+    _name = std::move(si._name);
+    _type = std::move(si._type);
     _nodes = si._nodes;
+    si._nodes = 0;
 
     if (si._t == t_string)
-        _setString( si._String() );
+    {
+        if (_t != t_string)
+        {
+            _releaseValue();
+            new (_StringPtr()) String();
+            _t = t_string;
+        }
+
+        _String().swap(si._String());
+    }
     else if (si._t == t_string8)
-        _setString8( si._String8() );
+    {
+        if (_t != t_string8)
+        {
+            _releaseValue();
+            new (_String8Ptr()) std::string();
+            _t = t_string8;
+        }
+
+        _String8().swap(si._String8());
+    }
     else
     {
         _releaseValue();
-        _u = si._u;
         _t = si._t;
+        _u = si._u;
     }
 
     return *this;
 }
 
 
-void SerializationInfo::reserve(size_t n)
-{
-    _nodes.reserve(n);
-}
+#endif
 
+SerializationInfo::~SerializationInfo()
+{
+    _releaseValue();
+    delete _nodes;
+}
 
 SerializationInfo& SerializationInfo::addMember(const std::string& name)
 {
-    _nodes.resize(_nodes.size() + 1);
-    _nodes.back().setParent(*this);
-    _nodes.back().setName(name);
+    log_debug("addMember(\"" << name << "\")");
 
-    // category Array overrides Object (is this a hack?)
+    Nodes& n = nodes();
+    n.push_back(SerializationInfo());
+    n.back().setName(name);
+
+    // category Array overrides Object
     // This is needed for xmldeserialization. In the xml file the root node of a array
     // has a category attribute. When the serializationinfo of the array is created
     // it is known, that it is of category Array. When the members of the array are read,
     // they should not make an object out of the array.
-    if (_category != Array)
-        _category = Object;
+    if (_category != Array && _category != Object)
+        _category = name.empty() ? Array : Object;
 
-    return _nodes.back();
-}
-
-
-SerializationInfo::Iterator SerializationInfo::begin()
-{
-    if(_nodes.size() == 0)
-        return 0;
-
-    return &( _nodes[0] );
-}
-
-
-SerializationInfo::Iterator SerializationInfo::end()
-{
-    if(_nodes.size() == 0)
-        return 0;
-
-    return &_nodes[0] + _nodes.size();
-}
-
-
-SerializationInfo::ConstIterator SerializationInfo::begin() const
-{
-    if(_nodes.size() == 0)
-        return 0;
-
-    return &( _nodes[0] );
-}
-
-
-SerializationInfo::ConstIterator SerializationInfo::end() const
-{
-    if(_nodes.size() == 0)
-        return 0;
-
-    return &_nodes[0] + _nodes.size();
+    return n.back();
 }
 
 
 const SerializationInfo& SerializationInfo::getMember(const std::string& name) const
 {
-    Nodes::const_iterator it = _nodes.begin();
-    for(; it != _nodes.end(); ++it)
+    log_debug("getMember(\"" << name << "\")");
+
+    const Nodes& n = nodes();
+
+    for (Nodes::const_iterator it = n.begin(); it != n.end(); ++it)
     {
         if( it->name() == name )
             return *it;
     }
 
-    throw SerializationMemberNotFound(name);
+    throw SerializationMemberNotFound(*this, name);
+}
+
+
+SerializationInfo& SerializationInfo::getMember(const std::string& name)
+{
+    log_debug("getMember(\"" << name << "\")");
+
+    Nodes& n = nodes();
+
+    for (Nodes::iterator it = n.begin(); it != n.end(); ++it)
+    {
+        if( it->name() == name )
+            return *it;
+    }
+
+    throw SerializationMemberNotFound(*this, name);
 }
 
 
 const SerializationInfo& SerializationInfo::getMember(unsigned idx) const
 {
-    if (idx >= _nodes.size())
-    {
-        std::ostringstream msg;
-        msg << "requested member index " << idx << " exceeds number of members " << _nodes.size();
-        throw std::range_error(msg.str());
-    }
+    log_debug("getMember(" << idx << ')');
 
-    return _nodes[idx];
+    const Nodes& n = nodes();
+
+    if (idx >= n.size())
+        throw SerializationMemberNotFound(*this, idx);
+
+    return n[idx];
+}
+
+
+SerializationInfo& SerializationInfo::getMember(unsigned idx)
+{
+    log_debug("getMember(" << idx << ')');
+
+    Nodes& n = nodes();
+
+    if (idx >= n.size())
+        throw SerializationMemberNotFound(*this, idx);
+
+    return n[idx];
 }
 
 
 const SerializationInfo* SerializationInfo::findMember(const std::string& name) const
 {
-    Nodes::const_iterator it = _nodes.begin();
-    for(; it != _nodes.end(); ++it)
+    log_debug("findMember(\"" << name << "\")");
+
+    const Nodes& n = nodes();
+
+    for (Nodes::const_iterator it = n.begin(); it != n.end(); ++it)
     {
         if( it->name() == name )
             return &(*it);
@@ -188,8 +245,11 @@ const SerializationInfo* SerializationInfo::findMember(const std::string& name)
 
 SerializationInfo* SerializationInfo::findMember(const std::string& name)
 {
-    Nodes::iterator it = _nodes.begin();
-    for(; it != _nodes.end(); ++it)
+    log_debug("findMember(\"" << name << "\")");
+
+    Nodes& n = nodes();
+
+    for (Nodes::iterator it = n.begin(); it != n.end(); ++it)
     {
         if( it->name() == name )
             return &(*it);
@@ -203,13 +263,8 @@ void SerializationInfo::clear()
     _category = Void;
     _name.clear();
     _type.clear();
-    _nodes.clear();
-    switch (_t)
-    {
-        case t_string: _String().clear(); break;
-        case t_string8: _String8().clear(); break;
-        default: _t = t_none; break;
-    }
+    nodes().clear();
+    _releaseValue();
 }
 
 void SerializationInfo::swap(SerializationInfo& si)
@@ -217,7 +272,6 @@ void SerializationInfo::swap(SerializationInfo& si)
     if (this == &si)
         return;
 
-    std::swap(_parent, si._parent);
     std::swap(_category, si._category);
     std::swap(_name, si._name);
     std::swap(_type, si._type);
@@ -307,26 +361,26 @@ void SerializationInfo::swap(SerializationInfo& si)
         }
     }
 
-    _nodes.swap(si._nodes);
+    std::swap(_nodes, si._nodes);
 }
 
-void SerializationInfo::dump(std::ostream& out, const std::string& praefix) const
+void SerializationInfo::dump(std::ostream& out, const std::string& prefix) const
 {
     if (!_name.empty())
-        out << praefix << "name = \"" << _name << "\"\n";
+        out << prefix << "name = \"" << _name << "\"\n";
 
     if (_t != t_none)
     {
-        out << praefix << "type = " << (_t == t_none ? "none" :
-                                        _t == t_string ? "string" :
-                                        _t == t_string8 ? "string8" :
-                                        _t == t_char ? "char" :
-                                        _t == t_bool ? "bool" :
-                                        _t == t_int ? "int" :
-                                        _t == t_uint ? "uint" :
-                                        _t == t_float ? "float" : "?") << '\n';
+        out << prefix << "type = " << (_t == t_none ? "none" :
+                                       _t == t_string ? "string" :
+                                       _t == t_string8 ? "string8" :
+                                       _t == t_char ? "char" :
+                                       _t == t_bool ? "bool" :
+                                       _t == t_int ? "int" :
+                                       _t == t_uint ? "uint" :
+                                       _t == t_ldouble ? "double" : "?") << '\n';
 
-        out << praefix << "value = ";
+        out << prefix << "value = ";
 
         switch (_t)
         {
@@ -338,20 +392,25 @@ void SerializationInfo::dump(std::ostream& out, const std::string& praefix) cons
             case t_int:     out << _u._i; break;
             case t_uint:    out << _u._u; break;
             case t_float:   out << _u._f; break;
+            case t_double:  out << _u._d; break;
+            case t_ldouble: out << _u._ld; break;
         }
 
         out << '\n';
     }
 
     if (!_type.empty())
-        out << praefix << "typeName = " << _type << '\n';
-    if (!_nodes.empty())
+        out << prefix << "typeName = " << _type << '\n';
+    out << prefix << "category = " << static_cast<unsigned>(_category) << '\n';
+
+    const Nodes& n = nodes();
+    if (!n.empty())
     {
-        std::string p = praefix + '\t';
-        for (std::vector<SerializationInfo>::size_type n = 0; n < _nodes.size(); ++n)
+        std::string p = prefix + '\t';
+        for (Nodes::size_type i = 0; i < n.size(); ++i)
         {
-            out << praefix << "node[" << n << "]\n";
-            _nodes[n].dump(out, p);
+            out << prefix << "node[" << i << "]\n";
+            n[i].dump(out, p);
         }
     }
 }
@@ -380,7 +439,7 @@ void SerializationInfo::setNull()
 {
     _releaseValue();
     _t = t_none;
-    _category = Void;
+    _category = Value;
 }
 
 void SerializationInfo::_setString(const String& value)
@@ -483,7 +542,7 @@ void SerializationInfo::_setUInt(unsigned_type value)
     _category = Value;
 }
 
-void SerializationInfo::_setFloat(long double value)
+void SerializationInfo::_setFloat(float value)
 {
     if (_t != t_float)
     {
@@ -496,6 +555,31 @@ void SerializationInfo::_setFloat(long double value)
     _category = Value;
 }
 
+void SerializationInfo::_setDouble(double value)
+{
+    if (_t != t_double)
+    {
+        _releaseValue();
+        _t = t_double;
+    }
+
+    _u._d = value;
+
+    _category = Value;
+}
+
+void SerializationInfo::_setLongDouble(long double value)
+{
+    if (_t != t_ldouble)
+    {
+        _releaseValue();
+        _t = t_ldouble;
+    }
+
+    _u._ld = value;
+
+    _category = Value;
+}
 
 void SerializationInfo::getValue(String& value) const
 {
@@ -509,6 +593,8 @@ void SerializationInfo::getValue(String& value) const
         case t_int:     convert(value, _u._i); break;
         case t_uint:    convert(value, _u._u); break;
         case t_float:   convert(value, _u._f); break;
+        case t_double:  convert(value, _u._d); break;
+        case t_ldouble: convert(value, _u._ld); break;
     }
 }
 
@@ -524,6 +610,8 @@ void SerializationInfo::getValue(std::string& value) const
         case t_int:     convert(value, _u._i); break;
         case t_uint:    convert(value, _u._u); break;
         case t_float:   convert(value, _u._f); break;
+        case t_double:  convert(value, _u._d); break;
+        case t_ldouble: convert(value, _u._ld); break;
     }
 }
 
@@ -552,6 +640,8 @@ bool SerializationInfo::_getBool() const
         case t_int:     return _u._i;
         case t_uint:    return _u._u;
         case t_float:   return _u._f;
+        case t_double:  return _u._d;
+        case t_ldouble: return _u._ld;
     }
 
     // never reached
@@ -570,6 +660,8 @@ wchar_t SerializationInfo::_getWChar() const
         case t_int:     return _u._i;
         case t_uint:    return _u._u;
         case t_float:   return static_cast<wchar_t>(_u._f);
+        case t_double:  return static_cast<wchar_t>(_u._d);
+        case t_ldouble: return static_cast<wchar_t>(_u._ld);
     }
 
     // never reached
@@ -588,6 +680,8 @@ char SerializationInfo::_getChar() const
         case t_int:     return _u._i; break;
         case t_uint:    return _u._u; break;
         case t_float:   return static_cast<char>(_u._f); break;
+        case t_double:   return static_cast<char>(_u._d); break;
+        case t_ldouble:   return static_cast<char>(_u._ld); break;
     }
     // never reached
     return 0;
@@ -607,7 +701,7 @@ SerializationInfo::int_type SerializationInfo::_getInt(const char* type, int_typ
                         }
                         catch (const ConversionError&)
                         {
-                            ConversionError::doThrow(type, "String", _String().narrow().c_str());
+                            SerializationConversionError::doThrow(*this, type, "String", _String().narrow());
                         }
                         break;
 
@@ -617,7 +711,7 @@ SerializationInfo::int_type SerializationInfo::_getInt(const char* type, int_typ
                         }
                         catch (const ConversionError&)
                         {
-                            ConversionError::doThrow(type, "string", _String8().c_str());
+                            SerializationConversionError::doThrow(*this, type, "string", _String8());
                         }
                         break;
 
@@ -632,6 +726,8 @@ SerializationInfo::int_type SerializationInfo::_getInt(const char* type, int_typ
                         }
                         ret = _u._u; break;
         case t_float:   ret = static_cast<int_type>(_u._f); break;
+        case t_double:   ret = static_cast<int_type>(_u._d); break;
+        case t_ldouble:   ret = static_cast<int_type>(_u._ld); break;
     }
 
     if (ret < min || ret > max)
@@ -658,7 +754,7 @@ SerializationInfo::unsigned_type SerializationInfo::_getUInt(const char* type, u
                         }
                         catch (const ConversionError&)
                         {
-                            ConversionError::doThrow(type, "String", _String().narrow().c_str());
+                            SerializationConversionError::doThrow(*this, type, "String", _String().narrow());
                         }
                         break;
 
@@ -668,7 +764,7 @@ SerializationInfo::unsigned_type SerializationInfo::_getUInt(const char* type, u
                         }
                         catch (const ConversionError&)
                         {
-                            ConversionError::doThrow(type, "string", _String8().c_str());
+                            SerializationConversionError::doThrow(*this, type, "string", _String8());
                         }
                         break;
 
@@ -680,6 +776,8 @@ SerializationInfo::unsigned_type SerializationInfo::_getUInt(const char* type, u
                         break;
         case t_uint:    ret = _u._u; break;
         case t_float:   ret = static_cast<unsigned_type>(_u._f); break;
+        case t_double:  ret = static_cast<unsigned_type>(_u._d); break;
+        case t_ldouble: ret = static_cast<unsigned_type>(_u._ld); break;
     }
 
     if (ret > max)
@@ -692,7 +790,138 @@ SerializationInfo::unsigned_type SerializationInfo::_getUInt(const char* type, u
     return ret;
 }
 
-long double SerializationInfo::_getFloat(const char* type, long double max) const
+float SerializationInfo::_getFloat() const
+{
+    float ret = 0;
+
+    switch (_t)
+    {
+        case t_none:    break;
+
+        case t_string:  try
+                        {
+                            ret = convert<float>(_String());
+                        }
+                        catch (const ConversionError&)
+                        {
+                            SerializationConversionError::doThrow(*this, "float", "String", _String().narrow());
+                        }
+                        break;
+
+        case t_string8: try
+                        {
+                            ret = convert<float>(_String8());
+                        }
+                        catch (const ConversionError&)
+                        {
+                            SerializationConversionError::doThrow(*this, "float", "string", _String8());
+                        }
+                        break;
+
+        case t_char:    ret = _u._c - '0'; break;
+        case t_bool:    ret = _u._b; break;
+        case t_int:     ret = _u._i; break;
+        case t_uint:    ret = _u._u; break;
+        case t_float:   ret = _u._f; break;
+
+        case t_double:
+            if (_u._d == std::numeric_limits<double>::infinity())
+                ret = std::numeric_limits<float>::infinity();
+            else if (_u._d == -std::numeric_limits<double>::infinity())
+                ret = -std::numeric_limits<float>::infinity();
+            else if (_u._d != _u._d)  // NaN
+                ret = _u._d;
+            else if (_u._d > std::numeric_limits<float>::max()
+                  || _u._d < -std::numeric_limits<float>::max())
+            {
+                std::ostringstream msg;
+                msg << "value " << _u._d << " does not fit into float";
+                throw std::range_error(msg.str());
+            }
+            else
+                ret = static_cast<float>(_u._d);
+            break;
+
+        case t_ldouble:
+            if (_u._ld == std::numeric_limits<long double>::infinity())
+                ret = std::numeric_limits<float>::infinity();
+            else if (_u._ld == -std::numeric_limits<long double>::infinity())
+                ret = -std::numeric_limits<float>::infinity();
+            else if (_u._ld != _u._ld)  // NaN
+                ret = _u._ld;
+            else if (_u._ld > std::numeric_limits<float>::max()
+                  || _u._ld < -std::numeric_limits<float>::max())
+            {
+                std::ostringstream msg;
+                msg << "value " << _u._ld << " does not fit into float";
+                throw std::range_error(msg.str());
+            }
+            else
+                ret = static_cast<float>(_u._ld);
+            break;
+    }
+
+    return ret;
+}
+
+double SerializationInfo::_getDouble() const
+{
+    double ret = 0;
+
+    switch (_t)
+    {
+        case t_none:    break;
+
+        case t_string:  try
+                        {
+                            ret = convert<double>(_String());
+                        }
+                        catch (const ConversionError&)
+                        {
+                            SerializationConversionError::doThrow(*this, "double", "String", _String().narrow());
+                        }
+                        break;
+
+        case t_string8: try
+                        {
+                            ret = convert<double>(_String8());
+                        }
+                        catch (const ConversionError&)
+                        {
+                            SerializationConversionError::doThrow(*this, "double", "string", _String8());
+                        }
+                        break;
+
+        case t_char:    ret = _u._c - '0'; break;
+        case t_bool:    ret = _u._b; break;
+        case t_int:     ret = _u._i; break;
+        case t_uint:    ret = _u._u; break;
+        case t_float:   ret = _u._f; break;
+        case t_double:  ret = _u._d; break;
+
+        case t_ldouble:
+            if (_u._ld == std::numeric_limits<long double>::infinity())
+                ret = std::numeric_limits<double>::infinity();
+            else if (_u._ld == -std::numeric_limits<long double>::infinity())
+                ret = -std::numeric_limits<double>::infinity();
+            else if (_u._ld != _u._ld)  // NaN
+                ret = _u._ld;
+            else if (_u._ld > std::numeric_limits<double>::max()
+                  || _u._ld < -std::numeric_limits<double>::max())
+            {
+                std::ostringstream msg;
+                msg << "value " << _u._ld << " does not fit into double";
+                throw std::range_error(msg.str());
+            }
+            else
+                ret = static_cast<double>(_u._ld);
+            break;
+    }
+
+    return ret;
+}
+
+long double SerializationInfo::_getLongDouble() const
 {
     long double ret = 0;
 
@@ -706,7 +935,7 @@ long double SerializationInfo::_getFloat(const char* type, long double max) cons
                         }
                         catch (const ConversionError&)
                         {
-                            ConversionError::doThrow(type, "String", _String().narrow().c_str());
+                            SerializationConversionError::doThrow(*this, "long double", "String", _String().narrow());
                         }
                         break;
 
@@ -716,7 +945,7 @@ long double SerializationInfo::_getFloat(const char* type, long double max) cons
                         }
                         catch (const ConversionError&)
                         {
-                            ConversionError::doThrow(type, "string", _String8().c_str());
+                            SerializationConversionError::doThrow(*this, "long double", "string", _String8());
                         }
                         break;
 
@@ -725,20 +954,59 @@ long double SerializationInfo::_getFloat(const char* type, long double max) cons
         case t_int:     ret = _u._i; break;
         case t_uint:    ret = _u._u; break;
         case t_float:   ret = _u._f; break;
-    }
-
-    if (ret != std::numeric_limits<long double>::infinity()
-        && ret != -std::numeric_limits<long double>::infinity()
-        && ret == ret        // check for NaN
-        && (ret < -max || ret > max))
-    {
-        std::ostringstream msg;
-        msg << "value " << ret << " does not fit into " << type;
-        throw std::range_error(msg.str());
+        case t_double:  ret = _u._d; break;
+        case t_ldouble: ret = _u._ld; break;
     }
 
     return ret;
 }
 
+SerializationInfo::Nodes& SerializationInfo::nodes()
+{
+    if (!_nodes)
+        _nodes = new Nodes;
+
+    return *_nodes;
+}
+
+const SerializationInfo::Nodes& SerializationInfo::nodes() const
+{
+    static const Nodes emptyNodes;
+
+    if (_nodes)
+        return *_nodes;
+    else
+        return emptyNodes;
+}
+
+void SerializationInfo::assignData(const SerializationInfo& si)
+{
+    _category = si._category;
+    _type = si._type;
+
+    delete _nodes;
+    _nodes = 0;
+    if (si._nodes)
+        _nodes = new Nodes(*si._nodes);
+
+    if (si._t == t_string)
+        _setString( si._String() );
+    else if (si._t == t_string8)
+        _setString8( si._String8() );
+    else
+    {
+        _releaseValue();
+        _u = si._u;
+        _t = si._t;
+    }
+
+}
+
+void operator <<=(SerializationInfo& si, const SerializationInfo& ssi)
+{
+    si.assignData(ssi);
+}
+
+
 
 } // namespace cxxtools
