@@ -33,6 +33,7 @@
 #include "cxxtools/serializationinfo.h"
 #include <iostream>
 #include <cctype>
+#include <stack>
 
 namespace cxxtools {
 
@@ -106,73 +107,73 @@ class SettingsReader
                 {}
 
             private:
-                virtual State* onSpace(cxxtools::Char c, SettingsReader& reader)
+                virtual State* onSpace(cxxtools::Char /*c*/, SettingsReader& reader)
                 {
                     this->syntaxError(reader.line());
                     return this;
                 }
 
-                virtual State* onQuote(cxxtools::Char c, SettingsReader& reader)
+                virtual State* onQuote(cxxtools::Char /*c*/, SettingsReader& reader)
                 {
                     this->syntaxError(reader.line());
                     return this;
                 }
 
-                virtual State* onComma(cxxtools::Char c, SettingsReader& reader)
+                virtual State* onComma(cxxtools::Char /*c*/, SettingsReader& reader)
                 {
                     this->syntaxError(reader.line());
                     return this;
                 }
 
-                virtual State* onEqual(cxxtools::Char c, SettingsReader& reader)
+                virtual State* onEqual(cxxtools::Char /*c*/, SettingsReader& reader)
                 {
                     this->syntaxError(reader.line());
                     return this;
                 }
 
-                virtual State* onOpenCurlyBrace(cxxtools::Char c, SettingsReader& reader)
+                virtual State* onOpenCurlyBrace(cxxtools::Char /*c*/, SettingsReader& reader)
                 {
                     this->syntaxError(reader.line());
                     return this;
                 }
 
-                virtual State* onCloseCurlyBrace(cxxtools::Char c, SettingsReader& reader)
+                virtual State* onCloseCurlyBrace(cxxtools::Char /*c*/, SettingsReader& reader)
                 {
                     this->syntaxError(reader.line());
                     return this;
                 }
 
-                virtual State* onOpenBrace(cxxtools::Char c, SettingsReader& reader)
+                virtual State* onOpenBrace(cxxtools::Char /*c*/, SettingsReader& reader)
                 {
                     this->syntaxError(reader.line());
                     return this;
                 }
 
-                virtual State* onCloseBrace(cxxtools::Char c, SettingsReader& reader)
+                virtual State* onCloseBrace(cxxtools::Char /*c*/, SettingsReader& reader)
                 {
                     this->syntaxError(reader.line());
                     return this;
                 }
 
-                virtual State* onOpenSquareBrace(cxxtools::Char c, SettingsReader& reader)
+                virtual State* onOpenSquareBrace(cxxtools::Char /*c*/, SettingsReader& reader)
                 {
                     this->syntaxError(reader.line());
                     return this;
                 }
 
-                virtual State* onCloseSquareBrace(cxxtools::Char c, SettingsReader& reader)
+                virtual State* onCloseSquareBrace(cxxtools::Char /*c*/, SettingsReader& reader)
                 {
                     this->syntaxError(reader.line());
                     return this;
                 }
 
-                virtual State* onHash(cxxtools::Char c, SettingsReader& reader)
+                virtual State* onHash(cxxtools::Char /*c*/, SettingsReader& reader)
                 {
                     reader.beginComment(); // save current state
                     return OnComment::instance();
                 }
 
-                virtual State* onAlpha(cxxtools::Char c, SettingsReader& reader)
+                virtual State* onAlpha(cxxtools::Char /*c*/, SettingsReader& reader)
                 {
                     this->syntaxError(reader.line());
                     return this;
@@ -207,12 +208,12 @@ class SettingsReader
 
         class BeginStatement : public State
         {
-            virtual State* onSpace(cxxtools::Char c, SettingsReader& reader)
+            virtual State* onSpace(cxxtools::Char /*c*/, SettingsReader& /*reader*/)
             {
                 return this;
             }
 
-            virtual State* onQuote(cxxtools::Char c, SettingsReader& reader)
+            virtual State* onQuote(cxxtools::Char /*c*/, SettingsReader& reader)
             {
                 if(reader.depth() == 0)
                     this->syntaxError(reader.line());
@@ -220,13 +221,13 @@ class SettingsReader
                 return OnQuotedValue::instance();
             }
 
-            virtual State* onOpenSquareBrace(cxxtools::Char c, SettingsReader& reader)
+            virtual State* onOpenSquareBrace(cxxtools::Char /*c*/, SettingsReader& reader)
             {
                 reader.beginSection();
                 return OnSection::instance();
             }
 
-            virtual State* onOpenCurlyBrace(cxxtools::Char c, SettingsReader& reader)
+            virtual State* onOpenCurlyBrace(cxxtools::Char /*c*/, SettingsReader& reader)
             {
                 reader.enterMember();
                 return OnCurly::instance();
@@ -238,7 +239,7 @@ class SettingsReader
                 return BeginType::instance();
             }
 
-            virtual State* onEof(SettingsReader& reader)
+            virtual State* onEof(SettingsReader& /*reader*/)
             {
                 return this;
             }
@@ -254,12 +255,12 @@ class SettingsReader
 
         class OnSection : public State
         {
-            virtual State* onSpace(cxxtools::Char c, SettingsReader& reader)
+            virtual State* onSpace(cxxtools::Char /*c*/, SettingsReader& /*reader*/)
             {
                 return this;
             }
 
-            virtual State* onCloseSquareBrace(cxxtools::Char c, SettingsReader& reader)
+            virtual State* onCloseSquareBrace(cxxtools::Char /*c*/, SettingsReader& /*reader*/)
             {
                 return BeginStatement::instance();
             }
@@ -281,14 +282,14 @@ class SettingsReader
 
         class BeginType : public State
         {
-            virtual State* onSpace(cxxtools::Char c, SettingsReader& reader)
+            virtual State* onSpace(cxxtools::Char /*c*/, SettingsReader& /*reader*/)
             {
                 return AfterName::instance();
             }
 
-            virtual State* onEqual(cxxtools::Char c, SettingsReader& reader)
+            virtual State* onEqual(cxxtools::Char /*c*/, SettingsReader& reader)
             {
-                if(reader.depth() == 0)
+                if (reader.depth() == 0)
                     reader.enterMember();
                 else
                     reader.pushName();
@@ -296,7 +297,7 @@ class SettingsReader
                 return OnEqual::instance();
             }
 
-            virtual State* onComma(cxxtools::Char c, SettingsReader& reader)
+            virtual State* onComma(cxxtools::Char /*c*/, SettingsReader& reader)
             {
                 reader.pushValue();
                 reader.leaveMember();
@@ -304,13 +305,13 @@ class SettingsReader
                 return BeginStatement::instance();
             }
 
-            virtual State* onOpenBrace(cxxtools::Char c, SettingsReader& reader)
+            virtual State* onOpenBrace(cxxtools::Char /*c*/, SettingsReader& reader)
             {
                 reader.pushTypeName();
                 return BeginTypedValue::instance();
             }
 
-            virtual State* onOpenCurlyBrace(cxxtools::Char c, SettingsReader& reader)
+            virtual State* onOpenCurlyBrace(cxxtools::Char /*c*/, SettingsReader& reader)
             {
                 if(reader.depth() == 0)
                     this->syntaxError(reader.line());
@@ -320,7 +321,7 @@ class SettingsReader
                 return OnCurly::instance();
             }
 
-            virtual State* onCloseCurlyBrace(cxxtools::Char c, SettingsReader& reader)
+            virtual State* onCloseCurlyBrace(cxxtools::Char /*c*/, SettingsReader& reader)
             {
                 reader.pushValue();
                 reader.leaveMember();
@@ -345,12 +346,12 @@ class SettingsReader
 
         class AfterName : public BeginType
         {
-            virtual State* onSpace(cxxtools::Char c, SettingsReader& reader)
+            virtual State* onSpace(cxxtools::Char /*c*/, SettingsReader& /*reader*/)
             {
                 return this;
             }
 
-            virtual State* onAlpha(cxxtools::Char c, SettingsReader& reader)
+            virtual State* onAlpha(cxxtools::Char /*c*/, SettingsReader& reader)
             {
                 this->syntaxError(reader.line());
                 return this;
@@ -378,12 +379,12 @@ class SettingsReader
                     return this;
             }
 
-            virtual State* onQuote(cxxtools::Char c, SettingsReader& reader)
+            virtual State* onQuote(cxxtools::Char /*c*/, SettingsReader& /*reader*/)
             {
                 return OnQuotedValue::instance();
             }
 
-            virtual State* onOpenCurlyBrace(cxxtools::Char c, SettingsReader& reader)
+            virtual State* onOpenCurlyBrace(cxxtools::Char /*c*/, SettingsReader& reader)
             {
                 reader.enterMember();
                 return OnCurly::instance();
@@ -412,7 +413,7 @@ class SettingsReader
                 return this;
             }
 
-            virtual State* onQuote(cxxtools::Char c, SettingsReader& reader)
+            virtual State* onQuote(cxxtools::Char /*c*/, SettingsReader& reader)
             {
                 reader.pushValue();
                 return AfterQuotedValue::instance();
@@ -492,19 +493,19 @@ class SettingsReader
 
         class AfterValue : public State
         {
-            virtual State* onSpace(cxxtools::Char c, SettingsReader& reader)
+            virtual State* onSpace(cxxtools::Char /*c*/, SettingsReader& /*reader*/)
             {
                 return this;
             }
 
-            virtual State* onComma(cxxtools::Char c, SettingsReader& reader)
+            virtual State* onComma(cxxtools::Char /*c*/, SettingsReader& reader)
             {
                 reader.leaveMember();
                 reader.enterMember();
                 return BeginStatement::instance();
             }
 
-            virtual State* onCloseCurlyBrace(cxxtools::Char c, SettingsReader& reader)
+            virtual State* onCloseCurlyBrace(cxxtools::Char /*c*/, SettingsReader& reader)
             {
                 reader.leaveMember();
                 reader.leaveMember();
@@ -538,13 +539,13 @@ class SettingsReader
 
         class AfterQuotedValue : public AfterValue
         {
-            virtual State* onQuote(cxxtools::Char c, SettingsReader& reader)
+            virtual State* onQuote(cxxtools::Char /*c*/, SettingsReader& /*reader*/)
             {
                 /// TODO: multi-line strings
                 return this;
             }
 
-            virtual State* onOpenSquareBrace(cxxtools::Char c, SettingsReader& reader)
+            virtual State* onOpenSquareBrace(cxxtools::Char /*c*/, SettingsReader& reader)
             {
                 reader.leaveMember();
                 reader.beginSection();
@@ -562,12 +563,12 @@ class SettingsReader
 
         class OnRValue : public State
         {
-            virtual State* onSpace(cxxtools::Char c, SettingsReader& reader)
+            virtual State* onSpace(cxxtools::Char /*c*/, SettingsReader& /*reader*/)
             {
                 return AfterRValue::instance();
             }
 
-            virtual State* onOpenSquareBrace(cxxtools::Char c, SettingsReader& reader)
+            virtual State* onOpenSquareBrace(cxxtools::Char /*c*/, SettingsReader& reader)
             {
                 reader.pushValue();
                 reader.leaveMember();
@@ -576,20 +577,20 @@ class SettingsReader
                 return OnSection::instance();
             }
 
-            virtual State* onOpenCurlyBrace(cxxtools::Char c, SettingsReader& reader)
+            virtual State* onOpenCurlyBrace(cxxtools::Char /*c*/, SettingsReader& reader)
             {
                 reader.pushTypeName();
                 reader.enterMember();
                 return OnCurly::instance();
             }
 
-            virtual State* onOpenBrace(cxxtools::Char c, SettingsReader& reader)
+            virtual State* onOpenBrace(cxxtools::Char /*c*/, SettingsReader& reader)
             {
                 reader.pushTypeName();
                 return BeginTypedValue::instance();
             }
 
-            virtual State* onCloseCurlyBrace(cxxtools::Char c, SettingsReader& reader)
+            virtual State* onCloseCurlyBrace(cxxtools::Char /*c*/, SettingsReader& reader)
             {
                 reader.pushValue();
                 reader.leaveMember();
@@ -597,7 +598,7 @@ class SettingsReader
                 return OnCloseCurly::instance();
             }
 
-            virtual State* onComma(cxxtools::Char c, SettingsReader& reader)
+            virtual State* onComma(cxxtools::Char /*c*/, SettingsReader& reader)
             {
                 reader.pushValue();
                 reader.leaveMember();
@@ -629,7 +630,7 @@ class SettingsReader
 
         class AfterRValue : public OnRValue
         {
-            virtual State* onSpace(cxxtools::Char c, SettingsReader& reader)
+            virtual State* onSpace(cxxtools::Char /*c*/, SettingsReader& /*reader*/)
             {
                 return this;
             }
@@ -653,24 +654,24 @@ class SettingsReader
 
         class OnCurly : public State
         {
-            virtual State* onSpace(cxxtools::Char c, SettingsReader& reader)
+            virtual State* onSpace(cxxtools::Char /*c*/, SettingsReader& /*reader*/)
             {
                 return this;
             }
 
-            virtual State* onOpenCurlyBrace(cxxtools::Char c, SettingsReader& reader)
+            virtual State* onOpenCurlyBrace(cxxtools::Char /*c*/, SettingsReader& reader)
             {
                 reader.enterMember();
                 return OnCurly::instance();
             }
 
-            virtual State* onCloseCurlyBrace(cxxtools::Char c, SettingsReader& reader)
+            virtual State* onCloseCurlyBrace(cxxtools::Char /*c*/, SettingsReader& reader)
             {
                 reader.leaveMember();
                 return OnCloseCurly::instance();
             }
 
-            virtual State* onQuote(cxxtools::Char c, SettingsReader& reader)
+            virtual State* onQuote(cxxtools::Char /*c*/, SettingsReader& /*reader*/)
             {
                 return OnQuotedValue::instance();
             }
@@ -692,24 +693,24 @@ class SettingsReader
 
         class OnCloseCurly : public State
         {
-            virtual State* onSpace(cxxtools::Char c, SettingsReader& reader)
+            virtual State* onSpace(cxxtools::Char /*c*/, SettingsReader& /*reader*/)
             {
                 return this;
             }
 
-            virtual State* onOpenSquareBrace(cxxtools::Char c, SettingsReader& reader)
+            virtual State* onOpenSquareBrace(cxxtools::Char /*c*/, SettingsReader& reader)
             {
                 reader.beginSection();
                 return OnSection::instance();
             }
 
-            virtual State* onCloseCurlyBrace(cxxtools::Char c, SettingsReader& reader)
+            virtual State* onCloseCurlyBrace(cxxtools::Char /*c*/, SettingsReader& reader)
             {
                 reader.leaveMember();
                 return this;
             }
 
-            virtual State* onComma(cxxtools::Char c, SettingsReader& reader)
+            virtual State* onComma(cxxtools::Char /*c*/, SettingsReader& reader)
             {
                 if(reader.depth() == 0)
                 {
@@ -745,12 +746,12 @@ class SettingsReader
 
         class BeginTypedValue : public State
         {
-            virtual State* onSpace(cxxtools::Char c, SettingsReader& reader)
+            virtual State* onSpace(cxxtools::Char /*c*/, SettingsReader& /*reader*/)
             {
                 return this;
             }
 
-            virtual State* onQuote(cxxtools::Char c, SettingsReader& reader)
+            virtual State* onQuote(cxxtools::Char /*c*/, SettingsReader& /*reader*/)
             {
                 return OnQuotedTypedValue::instance();
             }
@@ -772,12 +773,12 @@ class SettingsReader
 
         class OnTypedValue : public State
         {
-            virtual State* onSpace(cxxtools::Char c, SettingsReader& reader)
+            virtual State* onSpace(cxxtools::Char /*c*/, SettingsReader& /*reader*/)
             {
                 return EndTypedValue::instance();
             }
 
-            virtual State* onCloseBrace(cxxtools::Char c, SettingsReader& reader)
+            virtual State* onCloseBrace(cxxtools::Char /*c*/, SettingsReader& reader)
             {
                 reader.pushValue();
                 return AfterValue::instance();
@@ -800,7 +801,7 @@ class SettingsReader
 
         class OnQuotedTypedValue : public OnQuotedValue
         {
-            virtual State* onQuote(cxxtools::Char c, SettingsReader& reader)
+            virtual State* onQuote(cxxtools::Char /*c*/, SettingsReader& /*reader*/)
             {
                 return EndTypedValue::instance();
             }
@@ -816,7 +817,7 @@ class SettingsReader
 
         class EndTypedValue : public OnTypedValue
         {
-            virtual State* onAlpha(cxxtools::Char c, SettingsReader& reader)
+            virtual State* onAlpha(cxxtools::Char /*c*/, SettingsReader& reader)
             {
                 this->syntaxError(reader.line());
                 return this;
@@ -834,7 +835,6 @@ class SettingsReader
         SettingsReader(std::basic_istream<cxxtools::Char>& is)
         : state(0)
         , _beforeComment(0)
-        , _current(0)
         , _is(&is)
         , _line(1)
         , _depth(0)
@@ -878,11 +878,14 @@ class SettingsReader
         cxxtools::Char getEscaped();
 
     private:
+        SerializationInfo* current()
+        { return _current.top(); }
+
         State* state;
 
         State* _beforeComment;
 
-        cxxtools::SerializationInfo* _current;
+        std::stack<cxxtools::SerializationInfo*> _current;
 
         std::basic_istream<cxxtools::Char>* _is;
 
