@@ -28,7 +28,6 @@
 
 #include "tcpserverimpl.h"
 #include <cxxtools/net/tcpserver.h>
-#include <memory>
 #include <sstream>
 
 namespace cxxtools {
@@ -61,11 +60,16 @@ TcpServer::TcpServer(const std::string& ipaddr, unsigned short int port, int bac
 : _impl(0)
 {
     _impl = new TcpServerImpl(*this);
-    std::auto_ptr<TcpServerImpl> impl(_impl);
 
-    this->listen(ipaddr, port, backlog, flags);
-
-    impl.release();
+    try
+    {
+        listen(ipaddr, port, backlog, flags);
+    }
+    catch (...)
+    {
+        delete _impl;
+        throw;
+    }
 }
 
 
@@ -114,9 +118,9 @@ void TcpServer::onClose()
 }
 
 
-bool TcpServer::onWait(std::size_t msecs)
+bool TcpServer::onWait(Timespan timeout)
 {
-    return _impl->wait(msecs);
+    return _impl->wait(timeout);
 }
 
 
