@@ -28,12 +28,15 @@
 
 #include "tcpsocketimpl.h"
 #include "cxxtools/net/tcpsocket.h"
-#include <stdexcept>
-#include <memory>
 #include "cxxtools/log.h"
+#include "cxxtools/systemerror.h"
+#include "cxxtools/ioerror.h"
+#include "cxxtools/sslcertificate.h"
+
+#include <stdexcept>
 #include <errno.h>
-#include <cxxtools/systemerror.h>
-#include <cxxtools/ioerror.h>
+
+#include "config.h"
 
 log_define("cxxtools.net.tcpsocket")
 
@@ -42,45 +45,53 @@ namespace cxxtools {
 namespace net {
 
 TcpSocket::TcpSocket()
-: _impl(0)
+: _impl(new TcpSocketImpl(*this))
 {
-    _impl = new TcpSocketImpl(*this);
 }
 
 
 TcpSocket::TcpSocket(const TcpServer& server, unsigned flags)
-: _impl(0)
+: _impl(new TcpSocketImpl(*this))
 {
-    _impl = new TcpSocketImpl(*this);
-    std::auto_ptr<TcpSocketImpl> impl(_impl);
-
-    this->accept(server, flags);
-
-    impl.release();
+    try
+    {
+        accept(server, flags);
+    }
+    catch (...)
+    {
+        delete _impl;
+        throw;
+    }
 }
 
 
 TcpSocket::TcpSocket(const std::string& ipaddr, unsigned short int port)
-: _impl(0)
+: _impl(new TcpSocketImpl(*this))
 {
-    _impl = new TcpSocketImpl(*this);
-    std::auto_ptr<TcpSocketImpl> impl(_impl);
-
-    this->connect(ipaddr, port);
-
-    impl.release();
+    try
+    {
+        connect(ipaddr, port);
+    }
+    catch (...)
+    {
+        delete _impl;
+        throw;
+    }
 }
 
 
 TcpSocket::TcpSocket(const AddrInfo& addrinfo)
-: _impl(0)
+: _impl(new TcpSocketImpl(*this))
 {
-    _impl = new TcpSocketImpl(*this);
-    std::auto_ptr<TcpSocketImpl> impl(_impl);
-
-    this->connect(addrinfo);
-
-    impl.release();
+    try
+    {
+        connect(addrinfo);
+    }
+    catch (...)
+    {
+        delete _impl;
+        throw;
+    }
 }
 
 
@@ -88,7 +99,7 @@ TcpSocket::~TcpSocket()
 {
     try
     {
-        this->close();
+        close();
     }
     catch(const std::exception& e)
     {
@@ -111,13 +122,13 @@ std::string TcpSocket::getPeerAddr() const
 }
 
 
-void TcpSocket::setTimeout(std::size_t msecs)
+void TcpSocket::setTimeout(Milliseconds timeout)
 {
-    _impl->setTimeout(msecs);
+    _impl->setTimeout(timeout);
 }
 
 
-std::size_t TcpSocket::timeout() const
+Milliseconds TcpSocket::timeout() const
 {
     return _impl->timeout();
 }
@@ -125,23 +136,24 @@ std::size_t TcpSocket::timeout() const
 
 void TcpSocket::connect(const AddrInfo& addrinfo)
 {
-    this->close();
-    _impl->connect(addrinfo);
-    this->setEnabled(true);
-    this->setAsync(true);
-    this->setEof(false);
+    close();
+    _impl->beginConnect(addrinfo);
+    _impl->endConnect();
+    setEnabled(true);
+    setAsync(true);
+    setEof(false);
 }
 
 
 bool TcpSocket::beginConnect(const AddrInfo& addrinfo)
 {
-    this->close();
+    close();
     bool ret = _impl->beginConnect(addrinfo);
-    this->setEnabled(true);
-    this->setAsync(true);
-    this->setEof(false);
+    setEnabled(true);
+    setAsync(true);
+    setEof(false);
 
-    if(ret)
+    if (ret)
         connected(*this);
     return ret;
 }
@@ -167,19 +179,152 @@ bool TcpSocket::isConnected() const
 }
 
 
+bool TcpSocket::isSslConnected() const
+{
+#ifdef WITH_SSL
+    return _impl->isSslConnected();
+#else
+    return false;
+#endif
+}
+
+
 int TcpSocket::getFd() const
 {
     return _impl->fd();
 }
 
 
+void TcpSocket::loadSslCertificateFile(const std::string& certFile, const std::string& privateKeyFile)
+{
+#ifdef WITH_SSL
+    _impl->loadSslCertificateFile(certFile, privateKeyFile);
+#else
+    log_warn("can't load certificate file since ssl is disabled");
+#endif
+}
+
+void TcpSocket::setSslVerify(int level, const std::string& ca)
+{
+#ifdef WITH_SSL
+    _impl->setSslVerify(level, ca);
+#else
+    log_warn("can't set ssl verify level since ssl is disabled");
+#endif
+}
+
+bool TcpSocket::hasSslPeerCertificate() const
+{
+#ifdef WITH_SSL
+    return getSslPeerCertificate();
+#else
+    return false;
+#endif
+}
+
+const SslCertificate& TcpSocket::getSslPeerCertificate() const
+{
+#ifdef WITH_SSL
+    return _impl->getSslPeerCertificate();
+#else
+    log_warn("can't get ssl peer certificate since ssl is disabled");
+    static SslCertificate cert;
+    return cert;
+#endif
+}
+
+void TcpSocket::beginSslConnect()
+{
+#ifdef WITH_SSL
+    if (_impl->beginSslConnect())
+        sslConnected(*this);
+#else
+    log_warn("can't connect ssl since ssl is disabled");
+    sslConnected(*this);
+#endif
+}
+
+void TcpSocket::endSslConnect()
+{
+#ifdef WITH_SSL
+    _impl->endSslConnect();
+#endif
+}
+
+void TcpSocket::sslConnect()
+{
+#ifdef WITH_SSL
+    _impl->beginSslConnect();
+    _impl->endSslConnect();
+#else
+    log_warn("can't connect ssl since ssl is disabled");
+    sslConnected(*this);
+#endif
+}
+
+void TcpSocket::beginSslAccept()
+{
+#ifdef WITH_SSL
+    if (_impl->beginSslAccept())
+        sslAccepted(*this);
+#else
+    log_warn("can't accept ssl connection since ssl is disabled");
+    sslAccepted(*this);
+#endif
+}
+
+void TcpSocket::endSslAccept()
+{
+#ifdef WITH_SSL
+    _impl->endSslAccept();
+#endif
+}
+
+void TcpSocket::sslAccept()
+{
+#ifdef WITH_SSL
+    _impl->beginSslAccept();
+    _impl->endSslAccept();
+#else
+    log_warn("can't accept ssl connection since ssl is disabled");
+#endif
+}
+
+void TcpSocket::beginSslShutdown()
+{
+#ifdef WITH_SSL
+    if (_impl->beginSslShutdown())
+        sslClosed(*this);
+#else
+    log_warn("can't shutdown ssl connection since ssl is disabled");
+    sslClosed(*this);
+#endif
+}
+
+void TcpSocket::endSslShutdown()
+{
+#ifdef WITH_SSL
+    _impl->endSslShutdown();
+#endif
+}
+
+void TcpSocket::sslShutdown()
+{
+#ifdef WITH_SSL
+    _impl->beginSslShutdown();
+    _impl->endSslShutdown();
+#else
+    log_warn("can't shutdown ssl connection since ssl is disabled");
+#endif
+}
+
 void TcpSocket::accept(const TcpServer& server, unsigned flags)
 {
-    this->close();
+    close();
     _impl->accept(server, flags);
-    this->setEnabled(true);
-    this->setAsync(true);
-    this->setEof(false);
+    setEnabled(true);
+    setAsync(true);
+    setEof(false);
 }
 
 
@@ -189,93 +334,25 @@ SelectableImpl& TcpSocket::simpl()
 }
 
 
-void TcpSocket::onClose()
-{
-    cancel();
-    _impl->close();
-}
-
-
-bool TcpSocket::onWait(std::size_t msecs)
-{
-    return _impl->wait(msecs);
-}
-
-
-void TcpSocket::onAttach(SelectorBase& sb)
-{
-    _impl->attach(sb);
-}
-
-
-void TcpSocket::onDetach(SelectorBase& sb)
-{
-    _impl->detach(sb);
-}
-
-
 size_t TcpSocket::onBeginRead(char* buffer, size_t n, bool& eof)
 {
     if (!_impl->isConnected())
-        throw IOPending("connect operation pending");
+        throw IOError("socket not connected when trying to read");
 
     return _impl->beginRead(buffer, n, eof);
 }
 
-
-size_t TcpSocket::onEndRead(bool& eof)
-{
-    return _impl->endRead(eof);
-}
-
-
-size_t TcpSocket::onRead(char* buffer, size_t count, bool& eof)
-{
-    return _impl->read(buffer, count, eof);
-}
-
-
 size_t TcpSocket::onBeginWrite(const char* buffer, size_t n)
 {
     if (!_impl->isConnected())
-        throw IOPending("connect operation pending");
+        throw IOError("socket not connected when trying to write");
 
     return _impl->beginWrite(buffer, n);
 }
 
-
-size_t TcpSocket::onEndWrite()
-{
-    return _impl->endWrite();
-}
-
-
-size_t TcpSocket::onWrite(const char* buffer, size_t count)
-{
-    return _impl->write(buffer, count);
-}
-
-
-void TcpSocket::onCancel()
-{
-    if (_impl->isConnected())
-    {
-        _impl->cancel();
-    }
-    else if (enabled())
-    {
-        // we are in connecting state
-        _impl->close();
-        setEnabled(false);
-    }
-}
-
-
 IODeviceImpl& TcpSocket::ioimpl()
 {
-    throw std::runtime_error("TcpSocket::ioimpl() not implemented");
-    IODeviceImpl* impl = 0;
-    return *impl;
+    return *_impl;
 }
 
 short TcpSocket::poll(short events) const
@@ -284,9 +361,11 @@ short TcpSocket::poll(short events) const
     fds.fd = _impl->fd();
     fds.events = events;
 
-    log_debug("poll timeout " << getTimeout());
+    int timeout = getTimeout().ceil();
 
-    int p = ::poll(&fds, 1, getTimeout());
+    log_debug("poll timeout " << timeout);
+
+    int p = ::poll(&fds, 1, timeout);
 
     log_debug("poll returns " << p << " revents " << fds.revents);
 
@@ -297,7 +376,7 @@ short TcpSocket::poll(short events) const
     }
     else if (p == 0)
     {
-      log_debug("poll timeout (" << getTimeout() << ')');
+      log_debug("poll timeout (" << timeout << ')');
       throw IOTimeout();
     }
 
