@@ -53,16 +53,26 @@ void Client::call(IComposer& r, IRemoteProcedure& method, IDecomposer** argv, un
     _impl->call(r, method, argv, argc);
 }
 
-std::size_t Client::timeout() const
+Milliseconds Client::timeout() const
 {
     return _impl->timeout();
 }
 
-void Client::timeout(std::size_t t)
+void Client::timeout(Milliseconds t)
 {
     _impl->timeout(t);
 }
 
+Milliseconds Client::connectTimeout() const
+{
+    return _impl->connectTimeout();
+}
+
+void Client::connectTimeout(Milliseconds t)
+{
+    _impl->connectTimeout(t);
+}
+
 std::string Client::url() const
 {
     return _impl->url();
@@ -70,12 +80,13 @@ std::string Client::url() const
 
 const IRemoteProcedure* Client::activeProcedure() const
 {
-    return _impl->activeProcedure();
+    return _impl == 0 ? 0 : _impl->activeProcedure();
 }
 
 void Client::cancel()
 {
-    _impl->cancel();
+    if (_impl)
+        _impl->cancel();
 }
 
 }
