@@ -28,7 +28,6 @@
 #ifndef cxxtools_xmlrpc_ClientImpl_h
 #define cxxtools_xmlrpc_ClientImpl_h
 
-#include <cxxtools/xmlrpc/api.h>
 #include <cxxtools/remoteexception.h>
 #include <cxxtools/xmlrpc/formatter.h>
 #include <cxxtools/xmlrpc/scanner.h>
@@ -76,9 +75,11 @@ class ClientImpl : public cxxtools::Connectable
 
         void call(IComposer& r, IRemoteProcedure& method, IDecomposer** argv, unsigned argc);
 
-        std::size_t timeout() const  { return _timeout; }
+        Milliseconds timeout() const  { return _timeout; }
+        void timeout(Milliseconds t)  { _timeout = t; if (!_connectTimeoutSet) _connectTimeout = t; }
 
-        void timeout(std::size_t t)  { _timeout = t; }
+        Milliseconds connectTimeout() const  { return _connectTimeout; }
+        void connectTimeout(Milliseconds t)  { _connectTimeout = t; _connectTimeoutSet = true; }
 
         virtual std::string url() const = 0;
 
@@ -97,7 +98,7 @@ class ClientImpl : public cxxtools::Connectable
 
         virtual void endExecute() = 0;
 
-        virtual std::string execute() = 0;
+        virtual std::istream& execute() = 0;
 
         virtual std::ostream& prepareRequest() = 0;
 
@@ -111,12 +112,15 @@ class ClientImpl : public cxxtools::Connectable
         xml::XmlReader _reader;
         xml::XmlWriter _writer;
         Formatter _formatter;
-        DeserializerBase _deserializer;
+        Deserializer _deserializer;
         Scanner _scanner;
         IRemoteProcedure* _method;
         RemoteException _fault;
         Composer<RemoteException> _fh;
-        std::size_t _timeout;
+        Milliseconds _timeout;
+        bool _connectTimeoutSet;  // indicates if connectTimeout is explicitely set
+                                  // when not, it follows the setting of _timeout
+        Milliseconds _connectTimeout;
         bool _errorPending;
 };
 
