@@ -30,13 +30,17 @@
 #include "cxxtools/base64stream.h"
 #include "cxxtools/unit/testsuite.h"
 #include "cxxtools/unit/registertest.h"
+#include "cxxtools/log.h"
+
+log_define("cxxtools.test.base64")
 
 class Base64Test : public cxxtools::unit::TestSuite
 {
         static std::string encodeDecode(const std::string& data)
         {
-            return cxxtools::decode<cxxtools::Base64Codec>(
-                      cxxtools::encode<cxxtools::Base64Codec>(data));
+            std::string b = cxxtools::encode<cxxtools::Base64Codec>(data);
+            log_debug("data.size=" << data.size() << " data=\"" << data << "\" base64.size=" << b.size() << " base64=" << b);
+            return cxxtools::decode<cxxtools::Base64Codec>(b);
         }
 
     public:
@@ -50,7 +54,12 @@ class Base64Test : public cxxtools::unit::TestSuite
             registerMethod("encodeStreamTest1", *this, &Base64Test::encodeStreamTest1);
             registerMethod("encodeStreamTest2", *this, &Base64Test::encodeStreamTest2);
             registerMethod("encodeDecodeTest", *this, &Base64Test::encodeDecodeTest);
+            registerMethod("whitespaceDecodeTest", *this, &Base64Test::whitespaceDecodeTest);
             registerMethod("binaryTest", *this, &Base64Test::binaryTest);
+            registerMethod("artemTest", *this, &Base64Test::artemTest);
+            registerMethod("maxcolTest", *this, &Base64Test::maxcolTest);
+            registerMethod("lineendTest", *this, &Base64Test::lineendTest);
+            registerMethod("paddingTest", *this, &Base64Test::paddingTest);
         }
 
         void encodeTest0()
@@ -120,12 +129,20 @@ class Base64Test : public cxxtools::unit::TestSuite
             CXXTOOLS_UNIT_ASSERT_EQUALS(data, data2);
         }
 
+        void whitespaceDecodeTest()
+        {
+            std::string data =
+                "SGk  gd\tGhl\n\ncmU=\n";
+            std::string w = cxxtools::Base64Codec::decode(data);
+            CXXTOOLS_UNIT_ASSERT_EQUALS(w, "Hi there");
+        }
+
         void binaryTest()
         {
             std::string data2;
             std::string data;
             
-            data.assign("\xff\xd8\xff\xe0\x00\x10\x4a\x46\x00\x01\x01\x00\x48", 16);
+            data.assign("\xff\xd8\xff\xe0\x00\x10\x4a\x46\x00\x01\x01\x00\x48", 13);
             data2 = encodeDecode(data);
             CXXTOOLS_UNIT_ASSERT_EQUALS(data, data2);
 
@@ -183,6 +200,53 @@ class Base64Test : public cxxtools::unit::TestSuite
             CXXTOOLS_UNIT_ASSERT_EQUALS(data, data2);
         }
 
+        void artemTest()
+        {
+            for (unsigned n = 0; n < 256; ++n)
+            {
+                std::string data(n, 'a');
+                std::string data2 = encodeDecode(data);
+                CXXTOOLS_UNIT_ASSERT_EQUALS(data, data2);
+            }
+        }
+
+        void maxcolTest()
+        {
+            std::ostringstream s;
+            
+            cxxtools::Base64ostream encoder(s);
+            encoder.maxcol(8);
+            encoder << "123456789";
+            encoder.terminate();
+
+            CXXTOOLS_UNIT_ASSERT_EQUALS(s.str(), "MTIzNDU2\r\nNzg5");
+        }
+
+        void lineendTest()
+        {
+            std::ostringstream s;
+            
+            cxxtools::Base64ostream encoder(s);
+            encoder.maxcol(8);
+            encoder.lineend("\n");
+            encoder << "123456789";
+            encoder.terminate();
+
+            CXXTOOLS_UNIT_ASSERT_EQUALS(s.str(), "MTIzNDU2\nNzg5");
+        }
+
+        void paddingTest()
+        {
+            std::ostringstream s;
+            
+            cxxtools::Base64ostream encoder(s);
+            encoder.padding(false);
+            encoder << "12345678";
+            encoder.terminate();
+
+            CXXTOOLS_UNIT_ASSERT_EQUALS(s.str(), "MTIzNDU2Nzg");
+        }
+
 };
 
 cxxtools::unit::RegisterTest<Base64Test> register_Base64Test;
