@@ -33,42 +33,22 @@
 #include "cxxtools/remoteprocedure.h"
 #include "cxxtools/eventloop.h"
 #include "cxxtools/log.h"
+#include "cxxtools/ioerror.h"
+#include "cxxtools/net/uri.h"
+#include "cxxtools/net/addrinfo.h"
 #include <stdlib.h>
 #include <sstream>
 
+#include "color.h"
+
 log_define("cxxtools.test.binrpc")
 
 namespace
 {
-    struct Color
-    {
-        int red;
-        int green;
-        int blue;
-    };
-
-
     typedef std::set<int> IntSet;
     typedef std::multiset<int> IntMultiset;
     typedef std::map<int, int> IntMap;
     typedef std::multimap<int, int> IntMultimap;
-
-
-    void operator >>=(const cxxtools::SerializationInfo& si, Color& color)
-    {
-        si.getMember("red") >>= color.red;
-        si.getMember("green") >>= color.green;
-        si.getMember("blue") >>= color.blue;
-    }
-
-
-    void operator <<=(cxxtools::SerializationInfo& si, const Color& color)
-    {
-        si.addMember("red") <<= color.red;
-        si.addMember("green") <<= color.green;
-        si.addMember("blue") <<= color.blue;
-    }
-
 }
 
 class BinRpcTest : public cxxtools::unit::TestSuite
@@ -77,6 +57,7 @@ class BinRpcTest : public cxxtools::unit::TestSuite
         cxxtools::EventLoop _loop;
         cxxtools::bin::RpcServer* _server;
         unsigned _count;
+        std::string _listen;
         unsigned short _port;
 
     public:
@@ -105,6 +86,9 @@ class BinRpcTest : public cxxtools::unit::TestSuite
             registerMethod("CallbackException", *this, &BinRpcTest::CallbackException);
             registerMethod("ConnectError", *this, &BinRpcTest::ConnectError);
             registerMethod("BigRequest", *this, &BinRpcTest::BigRequest);
+            registerMethod("PrepareConnect", *this, &BinRpcTest::PrepareConnect);
+            registerMethod("Connect", *this, &BinRpcTest::Connect);
+            registerMethod("Multiple", *this, &BinRpcTest::Multiple);
 
             char* PORT = getenv("UTEST_PORT");
             if (PORT)
@@ -112,6 +96,10 @@ class BinRpcTest : public cxxtools::unit::TestSuite
                 std::istringstream s(PORT);
                 s >> _port;
             }
+
+            char* LISTEN = getenv("UTEST_LISTEN");
+            if (LISTEN)
+                _listen = LISTEN;
         }
 
         void failTest()
@@ -125,7 +113,7 @@ class BinRpcTest : public cxxtools::unit::TestSuite
             connect(_loop.timeout, *this, &BinRpcTest::failTest);
             connect(_loop.timeout, _loop, &cxxtools::EventLoop::exit);
 
-            _server = new cxxtools::bin::RpcServer(_loop, _port);
+            _server = new cxxtools::bin::RpcServer(_loop, _listen, _port);
             _server->minThreads(1);
         }
 
@@ -141,7 +129,7 @@ class BinRpcTest : public cxxtools::unit::TestSuite
         {
             _server->registerMethod("multiply", *this, &BinRpcTest::multiplyNothing);
 
-            cxxtools::bin::RpcClient client(_loop, "", _port);
+            cxxtools::bin::RpcClient client(_loop, _listen, _port);
             cxxtools::RemoteProcedure<bool> multiply(client, "multiply");
 
             multiply.begin();
@@ -160,7 +148,7 @@ class BinRpcTest : public cxxtools::unit::TestSuite
         {
             _server->registerMethod("multiply", *this, &BinRpcTest::multiplyNothing);
 
-            cxxtools::bin::RpcClient client(_loop, "", _port);
+            cxxtools::bin::RpcClient client(_loop, _listen, _port);
             cxxtools::RemoteProcedure<bool> multiply(client, "multiply");
             connect( multiply.finished, *this, &BinRpcTest::onExceptionCallback );
 
@@ -171,7 +159,7 @@ class BinRpcTest : public cxxtools::unit::TestSuite
             CXXTOOLS_UNIT_ASSERT_EQUALS(_count, 1);
         }
 
-        void onExceptionCallback(const cxxtools::RemoteResult<bool>& r)
+        void onExceptionCallback(cxxtools::RemoteResult<bool>& /*r*/)
         {
             log_warn("exception callback");
             ++_count;
@@ -186,7 +174,7 @@ class BinRpcTest : public cxxtools::unit::TestSuite
         {
             log_trace("ConnectError");
 
-            cxxtools::bin::RpcClient client(_loop, "", _port + 1);
+            cxxtools::bin::RpcClient client(_loop, _listen, _port + 1);
             cxxtools::RemoteProcedure<bool> multiply(client, "multiply");
             connect( multiply.finished, *this, &BinRpcTest::onConnectErrorCallback );
 
@@ -203,7 +191,7 @@ class BinRpcTest : public cxxtools::unit::TestSuite
             }
         }
 
-        void onConnectErrorCallback(const cxxtools::RemoteResult<bool>& r)
+        void onConnectErrorCallback(cxxtools::RemoteResult<bool>& r)
         {
             log_debug("onConnectErrorCallback");
             _loop.exit();
@@ -217,7 +205,7 @@ class BinRpcTest : public cxxtools::unit::TestSuite
         {
             _server->registerMethod("boolean", *this, &BinRpcTest::boolean);
 
-            cxxtools::bin::RpcClient client(_loop, "", _port);
+            cxxtools::bin::RpcClient client(_loop, _listen, _port);
             cxxtools::RemoteProcedure<bool, bool, bool> multiply(client, "boolean");
 
             multiply.begin(true, true);
@@ -238,7 +226,7 @@ class BinRpcTest : public cxxtools::unit::TestSuite
         {
             _server->registerMethod("multiply", *this, &BinRpcTest::multiplyInt);
 
-            cxxtools::bin::RpcClient client(_loop, "", _port);
+            cxxtools::bin::RpcClient client(_loop, _listen, _port);
             cxxtools::RemoteProcedure<int, int, int> multiply(client, "multiply");
 
             multiply.begin(2, 3);
@@ -257,7 +245,7 @@ class BinRpcTest : public cxxtools::unit::TestSuite
         {
             _server->registerMethod("multiply", *this, &BinRpcTest::multiplyDouble);
 
-            cxxtools::bin::RpcClient client(_loop, "", _port);
+            cxxtools::bin::RpcClient client(_loop, _listen, _port);
             cxxtools::RemoteProcedure<double, double, double> multiply(client, "multiply");
 
             multiply.begin(2.0, 3.0);
@@ -276,7 +264,7 @@ class BinRpcTest : public cxxtools::unit::TestSuite
         {
             _server->registerMethod("echoString", *this, &BinRpcTest::echoString);
 
-            cxxtools::bin::RpcClient client(_loop, "", _port);
+            cxxtools::bin::RpcClient client(_loop, _listen, _port);
             cxxtools::RemoteProcedure<std::string, std::string> echo(client, "echoString");
 
             echo.begin("\xc3\xaf\xc2\xbb\xc2\xbf'\"&<> foo?");
@@ -295,7 +283,7 @@ class BinRpcTest : public cxxtools::unit::TestSuite
         {
             _server->registerMethod("multiply", *this, &BinRpcTest::multiplyEmpty);
 
-            cxxtools::bin::RpcClient client(_loop, "", _port);
+            cxxtools::bin::RpcClient client(_loop, _listen, _port);
             cxxtools::RemoteProcedure<std::string, std::string, std::string> multiply(client, "multiply");
 
             multiply.begin("", "");
@@ -316,7 +304,7 @@ class BinRpcTest : public cxxtools::unit::TestSuite
         {
             _server->registerMethod("multiply", *this, &BinRpcTest::multiplyVector);
 
-            cxxtools::bin::RpcClient client(_loop, "", _port);
+            cxxtools::bin::RpcClient client(_loop, _listen, _port);
             cxxtools::RemoteProcedure< std::vector<int>, std::vector<int>, std::vector<int> > multiply(client, "multiply");
 
             std::vector<int> vec;
@@ -349,7 +337,7 @@ class BinRpcTest : public cxxtools::unit::TestSuite
         {
             _server->registerMethod("multiply", *this, &BinRpcTest::multiplyVector);
 
-            cxxtools::bin::RpcClient client(_loop, "", _port);
+            cxxtools::bin::RpcClient client(_loop, _listen, _port);
             cxxtools::RemoteProcedure< std::vector<int>, std::vector<int>, std::vector<int> > multiply(client, "multiply");
 
             std::vector<int> vec;
@@ -364,7 +352,7 @@ class BinRpcTest : public cxxtools::unit::TestSuite
         {
             _server->registerMethod("multiply", *this, &BinRpcTest::multiplyColor);
 
-            cxxtools::bin::RpcClient client(_loop, "", _port);
+            cxxtools::bin::RpcClient client(_loop, _listen, _port);
             cxxtools::RemoteProcedure< Color, Color, Color > multiply(client, "multiply");
 
             Color a;
@@ -400,7 +388,7 @@ class BinRpcTest : public cxxtools::unit::TestSuite
         {
             _server->registerMethod("multiplyset", *this, &BinRpcTest::multiplySet);
 
-            cxxtools::bin::RpcClient client(_loop, "", _port);
+            cxxtools::bin::RpcClient client(_loop, _listen, _port);
             cxxtools::RemoteProcedure<IntSet, IntSet, int> multiply(client, "multiplyset");
 
             IntSet myset;
@@ -432,7 +420,7 @@ class BinRpcTest : public cxxtools::unit::TestSuite
         {
             _server->registerMethod("multiplyset", *this, &BinRpcTest::multiplyMultiset);
 
-            cxxtools::bin::RpcClient client(_loop, "", _port);
+            cxxtools::bin::RpcClient client(_loop, _listen, _port);
             cxxtools::RemoteProcedure<IntMultiset, IntMultiset, int> multiply(client, "multiplyset");
 
             IntMultiset myset;
@@ -464,7 +452,7 @@ class BinRpcTest : public cxxtools::unit::TestSuite
         {
             _server->registerMethod("multiplymap", *this, &BinRpcTest::multiplyMap);
 
-            cxxtools::bin::RpcClient client(_loop, "", _port);
+            cxxtools::bin::RpcClient client(_loop, _listen, _port);
             cxxtools::RemoteProcedure<IntMap, IntMap, int> multiply(client, "multiplymap");
 
             IntMap mymap;
@@ -502,7 +490,7 @@ class BinRpcTest : public cxxtools::unit::TestSuite
         {
             _server->registerMethod("multiplymultimap", *this, &BinRpcTest::multiplyMultimap);
 
-            cxxtools::bin::RpcClient client(_loop, "", _port);
+            cxxtools::bin::RpcClient client(_loop, _listen, _port);
             cxxtools::RemoteProcedure<IntMultimap, IntMultimap, int> multiply(client, "multiplymultimap");
 
             IntMultimap mymap;
@@ -548,7 +536,8 @@ class BinRpcTest : public cxxtools::unit::TestSuite
             registry.registerMethod("multiply", *this, &BinRpcTest::multiplyInt);
             _server->addService("myDomain", registry);
 
-            cxxtools::bin::RpcClient client(_loop, "", _port, "myDomain");
+            cxxtools::bin::RpcClient client(_loop, _listen, _port);
+            client.domain("myDomain");
             cxxtools::RemoteProcedure<int, int, int> multiply(client, "multiply");
 
             multiply.begin(2, 3);
@@ -564,7 +553,8 @@ class BinRpcTest : public cxxtools::unit::TestSuite
             registry.registerMethod("multiply", *this, &BinRpcTest::multiplyInt);
             _server->addService("myDomain", registry);
 
-            cxxtools::bin::RpcClient client(_loop, "", _port, "unknownDomain");
+            cxxtools::bin::RpcClient client(_loop, _listen, _port);
+            client.domain("unknownDomain");
             cxxtools::RemoteProcedure<int, int, int> multiply(client, "multiply");
 
             multiply.begin(2, 3);
@@ -576,7 +566,7 @@ class BinRpcTest : public cxxtools::unit::TestSuite
         //
         void UnknownMethod()
         {
-            cxxtools::bin::RpcClient client(_loop, "", _port);
+            cxxtools::bin::RpcClient client(_loop, _listen, _port);
             cxxtools::RemoteProcedure<bool> unknownMethod(client, "unknownMethod");
 
             unknownMethod.begin();
@@ -591,7 +581,7 @@ class BinRpcTest : public cxxtools::unit::TestSuite
         {
             _server->registerMethod("multiply", *this, &BinRpcTest::throwFault);
 
-            cxxtools::bin::RpcClient client(_loop, "", _port);
+            cxxtools::bin::RpcClient client(_loop, _listen, _port);
             cxxtools::RemoteProcedure<bool> multiply(client, "multiply");
             multiply.begin();
 
@@ -620,7 +610,7 @@ class BinRpcTest : public cxxtools::unit::TestSuite
         {
             _server->registerMethod("multiply", *this, &BinRpcTest::throwException);
 
-            cxxtools::bin::RpcClient client(_loop, "", _port);
+            cxxtools::bin::RpcClient client(_loop, _listen, _port);
             cxxtools::RemoteProcedure<bool> multiply(client, "multiply");
 
             multiply.begin();
@@ -648,11 +638,9 @@ class BinRpcTest : public cxxtools::unit::TestSuite
         //
         void BigRequest()
         {
-            log_trace("ConnectError");
-
             _server->registerMethod("countSize", *this, &BinRpcTest::countSize);
 
-            cxxtools::bin::RpcClient client(_loop, "", _port);
+            cxxtools::bin::RpcClient client(_loop, _listen, _port);
             cxxtools::RemoteProcedure<unsigned, std::vector<int> > countSize(client, "countSize");
 
             std::vector<int> v;
@@ -676,6 +664,146 @@ class BinRpcTest : public cxxtools::unit::TestSuite
             return v.size();
         }
 
+        ////////////////////////////////////////////////////////////
+        // PrepareConnect
+        //
+        void PrepareConnect()
+        {
+            log_trace("PrepareConnect");
+
+            _server->registerMethod("boolean", *this, &BinRpcTest::boolean);
+
+            // test connect using cxxtools::net::AddrInfo
+            {
+                cxxtools::bin::RpcClient client(_loop);
+                cxxtools::RemoteProcedure<bool, bool, bool> boolean(client, "boolean");
+
+                client.prepareConnect(cxxtools::net::AddrInfo(_listen, _port));
+                boolean.begin(true, true);
+                CXXTOOLS_UNIT_ASSERT_EQUALS(boolean.end(2000), true);
+            }
+
+            // test connect using host and port
+            {
+                cxxtools::bin::RpcClient client(_loop);
+                cxxtools::RemoteProcedure<bool, bool, bool> boolean(client, "boolean");
+
+                client.prepareConnect(_listen, _port);
+                boolean.begin(true, true);
+                CXXTOOLS_UNIT_ASSERT_EQUALS(boolean.end(2000), true);
+            }
+
+            // test connect using uri
+            {
+                cxxtools::bin::RpcClient client(_loop);
+                cxxtools::RemoteProcedure<bool, bool, bool> boolean(client, "boolean");
+
+                std::ostringstream uri;
+                uri << "bin://localhost:" << _port << '/';
+                client.prepareConnect(cxxtools::net::Uri(uri.str()));
+                boolean.begin(true, true);
+                CXXTOOLS_UNIT_ASSERT_EQUALS(boolean.end(2000), true);
+            }
+
+            // test failing connect in connect
+            {
+                cxxtools::bin::RpcClient client(_loop);
+                cxxtools::RemoteProcedure<bool, bool, bool> boolean(client, "boolean");
+
+                CXXTOOLS_UNIT_ASSERT_NOTHROW(client.prepareConnect(_listen, _port + 1));
+                CXXTOOLS_UNIT_ASSERT_THROW(client.connect(), cxxtools::IOError);
+            }
+
+            // test failing connect when calling function
+            {
+                cxxtools::bin::RpcClient client(_loop);
+                cxxtools::RemoteProcedure<bool, bool, bool> boolean(client, "boolean");
+                CXXTOOLS_UNIT_ASSERT_NOTHROW(client.prepareConnect(_listen, _port + 1));
+
+                boolean.begin(true, true);
+                CXXTOOLS_UNIT_ASSERT_THROW(boolean.end(2000), cxxtools::IOError);
+            }
+
+        }
+
+        ////////////////////////////////////////////////////////////
+        // Connect
+        //
+        void Connect()
+        {
+            _server->registerMethod("boolean", *this, &BinRpcTest::boolean);
+
+            // test connect using cxxtools::net::AddrInfo
+            {
+                cxxtools::bin::RpcClient client(_loop);
+                cxxtools::RemoteProcedure<bool, bool, bool> boolean(client, "boolean");
+
+                client.connect(cxxtools::net::AddrInfo(_listen, _port));
+                boolean.begin(true, true);
+                CXXTOOLS_UNIT_ASSERT_EQUALS(boolean.end(2000), true);
+            }
+
+            // test connect using host and port
+            {
+                cxxtools::bin::RpcClient client(_loop);
+                cxxtools::RemoteProcedure<bool, bool, bool> boolean(client, "boolean");
+
+                client.connect(_listen, _port);
+                boolean.begin(true, true);
+                CXXTOOLS_UNIT_ASSERT_EQUALS(boolean.end(2000), true);
+            }
+
+            // test connect using uri
+            {
+                cxxtools::bin::RpcClient client(_loop);
+                cxxtools::RemoteProcedure<bool, bool, bool> boolean(client, "boolean");
+
+                std::ostringstream uri;
+                uri << "bin://localhost:" << _port << '/';
+                client.connect(cxxtools::net::Uri(uri.str()));
+                boolean.begin(true, true);
+                CXXTOOLS_UNIT_ASSERT_EQUALS(boolean.end(2000), true);
+            }
+
+            // test failing connect
+            {
+                cxxtools::bin::RpcClient client(_loop);
+                cxxtools::RemoteProcedure<bool, bool, bool> boolean(client, "boolean");
+
+                CXXTOOLS_UNIT_ASSERT_THROW(client.connect(_listen, _port + 1), cxxtools::IOError);
+            }
+
+        }
+
+        ////////////////////////////////////////////////////////////
+        // Multiple calls
+        //
+        void Multiple()
+        {
+            _server->registerMethod("multiply", *this, &BinRpcTest::multiplyDouble);
+
+            typedef cxxtools::RemoteProcedure<double, double, double> Multiply;
+
+            std::vector<cxxtools::bin::RpcClient> clients;
+            std::vector<Multiply> procs;
+
+            clients.reserve(16);
+            procs.reserve(16);
+
+            for (unsigned i = 0; i < 16; ++i)
+            {
+                clients.push_back(cxxtools::bin::RpcClient(_loop, _listen, _port));
+                procs.push_back(Multiply(clients.back(), "multiply"));
+                procs.back().begin(i, i);
+            }
+
+            for (unsigned i = 0; i < 16; ++i)
+            {
+                CXXTOOLS_UNIT_ASSERT_EQUALS(procs[i].end(2000), i*i);
+            }
+
+        }
+
 };
 
 cxxtools::unit::RegisterTest<BinRpcTest> register_BinRpcTest;
