@@ -29,10 +29,14 @@
 #include "cxxtools/unit/testsuite.h"
 #include "cxxtools/unit/registertest.h"
 #include "cxxtools/serializationinfo.h"
-#include "cxxtools/bin/serializer.h"
-#include "cxxtools/bin/deserializer.h"
+#include "cxxtools/bin/bin.h"
+#include "cxxtools/bin/formatter.h"
 #include "cxxtools/log.h"
-#include "cxxtools/hdstream.h"
+#include "cxxtools/date.h"
+#include "cxxtools/time.h"
+#include "cxxtools/datetime.h"
+#include "cxxtools/timespan.h"
+#include "cxxtools/hexdump.h"
 #include <limits>
 #include <stdint.h>
 #include <config.h>
@@ -108,6 +112,24 @@ namespace
             && obj1.setValue == obj2.setValue
             && obj1.mapValue == obj2.mapValue;
     }
+
+    template <typename T>
+    std::string toBin(const T& t)
+    {
+        std::ostringstream out;
+        out << cxxtools::bin::Bin(t);
+        return out.str();
+    }
+
+    template <typename T>
+    T fromBin(const std::string& bin)
+    {
+        std::istringstream in(bin);
+        T t;
+        in >> cxxtools::bin::Bin(t);
+        return t;
+    }
+
 }
 
 class BinSerializerTest : public cxxtools::unit::TestSuite
@@ -118,25 +140,36 @@ class BinSerializerTest : public cxxtools::unit::TestSuite
         {
             registerMethod("testScalar", *this, &BinSerializerTest::testScalar);
             registerMethod("testInt", *this, &BinSerializerTest::testInt);
+            registerMethod("testChar", *this, &BinSerializerTest::testChar);
+            registerMethod("testChar_0", *this, &BinSerializerTest::testChar_0);
+            registerMethod("testString8", *this, &BinSerializerTest::testString8);
+            registerMethod("testString8_0", *this, &BinSerializerTest::testString8_0);
+            registerMethod("testString", *this, &BinSerializerTest::testString);
+            registerMethod("testString_0", *this, &BinSerializerTest::testString_0);
             registerMethod("testDouble", *this, &BinSerializerTest::testDouble);
             registerMethod("testArray", *this, &BinSerializerTest::testArray);
             registerMethod("testObject", *this, &BinSerializerTest::testObject);
             registerMethod("testComplexObject", *this, &BinSerializerTest::testComplexObject);
             registerMethod("testObjectVector", *this, &BinSerializerTest::testObjectVector);
             registerMethod("testBinaryData", *this, &BinSerializerTest::testBinaryData);
+            registerMethod("testReuse", *this, &BinSerializerTest::testReuse);
+            registerMethod("testNamedVector", *this, &BinSerializerTest::testNamedVector);
+            registerMethod("testDate", *this, &BinSerializerTest::testDate);
+            registerMethod("testTime", *this, &BinSerializerTest::testTime);
+            registerMethod("testDatetime", *this, &BinSerializerTest::testDatetime);
+            registerMethod("testTimespan", *this, &BinSerializerTest::testTimespan);
+            registerMethod("testRawString", *this, &BinSerializerTest::testRawString);
         }
 
         void testScalar()
         {
             std::stringstream data;
-            cxxtools::bin::Serializer serializer(data);
-            cxxtools::bin::Deserializer deserializer(data);
 
             int value = 5;
-            serializer.serialize(value);
+            data << cxxtools::bin::Bin(value);
 
             int value2 = 0;
-            deserializer.deserialize(value2);
+            data >> cxxtools::bin::Bin(value2);
 
             CXXTOOLS_UNIT_ASSERT_EQUALS(value, value2);
         }
@@ -145,15 +178,24 @@ class BinSerializerTest : public cxxtools::unit::TestSuite
         void testIntValue(IntT value)
         {
             std::stringstream data;
-            cxxtools::bin::Serializer serializer(data);
-            cxxtools::bin::Deserializer deserializer(data);
 
-            serializer.serialize(value);
-
-            log_debug("int: " << cxxtools::hexDump(data.str()));
+            data << cxxtools::bin::Bin(value);
 
             IntT result = 0;
-            deserializer.deserialize(result);
+            data >> cxxtools::bin::Bin(result);
+
+            CXXTOOLS_UNIT_ASSERT_EQUALS(value, result);
+        }
+
+        template <typename T>
+        void testStreamableValue(T value, T initvalue)
+        {
+            std::stringstream data;
+
+            data << cxxtools::bin::Bin(value);
+
+            T result = initvalue;
+            data >> cxxtools::bin::Bin(result);
 
             CXXTOOLS_UNIT_ASSERT_EQUALS(value, result);
         }
@@ -193,17 +235,49 @@ class BinSerializerTest : public cxxtools::unit::TestSuite
 #endif
         }
 
+        void testChar()
+        {
+            testStreamableValue('a', '\xff');
+            testStreamableValue('\1', '\xff');
+        }
+
+        void testChar_0()
+        {
+            testStreamableValue('\0', '\xff');
+        }
+
+        void testString8()
+        {
+            testStreamableValue(std::string(""), std::string("init"));
+            testStreamableValue(std::string("Hi there"), std::string("init"));
+        }
+
+        void testString8_0()
+        {
+            testStreamableValue(std::string(1, '\0'), std::string("init"));
+        }
+
+        void testString()
+        {
+            testStreamableValue(cxxtools::String(L""), cxxtools::String(L"init"));
+            testStreamableValue(cxxtools::String(L"Hi there"), cxxtools::String(L"init"));
+        }
+
+        void testString_0()
+        {
+            testStreamableValue(cxxtools::String(1, L'\0'), cxxtools::String(L"init"));
+        }
+
         void testDoubleValue(double value)
         {
             std::stringstream data;
-            cxxtools::bin::Serializer serializer(data);
-            cxxtools::bin::Deserializer deserializer(data);
 
-            serializer.serialize(value);
+            data << cxxtools::bin::Bin(value);
+
             double result = 0.0;
-            deserializer.deserialize(result);
+            data >> cxxtools::bin::Bin(result);
 
-            log_debug("test double value " << value << " => " << cxxtools::hexDump(data.str()) << " => " << result);
+            log_debug("test double value " << value << " =>\n" << cxxtools::hexDump(data.str()) << "\n=> " << result);
 
             if (value != value) // check for nan
                 CXXTOOLS_UNIT_ASSERT(result != result);
@@ -232,12 +306,11 @@ class BinSerializerTest : public cxxtools::unit::TestSuite
             testDoubleValue(std::numeric_limits<double>::quiet_NaN());
 
             std::stringstream data;
-            cxxtools::bin::Serializer serializer(data);
-            cxxtools::bin::Deserializer deserializer(data);
 
-            serializer.serialize(std::numeric_limits<double>::quiet_NaN());
+            data << cxxtools::bin::Bin(std::numeric_limits<double>::quiet_NaN());
+
             double result = 0.0;
-            deserializer.deserialize(result);
+            data >> cxxtools::bin::Bin(result);
 
             CXXTOOLS_UNIT_ASSERT(result != result); 
 
@@ -246,8 +319,6 @@ class BinSerializerTest : public cxxtools::unit::TestSuite
         void testArray()
         {
             std::stringstream data;
-            cxxtools::bin::Serializer serializer(data);
-            cxxtools::bin::Deserializer deserializer(data);
 
             std::vector<int> intvector;
             intvector.push_back(4711);
@@ -255,12 +326,11 @@ class BinSerializerTest : public cxxtools::unit::TestSuite
             intvector.push_back(-3);
             intvector.push_back(-257);
 
-            serializer.serialize(intvector);
-
-            log_debug("intvector: " << cxxtools::hexDump(data.str()));
+            data << cxxtools::bin::Bin(intvector);
+            log_debug("intvector:\n" << cxxtools::hexDump(data.str()));
 
             std::vector<int> intvector2;
-            deserializer.deserialize(intvector2);
+            data >> cxxtools::bin::Bin(intvector2);
 
             CXXTOOLS_UNIT_ASSERT_EQUALS(intvector.size(), intvector2.size());
             CXXTOOLS_UNIT_ASSERT_EQUALS(intvector[0], intvector2[0]);
@@ -272,8 +342,6 @@ class BinSerializerTest : public cxxtools::unit::TestSuite
         void testObject()
         {
             std::stringstream data;
-            cxxtools::bin::Serializer serializer(data);
-            cxxtools::bin::Deserializer deserializer(data);
 
             TestObject obj;
             obj.intValue = 17;
@@ -281,12 +349,12 @@ class BinSerializerTest : public cxxtools::unit::TestSuite
             obj.doubleValue = 3.125;
             obj.boolValue = true;
             obj.nullValue = true;
-            serializer.serialize(obj);
 
-            log_debug("bindata testobject: " << cxxtools::hexDump(data.str()));
+            data << cxxtools::bin::Bin(obj);
+            log_debug("bindata testobject:\n" << cxxtools::hexDump(data.str()));
 
             TestObject obj2;
-            deserializer.deserialize(obj2);
+            data >> cxxtools::bin::Bin(obj2);
 
             CXXTOOLS_UNIT_ASSERT_EQUALS(obj.intValue, obj2.intValue);
             CXXTOOLS_UNIT_ASSERT_EQUALS(obj.stringValue, obj2.stringValue);
@@ -299,8 +367,6 @@ class BinSerializerTest : public cxxtools::unit::TestSuite
         void testComplexObject()
         {
             std::stringstream data;
-            cxxtools::bin::Serializer serializer(data);
-            cxxtools::bin::Deserializer deserializer(data);
 
             std::vector<TestObject2> v;
             TestObject2 obj;
@@ -319,12 +385,11 @@ class BinSerializerTest : public cxxtools::unit::TestSuite
             obj.setValue.insert(88);
             v.push_back(obj);
 
-            serializer.serialize(v);
-
-            log_debug("bindata complex object: " << cxxtools::hexDump(data.str()));
+            data << cxxtools::bin::Bin(v);
+            log_debug("bindata complex object:\n" << cxxtools::hexDump(data.str()));
 
             std::vector<TestObject2> v2;
-            deserializer.deserialize(v2);
+            data >> cxxtools::bin::Bin(v2);
 
             CXXTOOLS_UNIT_ASSERT(v == v2);
         }
@@ -332,8 +397,6 @@ class BinSerializerTest : public cxxtools::unit::TestSuite
         void testObjectVector()
         {
             std::stringstream data;
-            cxxtools::bin::Serializer serializer(data);
-            cxxtools::bin::Deserializer deserializer(data);
 
             std::vector<TestObject> obj;
             obj.resize(2);
@@ -348,10 +411,10 @@ class BinSerializerTest : public cxxtools::unit::TestSuite
             obj[1].boolValue = false;
             obj[1].nullValue = true;
 
-            serializer.serialize(obj);
+            data << cxxtools::bin::Bin(obj);
 
             std::vector<TestObject> obj2;
-            deserializer.deserialize(obj2);
+            data >> cxxtools::bin::Bin(obj2);
 
             CXXTOOLS_UNIT_ASSERT_EQUALS(obj2.size(), 2);
             CXXTOOLS_UNIT_ASSERT_EQUALS(obj[0].intValue, obj2[0].intValue);
@@ -370,36 +433,206 @@ class BinSerializerTest : public cxxtools::unit::TestSuite
         void testBinaryData()
         {
             std::stringstream data;
-            cxxtools::bin::Serializer serializer(data);
-            cxxtools::bin::Deserializer deserializer(data);
 
             std::string v;
             for (unsigned n = 0; n < 1024; ++n)
                 v.push_back(static_cast<char>(n));
 
-            serializer.serialize(v);
-            log_debug("v.data=" << cxxtools::hexDump(data.str()));
+            data << cxxtools::bin::Bin(v);
 
             std::string v2;
-            deserializer.deserialize(v2);
+            data >> cxxtools::bin::Bin(v2);
 
             CXXTOOLS_UNIT_ASSERT_EQUALS(v2.size(), 1024);
             CXXTOOLS_UNIT_ASSERT(v == v2);
 
-            data.str(std::string());
-            deserializer.clear();
-
             for (unsigned n = 0; n < 0xffff; ++n)
                 v.push_back(static_cast<char>(n));
 
-            serializer.serialize(v);
-            deserializer.deserialize(v2);
+            data << cxxtools::bin::Bin(v);
+            data >> cxxtools::bin::Bin(v2);
 
             CXXTOOLS_UNIT_ASSERT_EQUALS(v.size(), 0xffff + 1024);
             CXXTOOLS_UNIT_ASSERT_EQUALS(v2.size(), 0xffff + 1024);
             CXXTOOLS_UNIT_ASSERT(v == v2);
 
         }
+
+        void testReuse();
+        void testNamedVector();
+
+        void testDate()
+        {
+            std::stringstream data;
+
+            cxxtools::Date d(1966, 5, 3);
+            data << cxxtools::bin::Bin(d);
+            cxxtools::Date d2;
+            data >> cxxtools::bin::Bin(d2);
+
+            CXXTOOLS_UNIT_ASSERT(d2 == d);
+        }
+
+        void testTime()
+        {
+            std::stringstream data;
+
+            cxxtools::Time d(23, 5, 17, 345);
+            data << cxxtools::bin::Bin(d);
+            cxxtools::Time d2;
+            data >> cxxtools::bin::Bin(d2);
+
+            CXXTOOLS_UNIT_ASSERT(d2 == d);
+        }
+
+        void testDatetime()
+        {
+            std::stringstream data;
+
+            cxxtools::DateTime d(1966, 5, 3, 23, 5, 17, 345);
+            data << cxxtools::bin::Bin(d);
+            cxxtools::DateTime d2;
+            data >> cxxtools::bin::Bin(d2);
+
+            CXXTOOLS_UNIT_ASSERT(d2 == d);
+        }
+
+        template <typename TS>
+        static void testTs(TS t)
+        {
+            std::stringstream s;
+            s << cxxtools::bin::Bin(t);
+
+            TS ts;
+            s >> cxxtools::bin::Bin(ts);
+
+            CXXTOOLS_UNIT_ASSERT_EQUALS(t, ts);
+            log_info(s.str().size());
+        }
+
+        void testTimespan()
+        {
+            testTs(cxxtools::Microseconds(34));
+            testTs(cxxtools::Milliseconds(124));
+            testTs(cxxtools::Seconds(cxxtools::Microseconds(34565432)));
+            testTs(cxxtools::Seconds(3456));
+            testTs(cxxtools::Minutes(18));
+            testTs(cxxtools::Hours(67));
+        }
+
+        void testRawString()
+        {
+            {
+                std::string str = "FOObar";
+                std::stringstream ss;
+                ss << cxxtools::bin::Bin(str);
+
+                std::string binData = ss.str();
+
+                std::string raw = cxxtools::bin::Formatter::rawString(str);
+                CXXTOOLS_UNIT_ASSERT(binData.find(str) != std::string::npos);
+                CXXTOOLS_UNIT_ASSERT(binData.find(raw) != std::string::npos);
+            }
+            
+            {
+                std::string str = "123:45.";
+                std::stringstream ss;
+                ss << cxxtools::bin::Bin(str);
+
+                std::string binData = ss.str();
+
+                std::string raw = cxxtools::bin::Formatter::rawString(str);
+                CXXTOOLS_UNIT_ASSERT(binData.find(str) == std::string::npos);
+                CXXTOOLS_UNIT_ASSERT(binData.find(raw) != std::string::npos);
+            }
+        }
 };
 
+void BinSerializerTest::testReuse()
+{
+    cxxtools::bin::Deserializer d;
+
+    {
+        std::stringstream data;
+        cxxtools::SerializationInfo si;
+        si.addMember("foo") <<= "bar";
+        data << cxxtools::bin::Bin(si);
+
+        d.read(data);
+
+        cxxtools::SerializationInfo si2;
+        d.deserialize(si2);
+
+        std::string bar;
+        CXXTOOLS_UNIT_ASSERT_EQUALS(si2.memberCount(), 1);
+        CXXTOOLS_UNIT_ASSERT_NOTHROW(si2.getMember("foo") >>= bar);
+        CXXTOOLS_UNIT_ASSERT_EQUALS(bar, "bar");
+    }
+
+    {
+        std::stringstream data;
+        cxxtools::SerializationInfo si;
+        si.addMember("bar").addMember("bar") <<= 42;
+        data << cxxtools::bin::Bin(si);
+
+        d.read(data);
+
+        cxxtools::SerializationInfo si2;
+        d.deserialize(si2);
+
+        int bar;
+        CXXTOOLS_UNIT_ASSERT_EQUALS(si2.memberCount(), 1);
+        CXXTOOLS_UNIT_ASSERT_NOTHROW(si2.getMember("bar").getMember("bar") >>= bar);
+        CXXTOOLS_UNIT_ASSERT_EQUALS(bar, 42);
+    }
+
+}
+
+namespace foo
+{
+    typedef std::vector<int> FooVector;
+
+    void operator<<= (cxxtools::SerializationInfo& si, const FooVector& iv)
+    {
+        cxxtools::operator<<= (si, iv);
+        si.setTypeName("FooVector");
+    }
+
+    struct Foo
+    {
+        FooVector data;
+    };
+
+    void operator<<= (cxxtools::SerializationInfo& si, const Foo& foo)
+    {
+        si.addMember("data") <<= foo.data;
+        si.setTypeName("Foo");
+    }
+
+    void operator>>= (const cxxtools::SerializationInfo& si, Foo& foo)
+    {
+        si.getMember("data") >>= foo.data;
+    }
+}
+
+void BinSerializerTest::testNamedVector()
+{
+    foo::Foo f;
+    f.data.push_back(42);
+    f.data.push_back(12);
+
+    std::stringstream data;
+    data << cxxtools::bin::Bin(f);
+
+    log_debug("named foovector:\n" << cxxtools::hexDump(data.str()));
+
+    foo::Foo f2;
+    cxxtools::bin::Deserializer d(data);
+    d.deserialize(f2);
+
+    CXXTOOLS_UNIT_ASSERT_EQUALS(f2.data.size(), 2);
+    CXXTOOLS_UNIT_ASSERT_EQUALS(f2.data[0], 42);
+    CXXTOOLS_UNIT_ASSERT_EQUALS(f2.data[1], 12);
+}
+
 cxxtools::unit::RegisterTest<BinSerializerTest> register_BinSerializerTest;
