@@ -29,10 +29,9 @@
 #include "cxxtools/unit/testsuite.h"
 #include "cxxtools/unit/registertest.h"
 #include "cxxtools/serializationinfo.h"
-#include "cxxtools/jsonserializer.h"
-#include "cxxtools/jsondeserializer.h"
+#include "cxxtools/json.h"
 #include "cxxtools/log.h"
-#include "cxxtools/hdstream.h"
+#include "cxxtools/hexdump.h"
 #include <limits>
 #include <stdint.h>
 #include <config.h>
@@ -119,6 +118,7 @@ class JsonTest : public cxxtools::unit::TestSuite
             registerMethod("testScalar", *this, &JsonTest::testScalar);
             registerMethod("testInt", *this, &JsonTest::testInt);
             registerMethod("testDouble", *this, &JsonTest::testDouble);
+            registerMethod("testNan", *this, &JsonTest::testNan);
             registerMethod("testArray", *this, &JsonTest::testArray);
             registerMethod("testObject", *this, &JsonTest::testObject);
             registerMethod("testComplexObject", *this, &JsonTest::testComplexObject);
@@ -129,17 +129,14 @@ class JsonTest : public cxxtools::unit::TestSuite
         void testScalar()
         {
             std::stringstream data;
-            cxxtools::JsonSerializer serializer(data);
-            cxxtools::JsonDeserializer deserializer(data);
 
             int value = 5;
-            serializer.serialize(value);
-            serializer.finish();
+            data << cxxtools::Json(value);
 
             log_debug("scalar: " << data.str());
 
             int value2 = 0;
-            deserializer.deserialize(value2);
+            data >> cxxtools::Json(value2);
 
             CXXTOOLS_UNIT_ASSERT_EQUALS(value, value2);
         }
@@ -148,16 +145,12 @@ class JsonTest : public cxxtools::unit::TestSuite
         void testIntValue(IntT value)
         {
             std::stringstream data;
-            cxxtools::JsonSerializer serializer(data);
-            cxxtools::JsonDeserializer deserializer(data);
-
-            serializer.serialize(value);
-            serializer.finish();
 
+            data << cxxtools::Json(value);
             log_debug("int: " << data.str());
 
             IntT result = 0;
-            deserializer.deserialize(result);
+            data >> cxxtools::Json(result);
 
             CXXTOOLS_UNIT_ASSERT_EQUALS(value, result);
         }
@@ -200,14 +193,11 @@ class JsonTest : public cxxtools::unit::TestSuite
         void testDoubleValue(double value)
         {
             std::stringstream data;
-            cxxtools::JsonSerializer serializer(data);
-            cxxtools::JsonDeserializer deserializer(data);
 
-            serializer.serialize(value);
-            serializer.finish();
+            data << cxxtools::Json(value);
 
             double result = 0.0;
-            deserializer.deserialize(result);
+            data >> cxxtools::Json(result);
 
             log_debug("test double value " << value << " => " << result);
 
@@ -227,11 +217,28 @@ class JsonTest : public cxxtools::unit::TestSuite
             testDoubleValue(-3.877e-12);
         }
 
+        void testNan()
+        {
+            {
+                std::ostringstream data;
+                data << cxxtools::Json(std::numeric_limits<float>::quiet_NaN());
+                CXXTOOLS_UNIT_ASSERT_EQUALS(data.str(), "null");
+            }
+            {
+                std::ostringstream data;
+                data << cxxtools::Json(std::numeric_limits<double>::quiet_NaN());
+                CXXTOOLS_UNIT_ASSERT_EQUALS(data.str(), "null");
+            }
+            {
+                std::ostringstream data;
+                data << cxxtools::Json(std::numeric_limits<long double>::quiet_NaN());
+                CXXTOOLS_UNIT_ASSERT_EQUALS(data.str(), "null");
+            }
+        }
+
         void testArray()
         {
             std::stringstream data;
-            cxxtools::JsonSerializer serializer(data);
-            cxxtools::JsonDeserializer deserializer(data);
 
             std::vector<int> intvector;
             intvector.push_back(4711);
@@ -239,13 +246,11 @@ class JsonTest : public cxxtools::unit::TestSuite
             intvector.push_back(-3);
             intvector.push_back(-257);
 
-            serializer.serialize(intvector);
-            serializer.finish();
-
+            data << cxxtools::Json(intvector);
             log_debug("intvector: " << data.str());
 
             std::vector<int> intvector2;
-            deserializer.deserialize(intvector2);
+            data >> cxxtools::Json(intvector2);
 
             CXXTOOLS_UNIT_ASSERT_EQUALS(intvector.size(), intvector2.size());
             CXXTOOLS_UNIT_ASSERT_EQUALS(intvector[0], intvector2[0]);
@@ -257,8 +262,6 @@ class JsonTest : public cxxtools::unit::TestSuite
         void testObject()
         {
             std::stringstream data;
-            cxxtools::JsonSerializer serializer(data);
-            cxxtools::JsonDeserializer deserializer(data);
 
             TestObject obj;
             obj.intValue = 17;
@@ -266,11 +269,11 @@ class JsonTest : public cxxtools::unit::TestSuite
             obj.doubleValue = 3.125;
             obj.boolValue = true;
             obj.nullValue = true;
-            serializer.serialize(obj);
-            serializer.finish();
+
+            data << cxxtools::Json(obj);
 
             TestObject obj2;
-            deserializer.deserialize(obj2);
+            data >> cxxtools::Json(obj2);
 
             CXXTOOLS_UNIT_ASSERT_EQUALS(obj.intValue, obj2.intValue);
             CXXTOOLS_UNIT_ASSERT_EQUALS(obj.stringValue, obj2.stringValue);
@@ -283,8 +286,6 @@ class JsonTest : public cxxtools::unit::TestSuite
         void testComplexObject()
         {
             std::stringstream data;
-            cxxtools::JsonSerializer serializer(data);
-            cxxtools::JsonDeserializer deserializer(data);
 
             std::vector<TestObject2> v;
             TestObject2 obj;
@@ -303,11 +304,10 @@ class JsonTest : public cxxtools::unit::TestSuite
             obj.setValue.insert(88);
             v.push_back(obj);
 
-            serializer.serialize(v);
-            serializer.finish();
+            data << cxxtools::Json(v);
 
             std::vector<TestObject2> v2;
-            deserializer.deserialize(v2);
+            data >> cxxtools::Json(v2);
 
             CXXTOOLS_UNIT_ASSERT(v == v2);
         }
@@ -315,8 +315,6 @@ class JsonTest : public cxxtools::unit::TestSuite
         void testObjectVector()
         {
             std::stringstream data;
-            cxxtools::JsonSerializer serializer(data);
-            cxxtools::JsonDeserializer deserializer(data);
 
             std::vector<TestObject> obj;
             obj.resize(2);
@@ -331,11 +329,10 @@ class JsonTest : public cxxtools::unit::TestSuite
             obj[1].boolValue = false;
             obj[1].nullValue = true;
 
-            serializer.serialize(obj);
-            serializer.finish();
+            data << cxxtools::Json(obj);
 
             std::vector<TestObject> obj2;
-            deserializer.deserialize(obj2);
+            data >> cxxtools::Json(obj2);
 
             CXXTOOLS_UNIT_ASSERT_EQUALS(obj2.size(), 2);
             CXXTOOLS_UNIT_ASSERT_EQUALS(obj[0].intValue, obj2[0].intValue);
@@ -354,36 +351,29 @@ class JsonTest : public cxxtools::unit::TestSuite
         void testBinaryData()
         {
             std::stringstream data;
-            cxxtools::JsonSerializer serializer(data);
-            cxxtools::JsonDeserializer deserializer(data);
 
             std::string v;
             for (unsigned n = 0; n < 1024; ++n)
                 v.push_back(static_cast<char>(n));
 
-            serializer.serialize(v);
-            serializer.finish();
-
-            log_debug("v.data=" << cxxtools::hexDump(data.str()));
+            data << cxxtools::Json(v);
+            log_debug("v.data:\n" << cxxtools::hexDump(data.str()));
 
             std::string v2;
-            deserializer.deserialize(v2);
+            data >> cxxtools::Json(v2);
 
             CXXTOOLS_UNIT_ASSERT_EQUALS(v2.size(), 1024);
             CXXTOOLS_UNIT_ASSERT(v == v2);
 
             data.str(std::string());
-            deserializer.clear();
 
             for (unsigned n = 0; n < 0xffff; ++n)
                 v.push_back(static_cast<char>(n));
 
-            serializer.serialize(v);
-            serializer.finish();
+            data << cxxtools::Json(v);
+            log_debug("v.data:\n" << cxxtools::hexDump(data.str()));
 
-            log_debug("v.data=" << cxxtools::hexDump(data.str()));
-
-            deserializer.deserialize(v2);
+            data >> cxxtools::Json(v2);
 
             CXXTOOLS_UNIT_ASSERT_EQUALS(v2.size(), 0xffff + 1024);
             CXXTOOLS_UNIT_ASSERT(v == v2);
