@@ -33,6 +33,9 @@
 #include "cxxtools/remoteprocedure.h"
 #include "cxxtools/eventloop.h"
 #include "cxxtools/log.h"
+#include "cxxtools/ioerror.h"
+#include "cxxtools/net/uri.h"
+#include "cxxtools/net/addrinfo.h"
 #include <stdlib.h>
 #include <sstream>
 
@@ -77,6 +80,7 @@ class JsonRpcTest : public cxxtools::unit::TestSuite
         cxxtools::EventLoop _loop;
         cxxtools::json::RpcServer* _server;
         unsigned _count;
+        std::string _listen;
         unsigned short _port;
 
     public:
@@ -104,6 +108,9 @@ class JsonRpcTest : public cxxtools::unit::TestSuite
             registerMethod("CallbackException", *this, &JsonRpcTest::CallbackException);
             registerMethod("ConnectError", *this, &JsonRpcTest::ConnectError);
             registerMethod("BigRequest", *this, &JsonRpcTest::BigRequest);
+            registerMethod("PrepareConnect", *this, &JsonRpcTest::PrepareConnect);
+            registerMethod("Connect", *this, &JsonRpcTest::Connect);
+            registerMethod("Multiple", *this, &JsonRpcTest::Multiple);
 
             char* PORT = getenv("UTEST_PORT");
             if (PORT)
@@ -111,6 +118,10 @@ class JsonRpcTest : public cxxtools::unit::TestSuite
                 std::istringstream s(PORT);
                 s >> _port;
             }
+
+            char* LISTEN = getenv("UTEST_LISTEN");
+            if (LISTEN)
+                _listen = LISTEN;
         }
 
         void failTest()
@@ -124,7 +135,7 @@ class JsonRpcTest : public cxxtools::unit::TestSuite
             connect(_loop.timeout, *this, &JsonRpcTest::failTest);
             connect(_loop.timeout, _loop, &cxxtools::EventLoop::exit);
 
-            _server = new cxxtools::json::RpcServer(_loop, _port);
+            _server = new cxxtools::json::RpcServer(_loop, _listen, _port);
             _server->minThreads(1);
         }
 
@@ -140,7 +151,7 @@ class JsonRpcTest : public cxxtools::unit::TestSuite
         {
             _server->registerMethod("multiply", *this, &JsonRpcTest::multiplyNothing);
 
-            cxxtools::json::RpcClient client(_loop, "", _port);
+            cxxtools::json::RpcClient client(_loop, _listen, _port);
             cxxtools::RemoteProcedure<bool> multiply(client, "multiply");
 
             multiply.begin();
@@ -159,7 +170,7 @@ class JsonRpcTest : public cxxtools::unit::TestSuite
         {
             _server->registerMethod("multiply", *this, &JsonRpcTest::multiplyNothing);
 
-            cxxtools::json::RpcClient client(_loop, "", _port);
+            cxxtools::json::RpcClient client(_loop, _listen, _port);
             cxxtools::RemoteProcedure<bool> multiply(client, "multiply");
             connect( multiply.finished, *this, &JsonRpcTest::onExceptionCallback );
 
@@ -170,7 +181,7 @@ class JsonRpcTest : public cxxtools::unit::TestSuite
             CXXTOOLS_UNIT_ASSERT_EQUALS(_count, 1);
         }
 
-        void onExceptionCallback(const cxxtools::RemoteResult<bool>& r)
+        void onExceptionCallback(cxxtools::RemoteResult<bool>& /*r*/)
         {
             log_warn("exception callback");
             ++_count;
@@ -185,7 +196,7 @@ class JsonRpcTest : public cxxtools::unit::TestSuite
         {
             log_trace("ConnectError");
 
-            cxxtools::json::RpcClient client(_loop, "", _port + 1);
+            cxxtools::json::RpcClient client(_loop, _listen, _port + 1);
             cxxtools::RemoteProcedure<bool> multiply(client, "multiply");
             connect( multiply.finished, *this, &JsonRpcTest::onConnectErrorCallback );
 
@@ -202,7 +213,7 @@ class JsonRpcTest : public cxxtools::unit::TestSuite
             }
         }
 
-        void onConnectErrorCallback(const cxxtools::RemoteResult<bool>& r)
+        void onConnectErrorCallback(cxxtools::RemoteResult<bool>& r)
         {
             log_debug("onConnectErrorCallback");
             _loop.exit();
@@ -216,7 +227,7 @@ class JsonRpcTest : public cxxtools::unit::TestSuite
         {
             _server->registerMethod("boolean", *this, &JsonRpcTest::boolean);
 
-            cxxtools::json::RpcClient client(_loop, "", _port);
+            cxxtools::json::RpcClient client(_loop, _listen, _port);
             cxxtools::RemoteProcedure<bool, bool, bool> multiply(client, "boolean");
 
             multiply.begin(true, true);
@@ -237,7 +248,7 @@ class JsonRpcTest : public cxxtools::unit::TestSuite
         {
             _server->registerMethod("multiply", *this, &JsonRpcTest::multiplyInt);
 
-            cxxtools::json::RpcClient client(_loop, "", _port);
+            cxxtools::json::RpcClient client(_loop, _listen, _port);
             cxxtools::RemoteProcedure<int, int, int> multiply(client, "multiply");
 
             multiply.begin(2, 3);
@@ -256,7 +267,7 @@ class JsonRpcTest : public cxxtools::unit::TestSuite
         {
             _server->registerMethod("multiply", *this, &JsonRpcTest::multiplyDouble);
 
-            cxxtools::json::RpcClient client(_loop, "", _port);
+            cxxtools::json::RpcClient client(_loop, _listen, _port);
             cxxtools::RemoteProcedure<double, double, double> multiply(client, "multiply");
 
             multiply.begin(2.0, 3.0);
@@ -275,7 +286,7 @@ class JsonRpcTest : public cxxtools::unit::TestSuite
         {
             _server->registerMethod("echoString", *this, &JsonRpcTest::echoString);
 
-            cxxtools::json::RpcClient client(_loop, "", _port);
+            cxxtools::json::RpcClient client(_loop, _listen, _port);
             cxxtools::RemoteProcedure<std::string, std::string> echo(client, "echoString");
 
             echo.begin("\xc3\xaf\xc2\xbb\xc2\xbf'\"&<> foo?");
@@ -294,7 +305,7 @@ class JsonRpcTest : public cxxtools::unit::TestSuite
         {
             _server->registerMethod("multiply", *this, &JsonRpcTest::multiplyEmpty);
 
-            cxxtools::json::RpcClient client(_loop, "", _port);
+            cxxtools::json::RpcClient client(_loop, _listen, _port);
             cxxtools::RemoteProcedure<std::string, std::string, std::string> multiply(client, "multiply");
 
             multiply.begin("", "");
@@ -315,7 +326,7 @@ class JsonRpcTest : public cxxtools::unit::TestSuite
         {
             _server->registerMethod("multiply", *this, &JsonRpcTest::multiplyVector);
 
-            cxxtools::json::RpcClient client(_loop, "", _port);
+            cxxtools::json::RpcClient client(_loop, _listen, _port);
             cxxtools::RemoteProcedure< std::vector<int>, std::vector<int>, std::vector<int> > multiply(client, "multiply");
 
             std::vector<int> vec;
@@ -348,7 +359,7 @@ class JsonRpcTest : public cxxtools::unit::TestSuite
         {
             _server->registerMethod("multiply", *this, &JsonRpcTest::multiplyVector);
 
-            cxxtools::json::RpcClient client(_loop, "", _port);
+            cxxtools::json::RpcClient client(_loop, _listen, _port);
             cxxtools::RemoteProcedure< std::vector<int>, std::vector<int>, std::vector<int> > multiply(client, "multiply");
 
             std::vector<int> vec;
@@ -363,7 +374,7 @@ class JsonRpcTest : public cxxtools::unit::TestSuite
         {
             _server->registerMethod("multiply", *this, &JsonRpcTest::multiplyColor);
 
-            cxxtools::json::RpcClient client(_loop, "", _port);
+            cxxtools::json::RpcClient client(_loop, _listen, _port);
             cxxtools::RemoteProcedure< Color, Color, Color > multiply(client, "multiply");
 
             Color a;
@@ -399,7 +410,7 @@ class JsonRpcTest : public cxxtools::unit::TestSuite
         {
             _server->registerMethod("multiplyset", *this, &JsonRpcTest::multiplySet);
 
-            cxxtools::json::RpcClient client(_loop, "", _port);
+            cxxtools::json::RpcClient client(_loop, _listen, _port);
             cxxtools::RemoteProcedure<IntSet, IntSet, int> multiply(client, "multiplyset");
 
             IntSet myset;
@@ -431,7 +442,7 @@ class JsonRpcTest : public cxxtools::unit::TestSuite
         {
             _server->registerMethod("multiplyset", *this, &JsonRpcTest::multiplyMultiset);
 
-            cxxtools::json::RpcClient client(_loop, "", _port);
+            cxxtools::json::RpcClient client(_loop, _listen, _port);
             cxxtools::RemoteProcedure<IntMultiset, IntMultiset, int> multiply(client, "multiplyset");
 
             IntMultiset myset;
@@ -463,7 +474,7 @@ class JsonRpcTest : public cxxtools::unit::TestSuite
         {
             _server->registerMethod("multiplymap", *this, &JsonRpcTest::multiplyMap);
 
-            cxxtools::json::RpcClient client(_loop, "", _port);
+            cxxtools::json::RpcClient client(_loop, _listen, _port);
             cxxtools::RemoteProcedure<IntMap, IntMap, int> multiply(client, "multiplymap");
 
             IntMap mymap;
@@ -501,7 +512,7 @@ class JsonRpcTest : public cxxtools::unit::TestSuite
         {
             _server->registerMethod("multiplymultimap", *this, &JsonRpcTest::multiplyMultimap);
 
-            cxxtools::json::RpcClient client(_loop, "", _port);
+            cxxtools::json::RpcClient client(_loop, _listen, _port);
             cxxtools::RemoteProcedure<IntMultimap, IntMultimap, int> multiply(client, "multiplymultimap");
 
             IntMultimap mymap;
@@ -545,7 +556,7 @@ class JsonRpcTest : public cxxtools::unit::TestSuite
         {
             _server->registerMethod("somePrefix.multiply", *this, &JsonRpcTest::multiplyInt);
 
-            cxxtools::json::RpcClient client(_loop, "", _port);
+            cxxtools::json::RpcClient client(_loop, _listen, _port);
             client.prefix("somePrefix.");
             cxxtools::RemoteProcedure<int, int, int> multiply(client, "multiply");
 
@@ -555,7 +566,7 @@ class JsonRpcTest : public cxxtools::unit::TestSuite
 
         void UnknownMethod()
         {
-            cxxtools::json::RpcClient client(_loop, "", _port);
+            cxxtools::json::RpcClient client(_loop, _listen, _port);
             cxxtools::RemoteProcedure<bool> unknownMethod(client, "unknownMethod");
 
             unknownMethod.begin();
@@ -570,7 +581,7 @@ class JsonRpcTest : public cxxtools::unit::TestSuite
         {
             _server->registerMethod("multiply", *this, &JsonRpcTest::throwFault);
 
-            cxxtools::json::RpcClient client(_loop, "", _port);
+            cxxtools::json::RpcClient client(_loop, _listen, _port);
             cxxtools::RemoteProcedure<bool> multiply(client, "multiply");
             multiply.begin();
 
@@ -599,7 +610,7 @@ class JsonRpcTest : public cxxtools::unit::TestSuite
         {
             _server->registerMethod("multiply", *this, &JsonRpcTest::throwException);
 
-            cxxtools::json::RpcClient client(_loop, "", _port);
+            cxxtools::json::RpcClient client(_loop, _listen, _port);
             cxxtools::RemoteProcedure<bool> multiply(client, "multiply");
 
             multiply.begin();
@@ -611,7 +622,6 @@ class JsonRpcTest : public cxxtools::unit::TestSuite
             }
             catch (const cxxtools::RemoteException& e)
             {
-                CXXTOOLS_UNIT_ASSERT_EQUALS(e.rc(), 0);
                 CXXTOOLS_UNIT_ASSERT_EQUALS(e.text(), "Exception");
             }
         }
@@ -627,11 +637,9 @@ class JsonRpcTest : public cxxtools::unit::TestSuite
         //
         void BigRequest()
         {
-            log_trace("ConnectError");
-
             _server->registerMethod("countSize", *this, &JsonRpcTest::countSize);
 
-            cxxtools::json::RpcClient client(_loop, "", _port);
+            cxxtools::json::RpcClient client(_loop, _listen, _port);
             cxxtools::RemoteProcedure<unsigned, std::vector<int> > countSize(client, "countSize");
 
             std::vector<int> v;
@@ -655,6 +663,146 @@ class JsonRpcTest : public cxxtools::unit::TestSuite
             return v.size();
         }
 
+        ////////////////////////////////////////////////////////////
+        // PrepareConnect
+        //
+        void PrepareConnect()
+        {
+            log_trace("PrepareConnect");
+
+            _server->registerMethod("boolean", *this, &JsonRpcTest::boolean);
+
+            // test connect using cxxtools::net::AddrInfo
+            {
+                cxxtools::json::RpcClient client(_loop);
+                cxxtools::RemoteProcedure<bool, bool, bool> boolean(client, "boolean");
+
+                client.prepareConnect(cxxtools::net::AddrInfo(_listen, _port));
+                boolean.begin(true, true);
+                CXXTOOLS_UNIT_ASSERT_EQUALS(boolean.end(2000), true);
+            }
+
+            // test connect using host and port
+            {
+                cxxtools::json::RpcClient client(_loop);
+                cxxtools::RemoteProcedure<bool, bool, bool> boolean(client, "boolean");
+
+                client.prepareConnect(_listen, _port);
+                boolean.begin(true, true);
+                CXXTOOLS_UNIT_ASSERT_EQUALS(boolean.end(2000), true);
+            }
+
+            // test connect using uri
+            {
+                cxxtools::json::RpcClient client(_loop);
+                cxxtools::RemoteProcedure<bool, bool, bool> boolean(client, "boolean");
+
+                std::ostringstream uri;
+                uri << "json://localhost:" << _port << '/';
+                client.prepareConnect(cxxtools::net::Uri(uri.str()));
+                boolean.begin(true, true);
+                CXXTOOLS_UNIT_ASSERT_EQUALS(boolean.end(2000), true);
+            }
+
+            // test failing connect in connect
+            {
+                cxxtools::json::RpcClient client(_loop);
+                cxxtools::RemoteProcedure<bool, bool, bool> boolean(client, "boolean");
+
+                CXXTOOLS_UNIT_ASSERT_NOTHROW(client.prepareConnect(_listen, _port + 1));
+                CXXTOOLS_UNIT_ASSERT_THROW(client.connect(), cxxtools::IOError);
+            }
+
+            // test failing connect when calling function
+            {
+                cxxtools::json::RpcClient client(_loop);
+                cxxtools::RemoteProcedure<bool, bool, bool> boolean(client, "boolean");
+                CXXTOOLS_UNIT_ASSERT_NOTHROW(client.prepareConnect(_listen, _port + 1));
+
+                boolean.begin(true, true);
+                CXXTOOLS_UNIT_ASSERT_THROW(boolean.end(2000), cxxtools::IOError);
+            }
+
+        }
+
+        ////////////////////////////////////////////////////////////
+        // Connect
+        //
+        void Connect()
+        {
+            _server->registerMethod("boolean", *this, &JsonRpcTest::boolean);
+
+            // test connect using cxxtools::net::AddrInfo
+            {
+                cxxtools::json::RpcClient client(_loop);
+                cxxtools::RemoteProcedure<bool, bool, bool> boolean(client, "boolean");
+
+                client.connect(cxxtools::net::AddrInfo(_listen, _port));
+                boolean.begin(true, true);
+                CXXTOOLS_UNIT_ASSERT_EQUALS(boolean.end(2000), true);
+            }
+
+            // test connect using host and port
+            {
+                cxxtools::json::RpcClient client(_loop);
+                cxxtools::RemoteProcedure<bool, bool, bool> boolean(client, "boolean");
+
+                client.connect(_listen, _port);
+                boolean.begin(true, true);
+                CXXTOOLS_UNIT_ASSERT_EQUALS(boolean.end(2000), true);
+            }
+
+            // test connect using uri
+            {
+                cxxtools::json::RpcClient client(_loop);
+                cxxtools::RemoteProcedure<bool, bool, bool> boolean(client, "boolean");
+
+                std::ostringstream uri;
+                uri << "json://localhost:" << _port << '/';
+                client.connect(cxxtools::net::Uri(uri.str()));
+                boolean.begin(true, true);
+                CXXTOOLS_UNIT_ASSERT_EQUALS(boolean.end(2000), true);
+            }
+
+            // test failing connect
+            {
+                cxxtools::json::RpcClient client(_loop);
+                cxxtools::RemoteProcedure<bool, bool, bool> boolean(client, "boolean");
+
+                CXXTOOLS_UNIT_ASSERT_THROW(client.connect(_listen, _port + 1), cxxtools::IOError);
+            }
+
+        }
+
+        ////////////////////////////////////////////////////////////
+        // Multiple calls
+        //
+        void Multiple()
+        {
+            _server->registerMethod("multiply", *this, &JsonRpcTest::multiplyDouble);
+
+            typedef cxxtools::RemoteProcedure<double, double, double> Multiply;
+
+            std::vector<cxxtools::json::RpcClient> clients;
+            std::vector<Multiply> procs;
+
+            clients.reserve(16);
+            procs.reserve(16);
+
+            for (unsigned i = 0; i < 16; ++i)
+            {
+                clients.push_back(cxxtools::json::RpcClient(_loop, _listen, _port));
+                procs.push_back(Multiply(clients.back(), "multiply"));
+                procs.back().begin(i, i);
+            }
+
+            for (unsigned i = 0; i < 16; ++i)
+            {
+                CXXTOOLS_UNIT_ASSERT_EQUALS(procs[i].end(2000), i*i);
+            }
+
+        }
+
 };
 
 cxxtools::unit::RegisterTest<JsonRpcTest> register_JsonRpcTest;
