@@ -34,6 +34,9 @@
 #include "cxxtools/http/server.h"
 #include "cxxtools/eventloop.h"
 #include "cxxtools/log.h"
+#include "cxxtools/ioerror.h"
+#include "cxxtools/net/uri.h"
+#include "cxxtools/net/addrinfo.h"
 #include <stdlib.h>
 #include <sstream>
 
@@ -79,6 +82,7 @@ class JsonRpcHttpTest : public cxxtools::unit::TestSuite
         cxxtools::EventLoop _loop;
         cxxtools::http::Server* _server;
         unsigned _count;
+        std::string _listen;
         unsigned short _port;
 
     public:
@@ -104,6 +108,10 @@ class JsonRpcHttpTest : public cxxtools::unit::TestSuite
             registerMethod("Exception", *this, &JsonRpcHttpTest::Exception);
             registerMethod("CallbackException", *this, &JsonRpcHttpTest::CallbackException);
             registerMethod("ConnectError", *this, &JsonRpcHttpTest::ConnectError);
+            registerMethod("BigRequest", *this, &JsonRpcHttpTest::BigRequest);
+            registerMethod("PrepareConnect", *this, &JsonRpcHttpTest::PrepareConnect);
+            registerMethod("Connect", *this, &JsonRpcHttpTest::Connect);
+            registerMethod("Multiple", *this, &JsonRpcHttpTest::Multiple);
 
             char* PORT = getenv("UTEST_PORT");
             if (PORT)
@@ -112,6 +120,10 @@ class JsonRpcHttpTest : public cxxtools::unit::TestSuite
                 s >> _port;
             }
 
+            char* LISTEN = getenv("UTEST_LISTEN");
+            if (LISTEN)
+                _listen = LISTEN;
+
             _loop.setIdleTimeout(2000);
             connect(_loop.timeout, *this, &JsonRpcHttpTest::failTest);
             connect(_loop.timeout, _loop, &cxxtools::EventLoop::exit);
@@ -125,7 +137,7 @@ class JsonRpcHttpTest : public cxxtools::unit::TestSuite
 
         void setUp()
         {
-            _server = new cxxtools::http::Server(_loop, _port);
+            _server = new cxxtools::http::Server(_loop, _listen, _port);
             _server->minThreads(1);
         }
 
@@ -143,7 +155,7 @@ class JsonRpcHttpTest : public cxxtools::unit::TestSuite
             service.registerMethod("multiply", *this, &JsonRpcHttpTest::multiplyNothing);
             _server->addService("/calc", service);
 
-            cxxtools::json::HttpClient client(_loop, "", _port, "/calc");
+            cxxtools::json::HttpClient client(_loop, _listen, _port, "/calc");
             cxxtools::RemoteProcedure<bool> multiply(client, "multiply");
 
             multiply.begin();
@@ -164,7 +176,7 @@ class JsonRpcHttpTest : public cxxtools::unit::TestSuite
             service.registerMethod("multiply", *this, &JsonRpcHttpTest::multiplyNothing);
             _server->addService("/calc", service);
 
-            cxxtools::json::HttpClient client(_loop, "", _port, "/calc");
+            cxxtools::json::HttpClient client(_loop, _listen, _port, "/calc");
             cxxtools::RemoteProcedure<bool> multiply(client, "multiply");
             connect( multiply.finished, *this, &JsonRpcHttpTest::onExceptionCallback );
 
@@ -175,7 +187,7 @@ class JsonRpcHttpTest : public cxxtools::unit::TestSuite
             CXXTOOLS_UNIT_ASSERT_EQUALS(_count, 1);
         }
 
-        void onExceptionCallback(const cxxtools::RemoteResult<bool>& r)
+        void onExceptionCallback(cxxtools::RemoteResult<bool>& /*r*/)
         {
             log_warn("exception callback");
             ++_count;
@@ -190,7 +202,7 @@ class JsonRpcHttpTest : public cxxtools::unit::TestSuite
         {
             log_trace("ConnectError");
 
-            cxxtools::json::HttpClient client(_loop, "", _port + 1, "/calc");
+            cxxtools::json::HttpClient client(_loop, _listen, _port + 1, "/calc");
             cxxtools::RemoteProcedure<bool> multiply(client, "multiply");
             connect( multiply.finished, *this, &JsonRpcHttpTest::onConnectErrorCallback );
 
@@ -207,7 +219,7 @@ class JsonRpcHttpTest : public cxxtools::unit::TestSuite
             }
         }
 
-        void onConnectErrorCallback(const cxxtools::RemoteResult<bool>& r)
+        void onConnectErrorCallback(cxxtools::RemoteResult<bool>& r)
         {
             log_debug("onConnectErrorCallback");
             _loop.exit();
@@ -220,17 +232,17 @@ class JsonRpcHttpTest : public cxxtools::unit::TestSuite
         void Boolean()
         {
             cxxtools::json::HttpService service;
-            service.registerMethod("multiply", *this, &JsonRpcHttpTest::multiplyBoolean);
+            service.registerMethod("boolean", *this, &JsonRpcHttpTest::boolean);
             _server->addService("/calc", service);
 
-            cxxtools::json::HttpClient client(_loop, "", _port, "/calc");
-            cxxtools::RemoteProcedure<bool, bool, bool> multiply(client, "multiply");
-            multiply.begin(true, true);
-            bool r = multiply.end(2000);
+            cxxtools::json::HttpClient client(_loop, _listen, _port, "/calc");
+            cxxtools::RemoteProcedure<bool, bool, bool> boolean(client, "boolean");
+            boolean.begin(true, true);
+            bool r = boolean.end(2000);
             CXXTOOLS_UNIT_ASSERT_EQUALS(r, true);
         }
 
-        bool multiplyBoolean(bool a, bool b)
+        bool boolean(bool a, bool b)
         {
             CXXTOOLS_UNIT_ASSERT_EQUALS(a, true);
             CXXTOOLS_UNIT_ASSERT_EQUALS(b, true);
@@ -246,7 +258,7 @@ class JsonRpcHttpTest : public cxxtools::unit::TestSuite
             service.registerMethod("multiply", *this, &JsonRpcHttpTest::multiplyInt);
             _server->addService("/calc", service);
 
-            cxxtools::json::HttpClient client(_loop, "", _port, "/calc");
+            cxxtools::json::HttpClient client(_loop, _listen, _port, "/calc");
             cxxtools::RemoteProcedure<int, int, int> multiply(client, "multiply");
 
             multiply.begin(2, 3);
@@ -267,7 +279,7 @@ class JsonRpcHttpTest : public cxxtools::unit::TestSuite
             service.registerMethod("multiply", *this, &JsonRpcHttpTest::multiplyDouble);
             _server->addService("/calc", service);
 
-            cxxtools::json::HttpClient client(_loop, "", _port, "/calc");
+            cxxtools::json::HttpClient client(_loop, _listen, _port, "/calc");
             cxxtools::RemoteProcedure<double, double, double> multiply(client, "multiply");
 
             multiply.begin(2.0, 3.0);
@@ -288,7 +300,7 @@ class JsonRpcHttpTest : public cxxtools::unit::TestSuite
             service.registerMethod("echoString", *this, &JsonRpcHttpTest::echoString);
             _server->addService("/foo", service);
 
-            cxxtools::json::HttpClient client(_loop, "", _port, "/foo");
+            cxxtools::json::HttpClient client(_loop, _listen, _port, "/foo");
             cxxtools::RemoteProcedure<std::string, std::string> echo(client, "echoString");
 
             echo.begin("\xc3\xaf\xc2\xbb\xc2\xbf'\"&<> foo?");
@@ -309,7 +321,7 @@ class JsonRpcHttpTest : public cxxtools::unit::TestSuite
             service.registerMethod("multiply", *this, &JsonRpcHttpTest::multiplyEmpty);
             _server->addService("/calc", service);
 
-            cxxtools::json::HttpClient client(_loop, "", _port, "/calc");
+            cxxtools::json::HttpClient client(_loop, _listen, _port, "/calc");
             cxxtools::RemoteProcedure<std::string, std::string, std::string> multiply(client, "multiply");
 
             multiply.begin("", "");
@@ -332,7 +344,7 @@ class JsonRpcHttpTest : public cxxtools::unit::TestSuite
             service.registerMethod("multiply", *this, &JsonRpcHttpTest::multiplyVector);
             _server->addService("/calc", service);
 
-            cxxtools::json::HttpClient client(_loop, "", _port, "/calc");
+            cxxtools::json::HttpClient client(_loop, _listen, _port, "/calc");
             cxxtools::RemoteProcedure< std::vector<int>, std::vector<int>, std::vector<int> > multiply(client, "multiply");
 
             std::vector<int> vec;
@@ -367,7 +379,7 @@ class JsonRpcHttpTest : public cxxtools::unit::TestSuite
             service.registerMethod("multiply", *this, &JsonRpcHttpTest::multiplyVector);
             _server->addService("/calc", service);
 
-            cxxtools::json::HttpClient client(_loop, "", _port, "/calc");
+            cxxtools::json::HttpClient client(_loop, _listen, _port, "/calc");
             cxxtools::RemoteProcedure< std::vector<int>, std::vector<int>, std::vector<int> > multiply(client, "multiply");
 
             std::vector<int> vec;
@@ -384,7 +396,7 @@ class JsonRpcHttpTest : public cxxtools::unit::TestSuite
             service.registerMethod("multiply", *this, &JsonRpcHttpTest::multiplyColor);
             _server->addService("/calc", service);
 
-            cxxtools::json::HttpClient client(_loop, "", _port, "/calc");
+            cxxtools::json::HttpClient client(_loop, _listen, _port, "/calc");
             cxxtools::RemoteProcedure< Color, Color, Color > multiply(client, "multiply");
 
             Color a;
@@ -422,7 +434,7 @@ class JsonRpcHttpTest : public cxxtools::unit::TestSuite
             service.registerMethod("multiplyset", *this, &JsonRpcHttpTest::multiplySet);
             _server->addService("/test", service);
 
-            cxxtools::json::HttpClient client(_loop, "", _port, "/test");
+            cxxtools::json::HttpClient client(_loop, _listen, _port, "/test");
             cxxtools::RemoteProcedure<IntSet, IntSet, int> multiply(client, "multiplyset");
 
             IntSet myset;
@@ -456,7 +468,7 @@ class JsonRpcHttpTest : public cxxtools::unit::TestSuite
             service.registerMethod("multiplyset", *this, &JsonRpcHttpTest::multiplyMultiset);
             _server->addService("/test", service);
 
-            cxxtools::json::HttpClient client(_loop, "", _port, "/test");
+            cxxtools::json::HttpClient client(_loop, _listen, _port, "/test");
             cxxtools::RemoteProcedure<IntMultiset, IntMultiset, int> multiply(client, "multiplyset");
 
             IntMultiset myset;
@@ -491,7 +503,7 @@ class JsonRpcHttpTest : public cxxtools::unit::TestSuite
 
             _server->addService("/test", service);
 
-            cxxtools::json::HttpClient client(_loop, "", _port, "/test");
+            cxxtools::json::HttpClient client(_loop, _listen, _port, "/test");
             cxxtools::RemoteProcedure<IntMap, IntMap, int> multiply(client, "multiplymap");
 
             IntMap mymap;
@@ -532,7 +544,7 @@ class JsonRpcHttpTest : public cxxtools::unit::TestSuite
 
             _server->addService("/test", service);
 
-            cxxtools::json::HttpClient client(_loop, "", _port, "/test");
+            cxxtools::json::HttpClient client(_loop, _listen, _port, "/test");
             cxxtools::RemoteProcedure<IntMultimap, IntMultimap, int> multiply(client, "multiplymultimap");
 
             IntMultimap mymap;
@@ -574,7 +586,7 @@ class JsonRpcHttpTest : public cxxtools::unit::TestSuite
         //
         void UnknownMethod()
         {
-            cxxtools::json::HttpClient client(_loop, "", _port, "/test");
+            cxxtools::json::HttpClient client(_loop, _listen, _port, "/test");
             cxxtools::json::HttpService service;
             _server->addService("/test", service);
             cxxtools::RemoteProcedure<bool, bool, bool> unknownMethod(client, "unknownMethod");
@@ -591,7 +603,7 @@ class JsonRpcHttpTest : public cxxtools::unit::TestSuite
             service.registerMethod("multiply", *this, &JsonRpcHttpTest::throwFault);
             _server->addService("/calc", service);
 
-            cxxtools::json::HttpClient client(_loop, "", _port, "/calc");
+            cxxtools::json::HttpClient client(_loop, _listen, _port, "/calc");
             cxxtools::RemoteProcedure<bool> multiply(client, "multiply");
             multiply.begin();
 
@@ -622,7 +634,7 @@ class JsonRpcHttpTest : public cxxtools::unit::TestSuite
             service.registerMethod("multiply", *this, &JsonRpcHttpTest::throwException);
             _server->addService("/calc", service);
 
-            cxxtools::json::HttpClient client(_loop, "", _port, "/calc");
+            cxxtools::json::HttpClient client(_loop, _listen, _port, "/calc");
             cxxtools::RemoteProcedure<bool> multiply(client, "multiply");
 
             multiply.begin();
@@ -634,7 +646,6 @@ class JsonRpcHttpTest : public cxxtools::unit::TestSuite
             }
             catch (const cxxtools::RemoteException& e)
             {
-                CXXTOOLS_UNIT_ASSERT_EQUALS(e.rc(), 0);
                 CXXTOOLS_UNIT_ASSERT_EQUALS(e.text(), "Exception");
             }
         }
@@ -645,6 +656,185 @@ class JsonRpcHttpTest : public cxxtools::unit::TestSuite
             return false;
         }
 
+        ////////////////////////////////////////////////////////////
+        // BigRequest
+        //
+        void BigRequest()
+        {
+            cxxtools::json::HttpService service;
+            service.registerMethod("countSize", *this, &JsonRpcHttpTest::countSize);
+            _server->addService("/rpc", service);
+
+            cxxtools::json::HttpClient client(_loop, _listen, _port, "/rpc");
+            cxxtools::RemoteProcedure<unsigned, std::vector<int> > countSize(client, "countSize");
+
+            std::vector<int> v;
+            v.resize(5000);
+
+            countSize.begin(v);
+
+            try
+            {
+                CXXTOOLS_UNIT_ASSERT_EQUALS(countSize.end(2000), 5000);
+            }
+            catch (const std::exception& e)
+            {
+                log_error("loop exited with exception: " << e.what());
+                CXXTOOLS_UNIT_ASSERT_MSG(false, std::string("unexpected exception ") + typeid(e).name() + ": " + e.what());
+            }
+        }
+
+        unsigned countSize(const std::vector<int>& v)
+        {
+            return v.size();
+        }
+
+        ////////////////////////////////////////////////////////////
+        // PrepareConnect
+        //
+        void PrepareConnect()
+        {
+            log_trace("PrepareConnect");
+
+            cxxtools::json::HttpService service;
+            service.registerMethod("boolean", *this, &JsonRpcHttpTest::boolean);
+            _server->addService("/rpc", service);
+
+            // test connect using cxxtools::net::AddrInfo
+            {
+                cxxtools::json::HttpClient client(_loop);
+                cxxtools::RemoteProcedure<bool, bool, bool> boolean(client, "boolean");
+
+                client.prepareConnect(cxxtools::net::AddrInfo(_listen, _port), "/rpc");
+                boolean.begin(true, true);
+                CXXTOOLS_UNIT_ASSERT_EQUALS(boolean.end(2000), true);
+            }
+
+            // test connect using host and port
+            {
+                cxxtools::json::HttpClient client(_loop);
+                cxxtools::RemoteProcedure<bool, bool, bool> boolean(client, "boolean");
+
+                client.prepareConnect(_listen, _port, "/rpc");
+                boolean.begin(true, true);
+                CXXTOOLS_UNIT_ASSERT_EQUALS(boolean.end(2000), true);
+            }
+
+            // test connect using uri
+            {
+                cxxtools::json::HttpClient client(_loop);
+                cxxtools::RemoteProcedure<bool, bool, bool> boolean(client, "boolean");
+
+                std::ostringstream uri;
+                uri << "http://localhost:" << _port << "/rpc";
+                client.prepareConnect(cxxtools::net::Uri(uri.str()));
+                boolean.begin(true, true);
+                CXXTOOLS_UNIT_ASSERT_EQUALS(boolean.end(2000), true);
+            }
+
+            // test failing connect in connect
+            {
+                cxxtools::json::HttpClient client(_loop);
+                cxxtools::RemoteProcedure<bool, bool, bool> boolean(client, "boolean");
+
+                CXXTOOLS_UNIT_ASSERT_NOTHROW(client.prepareConnect(_listen, _port + 1, "/rpc"));
+                CXXTOOLS_UNIT_ASSERT_THROW(client.connect(), cxxtools::IOError);
+            }
+
+            // test failing connect when calling function
+            {
+                cxxtools::json::HttpClient client(_loop);
+                cxxtools::RemoteProcedure<bool, bool, bool> boolean(client, "boolean");
+                CXXTOOLS_UNIT_ASSERT_NOTHROW(client.prepareConnect(_listen, _port + 1, "/rpc"));
+
+                boolean.begin(true, true);
+                CXXTOOLS_UNIT_ASSERT_THROW(boolean.end(2000), cxxtools::IOError);
+            }
+
+        }
+
+        ////////////////////////////////////////////////////////////
+        // Connect
+        //
+        void Connect()
+        {
+            cxxtools::json::HttpService service;
+            service.registerMethod("boolean", *this, &JsonRpcHttpTest::boolean);
+            _server->addService("/rpc", service);
+
+            // test connect using cxxtools::net::AddrInfo
+            {
+                cxxtools::json::HttpClient client(_loop);
+                cxxtools::RemoteProcedure<bool, bool, bool> boolean(client, "boolean");
+
+                client.connect(cxxtools::net::AddrInfo(_listen, _port), "/rpc");
+                boolean.begin(true, true);
+                CXXTOOLS_UNIT_ASSERT_EQUALS(boolean.end(2000), true);
+            }
+
+            // test connect using host and port
+            {
+                cxxtools::json::HttpClient client(_loop);
+                cxxtools::RemoteProcedure<bool, bool, bool> boolean(client, "boolean");
+
+                client.connect(_listen, _port, "/rpc");
+                boolean.begin(true, true);
+                CXXTOOLS_UNIT_ASSERT_EQUALS(boolean.end(2000), true);
+            }
+
+            // test connect using uri
+            {
+                cxxtools::json::HttpClient client(_loop);
+                cxxtools::RemoteProcedure<bool, bool, bool> boolean(client, "boolean");
+
+                std::ostringstream uri;
+                uri << "http://localhost:" << _port << "/rpc";
+                client.connect(cxxtools::net::Uri(uri.str()));
+                boolean.begin(true, true);
+                CXXTOOLS_UNIT_ASSERT_EQUALS(boolean.end(2000), true);
+            }
+
+            // test failing connect
+            {
+                cxxtools::json::HttpClient client(_loop);
+                cxxtools::RemoteProcedure<bool, bool, bool> boolean(client, "boolean");
+
+                CXXTOOLS_UNIT_ASSERT_THROW(client.connect(_listen, _port + 1, "/rpc"), cxxtools::IOError);
+            }
+
+        }
+
+        ////////////////////////////////////////////////////////////
+        // Multiple calls
+        //
+        void Multiple()
+        {
+            cxxtools::json::HttpService service;
+            service.registerMethod("multiply", *this, &JsonRpcHttpTest::multiplyDouble);
+            _server->addService("/rpc", service);
+
+            typedef cxxtools::RemoteProcedure<double, double, double> Multiply;
+
+            std::vector<cxxtools::json::HttpClient> clients;
+            std::vector<Multiply> procs;
+
+            clients.reserve(16);
+            procs.reserve(16);
+
+            for (unsigned i = 0; i < 16; ++i)
+            {
+                clients.push_back(cxxtools::json::HttpClient(_loop, _listen, _port, "/rpc"));
+                procs.push_back(Multiply(clients.back(), "multiply"));
+                procs.back().begin(i, i);
+            }
+
+            for (unsigned i = 0; i < 16; ++i)
+            {
+                CXXTOOLS_UNIT_ASSERT_EQUALS(procs[i].end(2000), i*i);
+            }
+
+        }
+
 };
 
 cxxtools::unit::RegisterTest<JsonRpcHttpTest> register_JsonRpcHttpTest;
