@@ -29,6 +29,12 @@
 #include "cxxtools/unit/testsuite.h"
 #include "cxxtools/unit/registertest.h"
 #include "cxxtools/query_params.h"
+#include "cxxtools/serializationinfo.h"
+#include "cxxtools/log.h"
+#include <vector>
+#include <iterator>
+
+log_define("cxxtools.test.queryparams")
 
 class QueryParamsTest : public cxxtools::unit::TestSuite
 {
@@ -43,8 +49,11 @@ class QueryParamsTest : public cxxtools::unit::TestSuite
             registerMethod("testParseDoublePercent", *this, &QueryParamsTest::testParseDoublePercent);
             registerMethod("testCount", *this, &QueryParamsTest::testCount);
             registerMethod("testCombine", *this, &QueryParamsTest::testCombine);
+            registerMethod("testSet", *this, &QueryParamsTest::testSet);
             registerMethod("testIterator", *this, &QueryParamsTest::testIterator);
             registerMethod("testGetUrl", *this, &QueryParamsTest::testGetUrl);
+            registerMethod("testGetNames", *this, &QueryParamsTest::testGetNames);
+            registerMethod("testDeserialization", *this, &QueryParamsTest::testDeserialization);
         }
 
         void testQueryParams()
@@ -129,6 +138,33 @@ class QueryParamsTest : public cxxtools::unit::TestSuite
             q.add(q2);
         }
 
+        void testSet()
+        {
+            cxxtools::QueryParams q;
+
+            q.set("a", "Hi");
+            CXXTOOLS_UNIT_ASSERT_EQUALS(q.getUrl(), "a=Hi");
+
+            q.set("b", "foo");
+            CXXTOOLS_UNIT_ASSERT_EQUALS(q.getUrl(), "a=Hi&b=foo");
+
+            q.set("a", "There");
+            CXXTOOLS_UNIT_ASSERT_EQUALS(q.getUrl(), "b=foo&a=There");
+
+            q.clear();
+            CXXTOOLS_UNIT_ASSERT_EQUALS(q.getUrl(), "");
+
+            q.add("a", "Hi");
+            CXXTOOLS_UNIT_ASSERT_EQUALS(q.getUrl(), "a=Hi");
+
+            q.add("b", "foo");
+            CXXTOOLS_UNIT_ASSERT_EQUALS(q.getUrl(), "a=Hi&b=foo");
+
+            q.add("a", "There");
+            CXXTOOLS_UNIT_ASSERT_EQUALS(q.getUrl(), "a=Hi&b=foo&a=There");
+
+        }
+
         void testIterator()
         {
             cxxtools::QueryParams q;
@@ -175,6 +211,85 @@ class QueryParamsTest : public cxxtools::unit::TestSuite
             std::string url = q.getUrl();
             CXXTOOLS_UNIT_ASSERT_EQUALS(url, "first+name=Tommi&last+name=M%A4kitalo&some+value");
         }
+
+        void testGetNames()
+        {
+            cxxtools::QueryParams q;
+            q.add("p1", "value1");
+            q.add("p2", "value2");
+            q.add("value3");
+
+            std::vector<std::string> names;
+            q.getNames(std::back_inserter(names));
+            CXXTOOLS_UNIT_ASSERT_EQUALS(names.size(), 3);
+            CXXTOOLS_UNIT_ASSERT_EQUALS(names[0], "p1");
+            CXXTOOLS_UNIT_ASSERT_EQUALS(names[1], "p2");
+            CXXTOOLS_UNIT_ASSERT_EQUALS(names[2], "");
+        }
+
+        void testDeserialization();
 };
 
 cxxtools::unit::RegisterTest<QueryParamsTest> register_QueryParamsTest;
+
+// helper classes for deserialization
+
+namespace
+{
+    struct Columns
+    {
+        std::string name;
+        std::string value;
+    };
+
+    struct TableQuery
+    {
+        bool draw;
+        std::vector<Columns> columns;
+        unsigned start;
+
+        TableQuery() : draw(false), start(0) { }
+    };
+
+    void operator>>= (const cxxtools::SerializationInfo& si, Columns& column)
+    {
+        si.getMember("name") >>= column.name;
+        si.getMember("value") >>= column.value;
+    }
+
+    void operator>>= (const cxxtools::SerializationInfo& si, TableQuery& query)
+    {
+        si.getMember("draw") >>= query.draw;
+        si.getMember("columns") >>= query.columns;
+        si.getMember("start") >>= query.start;
+    }
+
+}
+
+void QueryParamsTest::testDeserialization()
+{
+    cxxtools::QueryParams qp;
+    qp.add("draw", "1");
+    qp.add("columns[0][name]", "somename");
+    qp.add("columns[0][value]", "somevalue");
+    qp.add("columns[1][name]", "othername");
+    qp.add("columns[1][value]", "othervalue");
+    qp.add("start", "42");
+
+    cxxtools::SerializationInfo si;
+    si <<= qp;
+    CXXTOOLS_UNIT_ASSERT_EQUALS(si.memberCount(), 3);
+    CXXTOOLS_UNIT_ASSERT(si.findMember("draw") != 0);
+    CXXTOOLS_UNIT_ASSERT(si.findMember("columns") != 0);
+    CXXTOOLS_UNIT_ASSERT(si.findMember("start") != 0);
+
+    TableQuery query;
+    si >>= query;
+    CXXTOOLS_UNIT_ASSERT(query.draw);
+    CXXTOOLS_UNIT_ASSERT_EQUALS(query.columns.size(), 2);
+    CXXTOOLS_UNIT_ASSERT_EQUALS(query.columns[0].name, "somename");
+    CXXTOOLS_UNIT_ASSERT_EQUALS(query.columns[0].value, "somevalue");
+    CXXTOOLS_UNIT_ASSERT_EQUALS(query.columns[1].name, "othername");
+    CXXTOOLS_UNIT_ASSERT_EQUALS(query.columns[1].value, "othervalue");
+    CXXTOOLS_UNIT_ASSERT_EQUALS(query.start, 42);
+}
