@@ -38,7 +38,11 @@
 #include <cxxtools/mutex.h>
 #include <cxxtools/clock.h>
 #include <cxxtools/timespan.h>
-#include <cxxtools/atomicity.h>
+#include <cxxtools/datetime.h>
+#include <atomic>
+
+#include "color.h"
+
 
 class BenchClient
 {
@@ -48,10 +52,12 @@ class BenchClient
     cxxtools::AttachedThread thread;
 
     static unsigned _numRequests;
+    static cxxtools::DateTime _until;
     static unsigned _vectorSize;
-    static cxxtools::atomic_t _requestsStarted;
-    static cxxtools::atomic_t _requestsFinished;
-    static cxxtools::atomic_t _requestsFailed;
+    static unsigned _objectsSize;
+    static std::atomic<unsigned> _requestsStarted;
+    static std::atomic<unsigned> _requestsFinished;
+    static std::atomic<unsigned> _requestsFailed;
 
   public:
     explicit BenchClient(cxxtools::RemoteClient* client_)
@@ -68,20 +74,32 @@ class BenchClient
     static void numRequests(unsigned n)
     { _numRequests = n; }
 
+    static cxxtools::DateTime until()
+    { return _until; }
+
+    static void until(cxxtools::DateTime n)
+    { _until = n; }
+
     static unsigned vectorSize()
     { return _vectorSize; }
 
     static void vectorSize(unsigned n)
     { _vectorSize = n; }
 
+    static unsigned objectsSize()
+    { return _objectsSize; }
+
+    static void objectsSize(unsigned n)
+    { _objectsSize = n; }
+
     static unsigned requestsStarted()
-    { return static_cast<unsigned>(cxxtools::atomicGet(_requestsStarted)); }
+    { return _requestsStarted; }
 
     static unsigned requestsFinished()
-    { return static_cast<unsigned>(cxxtools::atomicGet(_requestsFinished)); }
+    { return _requestsFinished; }
 
     static unsigned requestsFailed()
-    { return static_cast<unsigned>(cxxtools::atomicGet(_requestsFailed)); }
+    { return _requestsFailed; }
 
     void start()
     { thread.start(); }
@@ -90,11 +108,13 @@ class BenchClient
     { thread.join(); }
 };
 
-cxxtools::atomic_t BenchClient::_requestsStarted(0);
-cxxtools::atomic_t BenchClient::_requestsFinished(0);
-cxxtools::atomic_t BenchClient::_requestsFailed(0);
+std::atomic<unsigned> BenchClient::_requestsStarted(0);
+std::atomic<unsigned> BenchClient::_requestsFinished(0);
+std::atomic<unsigned> BenchClient::_requestsFailed(0);
 unsigned BenchClient::_numRequests = 0;
+cxxtools::DateTime BenchClient::_until(2999, 12, 31, 23, 59, 59, 999);
 unsigned BenchClient::_vectorSize = 0;
+unsigned BenchClient::_objectsSize = 0;
 typedef std::vector<BenchClient*> BenchClients;
 
 static cxxtools::Mutex mutex;
@@ -103,29 +123,41 @@ void BenchClient::exec()
 {
   cxxtools::RemoteProcedure<std::string, std::string> echo(*client, "echo");
   cxxtools::RemoteProcedure<std::vector<int>, int, int> seq(*client, "seq");
+  cxxtools::RemoteProcedure<std::vector<Color>, unsigned> objects(*client, "objects");
 
-  while (static_cast<unsigned>(cxxtools::atomicIncrement(_requestsStarted)) <= _numRequests)
+  while (_requestsStarted <= _numRequests
+      && cxxtools::DateTime::gmtime() < _until)
   {
     try
     {
       if (_vectorSize > 0)
       {
         std::vector<int> ret = seq(1, _vectorSize);
-        cxxtools::atomicIncrement(_requestsFinished);
+        ++_requestsFinished;
         if (ret.size() != _vectorSize)
         {
           std::cerr << "wrong response result size " << ret.size() << std::endl;
-          cxxtools::atomicIncrement(_requestsFailed);
+          ++_requestsFailed;
+        }
+      }
+      else if (_objectsSize > 0)
+      {
+        std::vector<Color> ret = objects(_objectsSize);
+        ++_requestsFinished;
+        if (ret.size() != _objectsSize)
+        {
+          std::cerr << "wrong response result size " << ret.size() << std::endl;
+          ++_requestsFailed;
         }
       }
       else
       {
         std::string ret = echo("hi");
-        cxxtools::atomicIncrement(_requestsFinished);
+        ++_requestsFinished;
         if (ret != "hi")
         {
           std::cerr << "wrong response result \"" << ret << '"' << std::endl;
-          cxxtools::atomicIncrement(_requestsFailed);
+          ++_requestsFailed;
         }
       }
     }
@@ -136,7 +168,7 @@ void BenchClient::exec()
         std::cerr << "request failed with error message \"" << e.what() << '"' << std::endl;
       }
 
-      cxxtools::atomicIncrement(_requestsFailed);
+      ++_requestsFailed;
     }
   }
 }
@@ -154,8 +186,15 @@ int main(int argc, char* argv[])
     cxxtools::Arg<bool> json(argc, argv, 'j');
     cxxtools::Arg<bool> jsonhttp(argc, argv, 'J');
     cxxtools::Arg<unsigned short> port(argc, argv, 'p', binary ? 7003 : json ? 7004 : 7002);
+    cxxtools::Arg<bool> ssl(argc, argv, 's');
+    cxxtools::Arg<cxxtools::Seconds> maxtime(argc, argv, 'T');
+
     BenchClient::numRequests(cxxtools::Arg<unsigned>(argc, argv, 'n', 10000));
     BenchClient::vectorSize(cxxtools::Arg<unsigned>(argc, argv, 'v', 0));
+    BenchClient::objectsSize(cxxtools::Arg<unsigned>(argc, argv, 'o', 0));
+
+    if (maxtime.isSet())
+        BenchClient::until(cxxtools::DateTime::gmtime() + maxtime);
 
     if (!xmlrpc && !binary && !json && !jsonhttp)
     {
@@ -167,8 +206,12 @@ int main(int argc, char* argv[])
                      "   -b         use binary rpc protocol\n"
                      "   -j         use json rpc protocol\n"
                      "   -J         use json rpc over http protocol\n"
+                     "   -s         enable ssl\n"
                      "   -t number  set number of threads (default: 4)\n"
                      "   -n number  set number of requests (default: 10000)\n"
+                     "   -T seconds set maximum runtime after which the test stops\n"
+                     "   -v number  test int vector with <number> of elements\n"
+                     "   -o number  test vector of objects\n"
                      "one protocol must be selected\n"
                   << std::endl;
         return -1;
@@ -180,13 +223,25 @@ int main(int argc, char* argv[])
     {
       cxxtools::RemoteClient* client;
       if (binary)
-        client = new cxxtools::bin::RpcClient(ip, port);
+      {
+        cxxtools::bin::RpcClient* c = new cxxtools::bin::RpcClient(ip, port, ssl);
+        client = c;
+      }
       else if (json)
-        client = new cxxtools::json::RpcClient(ip, port);
+      {
+        cxxtools::json::RpcClient* c = new cxxtools::json::RpcClient(ip, port, ssl);
+        client = c;
+      }
       else if (jsonhttp)
-        client = new cxxtools::json::HttpClient(ip, port, "/jsonrpc");
+      {
+        cxxtools::json::HttpClient* c = new cxxtools::json::HttpClient(ip, port, "/jsonrpc", ssl);
+        client = c;
+      }
       else // if (xmlrpc)
-        client = new cxxtools::xmlrpc::HttpClient(ip, port, "/xmlrpc");
+      {
+        cxxtools::xmlrpc::HttpClient* c = new cxxtools::xmlrpc::HttpClient(ip, port, "/xmlrpc", ssl);
+        client = c;
+      }
 
       clients.push_back(new BenchClient(client));
     }
@@ -202,7 +257,7 @@ int main(int argc, char* argv[])
 
     cxxtools::Timespan t = cl.stop();
 
-    std::cout << BenchClient::numRequests() << " requests in " << t.totalMSecs()/1e3 << " s => " << (BenchClient::requestsStarted() / (t.totalMSecs()/1e3)) << "#/s\n"
+    std::cout << BenchClient::requestsStarted() << " requests in " << t.totalMSecs()/1e3 << " s => " << (BenchClient::requestsStarted() / (t.totalMSecs()/1e3)) << "#/s\n"
               << BenchClient::requestsFinished() << " finished " << BenchClient::requestsFailed() << " failed" << std::endl;
 
     for (BenchClients::iterator it = clients.begin(); it != clients.end(); ++it)
