@@ -26,10 +26,18 @@
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-#include <iostream>
 #include "cxxtools/serializationinfo.h"
+#include "cxxtools/serializationerror.h"
 #include "cxxtools/unit/testsuite.h"
 #include "cxxtools/unit/registertest.h"
+#include "cxxtools/convert.h"
+#include "cxxtools/log.h"
+
+log_define("cxxtools.unit.serializationinfo")
+
+#if __cplusplus >= 201103L
+#include <utility>
+#endif
 
 template <typename T>
 T siValue(const cxxtools::SerializationInfo& si)
@@ -52,12 +60,18 @@ class SerializationInfoTest : public cxxtools::unit::TestSuite
             registerMethod("testChar", *this, &SerializationInfoTest::testChar);
             registerMethod("testInt", *this, &SerializationInfoTest::testInt);
             registerMethod("testUInt", *this, &SerializationInfoTest::testUInt);
+            registerMethod("testFloat", *this, &SerializationInfoTest::testFloat);
             registerMethod("testDouble", *this, &SerializationInfoTest::testDouble);
+            registerMethod("testLongDouble", *this, &SerializationInfoTest::testLongDouble);
             registerMethod("testSiAssign", *this, &SerializationInfoTest::testSiAssign);
             registerMethod("testSiCopy", *this, &SerializationInfoTest::testSiCopy);
             registerMethod("testSiSwap", *this, &SerializationInfoTest::testSiSwap);
+#if __cplusplus >= 201103L
+            registerMethod("testMove", *this, &SerializationInfoTest::testMove);
+#endif
             registerMethod("testStringToBool", *this, &SerializationInfoTest::testStringToBool);
             registerMethod("testRangeCheck", *this, &SerializationInfoTest::testRangeCheck);
+            registerMethod("testMember", *this, &SerializationInfoTest::testMember);
         }
 
         void testSiSet()
@@ -88,7 +102,7 @@ class SerializationInfoTest : public cxxtools::unit::TestSuite
             CXXTOOLS_UNIT_ASSERT_EQUALS(siValue<short>(si), -17);
 
             si.setValue(0.0);
-            CXXTOOLS_UNIT_ASSERT(si.isFloat());
+            CXXTOOLS_UNIT_ASSERT(si.isDouble());
             CXXTOOLS_UNIT_ASSERT_EQUALS(cxxtools::convert<double>(siValue<std::string>(si)), 0);
             CXXTOOLS_UNIT_ASSERT_EQUALS(cxxtools::convert<double>(siValue<cxxtools::String>(si)), 0);
             CXXTOOLS_UNIT_ASSERT_EQUALS(siValue<unsigned>(si), 0);
@@ -107,6 +121,8 @@ class SerializationInfoTest : public cxxtools::unit::TestSuite
             CXXTOOLS_UNIT_ASSERT(!si.isInt());
             CXXTOOLS_UNIT_ASSERT(!si.isUInt());
             CXXTOOLS_UNIT_ASSERT(!si.isFloat());
+            CXXTOOLS_UNIT_ASSERT(!si.isDouble());
+            CXXTOOLS_UNIT_ASSERT(!si.isLongDouble());
 
             CXXTOOLS_UNIT_ASSERT_EQUALS(siValue<cxxtools::String>(si), cxxtools::String(L"42"));
             CXXTOOLS_UNIT_ASSERT_EQUALS(siValue<std::string>(si), "42");
@@ -128,6 +144,8 @@ class SerializationInfoTest : public cxxtools::unit::TestSuite
             CXXTOOLS_UNIT_ASSERT(!si.isInt());
             CXXTOOLS_UNIT_ASSERT(!si.isUInt());
             CXXTOOLS_UNIT_ASSERT(!si.isFloat());
+            CXXTOOLS_UNIT_ASSERT(!si.isDouble());
+            CXXTOOLS_UNIT_ASSERT(!si.isLongDouble());
 
             CXXTOOLS_UNIT_ASSERT_EQUALS(siValue<cxxtools::String>(si), cxxtools::String(L"42"));
             CXXTOOLS_UNIT_ASSERT_EQUALS(siValue<std::string>(si), "42");
@@ -149,6 +167,8 @@ class SerializationInfoTest : public cxxtools::unit::TestSuite
             CXXTOOLS_UNIT_ASSERT(!si.isInt());
             CXXTOOLS_UNIT_ASSERT(!si.isUInt());
             CXXTOOLS_UNIT_ASSERT(!si.isFloat());
+            CXXTOOLS_UNIT_ASSERT(!si.isDouble());
+            CXXTOOLS_UNIT_ASSERT(!si.isLongDouble());
 
             CXXTOOLS_UNIT_ASSERT_EQUALS(siValue<cxxtools::String>(si), cxxtools::String(L"7"));
             CXXTOOLS_UNIT_ASSERT_EQUALS(siValue<std::string>(si), "7");
@@ -170,6 +190,8 @@ class SerializationInfoTest : public cxxtools::unit::TestSuite
             CXXTOOLS_UNIT_ASSERT(si.isInt());
             CXXTOOLS_UNIT_ASSERT(!si.isUInt());
             CXXTOOLS_UNIT_ASSERT(!si.isFloat());
+            CXXTOOLS_UNIT_ASSERT(!si.isDouble());
+            CXXTOOLS_UNIT_ASSERT(!si.isLongDouble());
 
             CXXTOOLS_UNIT_ASSERT_EQUALS(siValue<cxxtools::String>(si), cxxtools::String(L"42"));
             CXXTOOLS_UNIT_ASSERT_EQUALS(siValue<std::string>(si), "42");
@@ -191,6 +213,8 @@ class SerializationInfoTest : public cxxtools::unit::TestSuite
             CXXTOOLS_UNIT_ASSERT(!si.isInt());
             CXXTOOLS_UNIT_ASSERT(si.isUInt());
             CXXTOOLS_UNIT_ASSERT(!si.isFloat());
+            CXXTOOLS_UNIT_ASSERT(!si.isDouble());
+            CXXTOOLS_UNIT_ASSERT(!si.isLongDouble());
 
             CXXTOOLS_UNIT_ASSERT_EQUALS(siValue<cxxtools::String>(si), cxxtools::String(L"42"));
             CXXTOOLS_UNIT_ASSERT_EQUALS(siValue<std::string>(si), "42");
@@ -201,6 +225,29 @@ class SerializationInfoTest : public cxxtools::unit::TestSuite
             CXXTOOLS_UNIT_ASSERT_EQUALS(siValue<double>(si), 42.0);
         }
 
+        void testFloat()
+        {
+            cxxtools::SerializationInfo si;
+            si.setValue(static_cast<float>(48.0));
+
+            CXXTOOLS_UNIT_ASSERT(!si.isString());
+            CXXTOOLS_UNIT_ASSERT(!si.isString8());
+            CXXTOOLS_UNIT_ASSERT(!si.isChar());
+            CXXTOOLS_UNIT_ASSERT(!si.isInt());
+            CXXTOOLS_UNIT_ASSERT(!si.isUInt());
+            CXXTOOLS_UNIT_ASSERT(si.isFloat());
+            CXXTOOLS_UNIT_ASSERT(!si.isDouble());
+            CXXTOOLS_UNIT_ASSERT(!si.isLongDouble());
+
+            CXXTOOLS_UNIT_ASSERT_EQUALS(cxxtools::convert<float>(siValue<std::string>(si)), 48);
+            CXXTOOLS_UNIT_ASSERT_EQUALS(cxxtools::convert<float>(siValue<cxxtools::String>(si)), 48);
+            CXXTOOLS_UNIT_ASSERT_EQUALS(siValue<char>(si), 48);
+            CXXTOOLS_UNIT_ASSERT_EQUALS(siValue<unsigned char>(si), static_cast<unsigned char>(48));
+            CXXTOOLS_UNIT_ASSERT_EQUALS(siValue<bool>(si), true);
+            CXXTOOLS_UNIT_ASSERT_EQUALS(siValue<int>(si), 48);
+            CXXTOOLS_UNIT_ASSERT_EQUALS(siValue<float>(si), 48.0);
+        }
+
         void testDouble()
         {
             cxxtools::SerializationInfo si;
@@ -211,7 +258,9 @@ class SerializationInfoTest : public cxxtools::unit::TestSuite
             CXXTOOLS_UNIT_ASSERT(!si.isChar());
             CXXTOOLS_UNIT_ASSERT(!si.isInt());
             CXXTOOLS_UNIT_ASSERT(!si.isUInt());
-            CXXTOOLS_UNIT_ASSERT(si.isFloat());
+            CXXTOOLS_UNIT_ASSERT(!si.isFloat());
+            CXXTOOLS_UNIT_ASSERT(si.isDouble());
+            CXXTOOLS_UNIT_ASSERT(!si.isLongDouble());
 
             CXXTOOLS_UNIT_ASSERT_EQUALS(cxxtools::convert<double>(siValue<std::string>(si)), 48);
             CXXTOOLS_UNIT_ASSERT_EQUALS(cxxtools::convert<double>(siValue<cxxtools::String>(si)), 48);
@@ -222,6 +271,29 @@ class SerializationInfoTest : public cxxtools::unit::TestSuite
             CXXTOOLS_UNIT_ASSERT_EQUALS(siValue<double>(si), 48.0);
         }
 
+        void testLongDouble()
+        {
+            cxxtools::SerializationInfo si;
+            si.setValue(48.0l);
+
+            CXXTOOLS_UNIT_ASSERT(!si.isString());
+            CXXTOOLS_UNIT_ASSERT(!si.isString8());
+            CXXTOOLS_UNIT_ASSERT(!si.isChar());
+            CXXTOOLS_UNIT_ASSERT(!si.isInt());
+            CXXTOOLS_UNIT_ASSERT(!si.isUInt());
+            CXXTOOLS_UNIT_ASSERT(!si.isFloat());
+            CXXTOOLS_UNIT_ASSERT(!si.isDouble());
+            CXXTOOLS_UNIT_ASSERT(si.isLongDouble());
+
+            CXXTOOLS_UNIT_ASSERT_EQUALS(cxxtools::convert<long double>(siValue<std::string>(si)), 48);
+            CXXTOOLS_UNIT_ASSERT_EQUALS(cxxtools::convert<long double>(siValue<cxxtools::String>(si)), 48);
+            CXXTOOLS_UNIT_ASSERT_EQUALS(siValue<char>(si), 48);
+            CXXTOOLS_UNIT_ASSERT_EQUALS(siValue<unsigned char>(si), static_cast<unsigned char>(48));
+            CXXTOOLS_UNIT_ASSERT_EQUALS(siValue<bool>(si), true);
+            CXXTOOLS_UNIT_ASSERT_EQUALS(siValue<int>(si), 48);
+            CXXTOOLS_UNIT_ASSERT_EQUALS(siValue<long double>(si), 48.0);
+        }
+
         void testSiAssign()
         {
             cxxtools::SerializationInfo si;
@@ -297,6 +369,49 @@ class SerializationInfoTest : public cxxtools::unit::TestSuite
 
         }
 
+#if __cplusplus >= 201103L
+        void testMove()
+        {
+            {
+                cxxtools::SerializationInfo si;
+                si.setValue(static_cast<unsigned short>(42));
+                si.addMember("foo");
+                si.addMember("bar");
+
+                cxxtools::SerializationInfo si2(std::move(si));
+
+                CXXTOOLS_UNIT_ASSERT(si2.isUInt());
+                CXXTOOLS_UNIT_ASSERT_EQUALS(siValue<unsigned>(si2), 42);
+                CXXTOOLS_UNIT_ASSERT_EQUALS(si.memberCount(), 0);
+                CXXTOOLS_UNIT_ASSERT_EQUALS(si2.memberCount(), 2);
+            }
+
+            {
+                cxxtools::SerializationInfo si;
+                const char str[] = "this is a string with aribtrary content";
+                si.setValue(str);
+
+                cxxtools::SerializationInfo si2;
+                si2.setValue(L"string to be replaced");
+                si2 = std::move(si);
+                CXXTOOLS_UNIT_ASSERT_EQUALS(siValue<std::string>(si2), str);
+                CXXTOOLS_UNIT_ASSERT(siValue<std::string>(si) != str);
+            }
+
+            {
+                cxxtools::SerializationInfo si;
+                const char str[] = "this is a string with aribtrary content";
+                si.setValue(str);
+
+                cxxtools::SerializationInfo si2(std::move(si));
+                CXXTOOLS_UNIT_ASSERT_EQUALS(siValue<std::string>(si2), str);
+                CXXTOOLS_UNIT_ASSERT(siValue<std::string>(si) != str);
+            }
+
+        }
+
+#endif
+
         void testSiSwap()
         {
             cxxtools::SerializationInfo si;
@@ -307,7 +422,7 @@ class SerializationInfoTest : public cxxtools::unit::TestSuite
             si1.setValue(17);
             si2.setValue(1.5);
             si1.swap(si2);
-            CXXTOOLS_UNIT_ASSERT(si1.isFloat());
+            CXXTOOLS_UNIT_ASSERT(si1.isDouble());
             CXXTOOLS_UNIT_ASSERT_EQUALS(siValue<double>(si1), 1.5);
             CXXTOOLS_UNIT_ASSERT(si2.isInt());
             CXXTOOLS_UNIT_ASSERT_EQUALS(siValue<int>(si2), 17);
@@ -334,7 +449,7 @@ class SerializationInfoTest : public cxxtools::unit::TestSuite
             si1.setValue("Hi");
             si2.setValue(1.5);
             si1.swap(si2);
-            CXXTOOLS_UNIT_ASSERT(si1.isFloat());
+            CXXTOOLS_UNIT_ASSERT(si1.isDouble());
             CXXTOOLS_UNIT_ASSERT_EQUALS(siValue<double>(si1), 1.5);
             CXXTOOLS_UNIT_ASSERT(si2.isString8());
             CXXTOOLS_UNIT_ASSERT_EQUALS(siValue<std::string>(si2), "Hi");
@@ -361,7 +476,7 @@ class SerializationInfoTest : public cxxtools::unit::TestSuite
             si1.setValue(cxxtools::String("Hi"));
             si2.setValue(1.5);
             si1.swap(si2);
-            CXXTOOLS_UNIT_ASSERT(si1.isFloat());
+            CXXTOOLS_UNIT_ASSERT(si1.isDouble());
             CXXTOOLS_UNIT_ASSERT_EQUALS(siValue<double>(si1), 1.5);
             CXXTOOLS_UNIT_ASSERT(si2.isString());
             CXXTOOLS_UNIT_ASSERT_EQUALS(siValue<std::string>(si2), "Hi");
@@ -439,8 +554,54 @@ class SerializationInfoTest : public cxxtools::unit::TestSuite
             si.setValue(static_cast<long>(std::numeric_limits<short>::max()) + 1);
             CXXTOOLS_UNIT_ASSERT_THROW(siValue<short>(si), std::range_error);
             CXXTOOLS_UNIT_ASSERT_NOTHROW(siValue<long>(si));
+
+            if (std::numeric_limits<long double>::max() > static_cast<long double>(std::numeric_limits<double>::max()))
+            {
+                si.setValue(static_cast<long double>(std::numeric_limits<double>::max()) * 1.01);
+                CXXTOOLS_UNIT_ASSERT_THROW(siValue<double>(si), std::range_error);
+                CXXTOOLS_UNIT_ASSERT_NOTHROW(siValue<long double>(si));
+
+                si.setValue(static_cast<long double>(-std::numeric_limits<double>::max()) * 1.01);
+                CXXTOOLS_UNIT_ASSERT_THROW(siValue<double>(si), std::range_error);
+                CXXTOOLS_UNIT_ASSERT_NOTHROW(siValue<long double>(si));
+            }
+            else
+            {
+                log_info("range error for double skipped since long double is not larger than double");
+            }
+
+            if (std::numeric_limits<long double>::max() > static_cast<long double>(std::numeric_limits<float>::max()))
+            {
+                si.setValue(static_cast<long double>(std::numeric_limits<float>::max()) * 1.01);
+                CXXTOOLS_UNIT_ASSERT_THROW(siValue<float>(si), std::range_error);
+                CXXTOOLS_UNIT_ASSERT_NOTHROW(siValue<double>(si));
+
+                si.setValue(static_cast<long double>(-std::numeric_limits<float>::max()) * 1.01);
+                CXXTOOLS_UNIT_ASSERT_THROW(siValue<float>(si), std::range_error);
+                CXXTOOLS_UNIT_ASSERT_NOTHROW(siValue<double>(si));
+            }
+            else
+            {
+                log_info("range error for float skipped since long double is not larger than double");
+            }
         }
 
+        void testMember()
+        {
+            cxxtools::SerializationInfo si;
+            si.addMember("foo");
+            si.addMember("bar");
+            si.addMember("baz");
+            si.addMember("foo");
+
+            CXXTOOLS_UNIT_ASSERT_EQUALS(si.memberCount(), 4);
+            CXXTOOLS_UNIT_ASSERT_NOTHROW(si.getMember("foo"));
+            CXXTOOLS_UNIT_ASSERT_THROW(si.getMember("fooo"), cxxtools::SerializationError);
+            CXXTOOLS_UNIT_ASSERT_EQUALS(si.getMember(0).name(), "foo");
+            CXXTOOLS_UNIT_ASSERT_EQUALS(si.getMember(1).name(), "bar");
+            CXXTOOLS_UNIT_ASSERT_EQUALS(si.getMember(2).name(), "baz");
+            CXXTOOLS_UNIT_ASSERT_EQUALS(si.getMember(3).name(), "foo");
+        }
 };
 
 cxxtools::unit::RegisterTest<SerializationInfoTest> register_SerializationInfoTest;
