@@ -27,13 +27,16 @@
  */
 #undef CXXTOOLS_API_EXPORT
 
-#include "cxxtools/api.h"
 #include "cxxtools/unit/testsuite.h"
 #include "cxxtools/unit/registertest.h"
 #include "cxxtools/string.h"
 #include <vector>
 #include <sstream>
 
+#if __cplusplus >= 201103L
+#include <utility>
+#endif
+
 
 class StringTest : public cxxtools::unit::TestSuite
 {
@@ -65,6 +68,9 @@ class StringTest : public cxxtools::unit::TestSuite
             cxxtools::unit::TestSuite::registerMethod( "testReserve", *this, &StringTest::testReserve );
             cxxtools::unit::TestSuite::registerMethod( "testReserveEmpty", *this, &StringTest::testReserveEmpty );
             cxxtools::unit::TestSuite::registerMethod( "testLengthAndSize", *this, &StringTest::testLengthAndSize );
+#if __cplusplus >= 201103L
+            cxxtools::unit::TestSuite::registerMethod( "testMove", *this, &StringTest::testMove );
+#endif
         }
 
     protected:
@@ -93,6 +99,9 @@ class StringTest : public cxxtools::unit::TestSuite
         void testReserve();
         void testReserveEmpty();
         void testLengthAndSize();
+#if __cplusplus >= 201103L
+        void testMove();
+#endif
 };
 
 cxxtools::unit::RegisterTest<StringTest> register_StringTest;
@@ -149,9 +158,8 @@ void StringTest::testConstructor()
     cxxtools::String s16(s11, 1, 3);
     CXXTOOLS_UNIT_ASSERT(s16 == c16);
 
-//  TODO API not implemented yet.
-//    cxxtools::String s20(s2.begin(), s2.end());
-//    CXXTOOLS_UNIT_ASSERT(s20 == L"abcde");
+    cxxtools::String s20(s2.begin(), s2.end());
+    CXXTOOLS_UNIT_ASSERT(s20 == L"abcde");
 }
 
 void StringTest::testCompare()
@@ -182,6 +190,19 @@ void StringTest::testCompare()
     CXXTOOLS_UNIT_ASSERT(y1 == y2);
     CXXTOOLS_UNIT_ASSERT(y1 == empty);
     CXXTOOLS_UNIT_ASSERT(y2 == empty);
+
+    CXXTOOLS_UNIT_ASSERT_EQUALS(s.compare(L"abcd"), 0);
+    CXXTOOLS_UNIT_ASSERT_EQUALS(s.compare(L"abc"), 1);
+    CXXTOOLS_UNIT_ASSERT_EQUALS(s.compare(L"abcde"), -1);
+
+    cxxtools::String zz(L"abcd\0ef", 7);
+    CXXTOOLS_UNIT_ASSERT_EQUALS(zz.compare(L"abcd"), 1);
+    CXXTOOLS_UNIT_ASSERT_EQUALS(zz.compare(L"abcde"), -1);
+    CXXTOOLS_UNIT_ASSERT_EQUALS(zz.compare(L"abcd\0ef", 7), 0);
+    CXXTOOLS_UNIT_ASSERT_EQUALS(zz.compare(L"abcd\0ee", 7), 1);
+    CXXTOOLS_UNIT_ASSERT_EQUALS(zz.compare(L"abcd\0eg", 7), -1);
+    CXXTOOLS_UNIT_ASSERT_EQUALS(zz.compare(L"abcd\0eff", 9), -1);
+    CXXTOOLS_UNIT_ASSERT_EQUALS(zz.compare(L"abcd\0eff", 5), 0);
 }
 
 void StringTest::testCompareShort()
@@ -236,6 +257,9 @@ void StringTest::testAssign()
     s.assign(z + 1, 0, 3);
     CXXTOOLS_UNIT_ASSERT(s == L"bcd");
 
+    s.assign(z + 1, 0, 10);
+    CXXTOOLS_UNIT_ASSERT(s == L"bcde");
+
     s.assign(3, 'x');
     CXXTOOLS_UNIT_ASSERT(s == L"xxx");
 
@@ -245,12 +269,11 @@ void StringTest::testAssign()
     s.assign(t, 1, 3);
     CXXTOOLS_UNIT_ASSERT(s == L"bcd");
 
-/*  TODO API not implemented yet.
     s.assign(v.begin(), v.end());
     CXXTOOLS_UNIT_ASSERT(s == L"abcde");
-*/
+
     s = s;
-    CXXTOOLS_UNIT_ASSERT(s == L"bcd");
+    CXXTOOLS_UNIT_ASSERT(s == L"abcde");
     s.assign(t);
     s = s.c_str();
     CXXTOOLS_UNIT_ASSERT(s == L"abcde");
@@ -282,11 +305,10 @@ void StringTest::testAppend()
     s.append(t, 1, 3);
     CXXTOOLS_UNIT_ASSERT(s == L"ABCbcd");
 
-/*  TODO API not implemented yet.
     s = L"ABC";
     s.append(v.begin(), v.end());
     CXXTOOLS_UNIT_ASSERT(s == L"ABCabcde");
-*/
+
     // operator +=
     s = L"ABC";
     s += z;
@@ -558,9 +580,10 @@ void StringTest::testSubstr()
 {
     cxxtools::String s(L"abcdefg");
 
-    CXXTOOLS_UNIT_ASSERT(s.substr()     == L"abcdefg");
-    CXXTOOLS_UNIT_ASSERT(s.substr(2)    == L"cdefg");
-    CXXTOOLS_UNIT_ASSERT(s.substr(2, 3) == L"cde");
+    CXXTOOLS_UNIT_ASSERT(s.substr()      == L"abcdefg");
+    CXXTOOLS_UNIT_ASSERT(s.substr(2)     == L"cdefg");
+    CXXTOOLS_UNIT_ASSERT(s.substr(2, 3)  == L"cde");
+    CXXTOOLS_UNIT_ASSERT(s.substr(0, 32) == L"abcdefg");
 }
 
 void StringTest::testSwap()
@@ -664,7 +687,6 @@ void StringTest::testReserveEmpty()
 {
     cxxtools::String s;
     s.reserve(0);
-    CXXTOOLS_UNIT_ASSERT( s.capacity() >= 0 );
     CXXTOOLS_UNIT_ASSERT_EQUALS( s.size() , 0 );
 }
 
@@ -694,3 +716,31 @@ void StringTest::testLengthAndSize()
     CXXTOOLS_UNIT_ASSERT_EQUALS(s3.length() , 4);
     CXXTOOLS_UNIT_ASSERT_EQUALS(s3.size()   , 4);
 }
+
+#if __cplusplus >= 201103L
+void StringTest::testMove()
+{
+    {
+        // test move constructor
+        wchar_t str[] = L"hi";
+        cxxtools::String s1(str);
+        cxxtools::String s2(std::move(s1));
+        CXXTOOLS_UNIT_ASSERT(s2 == str);
+        CXXTOOLS_UNIT_ASSERT(s1 == str);  // this is true since s1 is a short string and moving is just copying the data without affecting the source
+    }
+
+    {
+        // test move assignment
+        wchar_t str[] = L"This is a quite long string constant, which do not fit into a short string";
+
+        cxxtools::String s1(str);
+        CXXTOOLS_UNIT_ASSERT(s1 == str);
+
+        cxxtools::String s2;
+        s2 = std::move(s1);
+        CXXTOOLS_UNIT_ASSERT(s2 == str);
+        CXXTOOLS_UNIT_ASSERT(s1 != str);  // this is true since s1 is a long string and moving is moving the data from s1 to s2
+    }
+}
+
+#endif
