@@ -79,6 +79,7 @@ class XmlRpcCallbackTest : public cxxtools::unit::TestSuite
         cxxtools::EventLoop _loop;
         cxxtools::http::Server* _server;
         unsigned _count;
+        std::string _listen;
         unsigned short _port;
 
     public:
@@ -112,6 +113,10 @@ class XmlRpcCallbackTest : public cxxtools::unit::TestSuite
                 std::istringstream s(PORT);
                 s >> _port;
             }
+
+            char* LISTEN = getenv("UTEST_LISTEN");
+            if (LISTEN)
+                _listen = LISTEN;
         }
 
         void failTest()
@@ -125,7 +130,7 @@ class XmlRpcCallbackTest : public cxxtools::unit::TestSuite
             connect(_loop.timeout, *this, &XmlRpcCallbackTest::failTest);
             connect(_loop.timeout, _loop, &cxxtools::EventLoop::exit);
 
-            _server = new cxxtools::http::Server(_loop, "", _port);
+            _server = new cxxtools::http::Server(_loop, _listen, _port);
             _server->minThreads(1);
         }
 
@@ -143,7 +148,7 @@ class XmlRpcCallbackTest : public cxxtools::unit::TestSuite
             service.registerMethod("multiply", *this, &XmlRpcCallbackTest::multiplyNothing);
             _server->addService("/rpc", service);
 
-            cxxtools::xmlrpc::HttpClient client(_loop, "", _port, "/rpc");
+            cxxtools::xmlrpc::HttpClient client(_loop, _listen, _port, "/rpc");
             cxxtools::RemoteProcedure<bool> multiply(client, "multiply");
             connect( multiply.finished, *this, &XmlRpcCallbackTest::onNothingFinished );
 
@@ -152,7 +157,7 @@ class XmlRpcCallbackTest : public cxxtools::unit::TestSuite
             _loop.run();
         }
 
-        void onNothingFinished(const cxxtools::RemoteResult<bool>& r)
+        void onNothingFinished(cxxtools::RemoteResult<bool>& r)
         {
             CXXTOOLS_UNIT_ASSERT_EQUALS(r.get(), false);
 
@@ -173,7 +178,7 @@ class XmlRpcCallbackTest : public cxxtools::unit::TestSuite
             service.registerMethod("multiply", *this, &XmlRpcCallbackTest::multiplyNothing);
             _server->addService("/rpc", service);
 
-            cxxtools::xmlrpc::HttpClient client(_loop, "", _port, "/rpc");
+            cxxtools::xmlrpc::HttpClient client(_loop, _listen, _port, "/rpc");
             cxxtools::RemoteProcedure<bool> multiply(client, "multiply");
             connect( multiply.finished, *this, &XmlRpcCallbackTest::onExceptionCallback );
 
@@ -184,7 +189,7 @@ class XmlRpcCallbackTest : public cxxtools::unit::TestSuite
             CXXTOOLS_UNIT_ASSERT_EQUALS(_count, 1);
         }
 
-        void onExceptionCallback(const cxxtools::RemoteResult<bool>& r)
+        void onExceptionCallback(cxxtools::RemoteResult<bool>& /*r*/)
         {
             log_warn("exception callback");
             ++_count;
@@ -199,7 +204,7 @@ class XmlRpcCallbackTest : public cxxtools::unit::TestSuite
         {
             log_trace("ConnectError");
 
-            cxxtools::xmlrpc::HttpClient client(_loop, "", _port + 1, "/rpc");
+            cxxtools::xmlrpc::HttpClient client(_loop, _listen, _port + 1, "/rpc");
             cxxtools::RemoteProcedure<bool> multiply(client, "multiply");
             connect( multiply.finished, *this, &XmlRpcCallbackTest::onConnectErrorCallback );
 
@@ -216,7 +221,7 @@ class XmlRpcCallbackTest : public cxxtools::unit::TestSuite
             }
         }
 
-        void onConnectErrorCallback(const cxxtools::RemoteResult<bool>& r)
+        void onConnectErrorCallback(cxxtools::RemoteResult<bool>& r)
         {
             log_debug("onConnectErrorCallback");
             _loop.exit();
@@ -232,7 +237,7 @@ class XmlRpcCallbackTest : public cxxtools::unit::TestSuite
             service.registerMethod("multiply", *this, &XmlRpcCallbackTest::multiplyBoolean);
             _server->addService("/rpc", service);
 
-            cxxtools::xmlrpc::HttpClient client(_loop, "", _port, "/rpc");
+            cxxtools::xmlrpc::HttpClient client(_loop, _listen, _port, "/rpc");
             cxxtools::RemoteProcedure<bool, bool, bool> multiply(client, "multiply");
             connect( multiply.finished, *this, &XmlRpcCallbackTest::onBooleanFinished );
 
@@ -241,7 +246,7 @@ class XmlRpcCallbackTest : public cxxtools::unit::TestSuite
             _loop.run();
         }
 
-        void onBooleanFinished(const cxxtools::RemoteResult<bool>& r)
+        void onBooleanFinished(cxxtools::RemoteResult<bool>& r)
         {
             CXXTOOLS_UNIT_ASSERT_EQUALS(r.get(), true);
 
@@ -264,7 +269,7 @@ class XmlRpcCallbackTest : public cxxtools::unit::TestSuite
             service.registerMethod("multiply", *this, &XmlRpcCallbackTest::multiplyInt);
             _server->addService("/rpc", service);
 
-            cxxtools::xmlrpc::HttpClient client(_loop, "", _port, "/rpc");
+            cxxtools::xmlrpc::HttpClient client(_loop, _listen, _port, "/rpc");
             cxxtools::RemoteProcedure<int, int, int> multiply(client, "multiply");
             connect( multiply.finished, *this, &XmlRpcCallbackTest::onIntegerFinished );
 
@@ -273,7 +278,7 @@ class XmlRpcCallbackTest : public cxxtools::unit::TestSuite
             _loop.run();
         }
 
-        void onIntegerFinished(const cxxtools::RemoteResult<int>& r)
+        void onIntegerFinished(cxxtools::RemoteResult<int>& r)
         {
             CXXTOOLS_UNIT_ASSERT_EQUALS(r.get(), 6);
 
@@ -294,7 +299,7 @@ class XmlRpcCallbackTest : public cxxtools::unit::TestSuite
             service.registerMethod("multiply", *this, &XmlRpcCallbackTest::multiplyDouble);
             _server->addService("/rpc", service);
 
-            cxxtools::xmlrpc::HttpClient client(_loop, "", _port, "/rpc");
+            cxxtools::xmlrpc::HttpClient client(_loop, _listen, _port, "/rpc");
             cxxtools::RemoteProcedure<double, double, double> multiply(client, "multiply");
             connect( multiply.finished, *this, &XmlRpcCallbackTest::onDoubleFinished );
 
@@ -303,7 +308,7 @@ class XmlRpcCallbackTest : public cxxtools::unit::TestSuite
             _loop.run();
         }
 
-        void onDoubleFinished(const cxxtools::RemoteResult<double>& r)
+        void onDoubleFinished(cxxtools::RemoteResult<double>& r)
         {
             CXXTOOLS_UNIT_ASSERT_EQUALS(r.get(), 6.0);
 
@@ -324,7 +329,7 @@ class XmlRpcCallbackTest : public cxxtools::unit::TestSuite
             service.registerMethod("echoString", *this, &XmlRpcCallbackTest::echoString);
             _server->addService("/foo", service);
 
-            cxxtools::xmlrpc::HttpClient client(_loop, "", _port, "/foo");
+            cxxtools::xmlrpc::HttpClient client(_loop, _listen, _port, "/foo");
             cxxtools::RemoteProcedure<std::string, std::string> echo(client, "echoString");
             connect( echo.finished, *this, &XmlRpcCallbackTest::onStringEchoFinished );
 
@@ -333,7 +338,7 @@ class XmlRpcCallbackTest : public cxxtools::unit::TestSuite
             _loop.run();
         }
 
-        void onStringEchoFinished(const cxxtools::RemoteResult<std::string>& r)
+        void onStringEchoFinished(cxxtools::RemoteResult<std::string>& r)
         {
             CXXTOOLS_UNIT_ASSERT_EQUALS(r.get(), "\xc3\xaf\xc2\xbb\xc2\xbf'\"&<> foo?");
 
@@ -354,7 +359,7 @@ class XmlRpcCallbackTest : public cxxtools::unit::TestSuite
             service.registerMethod("multiply", *this, &XmlRpcCallbackTest::multiplyEmpty);
             _server->addService("/rpc", service);
 
-            cxxtools::xmlrpc::HttpClient client(_loop, "", _port, "/rpc");
+            cxxtools::xmlrpc::HttpClient client(_loop, _listen, _port, "/rpc");
             cxxtools::RemoteProcedure<std::string, std::string, std::string> multiply(client, "multiply");
             connect( multiply.finished, *this, &XmlRpcCallbackTest::onEmptyFinished );
 
@@ -363,7 +368,7 @@ class XmlRpcCallbackTest : public cxxtools::unit::TestSuite
             _loop.run();
         }
 
-        void onEmptyFinished(const cxxtools::RemoteResult<std::string>& r)
+        void onEmptyFinished(cxxtools::RemoteResult<std::string>& r)
         {
             CXXTOOLS_UNIT_ASSERT_EQUALS(r.get(), "4");
             _loop.exit();
@@ -385,7 +390,7 @@ class XmlRpcCallbackTest : public cxxtools::unit::TestSuite
             service.registerMethod("multiply", *this, &XmlRpcCallbackTest::multiplyVector);
             _server->addService("/rpc", service);
 
-            cxxtools::xmlrpc::HttpClient client(_loop, "", _port, "/rpc");
+            cxxtools::xmlrpc::HttpClient client(_loop, _listen, _port, "/rpc");
             cxxtools::RemoteProcedure< std::vector<int>, std::vector<int>, std::vector<int> > multiply(client, "multiply");
             connect( multiply.finished, *this, &XmlRpcCallbackTest::onArrayFinished );
 
@@ -410,7 +415,7 @@ class XmlRpcCallbackTest : public cxxtools::unit::TestSuite
             return r;
         }
 
-        void onArrayFinished(const cxxtools::RemoteResult<std::vector<int> >& r)
+        void onArrayFinished(cxxtools::RemoteResult<std::vector<int> >& r)
         {
             CXXTOOLS_UNIT_ASSERT_EQUALS(r.get().size(), 2);
             CXXTOOLS_UNIT_ASSERT_EQUALS(r.get().at(0), 100);
@@ -428,7 +433,7 @@ class XmlRpcCallbackTest : public cxxtools::unit::TestSuite
             service.registerMethod("multiply", *this, &XmlRpcCallbackTest::multiplyVector);
             _server->addService("/rpc", service);
 
-            cxxtools::xmlrpc::HttpClient client(_loop, "", _port, "/rpc");
+            cxxtools::xmlrpc::HttpClient client(_loop, _listen, _port, "/rpc");
             cxxtools::RemoteProcedure< std::vector<int>, std::vector<int>, std::vector<int> > multiply(client, "multiply");
             connect( multiply.finished, *this, &XmlRpcCallbackTest::onEmptyArrayFinished );
 
@@ -438,7 +443,7 @@ class XmlRpcCallbackTest : public cxxtools::unit::TestSuite
             _loop.run();
         }
 
-        void onEmptyArrayFinished(const cxxtools::RemoteResult<std::vector<int> >& r)
+        void onEmptyArrayFinished(cxxtools::RemoteResult<std::vector<int> >& r)
         {
             CXXTOOLS_UNIT_ASSERT_EQUALS(r.get().size(), 0);
 
@@ -454,7 +459,7 @@ class XmlRpcCallbackTest : public cxxtools::unit::TestSuite
             service.registerMethod("multiply", *this, &XmlRpcCallbackTest::multiplyColor);
             _server->addService("/rpc", service);
 
-            cxxtools::xmlrpc::HttpClient client(_loop, "", _port, "/rpc");
+            cxxtools::xmlrpc::HttpClient client(_loop, _listen, _port, "/rpc");
             cxxtools::RemoteProcedure< Color, Color, Color > multiply(client, "multiply");
             connect( multiply.finished, *this, &XmlRpcCallbackTest::onStuctFinished );
 
@@ -473,7 +478,7 @@ class XmlRpcCallbackTest : public cxxtools::unit::TestSuite
             _loop.run();
         }
 
-        void onStuctFinished(const cxxtools::RemoteResult<Color>& color)
+        void onStuctFinished(cxxtools::RemoteResult<Color>& color)
         {
             CXXTOOLS_UNIT_ASSERT_EQUALS(color.get().red, 6);
             CXXTOOLS_UNIT_ASSERT_EQUALS(color.get().green, 12);
@@ -500,7 +505,7 @@ class XmlRpcCallbackTest : public cxxtools::unit::TestSuite
             service.registerMethod("multiplyset", *this, &XmlRpcCallbackTest::multiplySet);
             _server->addService("/test", service);
 
-            cxxtools::xmlrpc::HttpClient client(_loop, "", _port, "/test");
+            cxxtools::xmlrpc::HttpClient client(_loop, _listen, _port, "/test");
             cxxtools::RemoteProcedure<IntSet, IntSet, int> multiply(client, "multiplyset");
             connect( multiply.finished, *this, &XmlRpcCallbackTest::onSetFinished );
 
@@ -515,7 +520,7 @@ class XmlRpcCallbackTest : public cxxtools::unit::TestSuite
             _loop.run();
         }
 
-        void onSetFinished(const cxxtools::RemoteResult<IntSet>& result)
+        void onSetFinished(cxxtools::RemoteResult<IntSet>& result)
         {
             const IntSet& v = result.get();
             CXXTOOLS_UNIT_ASSERT_EQUALS(v.size(), 3);
@@ -543,7 +548,7 @@ class XmlRpcCallbackTest : public cxxtools::unit::TestSuite
             service.registerMethod("multiplyset", *this, &XmlRpcCallbackTest::multiplyMultiset);
             _server->addService("/test", service);
 
-            cxxtools::xmlrpc::HttpClient client(_loop, "", _port, "/test");
+            cxxtools::xmlrpc::HttpClient client(_loop, _listen, _port, "/test");
             cxxtools::RemoteProcedure<IntMultiset, IntMultiset, int> multiply(client, "multiplyset");
             connect( multiply.finished, *this, &XmlRpcCallbackTest::onMultisetFinished );
 
@@ -558,7 +563,7 @@ class XmlRpcCallbackTest : public cxxtools::unit::TestSuite
             _loop.run();
         }
 
-        void onMultisetFinished(const cxxtools::RemoteResult<IntMultiset>& result)
+        void onMultisetFinished(cxxtools::RemoteResult<IntMultiset>& result)
         {
             const IntMultiset& v = result.get();
             CXXTOOLS_UNIT_ASSERT_EQUALS(v.size(), 4);
@@ -587,7 +592,7 @@ class XmlRpcCallbackTest : public cxxtools::unit::TestSuite
 
             _server->addService("/test", service);
 
-            cxxtools::xmlrpc::HttpClient client(_loop, "", _port, "/test");
+            cxxtools::xmlrpc::HttpClient client(_loop, _listen, _port, "/test");
             cxxtools::RemoteProcedure<IntMap, IntMap, int> multiply(client, "multiplymap");
             connect( multiply.finished, *this, &XmlRpcCallbackTest::onMultiplyMapFinished );
 
@@ -601,7 +606,7 @@ class XmlRpcCallbackTest : public cxxtools::unit::TestSuite
             _loop.run();
         }
 
-        void onMultiplyMapFinished(const cxxtools::RemoteResult<IntMap>& result)
+        void onMultiplyMapFinished(cxxtools::RemoteResult<IntMap>& result)
         {
             const IntMap& v = result.get();
             CXXTOOLS_UNIT_ASSERT_EQUALS(v.size(), 3);
@@ -636,7 +641,7 @@ class XmlRpcCallbackTest : public cxxtools::unit::TestSuite
 
             _server->addService("/test", service);
 
-            cxxtools::xmlrpc::HttpClient client(_loop, "", _port, "/test");
+            cxxtools::xmlrpc::HttpClient client(_loop, _listen, _port, "/test");
             cxxtools::RemoteProcedure<IntMultimap, IntMultimap, int> multiply(client, "multiplymultimap");
             connect( multiply.finished, *this, &XmlRpcCallbackTest::onMultiplyMultimapFinished );
 
@@ -651,7 +656,7 @@ class XmlRpcCallbackTest : public cxxtools::unit::TestSuite
             _loop.run();
         }
 
-        void onMultiplyMultimapFinished(const cxxtools::RemoteResult<IntMultimap>& result)
+        void onMultiplyMultimapFinished(cxxtools::RemoteResult<IntMultimap>& result)
         {
             const IntMultimap& v = result.get();
             CXXTOOLS_UNIT_ASSERT_EQUALS(v.size(), 4);
@@ -683,7 +688,7 @@ class XmlRpcCallbackTest : public cxxtools::unit::TestSuite
 
         void UnknownMethod()
         {
-            cxxtools::xmlrpc::HttpClient client(_loop, "", _port, "/test");
+            cxxtools::xmlrpc::HttpClient client(_loop, _listen, _port, "/test");
             cxxtools::RemoteProcedure<bool, bool, bool> unknownMethod(client, "unknownMethod");
             connect( unknownMethod.finished, *this, &XmlRpcCallbackTest::onBooleanFinished );
 
@@ -701,7 +706,7 @@ class XmlRpcCallbackTest : public cxxtools::unit::TestSuite
             service.registerMethod("multiply", *this, &XmlRpcCallbackTest::throwFault);
             _server->addService("/rpc", service);
 
-            cxxtools::xmlrpc::HttpClient client(_loop, "", _port, "/rpc");
+            cxxtools::xmlrpc::HttpClient client(_loop, _listen, _port, "/rpc");
             cxxtools::RemoteProcedure<bool> multiply(client, "multiply");
             connect( multiply.finished, *this, &XmlRpcCallbackTest::onFault );
             multiply.begin();
@@ -709,7 +714,7 @@ class XmlRpcCallbackTest : public cxxtools::unit::TestSuite
             _loop.run();
         }
 
-        void onFault(const cxxtools::RemoteResult<bool>& result)
+        void onFault(cxxtools::RemoteResult<bool>& result)
         {
             try
             {
@@ -740,7 +745,7 @@ class XmlRpcCallbackTest : public cxxtools::unit::TestSuite
             service.registerMethod("multiply", *this, &XmlRpcCallbackTest::throwException);
             _server->addService("/rpc", service);
 
-            cxxtools::xmlrpc::HttpClient client(_loop, "", _port, "/rpc");
+            cxxtools::xmlrpc::HttpClient client(_loop, _listen, _port, "/rpc");
             cxxtools::RemoteProcedure<bool> multiply(client, "multiply");
             connect( multiply.finished, *this, &XmlRpcCallbackTest::onException );
             multiply.begin();
@@ -748,7 +753,7 @@ class XmlRpcCallbackTest : public cxxtools::unit::TestSuite
             _loop.run();
         }
 
-        void onException(const cxxtools::RemoteResult<bool>& result)
+        void onException(cxxtools::RemoteResult<bool>& result)
         {
             try
             {
@@ -775,13 +780,11 @@ class XmlRpcCallbackTest : public cxxtools::unit::TestSuite
         //
         void BigRequest()
         {
-            log_trace("ConnectError");
-
             cxxtools::xmlrpc::Service service;
             service.registerMethod("countSize", *this, &XmlRpcCallbackTest::countSize);
             _server->addService("/rpc", service);
 
-            cxxtools::xmlrpc::HttpClient client(_loop, "", _port, "/rpc");
+            cxxtools::xmlrpc::HttpClient client(_loop, _listen, _port, "/rpc");
             cxxtools::RemoteProcedure<unsigned, std::vector<int> > countSize(client, "countSize");
             connect( countSize.finished, *this, &XmlRpcCallbackTest::onCountSizeFinished );
 
@@ -801,7 +804,7 @@ class XmlRpcCallbackTest : public cxxtools::unit::TestSuite
             }
         }
 
-        void onCountSizeFinished(const cxxtools::RemoteResult<unsigned>& r)
+        void onCountSizeFinished(cxxtools::RemoteResult<unsigned>& r)
         {
             CXXTOOLS_UNIT_ASSERT_EQUALS(r.get(), 5000);
             _loop.exit();
